//This is the class used to pull up and Process the Coronal Options Window

//Import Statements
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;

public class coronalOptions extends JPanel implements ActionListener {
	//Private Members of this Class
	private JPanel outGrid, paramGrid;
	private JCheckBox vr, vt, vp, br, bt, bp, rho, T, P, jr, jt, jp, ar, at, ap;
	private JTextField tmax, out_rad, out_int, dt_max, dt_min, lund, visc;
	private JButton generate; 
	private GridBagLayout gridBag;
	private GridBagConstraints gridCon;
	private JLabel holder;	
	private JFrame myframe;
	private String output_params, tmax_s, out_rad_s, out_int_s, dt_max_s, dt_min_s, lund_s, visc_s;

	public coronalOptions(JFrame frm) {
		//Coronal Options Constructor
		//This Ctor will create a new window for the Coronal Options
		//Generate top level grid layout
		super();

		myframe = frm;

		gridBag = new GridBagLayout();
		gridCon = new GridBagConstraints();
		setLayout(gridBag);
		gridCon.gridwidth = GridBagConstraints.REMAINDER;

		//Add the first label to the top level grid
		holder = new JLabel("Output Parameters:");
		gridBag.setConstraints(holder, gridCon);
		add(holder);
		
		//Initialize output Grid
		outGrid = new JPanel(new GridLayout(3,5,10,10));
		
		//Initialize Check boxes for output grid
		vr = new JCheckBox("Vr", true);
		vt = new JCheckBox("Vt", true);
		vp = new JCheckBox("Vp", true);
		br = new JCheckBox("Br", true);
		bt = new JCheckBox("Bt", true);
		bp = new JCheckBox("Bp", true);
		rho = new JCheckBox("Rho", true);
		T = new JCheckBox("T");
		P = new JCheckBox("P");
		jr = new JCheckBox("Jr");
		jt = new JCheckBox("Jt");
		jp = new JCheckBox("Jp");
		ar = new JCheckBox("ar");
		at  = new JCheckBox("at");
		ap = new JCheckBox("ap");

		//Add check boxes to output grid
		outGrid.add(vr);
		outGrid.add(br);
		outGrid.add(rho);
		outGrid.add(jr);
		outGrid.add(ar);
		outGrid.add(vt);
        outGrid.add(bt);
        outGrid.add(T);
        outGrid.add(jt);
        outGrid.add(at);
		outGrid.add(vp);
        outGrid.add(bp);
        outGrid.add(P);
        outGrid.add(jp);
        outGrid.add(ap);

		//Add output grid to top level grid
		gridBag.setConstraints(outGrid, gridCon);
		add(outGrid);

		//Initialize Parameter Grid
		paramGrid = new JPanel(new GridLayout(7,2,10,10));		

		//Initialize Text Fields
		tmax = new JTextField("200");
		out_rad = new JTextField("30");
		out_int = new JTextField("20");
		dt_max = new JTextField("0.05");
		dt_min = new JTextField("0.005");
		lund = new JTextField("1.0e3");
		visc = new JTextField("0.01");

		//Add The Labels and the text fields to the parameter grid
		paramGrid.add(new JLabel("Maximum Time:"));
		paramGrid.add(tmax);
		paramGrid.add(new JLabel("Outer Radial Boundary:"));
		paramGrid.add(out_rad);
		paramGrid.add(new JLabel("Output Interval:"));
		paramGrid.add(out_int);
		paramGrid.add(new JLabel("dt Max:"));
		paramGrid.add(dt_max);
		paramGrid.add(new JLabel("dt Min:"));
		paramGrid.add(dt_min);
		paramGrid.add(new JLabel("Lundquist Number:"));
		paramGrid.add(lund);
		paramGrid.add(new JLabel("Viscosity"));
		paramGrid.add(visc);

		//add parameter grid to top level grid
		gridBag.setConstraints(paramGrid, gridCon);
		add(paramGrid);

		//Initialize & Add button to middle section
		generate = new JButton("Save Input File Parameters");
		generate.addActionListener(this);
		generate.setActionCommand("generate");
		gridBag.setConstraints(generate, gridCon);
		add(generate);

		//Now Initialize the default strings for imas input
		output_params = "'vr','vt','vp','br','bt','bp','rho'";
		tmax_s = "200";
		out_rad_s = "29";
		out_int_s = "20";
		dt_max_s = "0.05";
		dt_min_s = "0.005";
		lund_s = "1.e3";
		visc_s = "0.01";
	}

	public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand() == "generate") {
			//First thing to do it Validate and Clamp the input
			int tmax_parse, out_int_parse, out_rad_parse;
			double dt_min_parse, dt_max_parse, lund_parse, visc_parse;

			//Parse tmax
			try {
				tmax_parse = Integer.parseInt(tmax.getText());
			}
			catch (NumberFormatException exc) {
				//Reset offending input to its default
				tmax.setText("200");
				tmax_parse = 200;
			}
			if (tmax_parse > 200) {
				tmax_parse = 200;
				tmax.setText("200");
			}
			if (tmax_parse < 0) {
				tmax_parse = 0;
				tmax.setText("0");
			}
			tmax_s = tmax.getText();	//Passes all the tests

			//Parse out_rad
			try {
                out_rad_parse = Integer.parseInt(out_rad.getText());
            }
            catch (NumberFormatException exc) {
                //Reset offending input to its default
                out_rad.setText("30");
                out_rad_parse = 30;
            }
            if (out_rad_parse > 30) {
                out_rad_parse = 30;
                out_rad.setText("30");
            }
            if (out_rad_parse < 1) {
                out_rad_parse = 1;
                out_rad.setText("1");
            }
            out_rad_s = Integer.toString(out_rad_parse - 1);

			//Parse out_int
            try {   
                out_int_parse = Integer.parseInt(out_int.getText());
            }       
            catch (NumberFormatException exc) {
                //Reset offending input to its default
                out_int.setText("20");
                out_int_parse = 20;
            }       
            if (out_int_parse > tmax_parse) { 
                out_int_parse = tmax_parse;
                out_int.setText(tmax.getText());
            }       
            if (out_int_parse < 1) {
                out_int_parse = 1;
                out_int.setText("1");
            }       
            out_int_s = out_int.getText();

			//Parse dt_max
            try {   
                dt_max_parse = Double.parseDouble(dt_max.getText());
            }       
            catch (NumberFormatException exc) {
                //Reset offending input to its default
                dt_max.setText("0.05");
                dt_max_parse = 0.05;
            }       
            if (dt_max_parse > 1.0) { 
                dt_max_parse = 1.0;
                dt_max.setText("1.0");
            }       
            if (dt_max_parse < 0.01) {
                dt_max_parse = 0.01;
                dt_max.setText("0.01");
            }       
            dt_max_s = dt_max.getText();

			//Parse dt_min
            try {   
                dt_min_parse = Double.parseDouble(dt_min.getText());
            }       
            catch (NumberFormatException exc) {
                //Reset offending input to its default
                dt_min.setText("0.005");
                dt_min_parse = 0.005; 
            }       
            if (dt_min_parse > dt_max_parse) {  
                dt_min_parse = dt_max_parse;
                dt_min.setText(dt_max.getText());
            }       
            if (dt_min_parse < 0.001) { 
                dt_min_parse = 0.001; 
                dt_min.setText("0.001");
            }       
            dt_min_s = dt_min.getText();

			//Parse lund
            try {   
                lund_parse = Double.parseDouble(lund.getText());
            }       
            catch (NumberFormatException exc) {
                //Reset offending input to its default
                lund.setText("1.0e3");
                lund_parse = 1.0e3; 
            }       
            if (lund_parse <= 0) { 
                lund_parse = 1.0e3; 
                lund.setText("1.0e3");
            }       
            lund_s = lund.getText();

			//Parse visc
            try {   
                visc_parse = Double.parseDouble(visc.getText());
            }       
            catch (NumberFormatException exc) {
                //Reset offending input to its default
                visc.setText("0.01");
                visc_parse = 0.01; 
            }       
            if (visc_parse <= 0) { 
                visc_parse = 0.01; 
                visc.setText("0.01");
            }       
            visc_s = visc.getText();

			//Create Output Params String
			int counter = 0;
			output_params = "";
			if (vr.isSelected()) {
				if (counter == 0) {
					output_params += "'vr'";
				} else {
					output_params += ",'vr'";
				}
				if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (vt.isSelected()) {
                if (counter == 0) {
                    output_params += "'vt'";
                } else {
                    output_params += ",'vt'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (vp.isSelected()) {
                if (counter == 0) {
                    output_params += "'vp'";
                } else {
                    output_params += ",'vp'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (br.isSelected()) {
                if (counter == 0) {
                    output_params += "'br'";
                } else {
                    output_params += ",'br'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (bt.isSelected()) {
                if (counter == 0) {
                    output_params += "'bt'";
                } else {
                    output_params += ",'bt'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (bp.isSelected()) {
                if (counter == 0) {
                    output_params += "'bp'";
                } else {
                    output_params += ",'bp'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
            }
			if (rho.isSelected()) {
                if (counter == 0) {
                    output_params += "'rho'";
                } else {
                    output_params += ",'rho'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (T.isSelected()) {
                if (counter == 0) {
                    output_params += "'t'";
                } else {
                    output_params += ",'t'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (P.isSelected()) {
                if (counter == 0) {
                    output_params += "'p'";
                } else {
                    output_params += ",'p'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (jr.isSelected()) {
                if (counter == 0) {
                    output_params += "'jr'";
                } else {
                    output_params += ",'jr'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (jt.isSelected()) {
                if (counter == 0) {
                    output_params += "'jt'";
                } else {
                    output_params += ",'jt'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (jp.isSelected()) {
                if (counter == 0) {
                    output_params += "'jp'";
                } else {
                    output_params += ",'jp'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (ar.isSelected()) {
                if (counter == 0) {
                    output_params += "'ar'";
                } else {
                    output_params += ",'ar'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (at.isSelected()) {
                if (counter == 0) {
                    output_params += "'at'";
                } else {
                    output_params += ",'at'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}
			if (ap.isSelected()) {
                if (counter == 0) {
                    output_params += "'ap'";
                } else {
                    output_params += ",'ap'";
                }
                if (counter == 5 || counter == 10) output_params += "\n ";
				counter++;
			}

			myframe.setVisible(false);
		}
    }

	public String getOutputParams() {
		return output_params;
	}

	public String getMaxTime() {
		return tmax_s;
	}

	public String getOuterRadial() {
		return out_rad_s;
	}

	public String getOutputInt() {
		return out_int_s;
	}

	public String getDtMax() {
		return dt_max_s;
	}

	public String getDtMin() {
		return dt_min_s;
	}

	public String getLundNum() {
		return lund_s;
	}

	public String getViscosity() {
		return visc_s;
	}

	public void reset() {
		//Reset all of the coronal Options and reprocess inout
		vr.setSelected(true);
		vt.setSelected(true);
		vp.setSelected(true);
		br.setSelected(true);
		bt.setSelected(true);
		bp.setSelected(true);
		rho.setSelected(true);
		T.setSelected(false);
		P.setSelected(false);
		jr.setSelected(false);
		jt.setSelected(false);
		jp.setSelected(false);
		ar.setSelected(false);
		at.setSelected(false);
		ap.setSelected(false);

		tmax.setText("200");
		out_rad.setText("30");
		out_int.setText("20");
		dt_max.setText("0.05");
		dt_min.setText("0.005");
		lund.setText("1.0e3");
		visc.setText("0.01");

		//Reparse the Options
		actionPerformed(new ActionEvent(generate, ActionEvent.ACTION_PERFORMED, "generate"));
	}
};
