c
c-----------------------------------------------------------------------
c
c ****** Macro expander.
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        02/10/2000, ZM, Version 1.00:
c
c         - Original version of program.
c           Added a command-line iterface to the original version.
c
c        07/29/2003, ZM, Version 1.01:
c
c         - This version uses the ZM tools libraries.
c
c#######################################################################
c
c ****** Note that this tool uses modules from ZM's tools library.
c
c#######################################################################
      module params
c
c-----------------------------------------------------------------------
c ****** Parameters.
c-----------------------------------------------------------------------
c
      implicit none
c
      character(256) :: infile,outfile
      integer :: idebug
c
      end module
c#######################################################################
      program EXPMAC
c
c-----------------------------------------------------------------------
c
      use number_types
      use params
c
c-----------------------------------------------------------------------
c
      character(*), parameter :: cname='EXPMAC'
      character(*), parameter :: cvers='1.01'
      character(*), parameter :: cdate='07/29/2003'
c
c-----------------------------------------------------------------------
c
c ****** Input and output line buffers.
c
      parameter (LBUF=2048)
      character*(LBUF) ibuf,obuf
c
c ****** Macro definition tables.
c
      parameter (nmacmx=100)
      parameter (LMNAM=64)
      parameter (LMRPL=512)
      character*(LMNAM) macnam
      character*(LMRPL) macrpl
      common /mdefcom1/ macnam(nmacmx),macrpl(nmacmx)
      common /mdefcom2/ nmac,macnarg(nmacmx)
c
      logical :: mdef,mres
      integer :: ierr
c
      character*80 line
c
c-----------------------------------------------------------------------
c
      logical, external :: macdef,macres
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call set_parameters (cname,cvers,cdate)
c
c ****** Check that the input and output file names are not identical.
c
      if (infile.eq.outfile) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Input and output file names are identical.'
        call exit (1)
      end if
c
c ****** Open the input and output files.
c
      call ffopen (1,infile,'r',ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not open the input file.'
        write (*,*) 'File name: ',trim(infile)
        call exit (1)
      end if
c
      call ffopen (2,outfile,'rw',ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not open the output file.'
        write (*,*) 'File name: ',trim(outfile)
        call exit (1)
      end if
c
c ****** Initialization.
c
      ls=0
      nmac=0
      call getstmt (0,0,ibuf,ixi,ixfc,ierr)
c
c ****** Main loop.
c
  100 continue
c
c ****** Get the next statement.
c
      call getstmt (1,2,ibuf,ixi,ixfc,ierr)
      if (ierr.ne.0) go to 900
c
      ls=ls+1
c
      if (idebug.ge.3) then
        write (*,*) '### Read statement #',ls,':'
        write (*,*) ibuf(1:ixi-1)
      end if
c
c ****** Check the line for macro calls.
c
      ixo=1
      mres=macres(ibuf(1:ixi-1),obuf,ixo)
      if (mres) then
        if (idebug.ge.2) then
          write (*,*)
          write (*,*) '### Macro call in statement #',ls,':'
          write (*,*) 'Replacement text:'
          write (*,*) obuf(1:ixo-1)
        end if
      end if
c
c ****** Check if it is a macro definition.
c
      mdef=macdef(obuf(1:ixo-1))
      if (mdef) then
c
        if (idebug.ge.1) then
          write (*,*)
          write (*,*) '### Macro definition in statement #',ls,':'
          write (*,*) 'Macro number = ',nmac
          write (*,*) 'Macro name = ',macnam(nmac)
          write (*,*) 'Number of arguments = ',macnarg(nmac)
          write (*,*) 'Replacement text:'
          lrpl=lenstr(macrpl(nmac))
          write (*,*) macrpl(nmac)(1:lrpl)
        end if
c
      else
c
c ****** Write the line to the output file.
c
        line=' '
        ix=ixfc-1
        do 200 i=1,ixo-1
          ix=ix+1
          line(ix:ix)=obuf(i:i)
          if (ix.ge.72) then
            write (2,'(a)') line(1:ix)
            ix=6
            line=' '
            line(6:6)='&'
          end if
  200   continue
        if (ix.gt.0) then
          if (line(1:ix).ne.'     &') write (2,'(a)') line(1:ix)
        end if
c
      end if
c
      go to 100
c
  900 continue
c
      close (1)
      close (2)
c
      call exit (0)
c
      end
c#######################################################################
      logical function getname (line,ix,name,isl,delim)
c
c-----------------------------------------------------------------------
c
      character*(*) line,name
      logical delim
c
c-----------------------------------------------------------------------
c
      logical letter,alpha
c
c-----------------------------------------------------------------------
c
      getname=.false.
      delim=.false.
      name=' '
c
      lline=lenstr(line)
      lname=len(name)
c
c ****** Find the next name.
c
  100 continue
      if (ix.gt.lline) return
      if (.not.letter(line(ix:ix))) then
        if (isl.gt.0.and.line(ix:ix).eq.']') then
          delim=.true.
          return
        end if
        ix=ix+1
        go to 100
      end if
c
c ****** Get the name.
c
      ln=1
      name(1:1)=line(ix:ix)
      ix=ix+1
      getname=.true.
  200 continue
      if (ix.gt.lline) return
      if (alpha(line(ix:ix))) then
        ln=ln+1
        if (ln.gt.lname) then
          write (*,*)
          write (*,*) '### ERROR in GETNAME:'
          write (*,*) '### A variable name exceeds the maximum'//
     &                ' number of characters allowed.'
          write (*,*) 'Number of characters allowed = ',lname
          write (*,*) 'Truncated variable name: ',name
          call exit (1)
        end if
        name(ln:ln)=line(ix:ix)
        ix=ix+1
        go to 200
      end if
c
  300 continue
c
      return
      end
c#######################################################################
      logical function alpha (ch)
c
c-----------------------------------------------------------------------
c
      character ch
c
c-----------------------------------------------------------------------
c
      alpha=.false.
c
      ich=ichar(ch)
c
      if ((ich.ge.65.and.ich.le. 90).or.
     &    (ich.ge.97.and.ich.le.122).or. 
     &    (ich.ge.48.and.ich.le. 57).or.
     &    ich.eq.95) alpha=.true.
c
      return
      end
c#######################################################################
      logical function letter (ch)
c
c-----------------------------------------------------------------------
c
      character ch
c
c-----------------------------------------------------------------------
c
      letter=.false.
c
      ich=ichar(ch)
c
      if ((ich.ge.65.and.ich.le. 90).or.
     &    (ich.ge.97.and.ich.le.122)) letter=.true.
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do i=len(str),1,-1
        if (str(i:i).ne.' ') go to 100
      enddo
      i=0
  100 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      function match (table,n,alpha)
c
      character*(*) table(n),alpha
c
      do i=1,n
        if (table(i).eq.alpha) then
          match=i
          return
        end if
      enddo
c
      match=0
c
      return
      end
c#######################################################################
      logical function find (line,ch,ix)
c
c-----------------------------------------------------------------------
c
c ****** Find the index of the first occurence of
c ****** character CH in LINE, starting from position IX.
c
c ****** If found, set FIND=.true., and set IX to be its index.
c
c-----------------------------------------------------------------------
c
      character*(*) line
      character ch
c
c-----------------------------------------------------------------------
c
      find=.false.
c
      ll=lenstr(line)
c
      ix0=ix
c
      do 100 i=ix0,ll
        if (line(i:i).eq.ch) then
          ix=i
          find=.true.
          return
        end if
  100 continue
c
      return
      end
c#######################################################################
      function ifind (line,ch1,ch2,ix)
c
c-----------------------------------------------------------------------
c
c ****** Find the index of the first occurence of
c ****** character CH1 or CH2 in LINE, starting from
c ****** position IX.
c
c ****** Set IFIND=1 if CH1 was found, or set IFIND=2 if
c ****** CH2 was found, and set IX to be its index.
c ****** Set IFIND=0 if neither character was found.
c
c-----------------------------------------------------------------------
c
      character*(*) line
      character ch1,ch2
c
c-----------------------------------------------------------------------
c
      ifind=0
c
      ll=lenstr(line)
c
      ix0=ix
c
      do 100 i=ix0,ll
        if (line(i:i).eq.ch1) then
          ix=i
          ifind=1
          return
        else if (line(i:i).eq.ch2) then
          ix=i
          ifind=2
          return
        end if
  100 continue
c
      return
      end
c#######################################################################
      subroutine addtobuf (line,buf,ix,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Add the string in LINE to the buffer BUF, starting
c ****** at character position IX in BUF.
c
c-----------------------------------------------------------------------
c
c ****** When COMPRESS=.true., multiple spaces in LINE are
c ****** converted into a single space.
c
c-----------------------------------------------------------------------
c
      logical COMPRESS
      parameter (COMPRESS=.false.)
c
c-----------------------------------------------------------------------
c
      character*(*) line,buf
c
c-----------------------------------------------------------------------
c
      logical psp
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lbuf=len(buf)
      ll=len(line)
c
      psp=.false.
      do 100 i=1,ll
c
        if (COMPRESS) then
c
c ****** Compress multiple spaces into a single space.
c
          if (line(i:i).eq.' ') then
            if (psp) go to 100
            psp=.true.
          else
            psp=.false.
          end if
        end if
c
c ****** Check for a buffer overflow.
c
        if (ix.gt.lbuf) then
          ierr=1
          return
        end if
c
c ****** Write the character to the buffer.
c
        buf(ix:ix)=line(i:i)
        ix=ix+1
c
  100 continue
c
      return
      end
c#######################################################################
      integer function fchar (str)
c
c-----------------------------------------------------------------------
c
c ****** Find the position of the first non-blank character in STR.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do i=1,len(str)
        if (str(i:i).ne.' ') go to 100
      enddo
      i=0
  100 continue
c
      fchar=i
c
      return
      end
c#######################################################################
      subroutine getstmt (iun,iuno,buf,ix,ixfc,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Return the next whole FORTRAN statement from the
c ****** file connected to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** The statement is returned in BUF.  IX-1 is the index
c ****** of the last character filled in BUF.
c
c ****** If a statement is returned, IERR=0 is set.
c ****** IERR=1 is returned when no more statements can be
c ****** found in the file.
c
c ****** IXFC is the position of the first non-blank character
c ****** in the statement.  BUF is filled with the statement
c ****** without including any leading spaces.
c ****** Multiple spaces are combined into a single space.
c
c ****** This routine skips blank lines.
c ****** If a statement is a comment, it is written
c ****** (without any processing) to unit IUNO if IUNO>0.
c ****** Set IUNO=0 to exclude comments from the output file.
c
c ****** This routine must be initialized with IUN=0
c ****** before it is used to return the first statement.
c
c-----------------------------------------------------------------------
c
      character*(*) buf
c
c-----------------------------------------------------------------------
c
      parameter (LMX=132)
      character*(LMX) line
      common /gtstcom1/ line
c
      logical rdfil
      common /gtstcom2/ ln,rdfil
c
c-----------------------------------------------------------------------
c
      external rdlin,fchar
      logical rdlin,defcont
      integer fchar
c
c-----------------------------------------------------------------------
c
      character bslash
      parameter (bslash=char(92))
c
c ****** For CF77 on the Cray, define BSLASH as follows:
c 
ccc      parameter (bslash='\')
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Check if this is an initialization.
c
      if (iun.eq.0) then
        rdfil=.true.
        ln=0
        return
      end if
c
      lbuf=len(buf)
      defcont=.false.
      ix=1
c
  100 continue
c
c ****** Get the next line.
c
      if (rdfil) then
        if (.not.rdlin(iun,line)) then
          ierr=1
          return
        end if
        ln=ln+1
      end if
c
c ****** Discard blank lines.
c
      ll=lenstr(line)
      if (ll.eq.0) go to 100
c
c ****** Check if it is a continued "#define" line.
c
      if (defcont) then
        if (line(ll:ll).eq.bslash) then
          call addtobuf (line(1:ll-1),buf,ix,ierr)
          if (ierr.ne.0) go to 900
          go to 100
        else
          defcont=.false.
          call addtobuf (line(1:ll),buf,ix,ierr)
          if (ierr.ne.0) go to 900
          go to 300
        end if
      else
        if (ll.ge.8.and.line(1:7).eq.'#define'.and.
     &      line(ll:ll).eq.bslash) then
          defcont=.true.
          call addtobuf (line(1:ll-1),buf,ix,ierr)
          if (ierr.ne.0) go to 900
          rdfil=.true.
          go to 100
        end if
      end if
c
c ****** If it is a comment, write it out to unit IUNO
c ****** and process the next line.
c
      if (line(1:1).eq.'c'.or.line(1:1).eq.'C') then
        if (iuno.gt.0) write (iuno,'(a)') line(1:ll)
        rdfil=.true.
        go to 100
      end if
c
c ****** Find the position of the first non-blank character.
c
      ixfc=fchar(line(1:ll))
c
c ****** Check length of line.
c
      if (ll.gt.72) then
        write (*,*)
        write (*,*) '### WARNING from GETSTMT:'
        write (*,*) '### Line exceeds 72 characters.'
        write (*,*) 'Number of characters in line = ',ll
        write (*,*) 'Line number = ',ln
        write (*,*) 'The line is:'
        write (*,*) line(1:ll)
      end if
c
c ****** Add the line to the buffer.
c
      call addtobuf (line(ixfc:ll),buf,ix,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Read the next line to check if it is a continuation.
c
  200 continue
c
      if (.not.rdlin(iun,line)) then
        rdfil=.true.
        go to 300
      end if
c
      ln=ln+1
c
c ****** Discard blank lines.
c
      ll=lenstr(line)
      if (ll.eq.0) go to 200
c
c ****** Check if it is a comment.
c
ccc      if (line(1:1).eq.'c'.or.line(1:1).eq.'C') then
ccc        if (iuno.gt.0) write (iuno,'(a)') line(1:ll)
ccc        go to 200
ccc      end if
c
      if (line(1:1).eq.'c'.or.line(1:1).eq.'C') then
        rdfil=.false.
        return
      end if
c
c ****** Check if it is a continuation.
c
      if (ll.ge.6.and.line(1:5).eq.' '.and.line(6:6).ne.' ') then
        call addtobuf (line(7:ll),buf,ix,ierr)
        if (ierr.ne.0) go to 900
        go to 200
      else
        rdfil=.false.
      end if
c
  300 continue
c
      return
c
  900 continue
c
c ****** Error: output buffer overflow.
c
      write (*,*)
      write (*,*) '### ERROR in GETSTMT:'
      write (*,*) '### Line buffer overflow.'
      write (*,*) '[A statement may be too long.]'
      write (*,*) 'Line number = ',ln
      stop
c
      end
c#######################################################################
      logical function rdlin (iun,line)
c
c-----------------------------------------------------------------------
c
c ****** Read a line of text from unit IUN.
c
c ****** If a line was read successfully, return with
c ****** RDLIN=.true.; otherwise, set RDLIN=.false.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      read (iun,'(a)',end=100,err=100) line
c
      rdlin=.true.
      return
c
  100 continue
c
c ****** Error while reading the line.
c
      rdlin=.false.
c
      return
      end
c#######################################################################
      logical function macdef (buf)
c
c-----------------------------------------------------------------------
c
c ****** Check if the buffer BUF holds a macro definition.
c
c-----------------------------------------------------------------------
c
c ****** If this is a valid macro definition, add it to the
c ****** macro tables.
c
c ****** A valid macro has the following definition:
c ******
c ******    #define <name>(arg1[,arg2]*) <replacement text>
c ******
c ****** The line can be continued by a "\".  The "#define"
c ****** must start in column 1.  The macro can have 1 or more
c ****** arguments. A "##" in the replacement text concatenates
c ****** its neighbors.
c
c-----------------------------------------------------------------------
c
      character*(*) buf
c
c-----------------------------------------------------------------------
c
c ****** Macro definition tables.
c
      parameter (nmacmx=100)
      parameter (LMNAM=64)
      parameter (LMRPL=512)
      character*(LMNAM) macnam
      character*(LMRPL) macrpl
      common /mdefcom1/ macnam(nmacmx),macrpl(nmacmx)
      common /mdefcom2/ nmac,macnarg(nmacmx)
c
c-----------------------------------------------------------------------
c
      character*(LMNAM) name
c
      parameter (nargmx=20)
      parameter (LMARG=64)
      character*(LMARG) args(nargmx),targ
c
c-----------------------------------------------------------------------
c
      external getname,fchar,find
      logical getname,find
      integer fchar
c
c-----------------------------------------------------------------------
c
      character*3 ch3
c
c-----------------------------------------------------------------------
c
      macdef=.false.
c
      narg=0
      lb=len(buf)
c
      if (.not.(lb.gt.8.and.buf(1:8).eq.'#define ')) return
      ix=9
      ix0=ix
      if (.not.getname(buf,ix,name,0,delim)) return
      ln=lenstr(name)
      if (ix-1-ln.ge.ix0.and.buf(ix0:ix-1-ln).ne.' ') return
      ix0=ix
      if (.not.find(buf,'(',ix)) return
      if (ix-1.ge.ix0.and.buf(ix0:ix-1).ne.' ') return
      ix=ix+1
c
      nmac=nmac+1
c
      if (nmac.gt.nmacmx) then
        write (*,*)
        write (*,*) '### ERROR in MACDEF:'
        write (*,*) '### Too many macro definitions.'
        write (*,*) 'Macro number = ',nmac
        write (*,*) 'Input line is:'
        write (*,*) buf
        stop
      end if
c
      macnam(nmac)=name
c
c ****** Collect the arguments.
c
  100 continue
c
      ix0=ix
      ifnd=ifind(buf,')',',',ix)
      if (ifnd.eq.0) then
        nmac=nmac-1
        return
      end if
      narg=narg+1
      if (narg.gt.nargmx) then
        write (*,*)
        write (*,*) '### ERROR in MACDEF:'
        write (*,*) '### Too many macro arguments.'
        write (*,*) 'Argument number = ',narg
        write (*,*) 'Input line is:'
        write (*,*) buf
        stop
      end if
      larg=ix-ix0
      if (larg.gt.LMARG) then
        write (*,*)
        write (*,*) '### ERROR in MACDEF:'
        write (*,*) '### Macro argument is too long.'
        write (*,*) 'Argument number = ',narg
        write (*,*) 'Argument length = ',larg
        write (*,*) 'Input line is:'
        write (*,*) buf
        stop
      end if
      args(narg)=buf(ix0:ix-1)
      args(narg)=adjustl(args(narg))
      ix=ix+1
      if (ifnd.eq.2) go to 100
c
      macnarg(nmac)=narg
c
c ****** Store the replacement text, flagging the arguments
c ****** with $<argnum>, where <argnum> is a 2-digit number.
c
c ****** Remove leading spaces.
c
      ic0=fchar(buf(ix:))
      ix=ix+ic0-1
c
      ixr=1
      macrpl(nmac)=' '
c
  200 continue
c
      ix0=ix
      if (getname(buf,ix,targ,0,delim)) then
        iarg=match(args,narg,targ)
        if (iarg.ne.0) then
c
c ****** Write the text and code the argument.
c
          la=lenstr(targ)
          call addtobuf (buf(ix0:ix-1-la),macrpl(nmac),ixr,ierr)
          if (ierr.ne.0) go to 900
          ch3(1:1)='$'
          write (ch3(2:3),'(i2.2)') iarg
          call addtobuf (ch3,macrpl(nmac),ixr,ierr)
          if (ierr.ne.0) go to 900
c
        else
c
c ****** Write the text.
c
          call addtobuf (buf(ix0:ix-1),macrpl(nmac),ixr,ierr)
          if (ierr.ne.0) go to 900
c
        end if
        go to 200
      else
        call addtobuf (buf(ix0:lb),macrpl(nmac),ixr,ierr)
        if (ierr.ne.0) go to 900
      end if
c
c ****** Resolve the concatenation operator "##".
c
      call filtnn (macrpl(nmac)(1:ixr-1))
c
      macdef=.true.
c
      return
c
c ****** Error: replacement text is too long.
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in MACDEF:'
      write (*,*) '### Macro replacement text is too long.'
      write (*,*) 'Macro number = ',nmac
      write (*,*) 'Replacement text:'
      write (*,*) macrpl(nmac)
      write (*,*) 'Input line is:'
      write (*,*) buf
      stop
c
      end
c#######################################################################
      subroutine filtnn (line)
c
c-----------------------------------------------------------------------
c
c ****** Filter the ## operator.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      external fchar
      integer fchar
c
c-----------------------------------------------------------------------
c
      ll=lenstr(line)
c
      ix=1
  100 continue
      if (ix+1.gt.ll) go to 200
      if (line(ix:ix+1).eq.'##') then
        ln=lenstr(line(1:ix-1))
        ir=max0(0,fchar(line(ix+2:))-1)
        nd=ix-1-ln+2+ir
        line(ln+1:)=line(ln+1+nd:)
        ll=ll-nd
        ix=ln+1
      else
        ix=ix+1
      end if
      go to 100
c
  200 continue
c
      return
      end
c#######################################################################
      logical function macres (line,obuf,ixo)
c
c-----------------------------------------------------------------------
c
c ****** Resolve the text in LINE for macro calls.
c
c-----------------------------------------------------------------------
c
      use params
c
c-----------------------------------------------------------------------
c
      character*(*) line,obuf
c
c-----------------------------------------------------------------------
c
c ****** Macro definition tables.
c
      parameter (nmacmx=100)
      parameter (LMNAM=64)
      parameter (LMRPL=512)
      character*(LMNAM) macnam
      character*(LMRPL) macrpl
      common /mdefcom1/ macnam(nmacmx),macrpl(nmacmx)
      common /mdefcom2/ nmac,macnarg(nmacmx)
c
c-----------------------------------------------------------------------
c
      character*(LMNAM) name
c
      parameter (nargmx=20)
      parameter (LMARG=64)
      character*(LMARG) args(nargmx)
c
c-----------------------------------------------------------------------
c
      external getname
      logical getname
c
c-----------------------------------------------------------------------
c
      macres=.false.
c
c ****** If no macros are defined, just copy the line to the
c ****** output buffer and return.
c
      if (nmac.le.0) then
        call addtobuf (line,obuf,ixo,ierr)
        if (ierr.ne.0) go to 900
        return
      end if
c
      ix=1
c
  100 continue
c
      ix0=ix
c
      if (getname(line,ix,name,0,delim)) then
c
c ****** Write the text up to the name.
c
        ln=lenstr(name)
        call addtobuf (line(ix0:ix-1-ln),obuf,ixo,ierr)
        if (ierr.ne.0) go to 900
        ix0=ix-ln
c
c ****** Check if this is a macro call.
c
        imac=match(macnam,nmac,name)
c
        if (imac.ne.0) then
c
c ****** Get the arguments.
c
          ixhold=ix
          call getargs (line,ix,nargmx,args,narg,ierr)
          if (ierr.ne.0) then
            call addtobuf (name(1:ln),obuf,ixo,ierr)
            if (ierr.ne.0) go to 900
            go to 100
          end if
c
          if (idebug.ge.3) then
            write (*,*)
            write (*,*) '### Found a macro call:'
            write (*,*) 'Macro name = ',name
            write (*,*) 'Number of args = ',narg
            write (*,*) 'Arguments:'
            do 200 i=1,narg
              write (*,*) 'ARG#',i,': ',args(i)
  200       continue
          end if
c
c ****** Put in the replacement text.
c
          call putrpl (imac,args,narg,obuf,ixo,ierr)
          if (ierr.ne.0) then
            call addtobuf (name(1:ln),obuf,ixo,ierr)
            if (ierr.ne.0) go to 900
            ix=ixhold
            go to 100
          end if
c
          macres=.true.
c
        else
c
c ****** Write the name.
c
          call addtobuf (name(1:ln),obuf,ixo,ierr)
          if (ierr.ne.0) go to 900
c
        end if
        go to 100
c
      else
c
c ****** Write the rest of the line.
c
        call addtobuf (line(ix0:ix-1),obuf,ixo,ierr)
        if (ierr.ne.0) go to 900
c
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in MACRES:'
      write (*,*) '### Output buffer overflow.'
      stop
c
      end
c#######################################################################
      subroutine getargs (line,ixi,nargmx,args,narg,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Get the arguments for a macro call.
c
c-----------------------------------------------------------------------
c
      character*(*) line,args(nargmx)
c
c-----------------------------------------------------------------------
c
      external find
      logical find,found
c
c-----------------------------------------------------------------------
c
      ierr=1
c
      lmarg=len(args(1))
      narg=0
      ix=ixi
c
c ****** Find the first left parenthesis.
c
      ix0=ix
      found=find(line,'(',ix)
      if (.not.found) return
c
c ****** Check that it is the first non-blank character.
c
      if (ix-1.ge.ix0.and.line(ix0:ix-1).ne.' ') return
c
      ixl=ix
      ix=ix+1
c
c ****** Find the first right parenthesis.
c
      found=find(line,')',ix)
      if (.not.found) return
c
      ixr=ix
c
c ****** Get the arguments.
c
      ix=ixl+1
  100 continue
      ix0=ix
      found=find(line(1:ixr-1),',',ix)
      if (.not.found) ix=ixr
      narg=narg+1
      if (narg.gt.nargmx) then
        write (*,*)
        write (*,*) '### ERROR in GETARGS:'
        write (*,*) '### Too many macro arguments.'
        write (*,*) 'Argument number = ',narg
        write (*,*) 'Line is:'
        write (*,*) line
        stop
      end if
      larg=lenstr(line(ix0:ix-1))
      if (larg.gt.lmarg) then
        write (*,*)
        write (*,*) '### ERROR in GETARGS:'
        write (*,*) '### Macro argument is too long.'
        write (*,*) 'Argument number = ',narg
        write (*,*) 'Argument length = ',larg
        write (*,*) 'Line is:'
        write (*,*) line
        stop
      end if
      args(narg)=line(ix0:ix-1)
      args(narg)=adjustl(args(narg))
      if (found) then
        ix=ix+1
        go to 100
      end if
c
c ****** Arguments found successfully.
c
      ixi=ixr+1
      ierr=0
c
      return
      end
c#######################################################################
      subroutine putrpl (imac,args,narg,buf,ix,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Put the replacement text for a macro call into
c ****** the output buffer BUF.
c
c ****** The NARG actual arguments are in array ARGS.
c
c ****** IMAC is the macro name index.
c
c-----------------------------------------------------------------------
c
      character*(*) args(narg),buf
c
c-----------------------------------------------------------------------
c
c ****** Macro definition tables.
c
      parameter (nmacmx=100)
      parameter (LMNAM=64)
      parameter (LMRPL=512)
      character*(LMNAM) macnam
      character*(LMRPL) macrpl
      common /mdefcom1/ macnam(nmacmx),macrpl(nmacmx)
      common /mdefcom2/ nmac,macnarg(nmacmx)
c
c-----------------------------------------------------------------------
c
      ierr=1
c
c ****** Check the number of arguments.
c
      if (narg.ne.macnarg(imac)) return
c
c ****** Use the macro replacement text.
c
      ll=lenstr(macrpl(imac))
      ir=1
  100 continue
      if (ir.gt.ll) go to 200
c
c ****** Check if this is an argument.
c
      if (macrpl(imac)(ir:ir).eq.'$') then
c
c ****** Get the argument number.
c
        read (macrpl(imac)(ir+1:ir+2),'(i2)') iarg
        larg=lenstr(args(iarg))
c
c ****** Substitute the actual argument.
c
        call addtobuf (args(iarg)(1:larg),buf,ix,ierr)
        if (ierr.ne.0) go to 900
c
        ir=ir+3
c
      else
c
c ****** Write a character of the replacement text.
c
        call addtobuf (macrpl(imac)(ir:ir),buf,ix,ierr)
        if (ierr.ne.0) go to 900
        ir=ir+1
c
      end if
      go to 100
c
  200 continue
c
      ierr=0
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in PUTRPL:'
      write (*,*) '### Output buffer overflow.'
      stop
c
      end
c#######################################################################
      subroutine set_parameters (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command-line arguments.
c
c-----------------------------------------------------------------------
c
      use syntax
      use paragraph_def
      use get_usage_line_interface
      use print_par_interface
      use delete_par_interface
      use params
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*) :: cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Storage for the usage line.
c
      type(paragraph), pointer :: usage
c
c ****** Storage for the error message.
c
      character(72) :: errmsg
c
c-----------------------------------------------------------------------
c
      integer :: ierr
      character(256) :: arg
      logical :: set
c
c-----------------------------------------------------------------------
c
      integer, external :: intval
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (GROUP_KA,'-debug','0','<level>')
      call defarg (GROUP_A ,'infile',' ',' ')
      call defarg (GROUP_A ,'outfile',' ',' ')
c
c ****** Parse the command line.
c
      call parse (errmsg,ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Expand macros in a source file.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage line.
c
        call get_usage_line (usage)
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
c
        call print_par (usage)
c
        write (*,*)
        write (*,*) 'Use -debug to get diagnostic information:'
        write (*,*) '  debug.ge.1: lists macro definitions'
        write (*,*) '  debug.ge.2: lists macro call replacement text'
        write (*,*) '  debug.ge.3: lists every line read and'//
     &              ' detailed call diagnostic output'
c
c
        call delete_par (usage)
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Debug level.
c
      call fetcharg ('-debug',set,arg)
      idebug=intval(trim(arg),'-debug')
c
c ****** Input file name.
c
      call fetcharg ('infile',set,arg)
      infile=trim(arg)
c
c ****** Output file name.
c
      call fetcharg ('outfile',set,arg)
      outfile=trim(arg)
c
      return
      end
