c
c-----------------------------------------------------------------------
c
c ****** Print information about a scientific data set.
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        10/12/92, ZM, Version 1.00:
c
c         - Original version of program.
c
c        11/04/92, ZM, Version 1.01:
c
c         - Modified for SGI IRIS from CRAY UNICOS.
c
c        07/29/2003, ZM, Version 1.02:
c
c         - This version uses the ZM tools libraries.
c         - The program thus uses the new HDF read/write
c           routines that can handle 32-bit and 64-bit data files.
c
c-----------------------------------------------------------------------
c
c#######################################################################
c
c ****** Note that this tool uses modules from ZM's tools library.
c
c#######################################################################
      module params
c
c-----------------------------------------------------------------------
c ****** Parameters.
c-----------------------------------------------------------------------
c
      character(256) :: infile
      character(16) :: fmt
      logical :: verbose
c
      end module
c#######################################################################
      program INFO
c
c-----------------------------------------------------------------------
c
      use number_types
      use params
      use sds_def
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*), parameter :: cname='INFO'
      character(*), parameter :: cvers='1.02'
      character(*), parameter :: cdate='07/29/2003'
c
c-----------------------------------------------------------------------
c
      type(sds) :: s
c
      integer :: i,j,ierr
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call set_parameters (cname,cvers,cdate)
c
c ****** Read the SDS.
c
      call rdsds (fmt,infile,s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not read the data set.'
        write (*,*) 'IERR (from RDSDS) = ',ierr
        write (*,*) 'File name: ',trim(infile)
        call exit (1)
      end if
c
c ****** Write diagnostics.
c
      write (*,*)
      write (*,*) '### Information for: ',trim(infile)
      write (*,*) 'Number of dimensions = ',s%ndim
      write (*,900) 'Dimensions = ',(s%dims(i),i=1,s%ndim)
      write (*,910) 'Minimum value = ',minval(s%f)
      write (*,910) 'Maximum value = ',maxval(s%f)
      if (s%scale) then
        write (*,*) 'Scales are present.'
      else
        write (*,*) 'Scales are not present.'
      end if
      if (fmt.eq.'hdf') then
        if (s%hdf32) then
          write (*,*) 'The SDS contains 32-bit data.'
        else
          write (*,*) 'The SDS contains 64-bit data.'
        end if
      end if
c
      if (s%scale) then
        do i=1,s%ndim
          if (verbose) then
            write (*,*)
            write (*,920) 'Scale for dimension ',i,':'
            write (*,930) (s%scales(i)%f(j),j=1,s%dims(i))
          else
            write (*,940) 'Scale limits for dimension ',i,':',
     &                    s%scales(i)%f(1),s%scales(i)%f(s%dims(i))
          end if
        enddo
      end if
c
  900 format (1x,a,8(i8))
  910 format (1x,a,1pe20.13)
  920 format (1x,a,i1,a)
  930 format (5(1x,1pe13.6))
  940 format (1x,a,i1,a,2(1x,1pe20.13))
c
      call exit (0)
c
      end
c#######################################################################
      subroutine set_parameters (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command-line arguments.
c
c-----------------------------------------------------------------------
c
      use syntax
      use paragraph_def
      use get_usage_line_interface
      use print_par_interface
      use delete_par_interface
      use params
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*) :: cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Storage for the usage line.
c
      type(paragraph), pointer :: usage
c
c ****** Storage for the error message.
c
      character(72) :: errmsg
c
c-----------------------------------------------------------------------
c
      integer :: ierr
      character(256) :: arg
      logical :: set
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (GROUP_K ,'-v',' ',' ')
      call defarg (GROUP_KA,'-fmt','hdf','text|hdf')
      call defarg (GROUP_A ,'infile',' ',' ')
c
c ****** Parse the command line.
c
      call parse (errmsg,ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Print information about a data set.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage line.
c
        call get_usage_line (usage)
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
c
        call print_par (usage)
c
        write (*,*)
        write (*,*) '-fmt specifies the file format (default=hdf).'
        write (*,*) 'Option -v gives more information.'
c
        call delete_par (usage)
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Verbose flag.
c
      call fetcharg ('-v',set,arg)
      verbose=set
c
c ****** File format.
c
      call fetcharg ('-fmt',set,arg)
      fmt=trim(arg)
c
c ****** Input file name.
c
      call fetcharg ('infile',set,arg)
      infile=trim(arg)
c
      return
      end
