      program INTERP
c
c-----------------------------------------------------------------------
c
c ****** Interpolate a quantity from a staggered grid onto the
c ****** main grid.
c
c ****** This version is for SGI IRIS.
c
c-----------------------------------------------------------------------
c
c ****** Compile and link with:
c
c     f77 -O3 interp.f $LIBDF -o interp
c
c ****** When debugging, use the compile line:
c
c     f77 -g interp.f $LIBDF -o interp
c
c-----------------------------------------------------------------------
c
      character cname*(*),cvers*4,cdate*8
c
      parameter (cname='INTERP')
      parameter (cvers='1.04')
      parameter (cdate='04/10/93')
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        05/08/92, ZM, Version 1.00:
c
c         - Original version of program.
c
c        05/18/92, ZM, Version 1.01:
c
c         - Changed the program to use the new command
c           line parser.
c
c        10/26/92, ZM, Version 1.02:
c
c         - Added the ability to interpolate fields which are 1D,
c           2D or 3D.
c         - Changed the execute line options slightly to accomodate
c           the expanded fuctionality.
c         - Updated the parsing routines.
c
c        11/07/92, ZM, Version 1.03:
c
c         - Corrected a minor bug involving IERR in the
c           averaging routines.
c
c        04/10/93, ZM, Version 1.04:
c
c         - Changed options -x, -y, and -z so that they default to
c           the input SDS dimensions.  They do not have to be
c           specified now. 
c         - Extended the code to interpolate data sets which do
c           not contain scales.
c
c-----------------------------------------------------------------------
c
      cliche syntax
c
c-----------------------------------------------------------------------
c
c ****** Group definitions.
c
c        GROUP 1: <kw>
c        GROUP 2: <arg>
c        GROUP 3: <kw> <arg>
c        GROUP 4: <kw> <arg> <arg>
c
      parameter (ngroup=4)
c
      parameter (IGRPK  =1)
      parameter (IGRPA  =2)
      parameter (IGRPKA =3)
      parameter (IGRPKAA=4)
c
      parameter (mxarg=100)
c
      parameter (IXGRP=1)
      parameter (IXSET=2)
      parameter (IXREQ=3)
      parameter (IXKEY=4)
      parameter (IXARG=5)
      parameter (IXNAM=6)
c
      integer narg,info(6,mxarg)
      common /sntx1/ narg,info
c
c ****** Buffer for the usage line.
c
      character*512 ulbuf
      common /sntx2/ ulbuf
c
c ****** Buffer for the error message.
c
      character*512 errmsg
      common /sntx3/ errmsg
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      logical verbose,ifrs
      common /parcom1/ verbose,ifrs
c
      character*512 fmt,infile,outfile
      common /parcom2/ fmt,infile,outfile
c
      common /parcom3/ nxi,nyi,nzi
c
c-----------------------------------------------------------------------
c
c ****** Pointers to data arrays.
c
      pointer (ipf,f),(ipx,x),(ipy,y),(ipz,z)
      pointer (ipfi,fi),(ipxi,xi),(ipyi,yi),(ipzi,zi)
      pointer (ipav,av)
c
      parameter (mxdim=3)
      integer idim(mxdim),ips(mxdim),idimi(mxdim),ipsi(mxdim)
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call setpar (cname,cvers,cdate)
c
c ****** Read the data.
c
      lf=lenstr(infile)
c
      call rdsds (fmt,infile,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not read the data set.'
        write (*,*) 'IERR (from RDSDS) = ',ierr
        write (*,*) 'File name: ',infile(1:lf)
        call exit (1)
      end if
c
c ****** Check that scales are present when option -rs is set.
c
      if (ifrs.and.ifscale.eq.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not multiply by r*sin(theta).'
        write (*,*) '### The input data set does not contain scales.'
        write (*,*) 'File name: ',infile(1:lf)
        call exit (1)
      end if
c
c ****** Set the file data set dimensions.
c
      nx=idim(1)
c
      if (ndim.ge.2) then
        ny=idim(2)
      else
        ny=1
      end if
c
      if (ndim.ge.3) then
        nz=idim(3)
      else
        nz=1
      end if
c
c ****** Initialize the pointers to the scales.
c
      ipx=ipf
      ipy=ipf
      ipz=ipf
c
      if (ifscale.ne.0) then
        ipx=ips(1)
        if (ndim.ge.2) ipy=ips(2)
        if (ndim.ge.3) ipz=ips(3)
      end if
c
c ****** Set the default output dimensions.
c
      if (nxi.le.0) nxi=nx
      if (nyi.le.0) nyi=ny
      if (nzi.le.0) nzi=nz
c
      idimi(1)=nxi
      idimi(2)=nyi
      idimi(3)=nzi
c
c ****** Check that the dimensions match.
c
      if (.not.((nxi.eq.nx.or.nxi.eq.nx-1).and.
     &          (nyi.eq.ny.or.nyi.eq.ny-1).and.
     &          (nzi.eq.nz.or.nzi.eq.nz-1))) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Mismatch in dimensions.'
        write (*,100) 'Dimensions specified = ',(idimi(i),i=1,3)
        write (*,100) 'Dimensions in file   = ',(idim(i),i=1,ndim)
        write (*,*) 'File name: ',infile(1:lf)
  100   format (1x,a,3i8)
        call exit (1)
      end if
c
c ****** Allocate memory for the interpolated arrays.
c
c ****** Note that FI needs to be as large as F.
c
      nwd=nx*ny*nz
      ipfi=igetmem(nwd)
      if (ipfi.eq.0) go to 900
c
      if (ifscale.ne.0) then
c
c ****** Assign memory for the output scales.
c
        nwd=nxi
        ipxi=igetmem(nwd)
        if (ipxi.eq.0) go to 900
        ipsi(1)=ipxi
c
        if (ndim.ge.2) then
          nwd=nyi
          ipyi=igetmem(nwd)
          if (ipyi.eq.0) go to 900
          ipsi(2)=ipyi
        end if
c
        if (ndim.ge.3) then
          nwd=nzi
          ipzi=igetmem(nwd)
          if (ipzi.eq.0) go to 900
          ipsi(3)=ipzi
        end if
c
      else
c
c ****** Set the pointers to point to FI.  (These arrays
c ****** are not used when scales are not present.)
c
        ipxi=ipfi
        ipyi=ipfi
        ipzi=ipfi
        ipsi(1)=ipxi
        ipsi(2)=ipyi
        ipsi(3)=ipzi
c
      end if
c
      if (verbose) then
        write (*,*) '### Number of dimensions = ',ndim
        write (*,100) '### Original dimensions     = ',
     &                (idim(i),i=1,ndim)
        write (*,100) '### Interpolated dimensions = ',
     &                (idimi(i),i=1,ndim)
      end if
c
c ****** Do the average over x.
c ****** Put the result in FI.
c
      call avgx (ifscale,nx,ny,nz,x,f,nxi,ny,nz,xi,fi)
      ipav=ipfi
      if (verbose.and.nxi.ne.nx) then
        write (*,*) '### Performed an interpolation in x ...'
      end if
c
c ****** Do the average over y.
c ****** Put the result in F.
c
      if (ndim.ge.2) then
        call avgy (ifscale,nxi,ny,nz,y,fi,nxi,nyi,nz,yi,f)
        ipav=ipf
        if (verbose.and.nyi.ne.ny) then
          write (*,*) '### Performed an interpolation in y ...'
        end if
      end if
c
c ****** Do the average over z.
c ****** Put the result in FI.
c
      if (ndim.ge.3) then
        call avgz (ifscale,nxi,nyi,nz,z,f,nxi,nyi,nzi,zi,fi)
        ipav=ipfi
        if (verbose.and.nzi.ne.nz) then
          write (*,*) '### Performed an interpolation in z ...'
        end if
      end if
c
c ****** Check whether to multiply by r*sin(theta).
c
      if (ifrs) then
        call rst (nxi,nyi,nzi,xi,yi,zi,av)
        if (verbose) then
          write (*,*) '### Multiplied by r*sin(theta).'
        end if
      end if
c
c ****** Write the interpolated data to the output file.
c
      lf=lenstr(outfile)
c
      call wrsds (fmt,outfile,ndim,idimi,av,ifscale,ipsi,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not write the data set.'
        write (*,*) 'IERR (from WRSDS) = ',ierr
        write (*,*) 'File name: ',outfile(1:lf)
        call exit (1)
      end if
c
      call exit (0)
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in ',cname,':'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd 
      call exit (1)
c
      end
c#######################################################################
      subroutine avgx (ifscale,nx,ny,nz,x,f,nxi,nyi,nzi,xi,fi)
c
c-----------------------------------------------------------------------
c
      real x(nx),f(nx,ny,nz),xi(nxi),fi(nxi,nyi,nzi)
c
c-----------------------------------------------------------------------
c
      if (ifscale.ne.0) then
        call average (1,nx,x,1,nxi,xi)
      end if
c
      ns=1
      nsi=1
      do 200 k=1,nz
        do 100 j=1,ny
          call average (ns,nx,f(1,j,k),nsi,nxi,fi(1,j,k))
  100   continue
  200 continue
c
      return
      end
c#######################################################################
      subroutine avgy (ifscale,nx,ny,nz,y,f,nxi,nyi,nzi,yi,fi)
c
c-----------------------------------------------------------------------
c
      real y(ny),f(nx,ny,nz),yi(nyi),fi(nxi,nyi,nzi)
c
c-----------------------------------------------------------------------
c
      if (ifscale.ne.0) then
        call average (1,ny,y,1,nyi,yi)
      end if
c
      ns=nx
      nsi=nxi
      do 200 k=1,nz
        do 100 i=1,nx
          call average (ns,ny,f(i,1,k),nsi,nyi,fi(i,1,k))
  100   continue
  200 continue
c
      return
      end
c#######################################################################
      subroutine avgz (ifscale,nx,ny,nz,z,f,nxi,nyi,nzi,zi,fi)
c
c-----------------------------------------------------------------------
c
      real z(nz),f(nx,ny,nz),zi(nzi),fi(nxi,nyi,nzi)
c
c-----------------------------------------------------------------------
c
      if (ifscale.ne.0) then
        call average (1,nz,z,1,nzi,zi)
      end if
c
      ns=nx*ny
      nsi=nxi*nyi
      do 200 j=1,ny
        do 100 i=1,nx
          call average (ns,nz,f(i,j,1),nsi,nzi,fi(i,j,1))
  100   continue
  200 continue
c
      return
      end
c#######################################################################
      subroutine average (ndim1,n,f,ndim1i,ni,fi)
c
c-----------------------------------------------------------------------
c
c ****** If NI=N-1, average the function F(N) and put
c ****** the result in FI(NI).
c
c ****** If NI=N, copy the function F(N) into FI(NI).
c
c ****** Otherwise, exit with an error.
c
c-----------------------------------------------------------------------
c
      real f(ndim1,n),fi(ndim1i,ni)
c
c-----------------------------------------------------------------------
c
      if (ni.eq.n) then
        do 100 i=1,ni
          fi(1,i)=f(1,i)
  100   continue
      else if (ni.eq.n-1) then
        do 200 i=1,ni
          fi(1,i)=.5*(f(1,i)+f(1,i+1))
  200   continue
      else
        write (*,*)
        write (*,*) '### ERROR in AVERAGE:'
        write (*,*) '### Anomaly involving the mesh dimensions.'
        write (*,*) '[This is a fatal internal error.]'
        write (*,*) '[It should never occur!]'
        call exit (1)
        return
      end if
c
      return
      end
c#######################################################################
      subroutine rst (nx,ny,nz,x,y,z,f)
c
c-----------------------------------------------------------------------
c
c ****** Multiply F by r*sin(theta).
c
c-----------------------------------------------------------------------
c
      real x(nx),y(ny),z(nz),f(nx,ny,nz)
c
c-----------------------------------------------------------------------
c
      do 300 k=1,nz
        do 200 j=1,ny
          do 100 i=1,nx
            f(i,j,k)=f(i,j,k)*x(i)*sin(y(j))
  100     continue
  200   continue
  300 continue
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      subroutine rdsds (fmt,fname,mxdim,ndim,idim,
     &                  ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set.
c
c ****** Use routine RDTXT or RDHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in RDSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from a text file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    Text data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'r')
c
c ****** Get the number of dimensions.
c
      call rdint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 910
c
      if (ndim.lt.1.or.ndim.gt.mxdim) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT:'
        write (*,*) '### Invalid number of dimensions in file.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'MXDIM = ',mxdim
        ierr=1
        return
      end if
c
c ****** Read the dimensions.
c
      do 100 i=1,ndim
        call rdint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 910
        if (idim(i).le.0) go to 920
  100 continue
c
c ****** Check if the scales are present.
c
      call rdint (1,1,ifscale,ierr)
      if (ierr.ne.0) go to 910
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 300 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          call rdfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 910
  300   continue
      end if
c
c ****** Read the array.
c
      call rdfp (1,nwdf,f,ierr)
      if (ierr.ne.0) go to 910
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Error while reading text data.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=3
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine rdint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of INTEGER data into array I from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) i
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of REAL data into array F from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) f
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata
      external dsgdims,dsgdisc,dsgdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Get the dimensions of the SDS.
c
      lf=lenstr(fname)
      iret=dsgdims(fname(1:lf),ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check the dimensions.
c
      do 100 i=1,ndim
        if (idim(i).le.0) go to 910
  100 continue
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Check if all scales are present.
c ****** (Use array F for temporary storage.)
c
      nscal=0
      do 300 i=1,ndim
        iret=dsgdisc(i,idim(i),f)
        if (iret.eq.0) nscal=nscal+1
  300 continue
c
      if (nscal.eq.ndim) then
        ifscale=1
      else
        ifscale=0
      end if
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 400 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
  400   continue
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine wrsds (fmt,fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set.
c
c ****** Use routine WRTXT or WRHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in WRSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to text file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'rw')
c
c ****** Write the number of dimensions.
c
      call wrint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Write the dimensions.
c
      nwd=1
      do 100 i=1,ndim
        call wrint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 900
        nwd=nwd*idim(i)
  100 continue
c
c ****** Write the scales.
c
      if (ifscale.ne.0) then
        call wrint (1,1,1,ierr)
        if (ierr.ne.0) go to 900
        do 200 i=1,ndim
          ipx=ips(i)
          call wrfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 900
  200   continue
      else
        call wrint (1,1,0,ierr)
        if (ierr.ne.0) go to 900
      end if
c
c ****** Write the array.
c
      call wrfp (1,nwd,f,ierr)
      if (ierr.ne.0) go to 900
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRTXT:'
      write (*,*) '### Error in writing data to text file.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of INTEGER data into array I from unit IUN
c ****** using a free format text write.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,*,err=100) i
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of REAL data into array F from unit IUN.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,900,err=100) f
c
  900 format (5(1x,1pe13.6))
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to HDF file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dssdims,dssdisc,dspdata
      external dssdims,dssdisc,dspdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      iret=dssdims(ndim,idim)
      if (iret.ne.0) go to 900
c
      if (ifscale.ne.0) then
        do 100 i=1,ndim
          ipx=ips(i)
          iret=dssdisc(i,idim(i),x)
          if (iret.ne.0) go to 910
  100   continue
      end if
c
      iret=dspdata(fname,ndim,idim,f)
      if (iret.ne.0) go to 920
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDIMS) = ',iret,']'
      ierr=1
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the scales.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDISC) = ',iret,']'
      write (*,*) '[Dimension number = ',i,']'
      ierr=2
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not write the data array.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSPDATA) = ',iret,']'
      ierr=3
c
      return
      end
c#######################################################################
      integer function igetmem (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** The function returns the pointer to the array.
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for NWD words.
c
c     igetmem=malloc(%val(4*nwd))
      igetmem=malloc(4*nwd)
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do i=len(str),1,-1
        if (str(i:i).ne.' ') go to 100
      enddo
      i=1
  100 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      subroutine setpar (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      character*(*) cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      logical verbose,ifrs
      common /parcom1/ verbose,ifrs
c
      character*512 fmt,infile,outfile
      common /parcom2/ fmt,infile,outfile
c
      common /parcom3/ nxi,nyi,nzi
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*512 arg
c
c-----------------------------------------------------------------------
c
      logical set
      character*2 dim
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (IGRPK ,'-v',' ',' ')
      call defarg (IGRPKA,'-x','<default>','<nx>')
      call defarg (IGRPKA,'-y','<default>','<ny>')
      call defarg (IGRPKA,'-z','<default>','<nz>')
      call defarg (IGRPKA,'-fmt','hdf','<format>')
      call defarg (IGRPA  ,'infile',' ',' ')
      call defarg (IGRPA  ,'outfile',' ',' ')
      call defarg (IGRPK ,'-rs',' ',' ')
c
c ****** Parse the command line.
c
      call parse (ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Interpolate a field from a staggered mesh.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) trim(errmsg)
        end if
c
c ****** Print the usage.
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
        call wrulin (ulbuf)
c
        write (*,*)
        write (*,*) 'The file format is specified by -fmt text|hdf.'
        write (*,*) 'The default file format is hdf.'
        write (*,*)
        write (*,*) 'Set the interpolated data set dimensions '//
     &              'using -x, -y, and -z.'
        write (*,*)
        write (*,*) '-rs is a special option to multiply '//
     &              'the field by r*sin(theta).'
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Verbose flag.
c
      call fetcharg ('-v',set,arg)
      verbose=set
c
c ****** Dimensions.
c
      dim='-x'
      call fetcharg (dim,set,arg)
      ls=lenstr(arg)
      if (arg(1:ls).eq.'<default>') then
        nd=0
      else
        nd=intval(arg,dim)
        if (nd.le.0) go to 900
      end if
      nxi=nd
c
      dim='-y'
      call fetcharg (dim,set,arg)
      ls=lenstr(arg)
      if (arg(1:ls).eq.'<default>') then
        nd=0
      else
        nd=intval(arg,dim)
        if (nd.le.0) go to 900
      end if
      nyi=nd
c
      dim='-z'
      call fetcharg (dim,set,arg)
      ls=lenstr(arg)
      if (arg(1:ls).eq.'<default>') then
        nd=0
      else
        nd=intval(arg,dim)
        if (nd.le.0) go to 900
      end if
      nzi=nd
c
c ****** File format.
c
      call fetcharg ('-fmt',set,arg)
      ls=lenstr(arg)
      fmt=arg(1:ls)
c
c ****** Input file name.
c
      call fetcharg ('infile',set,arg)
      ls=lenstr(arg)
      infile=arg(1:ls)
c
c ****** Output file name.
c
      call fetcharg ('outfile',set,arg)
      ls=lenstr(arg)
      outfile=arg(1:ls)
c
c ****** Multiply-by-r*sin(theta) flag.
c
      call fetcharg ('-rs',set,arg)
      ifrs=set
c
      return
c
c ****** Error return.
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in SETPAR:'
      write (*,*) '### Error in setting dimension ',dim,':'
      write (*,*) 'Invalid value specified: ',nd
      call exit (1)
c
      return
      end
c#######################################################################
      block data bdsntx
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data narg/0/
c
      end
c#######################################################################
      function intval (avalue,name)
c
c-----------------------------------------------------------------------
c
c ****** Get the value of the integer in character string AVALUE.
c
c-----------------------------------------------------------------------
c
      character*(*) avalue,name
c
c-----------------------------------------------------------------------
c
      logical ifint
      external ifint
c
c-----------------------------------------------------------------------
c
      ls=lenstr(avalue)
c
      if (.not.ifint(avalue(1:ls),ivalue)) then
        write (*,*)
        write (*,*) '### ERROR in INTVAL:'
        write (*,*) '### Could not interpret an integer '//
     &              'while setting: ',name
        write (*,*) 'Invalid format: ',avalue(1:ls)
        call exit (1)
      end if
c
      intval=ivalue
c
      return
      end
c#######################################################################
      subroutine wrulin (line)
c
c-----------------------------------------------------------------------
c
c ****** Write the string LINE to the TTY, taking into account
c ****** any carriage returns in the line.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      ls=lenstr(line)
c
      i0=1
  100 continue
c
c ****** Find the next carriage return.
c
      icr=index(line(i0:ls),char(13))
c
      if (icr.gt.0) then
c
c ****** A CR was found.
c
        if (icr.eq.1) then
          write (*,*)
          i0=i0+1
        else
          write (*,*) line(i0:i0+icr-2)
          i0=i0+icr
        end if
        if (i0.le.ls) go to 100
c
      else
c
c ****** A CR was not found.
c
        write (*,*) line(i0:ls)
c
      end if
c
      return
      end
c#######################################################################
      subroutine parse (ierr)
c
c-----------------------------------------------------------------------
c
c ****** Parse the command line.
c
c-----------------------------------------------------------------------
c
c ****** The syntax for the keyword/argument items can be defined
c ****** by using routine DEFARG.
c
c ****** The usage line is constructed in character string ULBUF.
c ****** Use routine WRULIN to write the usage line, since it may
c ****** contain embedded carriage returns.
c
c ****** On return, IERR=0 indicates that the command line was
c ****** parsed successfully.
c
c ****** IERR=1 indicates that no arguments were present.  This
c ****** is usually used to print the usage line.
c
c ****** IERR=2 indicates that a syntax error occured.
c
c ****** IERR=3 indicates that one or more required arguments
c ****** was not supplied.
c
c ****** When IERR=2 or IERR=3, and error message is put into
c ****** character string ERRMSG.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Command line arguments.
c
      integer iargc
      external iargc,getarg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*512 arg
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      ierr=0
      errmsg=' '
c
c ****** Construct the usage line in ULBUF.
c
      ix=1
      ulbuf=' '
c
      call getarg (0,arg)
      ls=lenstr(arg)
      call addtobuf (arg(1:ls),ulbuf,ix)
      nc=ix-1
      ncc=min0(nc,7)
c
      i=1
      itried=0
  100 continue
      if (i.le.narg) then
        ix0=ix
        call addtobuf (' ',ulbuf,ix)
        if (info(IXREQ,i).eq.0) then
          call addtobuf ('[',ulbuf,ix)
        end if
        call getstr (info(IXKEY,i),arg)
        ls=lenstr(arg)
        call addtobuf (arg(1:ls),ulbuf,ix)
        if (info(IXNAM,i).gt.0) then
          call addtobuf (' ',ulbuf,ix)
          call getstr (info(IXNAM,i),arg)
          ls=lenstr(arg)
          call addtobuf (arg(1:ls),ulbuf,ix)
        end if
        if (info(IXREQ,i).eq.0) then
          call addtobuf (']',ulbuf,ix)
        end if
        nc=nc+ix-ix0
        if (nc.gt.78.and.itried.eq.0) then
          itried=1
          ix=ix0
          call addtobuf (char(13),ulbuf,ix)
          ulbuf(ix:ix+ncc-1)=' '
          ix=ix+ncc
          nc=ncc
          go to 100
        end if
        itried=0
        i=i+1
        go to 100
      end if
c
c ****** Get the number of command line arguments.
c
      na=iargc()
      if (na.eq.0) then
        ierr=1
        go to 900
      end if
c
      ia=1
  200 continue
c
      ia0=ia
c
c ****** Process arguments with syntax: <kw> <arg>
c
      if (na-ia+1.ge.2) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw> <arg> <arg>
c
      if (na-ia+1.ge.3) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKAA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            ls=ls+1
            arg(ls:ls)=' '
            ia=ia+1
            call getarg (ia,arg(ls+1:))
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw>
c
      if (na-ia+1.ge.1) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPK,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            info(IXARG,iarg)=0
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <arg>
c
      if (na-ia+1.ge.1) then
        iarg=nxtarg(IGRPA)
        if (iarg.gt.0) then
          call getarg (ia,arg)
          ls=lenstr(arg)
          info(IXARG,iarg)=iputstr(arg(1:ls))
          info(IXSET,iarg)=1
          ia=ia+1
          go to 300
        end if
      end if
c
  300 continue
c
c ****** Check that an argument was found.
c
      if (ia.eq.ia0) then
        ierr=2
        go to 900
      end if
c
c ****** Keep processing arguments until done.
c
      if (na-ia+1.gt.0) go to 200
c
c ****** Check that the required arguments were supplied.
c
      do 400 i=1,narg
        if (info(IXREQ,i).ne.0.and.info(IXSET,i).eq.0) then
          ierr=3
          go to 900
        end if
  400 continue
c
      return
c
c ****** Error exit.
c
  900 continue
c
      if (ierr.eq.2) then
        errmsg='### Syntax error.'
      else if (ierr.eq.3) then
        errmsg='### A required argument was not supplied.'
      end if
c
      return
      end
c#######################################################################
      subroutine defarg (igroup,kw,default,argname)
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax for a command line argument item.
c
c-----------------------------------------------------------------------
c
c ****** IGROUP is the syntax group index.
c ****** KW is the keyword.
c ****** DEFAULT is the default value of the argument.
c ****** ARGNAME is the name of the argument (for use in
c ****** constructing the usage line).
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw,default,argname
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Check the group index.
c
      if (igroup.lt.0.or.igroup.gt.ngroup) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Invalid group index specified.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Check for a null keyword.
c
      if (kw.eq.' ') then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### The keyword is null.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Increment the argument counter.
c
      if (narg.ge.mxarg) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Exceeded number of allowed arguments.'
        write (*,*) 'Maximum number of arguments = ',mxarg
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
      narg=narg+1
c
c ****** Store the group index and keyword.
c
c ****** For group IGRPA (single arguments), the name of the
c ****** argument is stored as the "keyword".
c
      info(IXGRP,narg)=igroup
      ls=lenstr(kw)
      info(IXKEY,narg)=iputstr(kw(1:ls))
c
c ****** Initialize the flag which indicates whether an argument
c ****** has been set.
c
      info(IXSET,narg)=0
c
c ****** If a default argument was supplied, the argument
c ****** does not have to be set.  Use DEFAULT=' ' to
c ****** indicate that an argument is required.
c
c ****** If a default argument has been supplied, store it and
c ****** put its pointer in info(IXARG,narg).  If there is no
c ****** default, set info(IXARG,narg)=0.
c
c ****** Since group IGRPK doesn't have an argument,
c ****** DEFAULT is ignored for this group.
c
      if (igroup.eq.IGRPK) then
        info(IXREQ,narg)=0
        info(IXARG,narg)=0
      else
        if (default.eq.' ') then
          info(IXREQ,narg)=1
          info(IXARG,narg)=0
        else
          info(IXREQ,narg)=0
          ls=lenstr(default)
          info(IXARG,narg)=iputstr(default(1:ls))
        end if
      end if
c
c ****** Store the argument name.  For groups IGRPK (keywords) 
c ****** and IGRPA (single arguments), there is no argument name,
c ****** so ARGNAME is ignored.
c
      if (igroup.eq.IGRPK.or.igroup.eq.IGRPA) then
        info(IXNAM,narg)=0
      else
        ls=lenstr(argname)
        info(IXNAM,narg)=iputstr(argname(1:ls))
      end if
c
      return
      end
c#######################################################################
      subroutine fetcharg (kw,set,arg)
c
c-----------------------------------------------------------------------
c
c ****** Fetch the argument corresponding to keyword KW.
c
c-----------------------------------------------------------------------
c
c ****** If the corresponding argument was set, return SET=.true.,
c ****** and copy the argument to ARG.
c
c ****** If the argument was not set, return SET=.false..
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      logical set
      character*(*) kw,arg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*512 tkw
c
c-----------------------------------------------------------------------
c
c ****** Find the index of the argument corresponding to keyword KW.
c
      do 100 i=narg,1,-1
        call getstr (info(IXKEY,i),tkw)
        if (tkw.eq.kw) then
          iarg=i
          go to 200
        end if
  100 continue
c
      ls=lenstr(kw)
      write (*,*)
      write (*,*) '### ERROR in FETCHARG:'
      write (*,*) '### Requested keyword could not be matched.'
      write (*,*) 'Keyword = ',kw(1:ls)
      call exit (1)
c
  200 continue
c
      set=info(IXSET,iarg).ne.0
      if (info(IXARG,iarg).gt.0) then
        call getstr (info(IXARG,iarg),arg)
      else
        arg=' '
      end if
c
      return
      end
c#######################################################################
      function matchkw (igroup,kw)
c
c-----------------------------------------------------------------------
c
c ****** Match keyword KW against the list of keywords in
c ****** group IGROUP.
c
c ****** If found, set the function value to the corresponding
c ****** argument number.  Otherwise, set MATCHKW=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*512 tkw
c
c-----------------------------------------------------------------------
c
      do 100 i=narg,1,-1
        if (info(IXGRP,i).eq.igroup) then
          call getstr (info(IXKEY,i),tkw)
          if (tkw.eq.kw) then
            matchkw=i
            return
          end if
        end if
  100 continue
c
      matchkw=0
c
      return
      end
c#######################################################################
      function nxtarg (igroup)
c
c-----------------------------------------------------------------------
c
c ****** Find the position of the next argument in group IGROUP
c ****** which has not been set.
c
c-----------------------------------------------------------------------
c
c ****** If an empty slot is found, set the function value
c ****** to the corresponding argument number.
c
c ****** Otherwise, set NXTARG=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      do 100 i=1,narg
        if (info(IXGRP,i).eq.igroup) then
          if (info(IXSET,i).eq.0) then
            nxtarg=i
            return
          end if
        end if
  100 continue
c
      nxtarg=0
c
      return
      end
c#######################################################################
      function iputstr (line)
c
c-----------------------------------------------------------------------
c
c ****** Put the string in LINE into internal storage, and return
c ****** a pointer to it as the function value.
c
c ****** The string can be retrieved by calling routine GETSTR
c ****** with the returned pointer.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
      ls=len(line)
c
c ****** Check that the string is not empty.
c
      if (ls.le.0) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Trying to store a zero-length string.'
        call exit (1)
      end if
c
c ****** Check that a free pointer exists.
c
      if (nstr.ge.mxstr) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for pointers.'
        write (*,*) 'Maximum number of pointers = ',mxstr
        call exit (1)
      end if
c
      nstr=nstr+1
c
c ****** Check that there is enough room to store the string.
c
      ix1=ix+ls-1
      if (ix1.gt.LENBUF) then
        nb=ix1-LENBUF
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for strings.'
        write (*,*) 'Number of additional bytes needed = ',nb
        call exit (1)
      end if
c
      strbuf(ix:ix1)=line
c
c ****** Set IX to point to the next available character.
c
      ix=ix1+1
      indx(nstr)=ix
c
c ****** Return the pointer to the string as the function value.
c
      iputstr=nstr
c
      return
      end
c#######################################################################
      block data bdstr
c
c-----------------------------------------------------------------------
c
      parameter (mxstr=100)
      common /pstr1/ nstr,ix,indx(mxstr)
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data nstr/0/
      data ix/1/
c
      end
c#######################################################################
      subroutine getstr (ip,line)
c
c-----------------------------------------------------------------------
c
c ****** Get the string associated with pointer IP and put it
c ****** into LINE.
c
c ****** This routine retrieves strings stored with routine IPUTSTR.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
c ****** Check that the pointer is valid.
c
      if (ip.lt.0.or.ip.gt.nstr) then
        write (*,*)
        write (*,*) '### ERROR in GETSTR:'
        write (*,*) '### Invalid string pointer.'
        write (*,*) 'Pointer = ',ip
        call exit (1)
      end if
c
c ****** Retrieve the string.
c
      if (ip.eq.1) then
        i0=1
      else
        i0=indx(ip-1)
      end if
      i1=indx(ip)-1
c
      line=strbuf(i0:i1)
c
      return
      end
c#######################################################################
      subroutine addtobuf (line,buf,ix)
c
c-----------------------------------------------------------------------
c
c ****** Add the string in LINE to the buffer BUF, starting
c ****** at character position IX in BUF.
c
c-----------------------------------------------------------------------
c
c ****** When COMPRESS=.true., multiple spaces in LINE are
c ****** converted into a single space.
c
c-----------------------------------------------------------------------
c
      logical COMPRESS
      parameter (COMPRESS=.false.)
c
c-----------------------------------------------------------------------
c
      character*(*) line,buf
c
c-----------------------------------------------------------------------
c
      logical psp
c
c-----------------------------------------------------------------------
c
      lbuf=len(buf)
      ll=len(line)
c
      psp=.false.
      do 100 i=1,ll
c
        if (compress) then
c
c ****** Compress multiple spaces into a single space.
c
          if (line(i:i).eq.' ') then
            if (psp) go to 100
            psp=.true.
          else
            psp=.false.
          end if
        end if
c
c ****** Check for a buffer overflow.
c
        if (ix.gt.lbuf) then
          write (*,*)
          write (*,*) '### ERROR in ADDTOBUF:'
          write (*,*) '### Internal buffer overflow.'
          write (*,*) '[This is an internal error.]'
          call exit (1)
        end if
c
c ****** Write the character to the buffer.
c
        buf(ix:ix)=line(i:i)
        ix=ix+1
c
  100 continue
c
      return
      end
c#######################################################################
      logical function ifint (alpha,ivalue)
c
c-----------------------------------------------------------------------
c
c ****** If ALPHA represents an integer, return IFINT=.true., and
c ****** put its value into IVALUE.
c
c ****** Otherwise, return IFINT=.false..
c
c-----------------------------------------------------------------------
c
c ****** A valid integer has the format:
c
c          ALPHA = '[A]B[B...B]',
c
c ****** where A represents a + or - sign, and B represents a digit
c ****** between 0 and 9, inclusive.
c
c-----------------------------------------------------------------------
c
      character*(*) alpha
      integer ivalue
c
c-----------------------------------------------------------------------
c
      character*5 fmt
c
c-----------------------------------------------------------------------
c
      ifint=.false.
c
      do 100 k=1,len(alpha)
c
        i=ichar(alpha(k:k))
c
c ****** Check for a sign in the first position.
c
        if (k.eq.1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Check for a digit.
c
        if (i.ge.48.and.i.le.57) then
          ifint=.true.
          go to 100
        end if
c
c ****** Failed check: fall through here.
c
        ifint=.false.
c
        return
c
  100 continue
c
c ****** Obtain its numeric value.
c
      fmt='(i  )'
      write (fmt(3:4),'(i2.2)') len(alpha)
c
      if (ifint) read (alpha,fmt) ivalue
c
      return
      end
