      program MKPER
c
c-----------------------------------------------------------------------
c
c ****** Extend an SDS periodically. 
c
c ****** This version is for SGI IRIS.
c
c-----------------------------------------------------------------------
c
c ****** Compile and link with:
c
c     f77 -O3 mkper.f $LIBDF -o mkper
c
c ****** When debugging, use the compile line:
c
c     f77 -g mkper.f $LIBDF -o mkper
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        02/25/97, ZM, Version 1.00:
c
c         - Original version of program.
c
c-----------------------------------------------------------------------
c
      character cname*(*),cvers*4,cdate*8
c
      parameter (cname='MKPER')
      parameter (cvers='1.00')
      parameter (cdate='02/25/97')
c
c-----------------------------------------------------------------------
c
      cliche syntax
c
c-----------------------------------------------------------------------
c
c ****** Group definitions.
c
c        GROUP 1: <kw>
c        GROUP 2: <arg>
c        GROUP 3: <kw> <arg>
c        GROUP 4: <kw> <arg> <arg>
c
      parameter (ngroup=4)
c
      parameter (IGRPK  =1)
      parameter (IGRPA  =2)
      parameter (IGRPKA =3)
      parameter (IGRPKAA=4)
c
      parameter (mxarg=100)
c
      parameter (IXGRP=1)
      parameter (IXSET=2)
      parameter (IXREQ=3)
      parameter (IXKEY=4)
      parameter (IXARG=5)
      parameter (IXNAM=6)
c
      integer narg,info(6,mxarg)
      common /sntx1/ narg,info
c
c ****** Buffer for the usage line.
c
      character*512 ulbuf
      common /sntx2/ ulbuf
c
c ****** Buffer for the error message.
c
      character*72 errmsg
      common /sntx3/ errmsg
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      character*16 fmt
      character*80 infile,outfile
      common /parcom1/ fmt,infile,outfile
c
      common /parcom2/ iper
c
c-----------------------------------------------------------------------
c
c ****** Pointers to the data arrays and the scales.
c
      pointer (ipx,x),(ipy,y),(ipz,z)
      pointer (ipf,f)
c
      pointer (ipxx,xx),(ipyy,yy),(ipzz,zz)
      pointer (ipff,ff)
c
      parameter (mxdim=3)
      integer idim(mxdim),ips(mxdim)
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call setpar (cname,cvers,cdate)
c
c ****** Read the SDS.
c
      lf=lenstr(infile)
c
      call rdsds (fmt,infile,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not read the data set.'
        write (*,*) 'IERR (from RDSDS) = ',ierr
        write (*,*) 'File name: ',infile(1:lf)
        call exit (1)
      end if
c
c ****** Check that the dimension to extend is valid.
c
      if (iper.gt.ndim) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Invalid dimension to extend.'
        write (*,*)
        write (*,*) 'The specified dimension is inconsistent with'//
     &              ' the dimensions of the SDS.'
        write (*,*)
        write (*,*) 'Number of dimensions of SDS = ',ndim
        call exit (1)
      end if
c
      if (ndim.eq.1) then
        nx=idim(1)
        ny=1
        nz=1
        ipx=ips(1)
        ipy=ips(1)
        ipz=ips(1)
      else if (ndim.eq.2) then
        nx=idim(1)
        ny=idim(2)
        nz=1
        ipx=ips(1)
        ipy=ips(2)
        ipz=ips(1)
      else if (ndim.eq.3) then
        nx=idim(1)
        ny=idim(2)
        nz=idim(3)
        ipx=ips(1)
        ipy=ips(2)
        ipz=ips(3)
      end if
c
c ****** Set the dimensions of the output array.
c
      nxx=nx
      nyy=ny
      nzz=nz
c
      if (iper.eq.1) then
        nxx=nx+1
      else if (iper.eq.2) then
        nyy=ny+1
      else if (iper.eq.3) then
        nzz=nz+1
      end if
c
c ****** Assign memory for the new array and its scales.
c
      nwd=nxx*nyy*nzz
      ipff=igetmem(nwd)
      if (ipff.eq.0) go to 900
c
      if (ifscale.ne.0) then
        nwd=nxx
        ipxx=igetmem(nwd)
        if (ipxx.eq.0) go to 900
        nwd=nyy
        ipyy=igetmem(nwd)
        if (ipyy.eq.0) go to 900
        nwd=nzz
        ipzz=igetmem(nwd)
        if (ipzz.eq.0) go to 900
      end if
c
c ****** Add the requested slice.
c
      call addslice  (iper,ifscale,nx,ny,nz,x,y,z,f,
     &                nxx,nyy,nzz,xx,yy,zz,ff)
c
c ****** Write the new array.
c
      idim(1)=nxx
      idim(2)=nyy
      idim(3)=nzz
c
      ips(1)=ipxx
      ips(2)=ipyy
      ips(3)=ipzz
c
      lf=lenstr(outfile)
c
      call wrsds (fmt,outfile,ndim,idim,ff,ifscale,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not write the output data file.'
        write (*,*) 'IERR (from WRSDS) = ',ierr
        write (*,*) 'File name: ',outfile(1:lf)
        call exit (1)
      end if
c
      call exit (0)
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in ',cname,':'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      call exit (1)
c
      end
c#######################################################################
      subroutine addslice (iper,ifscale,nx,ny,nz,x,y,z,f,
     &                     nxx,nyy,nzz,xx,yy,zz,ff)
c
c-----------------------------------------------------------------------
c
      real x(nx),y(ny),z(nz),f(nx,ny,nz)
      real xx(nxx),yy(nyy),zz(nzz),ff(nxx,nyy,nzz)
c
c-----------------------------------------------------------------------
c
c ****** Copy F into FF (including the scales if present).
c
      do k=1,nz
        do j=1,ny
          do i=1,nx
            ff(i,j,k)=f(i,j,k)
          enddo
        enddo
      enddo
c
      if (ifscale.ne.0) then
        do i=1,nx
          xx(i)=x(i)
        enddo
        do j=1,ny
          yy(j)=y(j)
        enddo
        do k=1,nz
          zz(k)=z(k)
        enddo
      end if
c
c ****** Add the requested slice.
c
      if (iper.eq.1) then
        do k=1,nz
          do j=1,ny
            ff(nx+1,j,k)=ff(1,j,k)
          enddo
        enddo
        if (ifscale.ne.0) then
          if (nx.le.1) then
            dx=1.
          else
            dx=xx(nx)-xx(nx-1)
          end if
          xx(nx+1)=xx(nx)+dx
        end if
      else if (iper.eq.2) then
        do k=1,nz
          do i=1,nx
            ff(i,ny+1,k)=ff(i,1,k)
          enddo
        enddo
        if (ifscale.ne.0) then
          if (ny.le.1) then
            dy=1.
          else
            dy=yy(ny)-yy(ny-1)
          end if
          yy(ny+1)=yy(ny)+dy
        end if
      else if (iper.eq.3) then
        do j=1,ny
          do i=1,nx
            ff(i,j,nz+1)=ff(i,j,1)
          enddo
        enddo
        if (ifscale.ne.0) then
          if (nz.le.1) then
            dz=1.
          else
            dz=zz(nz)-zz(nz-1)
          end if
          zz(nz+1)=zz(nz)+dz
        end if
      end if
c
      return
      end
c#######################################################################
      subroutine rdsds (fmt,fname,mxdim,ndim,idim,
     &                  ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set.
c
c ****** Use routine RDTXT or RDHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in RDSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from a text file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    Text data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'r')
c
c ****** Get the number of dimensions.
c
      call rdint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 910
c
      if (ndim.lt.1.or.ndim.gt.mxdim) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT:'
        write (*,*) '### Invalid number of dimensions in file.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'MXDIM = ',mxdim
        ierr=1
        return
      end if
c
c ****** Read the dimensions.
c
      do 100 i=1,ndim
        call rdint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 910
        if (idim(i).le.0) go to 920
  100 continue
c
c ****** Check if the scales are present.
c
      call rdint (1,1,ifscale,ierr)
      if (ierr.ne.0) go to 910
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 300 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          call rdfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 910
  300   continue
      end if
c
c ****** Read the array.
c
      call rdfp (1,nwdf,f,ierr)
      if (ierr.ne.0) go to 910
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Error while reading text data.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=3
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine rdint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of INTEGER data into array I from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) i
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of REAL data into array F from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) f
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata
      external dsgdims,dsgdisc,dsgdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Get the dimensions of the SDS.
c
      lf=lenstr(fname)
      iret=dsgdims(fname(1:lf),ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check the dimensions.
c
      do 100 i=1,ndim
        if (idim(i).le.0) go to 910
  100 continue
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Check if all scales are present.
c ****** (Use array F for temporary storage.)
c
      nscal=0
      do 300 i=1,ndim
        iret=dsgdisc(i,idim(i),f)
        if (iret.eq.0) nscal=nscal+1
  300 continue
c
      if (nscal.eq.ndim) then
        ifscale=1
      else
        ifscale=0
      end if
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 400 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
  400   continue
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine wrsds (fmt,fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set.
c
c ****** Use routine WRTXT or WRHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in WRSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to text file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'rw')
c
c ****** Write the number of dimensions.
c
      call wrint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Write the dimensions.
c
      nwd=1
      do 100 i=1,ndim
        call wrint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 900
        nwd=nwd*idim(i)
  100 continue
c
c ****** Write the scales.
c
      if (ifscale.ne.0) then
        call wrint (1,1,1,ierr)
        if (ierr.ne.0) go to 900
        do 200 i=1,ndim
          ipx=ips(i)
          call wrfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 900
  200   continue
      else
        call wrint (1,1,0,ierr)
        if (ierr.ne.0) go to 900
      end if
c
c ****** Write the array.
c
      call wrfp (1,nwd,f,ierr)
      if (ierr.ne.0) go to 900
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRTXT:'
      write (*,*) '### Error in writing data to text file.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of INTEGER data into array I from unit IUN
c ****** using a free format text write.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,*,err=100) i
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of REAL data into array F from unit IUN.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,900,err=100) f
c
  900 format (5(1x,1pe13.6))
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to HDF file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dssdims,dssdisc,dspdata
      external dssdims,dssdisc,dspdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      iret=dssdims(ndim,idim)
      if (iret.ne.0) go to 900
c
      if (ifscale.ne.0) then
        do 100 i=1,ndim
          ipx=ips(i)
          iret=dssdisc(i,idim(i),x)
          if (iret.ne.0) go to 910
  100   continue
      end if
c
      iret=dspdata(fname,ndim,idim,f)
      if (iret.ne.0) go to 920
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDIMS) = ',iret,']'
      ierr=1
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the scales.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDISC) = ',iret,']'
      write (*,*) '[Dimension number = ',i,']'
      ierr=2
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not write the data array.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSPDATA) = ',iret,']'
      ierr=3
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do i=len(str),1,-1
        if (str(i:i).ne.' ') go to 100
      enddo
      i=1
  100 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      integer function igetmem (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** The function returns the pointer to the array.
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for NWD words.
c
c     igetmem=malloc(%val(4*nwd))
      igetmem=malloc(4*nwd)
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      subroutine setpar (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      character*(*) cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      character*16 fmt
      character*80 infile,outfile
      common /parcom1/ fmt,infile,outfile
c
      common /parcom2/ iper
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*72 arg
c
c-----------------------------------------------------------------------
c
      logical set
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (IGRPKA,'-fmt','hdf','text|hdf')
      call defarg (IGRPK ,'-x',' ',' ')
      call defarg (IGRPK ,'-y',' ',' ')
      call defarg (IGRPK ,'-z',' ',' ')
      call defarg (IGRPA ,'infile',' ',' ')
      call defarg (IGRPA ,'outfile',' ',' ')
c
c ****** Parse the command line.
c
      call parse (ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Extend an SDS periodically.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage.
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
        call wrulin (ulbuf)
        write (*,*)
        write (*,*) 'Specify the dimension to extend using -x,'//
     &              ' -y, or -z.'
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** File format.
c
      call fetcharg ('-fmt',set,arg)
      ls=lenstr(arg)
      fmt=arg(1:ls)
c
c ****** Input file name.
c
      call fetcharg ('infile',set,arg)
      ls=lenstr(arg)
      infile=arg(1:ls)
c
c ****** Output file name.
c
      call fetcharg ('outfile',set,arg)
      ls=lenstr(arg)
      outfile=arg(1:ls)
c
c ****** Dimension to operate on.
c
      iper=0
      nset=0
c
      call fetcharg ('-x',set,arg)
      if (set) then
        iper=1
        nset=nset+1
      end if
c
      call fetcharg ('-y',set,arg)
      if (set) then
        iper=2
        nset=nset+1
      end if
c
      call fetcharg ('-z',set,arg)
      if (set) then
        iper=3
        nset=nset+1
      end if
c
      if (nset.gt.1) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Invalid dimension to extend.'
        write (*,*)
        write (*,*) 'You must specify only one of -x, -y, or -z.'
        call exit (1)
      end if
c
      return
      end
c#######################################################################
      block data bdsntx
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data narg/0/
c
      end
c#######################################################################
      subroutine wrulin (line)
c
c-----------------------------------------------------------------------
c
c ****** Write the string LINE to the TTY, taking into account
c ****** any carriage returns in the line.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      ls=lenstr(line)
c
      i0=1
  100 continue
c
c ****** Find the next carriage return.
c
      icr=index(line(i0:ls),char(13))
c
      if (icr.gt.0) then
c
c ****** A CR was found.
c
        if (icr.eq.1) then
          write (*,*)
          i0=i0+1
        else
          write (*,*) line(i0:i0+icr-2)
          i0=i0+icr
        end if
        if (i0.le.ls) go to 100
c
      else
c
c ****** A CR was not found.
c
        write (*,*) line(i0:ls)
c
      end if
c
      return
      end
c#######################################################################
      subroutine parse (ierr)
c
c-----------------------------------------------------------------------
c
c ****** Parse the command line.
c
c-----------------------------------------------------------------------
c
c ****** The syntax for the keyword/argument items can be defined
c ****** by using routine DEFARG.
c
c ****** The usage line is constructed in character string ULBUF.
c ****** Use routine WRULIN to write the usage line, since it may
c ****** contain embedded carriage returns.
c
c ****** On return, IERR=0 indicates that the command line was
c ****** parsed successfully.
c
c ****** IERR=1 indicates that no arguments were present.  This
c ****** is usually used to print the usage line.
c
c ****** IERR=2 indicates that a syntax error occured.
c
c ****** IERR=3 indicates that one or more required arguments
c ****** was not supplied.
c
c ****** When IERR=2 or IERR=3, and error message is put into
c ****** character string ERRMSG.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Command line arguments.
c
      integer iargc
      external iargc,getarg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*72 arg
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      ierr=0
      errmsg=' '
c
c ****** Construct the usage line in ULBUF.
c
      ix=1
      ulbuf=' '
c
      call getarg (0,arg)
      ls=lenstr(arg)
      call addtobuf (arg(1:ls),ulbuf,ix)
      nc=ix-1
      ncc=min0(nc,7)
c
      i=1
      itried=0
  100 continue
      if (i.le.narg) then
        ix0=ix
        call addtobuf (' ',ulbuf,ix)
        if (info(IXREQ,i).eq.0) then
          call addtobuf ('[',ulbuf,ix)
        end if
        call getstr (info(IXKEY,i),arg)
        ls=lenstr(arg)
        call addtobuf (arg(1:ls),ulbuf,ix)
        if (info(IXNAM,i).gt.0) then
          call addtobuf (' ',ulbuf,ix)
          call getstr (info(IXNAM,i),arg)
          ls=lenstr(arg)
          call addtobuf (arg(1:ls),ulbuf,ix)
        end if
        if (info(IXREQ,i).eq.0) then
          call addtobuf (']',ulbuf,ix)
        end if
        nc=nc+ix-ix0
        if (nc.gt.78.and.itried.eq.0) then
          itried=1
          ix=ix0
          call addtobuf (char(13),ulbuf,ix)
          ulbuf(ix:ix+ncc-1)=' '
          ix=ix+ncc
          nc=ncc
          go to 100
        end if
        itried=0
        i=i+1
        go to 100
      end if
c
c ****** Get the number of command line arguments.
c
      na=iargc()
      if (na.eq.0) then
        ierr=1
        go to 900
      end if
c
      ia=1
  200 continue
c
      ia0=ia
c
c ****** Process arguments with syntax: <kw> <arg>
c
      if (na-ia+1.ge.2) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw> <arg> <arg>
c
      if (na-ia+1.ge.3) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKAA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            ls=ls+1
            arg(ls:ls)=' '
            ia=ia+1
            call getarg (ia,arg(ls+1:))
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw>
c
      if (na-ia+1.ge.1) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPK,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            info(IXARG,iarg)=0
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <arg>
c
      if (na-ia+1.ge.1) then
        iarg=nxtarg(IGRPA)
        if (iarg.gt.0) then
          call getarg (ia,arg)
          ls=lenstr(arg)
          info(IXARG,iarg)=iputstr(arg(1:ls))
          info(IXSET,iarg)=1
          ia=ia+1
          go to 300
        end if
      end if
c
  300 continue
c
c ****** Check that an argument was found.
c
      if (ia.eq.ia0) then
        ierr=2
        go to 900
      end if
c
c ****** Keep processing arguments until done.
c
      if (na-ia+1.gt.0) go to 200
c
c ****** Check that the required arguments were supplied.
c
      do 400 i=1,narg
        if (info(IXREQ,i).ne.0.and.info(IXSET,i).eq.0) then
          ierr=3
          go to 900
        end if
  400 continue
c
      return
c
c ****** Error exit.
c
  900 continue
c
      if (ierr.eq.2) then
        errmsg='### Syntax error.'
      else if (ierr.eq.3) then
        errmsg='### A required argument was not supplied.'
      end if
c
      return
      end
c#######################################################################
      subroutine defarg (igroup,kw,default,argname)
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax for a command line argument item.
c
c-----------------------------------------------------------------------
c
c ****** IGROUP is the syntax group index.
c ****** KW is the keyword.
c ****** DEFAULT is the default value of the argument.
c ****** ARGNAME is the name of the argument (for use in
c ****** constructing the usage line).
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw,default,argname
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Check the group index.
c
      if (igroup.lt.0.or.igroup.gt.ngroup) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Invalid group index specified.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Check for a null keyword.
c
      if (kw.eq.' ') then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### The keyword is null.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Increment the argument counter.
c
      if (narg.ge.mxarg) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Exceeded number of allowed arguments.'
        write (*,*) 'Maximum number of arguments = ',mxarg
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
      narg=narg+1
c
c ****** Store the group index and keyword.
c
c ****** For group IGRPA (single arguments), the name of the
c ****** argument is stored as the "keyword".
c
      info(IXGRP,narg)=igroup
      ls=lenstr(kw)
      info(IXKEY,narg)=iputstr(kw(1:ls))
c
c ****** Initialize the flag which indicates whether an argument
c ****** has been set.
c
      info(IXSET,narg)=0
c
c ****** If a default argument was supplied, the argument
c ****** does not have to be set.  Use DEFAULT=' ' to
c ****** indicate that an argument is required.
c
c ****** If a default argument has been supplied, store it and
c ****** put its pointer in info(IXARG,narg).  If there is no
c ****** default, set info(IXARG,narg)=0.
c
c ****** Since group IGRPK doesn't have an argument,
c ****** DEFAULT is ignored for this group.
c
      if (igroup.eq.IGRPK) then
        info(IXREQ,narg)=0
        info(IXARG,narg)=0
      else
        if (default.eq.' ') then
          info(IXREQ,narg)=1
          info(IXARG,narg)=0
        else
          info(IXREQ,narg)=0
          ls=lenstr(default)
          info(IXARG,narg)=iputstr(default(1:ls))
        end if
      end if
c
c ****** Store the argument name.  For groups IGRPK (keywords) 
c ****** and IGRPA (single arguments), there is no argument name,
c ****** so ARGNAME is ignored.
c
      if (igroup.eq.IGRPK.or.igroup.eq.IGRPA) then
        info(IXNAM,narg)=0
      else
        ls=lenstr(argname)
        info(IXNAM,narg)=iputstr(argname(1:ls))
      end if
c
      return
      end
c#######################################################################
      subroutine fetcharg (kw,set,arg)
c
c-----------------------------------------------------------------------
c
c ****** Fetch the argument corresponding to keyword KW.
c
c-----------------------------------------------------------------------
c
c ****** If the corresponding argument was set, return SET=.true.,
c ****** and copy the argument to ARG.
c
c ****** If the argument was not set, return SET=.false..
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      logical set
      character*(*) kw,arg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
c ****** Find the index of the argument corresponding to keyword KW.
c
      do 100 i=narg,1,-1
        call getstr (info(IXKEY,i),tkw)
        if (tkw.eq.kw) then
          iarg=i
          go to 200
        end if
  100 continue
c
      ls=lenstr(kw)
      write (*,*)
      write (*,*) '### ERROR in FETCHARG:'
      write (*,*) '### Requested keyword could not be matched.'
      write (*,*) 'Keyword = ',kw(1:ls)
      call exit (1)
c
  200 continue
c
      set=info(IXSET,iarg).ne.0
      if (info(IXARG,iarg).gt.0) then
        call getstr (info(IXARG,iarg),arg)
      else
        arg=' '
      end if
c
      return
      end
c#######################################################################
      function matchkw (igroup,kw)
c
c-----------------------------------------------------------------------
c
c ****** Match keyword KW against the list of keywords in
c ****** group IGROUP.
c
c ****** If found, set the function value to the corresponding
c ****** argument number.  Otherwise, set MATCHKW=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
      do 100 i=narg,1,-1
        if (info(IXGRP,i).eq.igroup) then
          call getstr (info(IXKEY,i),tkw)
          if (tkw.eq.kw) then
            matchkw=i
            return
          end if
        end if
  100 continue
c
      matchkw=0
c
      return
      end
c#######################################################################
      function nxtarg (igroup)
c
c-----------------------------------------------------------------------
c
c ****** Find the position of the next argument in group IGROUP
c ****** which has not been set.
c
c-----------------------------------------------------------------------
c
c ****** If an empty slot is found, set the function value
c ****** to the corresponding argument number.
c
c ****** Otherwise, set NXTARG=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      do 100 i=1,narg
        if (info(IXGRP,i).eq.igroup) then
          if (info(IXSET,i).eq.0) then
            nxtarg=i
            return
          end if
        end if
  100 continue
c
      nxtarg=0
c
      return
      end
c#######################################################################
      function iputstr (line)
c
c-----------------------------------------------------------------------
c
c ****** Put the string in LINE into internal storage, and return
c ****** a pointer to it as the function value.
c
c ****** The string can be retrieved by calling routine GETSTR
c ****** with the returned pointer.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
      ls=len(line)
c
c ****** Check that the string is not empty.
c
      if (ls.le.0) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Trying to store a zero-length string.'
        call exit (1)
      end if
c
c ****** Check that a free pointer exists.
c
      if (nstr.ge.mxstr) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for pointers.'
        write (*,*) 'Maximum number of pointers = ',mxstr
        call exit (1)
      end if
c
      nstr=nstr+1
c
c ****** Check that there is enough room to store the string.
c
      ix1=ix+ls-1
      if (ix1.gt.LENBUF) then
        nb=ix1-LENBUF
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for strings.'
        write (*,*) 'Number of additional bytes needed = ',nb
        call exit (1)
      end if
c
      strbuf(ix:ix1)=line
c
c ****** Set IX to point to the next available character.
c
      ix=ix1+1
      indx(nstr)=ix
c
c ****** Return the pointer to the string as the function value.
c
      iputstr=nstr
c
      return
      end
c#######################################################################
      block data bdstr
c
c-----------------------------------------------------------------------
c
      parameter (mxstr=100)
      common /pstr1/ nstr,ix,indx(mxstr)
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data nstr/0/
      data ix/1/
c
      end
c#######################################################################
      subroutine getstr (ip,line)
c
c-----------------------------------------------------------------------
c
c ****** Get the string associated with pointer IP and put it
c ****** into LINE.
c
c ****** This routine retrieves strings stored with routine IPUTSTR.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
c ****** Check that the pointer is valid.
c
      if (ip.lt.0.or.ip.gt.nstr) then
        write (*,*)
        write (*,*) '### ERROR in GETSTR:'
        write (*,*) '### Invalid string pointer.'
        write (*,*) 'Pointer = ',ip
        call exit (1)
      end if
c
c ****** Retrieve the string.
c
      if (ip.eq.1) then
        i0=1
      else
        i0=indx(ip-1)
      end if
      i1=indx(ip)-1
c
      line=strbuf(i0:i1)
c
      return
      end
c#######################################################################
      subroutine addtobuf (line,buf,ix)
c
c-----------------------------------------------------------------------
c
c ****** Add the string in LINE to the buffer BUF, starting
c ****** at character position IX in BUF.
c
c-----------------------------------------------------------------------
c
c ****** When COMPRESS=.true., multiple spaces in LINE are
c ****** converted into a single space.
c
c-----------------------------------------------------------------------
c
      logical COMPRESS
      parameter (COMPRESS=.false.)
c
c-----------------------------------------------------------------------
c
      character*(*) line,buf
c
c-----------------------------------------------------------------------
c
      logical psp
c
c-----------------------------------------------------------------------
c
      lbuf=len(buf)
      ll=len(line)
c
      psp=.false.
      do 100 i=1,ll
c
        if (compress) then
c
c ****** Compress multiple spaces into a single space.
c
          if (line(i:i).eq.' ') then
            if (psp) go to 100
            psp=.true.
          else
            psp=.false.
          end if
        end if
c
c ****** Check for a buffer overflow.
c
        if (ix.gt.lbuf) then
          write (*,*)
          write (*,*) '### ERROR in ADDTOBUF:'
          write (*,*) '### Internal buffer overflow.'
          write (*,*) '[This is an internal error.]'
          call exit (1)
        end if
c
c ****** Write the character to the buffer.
c
        buf(ix:ix)=line(i:i)
        ix=ix+1
c
  100 continue
c
      return
      end
