!****h* e0pro/main
! NAME 
!   COMEVO
! PURPOSE
!   Evolution of values at given positions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
MODULE COMEVO
!---------------------------------------------------------------------
  USE commax,  ONLY: maxhel, maxsel, maxfld, maxfstep  !Obtain maximum array dimension
  IMPLICIT NONE
  PUBLIC
!---------------------------------------------------------------------
!L                  E.1      Values at points in heliosphere
  INTEGER, DIMENSION(maxhel) :: N1HEL, N2HEL, N3HEL
  INTEGER  NXHEL, NXSEL, NXGEO
  INTEGER, DIMENSION(1024) :: NNXHEL, NNXSEL, NNHEL, NNSEL
  REAL(kind=8), DIMENSION(maxhel) ::  &
#ifdef mhd
    B1HEL, B2HEL, B3HEL, BPHEL,  &
#endif
    DHEL,  DPHEL, THEL,  V1HEL, V2HEL, V3HEL, X1HEL, X2HEL, X3HEL
  REAL(kind=8), DIMENSION(maxhel) ::  &
    E1HEL, E2HEL, E3HEL, XJ1HEL, XJ2HEL, XJ3HEL, ETAHEL
  REAL(kind=8)  &
    TOTMAS, ENETOT, ENETHE, ENEKIN, ENEMAG, CROSCO, DIVB, ENSKIN,  &
    ENSMAG
  LOGICAL NLDIAV
!---------------------------------------------------------------------
!L                  E.2      Values at points around geospace
  INTEGER, DIMENSION(5) :: N1GEO, N2GEO, N3GEO
  REAL(kind=8), DIMENSION(5) ::  &
#ifdef mhd
    B1GEO, B2GEO, B3GEO, BPGEO,  &
#endif
    DGEO,  DPGEO, TGEO,  V1GEO, V2GEO, V3GEO, X1GEO, X2GEO, X3GEO
  REAL(kind=8), DIMENSION(5) ::  &
    E1GEO, E2GEO, E3GEO, XJ1GEO, XJ2GEO, XJ3GEO, ETAGEO
!---------------------------------------------------------------------
!L                  E.3      Values at points on Sun-Earth line         
  INTEGER, DIMENSION(maxsel) :: N1SEL, N2SEL, N3SEL 
  REAL(kind=8), DIMENSION(maxsel) ::  &
#ifdef mhd                                                              
    B1SEL, B2SEL, B3SEL, BPSEL,  &
#endif                                                                  
    DSEL,  DPSEL, TSEL,  V1SEL, V2SEL, V3SEL, X1SEL, X2SEL, X3SEL                                 
  REAL(kind=8), DIMENSION(maxsel) ::  &
    E1SEL, E2SEL, E3SEL, XJ1SEL, XJ2SEL, XJ3SEL, ETASEL
!---------------------------------------------------------------------
!L                  E.4      Values at planetary positions
  INTEGER, DIMENSION(9) :: N1PLA, N2PLA, N3PLA
  REAL(kind=8), DIMENSION(9) ::  &
#ifdef mhd
    B1PLA, B2PLA, B3PLA, BPPLA,  &
#endif
    DPLA,  DPPLA, TPLA,  V1PLA, V2PLA, V3PLA, X1PLA, X2PLA, X3PLA
  REAL(kind=8), DIMENSION(9) ::  &
    E1PLA, E2PLA, E3PLA, XJ1PLA, XJ2PLA, XJ3PLA, ETAPLA
!---------------------------------------------------------------------
!L                  E.4      Values at magnetic field lines
  REAL(kind=8), DIMENSION(maxfld,maxfstep) ::  &
#ifdef mhd
    B1FLD, B2FLD, B3FLD, BPFLD,  &
#endif
    DFLD,  DPFLD, TFLD,  V1FLD, V2FLD, V3FLD, X1FLD, X2FLD, X3FLD
! REAL(kind=8), DIMENSION(maxfld,maxfstep) ::  &
!   E1FLD, E2FLD, E3FLD, XJ1FLD, XJ2FLD, XJ3FLD, ETAFLD
!---------------------------------------------------------------------  
END MODULE COMEVO
