!****f* e0pro/main
! NAME 
!   ENLIL
! PURPOSE
!   Fortran main program
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
!PROGRAM ENLIL                                                    
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
    include "paramesh_preprocessor.fh"
#endif                                                                  
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE comout
  USE comglo
  USE comevo
  USE combnd
  USE commet
  USE comadm
  USE comtva
  IMPLICIT NONE
#ifdef MPI
    INTEGER ierr
    INCLUDE 'mpif.h'
#endif
#ifdef USE_VT
    INCLUDE 'VT.inc'
#endif
!---------------------------------------------------------------------
!      1.        Initialize the MPI system
#ifdef MPI
!        1.1       Do the MPI initialization
    call MPI_INIT(ierr)
!        1.2       Which processor I am
    call MPI_COMM_RANK(MPI_COMM_WORLD,myproc,ierr)
!        1.3       How many processors there are
    call MPI_COMM_SIZE(MPI_COMM_WORLD,nprocs,ierr)
#else
    myproc=0
    nprocs=1
#endif
!        1.4       Write
  write(*,*) "Process ", myproc, " of ", nprocs, " is alive"
!---------------------------------------------------------------------
!      2.        Initialize the diagnostic system
#ifdef DIAGNO
!        2.1       Vampir
#ifdef USE_VT
      call VTSETUP()
#endif
!        2.2       HPM
#ifdef HPM
      call F_HPM_INIT(myproc)
#endif
#endif
!---------------------------------------------------------------------
!      3.        Write the code parameters
  call PARAMS
!---------------------------------------------------------------------
!      4.        Enter the MASTER control
  call MASTER
!---------------------------------------------------------------------
!      5.        Terminate the diagnostic system
#ifdef DIAGNO
#ifdef HPM
!        5.1       HPM
      call F_HPM_TERMINATE(myproc)
#endif
#endif
!---------------------------------------------------------------------
!      6.        Finalize the MPI system
#ifdef MPI
    call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    call MPI_FINALIZE(ierr)
#endif
!---------------------------------------------------------------------
STOP
END


!****f* e0pro/params
! NAME 
!   PARAMS
! PURPOSE
!   Write the code parameters
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE PARAMS
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  IMPLICIT NONE
  INTEGER  ipos, inum, j
!---------------------------------------------------------------------
!      1.        Code and version
  if(myproc.eq.0) then
    write(*,'(a)') '=== CODE:'
    write(*,'(a)') '    program    = '//lprog
    write(*,'(a)') '    version    = '//lvers
!---------------------------------------------------------------------
!      2.        Options
    write(*,'(a)') '--- OPTIONS:'
    write(*,'(a)') '    model      = '//lmodel
    write(*,'(a)') '    hydcon     = '//lhydcon
    write(*,'(a)') '    mhdcon     = '//lmhdcon
    write(*,'(a)') '    resistivity= '//lresis
    write(*,'(a)') '    heating    = '//lheat
    write(*,'(a)') '    gam3d      = '//lgam3d
    write(*,'(a)') '    energy eq  = '//lenerg
    write(*,'(a)') '    momentum eq= '//lmomen
    write(*,'(a)') '    mag. field = '//lbdiv
    write(*,'(a)') '    geometry   = '//lgeom
    write(*,'(a)') '    boundary-t = '//lbctim
    write(*,'(a)') '    boundary-1 = '//lbc1l//'-'//lbc1r
    write(*,'(a)') '    boundary-2 = '//lbc2l//'-'//lbc2r
    write(*,'(a)') '    boundary-3 = '//lbc3l//'-'//lbc3r
    write(*,'(a)') '    scheme     = '//lschem
    write(*,'(a)') '    upwind/cent= '//lupwind
    write(*,'(a)') '    split/multi= '//lsplit
#ifdef PARAMESH
      write(*,'(a)') '    amr-flux   = '//lcflux
#endif
    write(*,'(a)') '    limiter    = '//limitr
    write(*,'(a)') '    vcen       = '//lvcen
    write(*,'(a)') '    vmax       = '//lvmax
    write(*,'(a)') '    powell cor.= '//lpowel
    write(*,'(a)') '    janhunen c.= '//ljanhun
    write(*,'(a)') '    toth cor.  = '//ltoth
    write(*,'(a)') '    difb       = '//ldifb
    write(*,'(a)') '    B-position = '//lstag
!---------------------------------------------------------------------
!      3.        Dimensions
    write(*,'(a)') '--- DIMENSIONS:'
    write(*,'(a,i6)') '    nbb1    =', nbb1
    write(*,'(a,i6)') '    ncc1    =', ncc1
    write(*,'(a,i6)') '    ncc2    =', ncc2
    write(*,'(a,i6)') '    ncc3    =', ncc3
    write(*,'(a,i6)') '    nnvar   =', nnvar
    write(*,'(a,i6)') '    maxblk  =', maxblk
    write(*,'(a,i6)') '    maxhel  =', maxhel
    write(*,'(a,i6)') '    maxsel  =', maxsel
    write(*,'(a,i6)') '    maxfld  =', maxfld
    write(*,'(a,i6)') '    maxfstep=', maxfstep
!---------------------------------------------------------------------
!      4.        Code label
    lacode=lblank(1:80)
    ipos=1
    inum=0
    do j=1,8
      if(lmodel(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=lmodel(1:inum)
    ipos=ipos+inum+1
    inum=0
    do j=1,8
      if(lenerg(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=lenerg(1:inum)
    ipos=ipos+inum+1
    inum=0
    do j=1,8
      if(lschem(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=lschem(1:inum)
    ipos=ipos+inum+1
    inum=0
    do j=1,8
      if(lsplit(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=lsplit(1:inum)
    ipos=ipos+inum+1
    inum=0
    do j=1,8
      if(limitr(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=limitr(1:inum)
    ipos=ipos+inum+1
    inum=0
    do j=1,8
      if(ltoth(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=ltoth(1:inum)
    ipos=ipos+inum+1
    inum=0
    do j=1,8
      if(ldifb(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=ldifb(1:inum)
    ipos=ipos+inum+1
    inum=0
    do j=1,8
      if(lstag(j:j).ne.' ') inum=inum+1
    enddo
    lacode(ipos:ipos+inum)=lstag(1:inum)
!---------------------------------------------------------------------
  endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE PARAMS


!****f* e0pro/expert
! NAME 
!   EXPERT
! PURPOSE
!   The OLYMPUS main control
! INPUTS
!   kmodul
!   konoff
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
#ifdef DIAGNO
SUBROUTINE EXPERT(kmodul,konoff)
!---------------------------------------------------------------------
  USE comoly
  IMPLICIT NONE
  INTEGER kmodul, konoff
#ifdef USE_VT
    INTEGER ierr
    INCLUDE 'VT.inc'
#endif
!---------------------------------------------------------------------
!      1.        Trace
#ifdef TRACE
    write(*,*) 'proc =', myproc, 'module =', kmodul, 'on =', konoff
#endif
!---------------------------------------------------------------------
!      2.        Vampir
#ifdef USE_VT
    if(konoff.eq.1) call VTBEGIN(kmodul,ierr)
    if(konoff.eq.0) call VTEND(kmodul,ierr)
#endif
!---------------------------------------------------------------------
!      3.        HPM
#ifdef HPM
    if(konoff.eq.1) call F_HPM_START(kmodul)
    if(konoff.eq.0) call F_HPM_STOP(kmodul)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE EXPERT
#endif                                                                  


!****f* e0pro/vtsetup
! NAME 
!   VTSETUP
! DESCRIPTION
!   Setup the VAMPIR trace instrumentation
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
#ifdef DIAGNO
#ifdef USE_VT
SUBROUTINE VTSETUP()
!---------------------------------------------------------------------
  IMPLICIT none
  INTEGER ierr
  INCLUDE 'VT.inc'
!---------------------------------------------------------------------
  call VTSYMDEF(2000,'ini','initial',ierr)
  call VTSYMDEF(2001,'dt','initial',ierr)
  call VTSYMDEF(2002,'evo','initial',ierr)
  call VTSYMDEF(2003,'bnd','initial',ierr)
!---------------------------------------------------------------------
  call VTSYMDEF(3001,'grid','region',ierr)
  call VTSYMDEF(3002,'grd1','region',ierr)
  call VTSYMDEF(3003,'grd2','region',ierr)
  call VTSYMDEF(3004,'grd3','region',ierr)
!---------------------------------------------------------------------
  call VTSYMDEF(4001,'grd','resume',ierr)
  call VTSYMDEF(4002,'res','resume',ierr)
  call VTSYMDEF(4002,'ini','resume',ierr)
  call VTSYMDEF(4003,'bnd','resume',ierr)
!---------------------------------------------------------------------
  call VTSYMDEF(5000,'out','output',ierr)
  call VTSYMDEF(5001,'dia','output',ierr)
  call VTSYMDEF(5002,'fin','output',ierr)
  call VTSYMDEF(5003,'val','output',ierr)
  call VTSYMDEF(5004,'tim','output',ierr)
  call VTSYMDEF(5005,'evh','output',ierr)
  call VTSYMDEF(5006,'evl','output',ierr)
  call VTSYMDEF(5007,'evg','output',ierr)
  call VTSYMDEF(5008,'evp','output',ierr)
  call VTSYMDEF(5009,'x1s','output',ierr)
  call VTSYMDEF(5010,'x2s','output',ierr)
  call VTSYMDEF(5011,'x3s','output',ierr)
!---------------------------------------------------------------------
  call VTSYMDEF(5001,'bnd1','boundary',ierr)
  call VTSYMDEF(5002,'bnd2','boundary',ierr)
  call VTSYMDEF(5003,'bnd3','boundary',ierr)
  call VTSYMDEF(5004,'rdr','boundary',ierr)
  call VTSYMDEF(5005,'map','boundary',ierr)
  call VTSYMDEF(5006,'in0','boundary',ierr)
  call VTSYMDEF(5007,'in1','boundary',ierr)
  call VTSYMDEF(5008,'mpi','boundary',ierr)
!---------------------------------------------------------------------
  call VTSYMDEF(6000,'cal','calculation',ierr)
  call VTSYMDEF(6001,'tim','calculation',ierr)
  call VTSYMDEF(6002,'evh','calculation',ierr)
  call VTSYMDEF(6003,'evl','calculation',ierr)
  call VTSYMDEF(6004,'evg','calculation',ierr)
  call VTSYMDEF(6005,'evp','calculation',ierr)
  call VTSYMDEF(6006,'dia','calculation',ierr)
  call VTSYMDEF(6007,'pas','calculation',ierr)
  call VTSYMDEF(6008,'tst','calculation',ierr)
!---------------------------------------------------------------------
  call VTSYMDEF(8001,'mhd1','dynamics',ierr)
  call VTSYMDEF(8002,'mhd2','dynamics',ierr)
  call VTSYMDEF(8003,'mhd3','dynamics',ierr)
  call VTSYMDEF(8004,'mhdc','dynamics',ierr)
  call VTSYMDEF(8005,'mhds','dynamics',ierr)
  call VTSYMDEF(8006,'toth1','dynamics',ierr)
  call VTSYMDEF(8007,'toth2','dynamics',ierr) 
!---------------------------------------------------------------------  
RETURN
END SUBROUTINE VTSETUP
#endif                                                                  
#endif                                                                  
