!****f* e0pro/main
! NAME 
!   RUNCLR
! PURPOSE
!   Clear variables and arrays                                       
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNCLR 
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comglo
  IMPLICIT NONE
!---------------------------------------------------------------------
!      3.        Clear variables
!        3.1       Clear
! unk(:,:,:,:,:)=zero
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RUNCLR


!****f* e0pro/main
! NAME 
!   RUNPRE
! PURPOSE
!   Set default variables
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNPRE
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comevo
  USE commet
  USE comout
  IMPLICIT NONE
  INTEGER  jsel, jhel
!---------------------------------------------------------------------
!      1.        Administration
!        1.1       Actual dimensions
  nhel=0
  nsel=0
  ngeo=0
  npla=0
  nfld=0
  nblk=maxblk
!---------------------------------------------------------------------
!      3.        Physical problem
!        3.1       Parameters defining a run
  vutime=1d0
#ifdef cartesian
    vux1=1d0
    vux2=1d0
    vux3=1d0
#endif
#ifdef cylindrical
    vux1=1d0
    vux2=0.017453293d0
    vux3=1d0
#endif
#ifdef spherical
    vux1=1d0
    vux2=0.017453293d0
    vux3=0.017453293d0
#endif
!        3.2       Parameters defining a run
  nbcmag=0
  xmgrav=1.991d30
  nlxn=.false.
  nlgrav=.true.
!        3.3       Parameters defining a run
  vrot=0d0
  ntest=0
!---------------------------------------------------------------------
!      4.        Computational region
!        4.1       Topographical parameters
!---------------------------------------------------------------------
!      5.        Boundary conditions
!        4.1       Boundary conditions
  nbndin=0
  npfrom=0
  nltbc=.false.
!        4.1       Boundary conditions
  do jhel=1,maxhel
    x1hel(jhel)=0d0
    x2hel(jhel)=0d0
    x3hel(jhel)=0d0
    n1hel(jhel)=1
    n2hel(jhel)=1
    n3hel(jhel)=1
  enddo
  nldiav=.false.
  do jsel=1,maxsel
    x1sel(jsel)=0d0
    n1sel(jsel)=1
  enddo
!---------------------------------------------------------------------
!      7.        Local variables
!        7.1       Perfect gas parameters
  gamcld=5d0/3d0
  xalpha=0d0
!---------------------------------------------------------------------
!      1.        Method of solution
#ifdef PARAMESH
    ctore=0.1d0
    ctode=0.01d0
#endif
!        1.2       Numerical control
  akcfl=0.4d0
  akdt=2d0
  akdtmn=0.1d0
  akdtmx=10d0
  dtzero=0d0
  dtmin=0d0
  dtmax=0d0
  nitmax=5
  nltimc=.true.
!        1.3       Numerical control
  cdifb=1d0
  ndivb=0
!---------------------------------------------------------------------
!      1.        Output control
  nstart=0
  nstop=0
  tstart=0d0
  tstop=0d0
!        1.1       Time control
  ndstep=0
  nefrom=0
  neto=0
  nestep=0
  nffrom=0
  nfto=0
  nfstep=0
  ntfrom=0
  ntto=0
  ntstep=0
  nvfrom=0
  nvto=0
  nvstep=0
  nsfrom=0
  nsto=0
  nsstep=0
  npfrom=0
  npto=0
  npstep=0
!        1.1       Time control
  tdstep=0d0
  tefrom=0d0
  teto=0d0
  testep=0d0
  tffrom=0d0
  tfto=0d0
  tfstep=0d0
  ttfrom=0d0
  ttto=0d0
  ttstep=0d0
  tvfrom=0d0
  tvto=0d0
  tvstep=0d0
  tsfrom=0d0
  tsto=0d0
  tsstep=0d0
  tpfrom=0d0
  tpto=0d0
  tpstep=0d0
!        1.2       Step control
  x1s=0d0
  x2s=0d0
  x3s=0d0
RETURN
END SUBROUTINE RUNPRE


!****f* e0pro/main
! NAME 
!   RUNPAR
! PURPOSE
!   Define data specific to run
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNPAR
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comout
  USE comevo
  USE combnd
  USE commet
  IMPLICIT NONE
#ifdef MPI
    include 'mpif.h'
    INTEGER  ie
#endif
!---------------------------------------------------------------------
  NAMELIST /namrun/ &
#ifdef PARAMESH
    CTORE, CTODE,  &
#endif
    VUTIME, VUX1, NTEST, NBCMAG, NLTBC,                                &
    NRECON, ETACON, YJZLIM, ETALIM, YJZANO, ETAANO, NLDIAV, NLXN,      &
    GAMMA, CDIFB, QHEAT, NHEAT,                                        &
    NLXP, NLGRAV, GAMCLD, XMGRAV, XALPHA, TSTART, TSTOP, NSTART, NSTOP,&
    TDSTEP, NDSTEP, TPSTEP, NPSTEP, TEFROM, TETO, TESTEP, NEFROM,      &
    NETO, NESTEP, TFFROM, TFTO, TFSTEP, NFFROM, NFTO, NFSTEP, TVFROM,  &
    TVTO, TVSTEP, NVFROM, NVTO, NVSTEP, TTFROM, TTTO, TTSTEP, NTFROM,  &
    NTTO, NTSTEP, TSFROM, TSTO, TSSTEP, NSFROM, NSTO, NSSTEP,          &
    NVRED, NB1, NBNDIN, NPFROM, VROT, X1S, X2S,                        &
    X3S,  X1HEL, X2HEL, X3HEL, X1SEL,                                  &
    NHEL, NSEL, NGEO, NPLA, NFLD,                                      &
    AKCFL, AKDT, DTZERO, DTMAX, DTMIN, NSOLVE, NITMAX, NLTIMC, NDIVB
!---------------------------------------------------------------------
!      1.        Read in parameters
  if(myproc.eq.0) then
    read(*,'(a)') lapar
    read(*, namrun)
  endif
!---------------------------------------------------------------------
!      2.        Label the run
  if(myproc.eq.0) then
    write(*,'(a)') '--- RUN: '//lapar
    write(*,'(4x,a8,1pe12.5)') 'vrot   =',vrot
#ifdef gam3d
      write(*,'(4x,a8,1pe12.5)') 'gamcld =',gamcld
#else
      write(*,'(4x,a8,1pe12.5)') 'gamma  =',gamma
#endif
    write(*,'(4x,a8,1pe12.5)') 'xalpha =',xalpha
    write(* ,'(4x,a8,1pe12.5,2x,a8,l12)') &
      'xmgrav =',xmgrav,'nlgrav =',nlgrav
    write(*,'(4x,a8,l12)') 'nlxn   =',nlxn
!   write(*,'(2x,3(2x,a8,1pe12.5))')                                 &
!     'etaano =',etaano,'etacon =',etacon,'etalim =',etalim
!   write(*,'(2x,2(2x,a8,1pe12.5))')                                 &
!     'yjzano =',yjzano,'yjzlim =',yjzlim
    write(*,'(4x,a8,i12)') 'nbndin =',nbndin
    write(*,'(4x,a8,i12)') 'nbcmag =',nbcmag
    write(*,'(2x,2(2x,a8,1pe12.5))') 'akcfl  =',akcfl,'akdt   =',akdt
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'dtzero =',dtzero,'dtmin  =',dtmin,'dtmax  =',dtmax
    write(*,'(4x,a8,i12)') 'ntest  =',ntest
    write(*,'(4x,a8,1pe12.5)') 'cdifb  =',cdifb
    write(*,'(4x,a8,i12)') 'ndivb  =',ndivb
    write(*,'(2x,2(2x,a8,1pe12.5))') &
      'vutime =',vutime,'vux1   =',vux1
    write(*,'(2x,2(2x,a8,1pe12.5))') &
      'tstart =',tstart,'tstop  =',tstop
    write(*,'(2x,2(2x,a8,i12))') 'nstart =',nstart,'nstop  =',nstop
    write(*,'(4x,a8,1pe12.5)') 'tdstep =',tdstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tefrom =',tefrom,'teto   =',teto, 'testep =',testep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tffrom =',tffrom,'tfto   =',tfto, 'tfstep =',tfstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'ttfrom =',ttfrom,'ttto   =',ttto, 'ttstep =',ttstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tvfrom =',tvfrom,'tvto   =',tvto, 'tvstep =',tvstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tsfrom =',tsfrom,'tsto   =',tsto, 'tsstep =',tsstep
    write(*,'(4x,a8,i12)') 'ndstep =',ndstep
    write(*,'(2x,3(2x,a8,i12))') &
      'nefrom =',nefrom,'neto   =',neto,'nestep =',nestep
    write(*,'(2x,3(2x,a8,i12))') &
      'nffrom =',nffrom,'nfto   =',nfto,'nfstep =',nfstep
    write(*,'(2x,3(2x,a8,i12))') &
      'ntfrom =',ntfrom,'ntto   =',ntto,'ntstep =',ntstep
    write(*,'(2x,3(2x,a8,i12))') &
      'nvfrom =',nvfrom,'nvto   =',nvto,'nvstep =',nvstep
    write(*,'(2x,3(2x,a8,i12))') &
      'nsfrom =',nsfrom,'nsto   =',nsto,'nsstep =',nsstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'x1s    =',x1s,'x2s    =',x2s,'x3s    =',x3s
    write(*,'(4x,a8,i12)') 'nhel   =',nhel
    write(*,'(4x,a8,i12)') 'nsel   =',nsel
    write(*,'(4x,a8,i12)') 'ngeo   =',ngeo
    write(*,'(4x,a8,i12)') 'npla   =',npla
    write(*,'(4x,a8,i12)') 'nfld   =',nfld
  endif
#ifdef MPI
!---------------------------------------------------------------------
!      3.        Wait until all procs come at this point
    call MPI_BARRIER(MPI_COMM_WORLD,ie)
!---------------------------------------------------------------------
!      4.        Broadcast parameters
    call MPI_BCAST(vrot,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(gamma,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(gamcld,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(xmgrav,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(xalpha,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nlgrav,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nlxn,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(etaano,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(etacon,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(etalim,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(yjzano,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(yjzlim,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nbndin,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ntest,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ndivb,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nltimc,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(akcfl,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(akdt,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(dtzero,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(dtmin,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(dtmax,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(cdifb,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(vutime,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(vux1,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tstart,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tstop,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tdstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tefrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(teto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(testep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tffrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tfto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tfstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ttfrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ttto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ttstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tvfrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tvto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tvstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tsfrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tsto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(tsstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nstart,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nstop,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ndstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nefrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(neto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nestep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nffrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nfto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nfstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ntfrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ntto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ntstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nvfrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nvto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nvstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nsfrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nsto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nsstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(x1s,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(x2s,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(x3s,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(x1hel,maxhel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(x2hel,maxhel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(x3hel,maxhel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nhel,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(x1sel,maxsel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nsel,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(ngeo,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(npla,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
    call MPI_BCAST(nfld,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
!---------------------------------------------------------------------
!      5.        Wait until all procs come at this point
    call MPI_BARRIER(MPI_COMM_WORLD,ie)
#endif
!---------------------------------------------------------------------
!      9.        Terminate
RETURN
END SUBROUTINE RUNPAR


!****f* e0pro/main
! NAME 
!   RUNCHE
! PURPOSE
!   Check input data
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNCHE
!---------------------------------------------------------------------
  USE comoly
  USE commax
  USE comdim
  USE comphy
  USE comreg
  USE comout
  USE comglo
  USE comevo
  USE combnd
  USE commet
  IMPLICIT NONE
  INTEGER  ierr
!---------------------------------------------------------------------
  if(myproc.eq.0) then
    ierr=0
!---------------------------------------------------------------------
!      3.        Physical problem
!        3.1       Parameters defining a run
!---------------------------------------------------------------------
!      4.        Computational region
!        4.1       Topographical parameters
!---------------------------------------------------------------------
!      7.        Local variables
!---------------------------------------------------------------------
!      1.        Boundary conditions
!        1.2       Hydrodynamic variables
!---------------------------------------------------------------------
!      1.        Method of solution
!        1.1       Numerical control
    if((akcfl.le.0d0).or.(akcfl.ge.1d0)) then
      write(*,*) '    akcfl (0d0,1d0) =', akcfl
      ierr=ierr+1
    endif
!---------------------------------------------------------------------
!      1.        Output control
    if((nhel.lt.0).or.(nhel.gt.maxhel)) then
      write(*,*) '    nhel (0,maxhel/ =', nhel
      ierr=ierr+1
    endif
    if((nsel.lt.0).or.(nsel.gt.maxsel)) then
      write(*,*) '    nsel (0,maxsel/ =', nsel
      ierr=ierr+1
    endif
    if((nfld.lt.0).or.(nfld.gt.maxfld)) then
      write(*,*) '    nhel (0,maxfld/ =', nfld
      ierr=ierr+1
    endif
!---------------------------------------------------------------------
!      1.        Output control
    if((tfstep.eq.zero).and.(nfstep.eq.0)) then
      write(*,'(a)') '*** No RES output at resume time levels'
    endif
    if((tpstep.eq.zero).and.(npstep.eq.0)) then
      write(*,'(a)') '*** No PAS output at passage positions'
    endif
    if((tvstep.eq.zero).and.(nvstep.eq.0)) then
      write(*,'(a)') '*** No VAL output at time levels'
    endif
    if((ttstep.eq.zero).and.(ntstep.eq.0)) then
      write(*,'(a)') '*** No TIM output at time levels'
    endif
    if((tsstep.eq.zero).and.(nsstep.eq.0)) then
      write(*,'(a)') '*** No X1S,X2S,X3S outputs at slices'
    endif
    if((testep.eq.zero).and.(nestep.eq.0)) then
      write(*,'(a)') '*** No EVH,EVL,EVG,EVP,EVF outputs at significant positions'
    endif
    if((tdstep.eq.zero).and.(ndstep.eq.0)) then
      write(*,'(a)') '*** No output of diagnostics'
    endif
!---------------------------------------------------------------------
!      9.        Terminate
!return if no error in input data
    if(ierr.eq.0)  RETURN
    write(*,*) '*** input data outside limits'
    write(*,*) '    errors', ierr
    write(*,*) '*** error in subroutine 1.6 check'
    call JOBEND
  endif
RETURN
END SUBROUTINE RUNCHE


!****f* e0pro/main
! NAME 
!   RUNAUX
! PURPOSE
!   Set auxiliary values
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNAUX
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE combnd
  USE comevo
  USE commet
  USE comout
  IMPLICIT NONE
  INTEGER  jsel, jhel
!---------------------------------------------------------------------
!      3.        Physical problem
!        3.1       Parameters defining a run
#ifdef cartesian
    vux2=vux1
    vux3=vux1
#endif
#ifdef cylindrical
    vux3=vux1
#endif
!        3.4       External forces
  grav=cgrav*xmgrav
!---------------------------------------------------------------------
!      7.        Local variables
!        7.1       Perfect gas parameters
  xmass=(1d0+4d0*xalpha)*cmass
  rgas=((2d0+3d0*xalpha)/(1d0+4d0*xalpha))*(ck/cmass)
!---------------------------------------------------------------------
!      7.        Output parameter
!---------------------------------------------------------------------
!      8.        Auxiliary values
#ifdef roe
    z13=zone/zthree
    z23=ztwo/zthree
#endif
  if(nlxn) then
    z05=zhalf*xmass
    zgas=ztwo*ck
    ztwom=ztwo/xmass
  else
    z05=zhalf
    zgas=rgas
    ztwom=ztwo
  endif
  zgrav=zero
  if(nlgrav) zgrav=grav
  if(nlxn) zgrav=zgrav*xmass
#ifdef mhd
    zmu=cmu
    zrmu=zone/cmu
    zr2mu=zhalf/cmu
#endif
!---------------------------------------------------------------------
!      9.        Units
  if(ntest.eq.0) then
    if(myproc.eq.0) then
      write(*,'(a)') '--- UNITS'
      write(*,'(a,1pe12.4)') '    vutime =',vutime
      write(*,'(a,1pe12.4)') '    vux1   =',vux1
      write(*,'(a,1pe12.4)') '    vux2   =',vux2
      write(*,'(a,1pe12.4)') '    vux3   =',vux3
    endif
    tstart=vutime*tstart
    tstop=vutime*tstop
    tdstep=vutime*tdstep
    tefrom=vutime*tefrom
    teto=vutime*teto
    testep=vutime*testep
    tffrom=vutime*tffrom
    tfto=vutime*tfto
    tfstep=vutime*tfstep
    ttfrom=vutime*ttfrom
    ttto=vutime*ttto
    ttstep=vutime*ttstep
    tvfrom=vutime*tvfrom
    tvto=vutime*tvto
    tvstep=vutime*tvstep
    tsfrom=vutime*tsfrom
    tsto=vutime*tsto
    tsstep=vutime*tsstep
    dtzero=vutime*dtzero
    dtmax=vutime*dtmax
    dtmin=vutime*dtmin
    vrot=vux3*vrot
    x1s=vux1*x1s
    x2s=vux2*x2s
    x3s=vux3*x3s
    do jhel=1,maxhel
      x1hel(jhel)=vux1*x1hel(jhel)
      x2hel(jhel)=vux2*x2hel(jhel)
      x3hel(jhel)=vux3*x3hel(jhel)
    enddo
    do jsel=1,maxsel
      x1sel(jsel)=vux1*x1sel(jsel)
    enddo
  endif 
!---------------------------------------------------------------------  
!      9.        Terminate                                    
RETURN
END SUBROUTINE RUNAUX
