!****f* e0pro/main
! NAME 
!   INPOS
! PURPOSE
!   Find mesh positions of the significant point
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE INPOS
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comevo
  USE comout
  IMPLICIT NONE
  INTEGER js
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(2003,1)
#endif
!---------------------------------------------------------------------
!      1.        Positions of significant points
  nxhel=0
  if(myproc.eq.0) then
    do js=1,nhel
      if((x1hel(js).ge.xh1(nc1l,1)).and.(x1hel(js).le.xh1(nc1r+1,1))) then
        nxhel=nxhel+1
      endif
    enddo
  else
    do js=1,nhel
      if((x1hel(js).gt.xh1(nc1l,1)).and.(x1hel(js).le.xh1(nc1r+1,1))) then
        nxhel=nxhel+1
      endif
    enddo
  endif
!---------------------------------------------------------------------
!      1.        Positions of Sun-Earth-line points
  nxsel=0
  if(myproc.eq.0) then
    do js=1,nsel
      if((x1sel(js).ge.xh1(nc1l,1)).and.(x1sel(js).le.xh1(nc1r+1,1))) then
        nxsel=nxsel+1
       endif
     enddo
  else
    do js=1,nsel
      if((x1sel(js).gt.xh1(nc1l,1)).and.(x1sel(js).le.xh1(nc1r+1,1))) then
        nxsel=nxsel+1
       endif
     enddo
  endif
!---------------------------------------------------------------------
!      1.        Positions of geospace points
  nxgeo=0
  if((1.496d11.ge.xh1(nc1l,1)).and.(1.496d11.le.xh1(nc1r+1,1))) then
    nxgeo=5
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(2003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE INPOS


!****f* e0pro/main
! NAME 
!   INEVO
! PURPOSE
!   Find mesh positions of the significant point
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE INEVO
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comevo
  USE comout
  IMPLICIT NONE
  INTEGER js, jhel, ihel, jsel, isel, j1, j2, j3
  INTEGER i1hel, i2hel, i3hel
  INTEGER i1geo, i3geo
  INTEGER i1sel, i3sel
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(2003,1)
#endif
!---------------------------------------------------------------------
!      1.        Positions of heliospheric points
  do jhel=1,nxhel
    ihel=nnhel(myproc+1)+jhel
    i1hel=1
    do j1=nc1l,nc1r
      if(xh1(j1,1).le.x1hel(ihel)) i1hel=j1
    enddo
#ifdef x2dim
      i2hel=1
      do j2=nc2l,nc2r
        if(xh2(j2,1).le.x2hel(ihel)) i2hel=j2
      enddo
      if(nc2l.eq.nc2r) i2hel=nc2l
#else
      i2hel=1
#endif
#ifdef x3dim
      i3hel=1
      do j3=nc3l,nc3r
        if(xh3(j3,1).le.x3hel(ihel)) i3hel=j3
      enddo
      if(nc3l.eq.nc3r) i3hel=nc3l
#else
      i3hel=1
#endif
    n1hel(ihel)=i1hel
    n2hel(ihel)=i2hel
    n3hel(ihel)=i3hel
  enddo
!---------------------------------------------------------------------
!      1.        Positions of Sun-Earth-line points
#ifdef x3dim
    i3sel=nc3l
    do j3=nc3l,nc3r-1
      if(xc3(j3,1).le.cpi) i3sel=j3
    enddo
#else
    i3sel=1
#endif
  do jsel=1,nxsel
    isel=nnsel(myproc+1)+jsel
    i1sel=1
    do j1=nc1l,nc1r
      if(xh1(j1,1).le.x1sel(isel)) i1sel=j1
    enddo
    n1sel(isel)=i1sel
    n3sel(isel)=i3sel
    x3sel(isel)=cpi
  enddo
!---------------------------------------------------------------------
!      1.        Positions of geospace points
  if(nxgeo.gt.0) then
    i1geo=nc1l
    do j1=nc1l,nc1r
      if(xh1(j1,1).le.1.496d11) i1geo=j1
    enddo
#ifdef x3dim
      i3geo=nc3l
      do j3=nc3l,nc3r-1
        if(xc3(j3,1).le.cpi) i3geo=j3
      enddo
#else
      i3geo=1
#endif
    x1geo(1)=1.496e11
    n1geo(1)=i1geo
    x3geo(1)=cpi
    n3geo(1)=i3geo
    do js=2,5
      x1geo(js)=1.496e11
      n1geo(js)=i1geo
      if((js.eq.2).or.(js.eq.3)) then
        x3geo(js)=xc3(i3geo,1)
        n3geo(js)=i3geo
      else
        x3geo(js)=xc3(i3geo+1,1)
        n3geo(js)=i3geo+1
      endif
    enddo
  endif
!---------------------------------------------------------------------
!      2.        Positions of slices
  if((tsstep.gt.zero).or.(nsstep.gt.0)) then
!        2.1       X1S
    do j1=1,nc1
      if(xc1(j1,1).le.x1s)  nx1s=j1
    enddo
!        2.2       X2S
    do j2=1,nc2
      if(xc2(j2,1).le.x2s)  nx2s=j2
   enddo
!        2.3       X3S
    do j3=1,nc3
      if(xc3(j3,1).le.x3s)  nx3s=j3
    enddo
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(2003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE INEVO
