!****f* e0pro/main
! NAME 
!   GRID
! PURPOSE
!   Set grid coordinates
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE GRID
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    USE paramesh_dimensions
    USE physicaldata
    USE tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comtva
  IMPLICIT NONE
  REAL(KIND=8)  zdx1, zdx2, zdx3
  INTEGER  j1, j2, j3, jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
    call EXPERT(1001,1)
#endif
!---------------------------------------------------------------------
!      1.        Coordinates
#ifdef PARAMESH
    do jb=1,lnblocks
      x1l=bnd_box(1,1,jb)
      x2l=bnd_box(1,2,jb)
      x3l=bnd_box(1,3,jb)
      x1r=bnd_box(2,1,jb)
      x2r=bnd_box(2,2,jb)
      x3r=bnd_box(2,3,jb)
#else
      jb=1
#endif
      zdx1=(x1r-x1l)/real(ncc1)
      zdx2=(x2r-x2l)/real(ncc2)
      zdx3=(x3r-x3l)/real(ncc3)
!        1.1       Cell centers
      do j1=1,nc1
        xc1(j1,jb)=x1l+zdx1*(real(j1-ng1)-0.5d0)
      enddo
      do j2=1,nc2
        xc2(j2,jb)=x2l+zdx2*(real(j2-ng2)-0.5d0)
      enddo
      do j3=1,nc3
        xc3(j3,jb)=x3l+zdx3*(real(j3-ng3)-0.5d0)
      enddo
!        1.1       Cell interfaces
      do j1=2,nc1
        xh1(j1,jb)=zhalf*(xc1(j1,jb)+xc1(j1-1,jb))
      enddo
      xh1(1,jb)=x1l-zdx1-zdx1
      xh1(nh1,jb)=x1r+zdx1+zdx1
#ifdef x2dim
        do j2=2,nc2
          xh2(j2,jb)=zhalf*(xc2(j2,jb)+xc2(j2-1,jb))
        enddo
#endif
      xh2(1,jb)=x2l-zdx2-zdx2
      xh2(nh2,jb)=x2r+zdx2+zdx2
#ifdef x3dim
        do j3=2,nc3
          xh3(j3,jb)=zhalf*(xc3(j3,jb)+xc3(j3-1,jb) )
        enddo
#endif
      xh3(1,jb)=x3l-zdx3-zdx3
      xh3(nh3,jb)=x3r+zdx3+zdx3
#ifdef PARAMESH
    enddo
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(1001,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE GRID


!****f* e0pro/main
! NAME 
!   GRID1
! PURPOSE
!   Set X1-grid variables and coefficients
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE GRID1
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    USE paramesh_dimensions
    USE physicaldata
    USE tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comtva
  IMPLICIT NONE
  INTEGER  jx, jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
    call EXPERT(1002,1)
#endif
!---------------------------------------------------------------------
!      4.        Coordinate system independent variables
#ifdef PARAMESH
    do jb=1,lnblocks
#else
      jb=1
      xc1(1,jb)=4d0*xh1(3,jb)-3d0*xc1(3,jb)
      xc1(2,jb)=2d0*xh1(3,jb)-xc1(3,jb)
      xc1(nc1-1,jb)=2d0*xh1(nc1-1,jb)-xc1(nc1-2,jb)
      xc1(nc1,jb)=4d0*xh1(nc1-1,jb)-3d0*xc1(nc1-2,jb)
      xh1(1,jb)=3d0*xh1(3,jb)-2d0*xh1(4,jb)
      xh1(2,jb)=2d0*xh1(3,jb)-xh1(4,jb)
      xh1(nh1-1,jb)=2d0*xh1(nh1-2,jb)-xh1(nh1-3,jb)
      xh1(nh1,jb)=3d0*xh1(nh1-2,jb)-2d0*xh1(nh1-3,jb)
#endif
      do jx=1,nc1
        rx1(jx,jb)=zone/xc1(jx,jb)
        rxx1(jx,jb)=rx1(jx,jb)*rx1(jx,jb)
      enddo
!---------------------------------------------------------------------
!      2.        Grid differences
      do jx=1,nc1
        dx1(jx,jb)=xh1(jx+1,jb)-xh1(jx,jb)
        rdx1(jx,jb)=zone/dx1(jx,jb)
      enddo
!---------------------------------------------------------------------
!      3.        Calculate the three coordinate systems
#ifdef cartesian
!        3.1       Cartesian coordinates
!     Interface areas
      do jx=1,nc1
        ax1(jx,jb)=zone
#ifdef mhd
          ax12(jx)=zone
          ax13(jx)=zone
#endif
      enddo
      do jx=1,nh1
        ah1(jx,jb)=zone
#ifdef mhd
          ah12(jx)=zone
          ah13(jx)=zone
#endif
      enddo
!     Cell volumes
      do jx=1,nc1
        vol1(jx,jb)=dx1(jx,jb)
        rvol1(jx,jb)=zone/vol1(jx,jb)
#ifdef mhd
          rvol12(jx)=rvol1(jx,jb)
          rvol13(jx)=rvol12(jx)
#endif
      enddo
#endif
#ifdef cylindrical
!        3.2       Cylindrical coordinates
!     Interface areas
      do jx=1,nc1
        ax1(jx,jb)=xc1(jx,jb)
#ifdef mhd
          ax12(jx)=zone
          ax13(jx)=xc1(jx,jb)
#endif
      enddo
      do jx=1,nh1
        ah1(jx,jb)=xh1(jx,jb)
#ifdef mhd
          ah12(jx)=zone
          ah13(jx)=xh1(jx,jb)
#endif
        difh(jx)=xh1(jx,jb)*xh1(jx,jb)
      enddo
!     Cell volumes
      do jx=1,nc1
        vol1(jx,jb)=zhalf*(difh(jx+1)-difh(jx))
        rvol1(jx,jb)=zone/vol1(jx,jb)
#ifdef mhd
          rvol12(jx)=rdx1(jx,jb)
          rvol13(jx)=rx1(jx,jb)*rvol12(jx)
#endif
      enddo
#endif
#ifdef spherical
!        3.3       Spherical coordinates
!     Interface areas
      do jx=1,nc1
        ax1(jx,jb)=xc1(jx,jb)*xc1(jx,jb)
#ifdef mhd
          ax12(jx)=xc1(jx,jb)
          ax13(jx)=ax12(jx)
#endif
      enddo
      do jx=1,nh1
        ah1(jx,jb)=xh1(jx,jb)*xh1(jx,jb)
#ifdef mhd
          ah12(jx)=xh1(jx,jb)
          ah13(jx)=ah12(jx)
#endif
        difh(jx)=xh1(jx,jb)*xh1(jx,jb)*xh1(jx,jb)
      enddo
!        3.4       Cell volumes
      do jx=1,nc1
        vol1(jx,jb)=(difh(jx+1)-difh(jx))/zthree
        rvol1(jx,jb)=zone/vol1(jx,jb)
#ifdef mhd
          rvol12(jx)=rx1(jx,jb)*rdx1(jx,jb)
          rvol13(jx)=rvol12(jx)
#endif
      enddo
#endif
#ifdef PARAMESH
      enddo
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(1002,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE GRID1


!****f* e0pro/main
! NAME 
!   GRID2
! PURPOSE
!   Set X2-grid variables and coefficients
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE GRID2
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    USE paramesh_dimensions
    USE physicaldata
    USE tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comtva
  IMPLICIT NONE
  INTEGER  jx, jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
    call EXPERT(1003,1)
#endif
!---------------------------------------------------------------------
!      2.        Grid differences
#ifdef PARAMESH
    do jb=1,lnblocks
#else
    jb=1
#ifdef x2dim
      xc2(1,jb)=4d0*xh2(3,jb)-3d0*xc2(3,jb)
      xc2(2,jb)=2d0*xh2(3,jb)-xc2(3,jb)
      xc2(nc2-1,jb)=2d0*xh2(nc2-1,jb)-xc2(nc2-2,jb)
      xc2(nc2,jb)=4d0*xh2(nc2-1,jb)-3d0*xc2(nc2-2,jb)
      xh2(1,jb)=3d0*xh2(3,jb)-2d0*xh2(4,jb)
      xh2(2,jb)=2d0*xh2(3,jb)-xh2(4,jb)
      xh2(nh2-1,jb)=2d0*xh2(nh2-2,jb)-xh2(nh2-3,jb)
      xh2(nh2,jb)=3d0*xh2(nh2-2,jb)-2d0*xh2(nh2-3,jb)
#endif
#endif
  do jx=1,nc2
    dx2(jx,jb)=xh2(jx+1,jb)-xh2(jx,jb)
    rdx2(jx,jb)=zone/dx2(jx,jb)
  enddo
!---------------------------------------------------------------------
!      3.        Calculate the three coordinate systems
#ifdef cartesian
!        3.1       Cartesian coordinates
!     Interface areas
       do jx=1,nh2
         ah2(jx,jb)=zone
       enddo
       do jx=1,nc2
         ax2(jx,jb)=zone
       enddo
#endif
#ifdef cylindrical
!        3.2       Cylindrical coordinates
!     Interface areas
       do jx=1,nh2
         ah2(jx,jb)=zone
       enddo
       do jx=1,nc2
         ax2(jx,jb)=zone
       enddo
#endif
#ifdef spherical
!        3.3       Spherical coordinates
!     Interface areas
       do jx=1,nh2
         ah2(jx,jb)=sin(xh2(jx,jb))
       enddo
       do jx=1,nc2
         ax2(jx,jb)=sin(xc2(jx,jb))
       enddo

!     Auxiliary values
       do jx=1,nc2
         cott(jx,jb)=zone/tan(xc2(jx,jb))
         sint(jx,jb)=sin(xc2(jx,jb))
         rsint(jx,jb)=zone/sint(jx,jb)
       enddo
#endif
#ifdef PARAMESH
    enddo
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(1003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE GRID2


!****f* e0pro/main
! NAME 
!   GRID3
! PURPOSE
!   Set X3-grid variables and coefficients
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE GRID3
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    USE paramesh_dimensions
    USE physicaldata
    USE tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comtva
  IMPLICIT NONE
  INTEGER  jx,jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
    call EXPERT(1004,1)
#endif
!---------------------------------------------------------------------
!      2.        Grid differences
#ifdef PARAMESH
    do jb=1,lnblocks
#else
    jb=1
#ifdef x3dim
      xc3(1,jb)=4d0*xh3(3,jb)-3d0*xc3(3,jb)
      xc3(2,jb)=2d0*xh3(3,jb)-xc3(3,jb)
      xc3(nc3-1,jb)=2d0*xh3(nc3-1,jb)-xc3(nc3-2,jb)
      xc3(nc3,jb)=4d0*xh3(nc3-1,jb)-3d0*xc3(nc3-2,jb)
      xh3(1,jb)=3d0*xh3(3,jb)-2d0*xh3(4,jb)
      xh3(2,jb)=2d0*xh3(3,jb)-xh3(4,jb)
      xh3(nh3-1,jb)=2d0*xh3(nh3-2,jb)-xh3(nh3-3,jb)
      xh3(nh3,jb)=3d0*xh3(nh3-2,jb)-2d0*xh3(nh3-3,jb)
#endif
#endif
    do jx=1,nc3
      dx3(jx,jb)=xh3(jx+1,jb)-xh3(jx,jb)
      rdx3(jx,jb)=zone/dx3(jx,jb)
    enddo
#ifdef PARAMESH
    enddo
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(1004,0) 
#endif                                                                  
!---------------------------------------------------------------------  
RETURN
END SUBROUTINE GRID3
