!****f* e0pro/main
! NAME 
!   RUNOUT
! PURPOSE
!   Control the output                                             
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNOUT(kmode) 
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comevo
  USE comout
  IMPLICIT NONE
  REAL(KIND=8)  ztime
  INTEGER  kmode, i
#ifdef DIAGNO
    call EXPERT(3000,1)
#endif
!---------------------------------------------------------------------
  if((tstop-tstart).lt.36000d0) then
    ztime=time
  elseif((tstop-tstart).gt.864000d0) then
    ztime=time/86400d0
  else
    ztime=time/3600d0
  endif
!---------------------------------------------------------------------
!      1.        Initial output
  if(kmode.eq.1) then
!        1.1       Values at final levels
    call OUTCHE(time,tfout,tffrom,tfto,tfstep, &
               nstep,nfout,nffrom,nfto,nfstep,kmode,mfin,i)
    if(i.eq.1) then
      call OUTFIN
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outfin',mfin,ztime
      endif
    endif
!        1.2       Values at boundary positions
    call OUTCHE(time,tpout,tpfrom,tpto,tpstep, &
               nstep,npout,npfrom,npto,npstep,kmode,mpass,i)
    if(i.eq.1) then
!     call OUTPAS
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outpas',mpass,ztime
      endif
    endif
!        1.7       Values at significant positions
    if((tvstep.ne.zero).or.(nvstep.ne.0)) then
      if(mfres.eq.0) then
        call OUTVAL(1)
      endif
      call OUTCHE(time,tvout,tvfrom,tvto,tvstep, &
                 nstep,nvout,nvfrom,nvto,nvstep,kmode,mval,i)
      if(i.eq.1) then
        call OUTVAL(2)
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outval',mval,ztime
        endif
      endif
    endif
!        1.3       Values at periodic levels
    if((ttstep.ne.zero).or.(ntstep.ne.0)) then
      call OUTCHE(time,ttout,ttfrom,ttto,ttstep, &
                 nstep,ntout,ntfrom,ntto,ntstep,kmode,mtim,i)
      if(i.eq.1) then
        call OUTTIM
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outtim',mtim,ztime
        endif
      endif
    endif
!        1.5       Values at significant positions
    if((tsstep.ne.zero).or.(nsstep.ne.0)) then
      call OUTX1S(1)
      call OUTX2S(1)
      call OUTX3S(1)
      call OUTCHE(time,tsout,tsfrom,tsto,tsstep, &
                 nstep,nsout,nsfrom,nsto,nsstep,kmode,mxs,i)
      if(i.eq.1) then
        call OUTX1S(2)
        call OUTX2S(2)
        call OUTX3S(2)
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outx*s',mxs,ztime
        endif
      endif
    endif
!        1.7       Values at significant positions
    if((testep.ne.zero).or.(nestep.ne.0)) then
      if(.not.nlres) then
        if(myproc.eq.0) then
          if(nhel.gt.0) call OUTEVH(1)
          if(nsel.gt.0) call OUTEVL(1)
          if(npla.gt.0) call OUTEVP(1)
          if(nfld.gt.0) call OUTEVF(1)
        endif
        if(nxgeo.gt.0) call OUTEVG(1)
      endif
      call OUTCHE(time,teout,tefrom,teto,testep, &
                 nstep,neout,nefrom,neto,nestep,kmode,mevo,i)
      if(i.eq.1) then
        if(myproc.eq.0) then
          if(nhel.gt.0) call OUTEVH(2)
          if(nsel.gt.0) call OUTEVL(2)
          if(npla.gt.0) call OUTEVP(2)
          if(nfld.gt.0) call OUTEVF(2)
        endif
        if(nxgeo.gt.0) call OUTEVG(2)
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outevo',mevo,ztime
        endif
      endif
    endif
!        1.8       Diagnostics
    if((tdstep.eq.zero).and.(ndstep.eq.0)) then
      if(myproc.eq.0) then
        write(*,'(a)') '*** No output of diagnostics'
      endif
    else
      call OUTDIA(1)
      call OUTDIA(2)
    endif
  endif
!---------------------------------------------------------------------
!      2.        Periodic output
  if(kmode.eq.2) then
!        2.1       Values at time levels
    call OUTCHE(time,tfout,tffrom,tfto,tfstep, &
               nstep,nfout,nffrom,nfto,nfstep,kmode,mfin,i)
    if(i.eq.1) then
      call OUTFIN
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outfin', mfin, ztime
      endif
    endif
!L       2.2       Values at time levels
    call OUTCHE(time,tvout,tvfrom,tvto,tvstep, &
               nstep,nvout,nvfrom,nvto,nvstep,kmode,mval,i)
    if(i.eq.1) then
      call OUTVAL(2)
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outval',mval,ztime
      endif
    endif
!L       2.3       Values at significant positions
    call OUTCHE(time,ttout,ttfrom,ttto,ttstep, &
               nstep,ntout,ntfrom,ntto,ntstep,kmode,mtim,i)
    if(i.eq.1) then
      call OUTTIM
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outtim', mtim, ztime
      endif
    endif
!L       2.4       Values at significant positions
    call OUTCHE(time,tsout,tsfrom,tsto,tsstep, &
               nstep,nsout,nsfrom,nsto,nsstep,kmode,mxs,i)
    if(i.eq.1) then
      call OUTX1S(2)
      call OUTX2S(2)
      call OUTX3S(2)
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outx*s', mxs, ztime
      endif
    endif
!L       2.7       Values at significant positions
    call OUTCHE(time,teout,tefrom,teto,testep, &
               nstep,neout,nefrom,neto,nestep,kmode,mevo,i)
    if(i.eq.1) then
      if(myproc.eq.0) then
        if(nhel.gt.0) call OUTEVH(2)
        if(nsel.gt.0) call OUTEVL(2)
        if(npla.gt.0) call OUTEVP(2)
        if(nfld.gt.0) call OUTEVF(2)
      endif
      if(nxgeo.gt.0) call OUTEVG(2)
    endif
!L       2.8       Diagnostics
    if((tdstep.ne.zero).or.(ndstep.ne.0)) then
!     if((time.ge.tdout).and.(nstep.ge.ndout)) then
        call OUTDIA(2)
        tdout=tdout+tdstep
        ndout=ndout+ndstep
!     endif
    endif
  endif
!---------------------------------------------------------------------
!      3.        Final output
  if(kmode.eq.3) then
!L       3.1       Diagnostics
    if((tdstep.ne.zero).or.(ndstep.ne.0)) then
      call OUTDIA(3)
    endif
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(3000,0)
#endif
RETURN
END SUBROUTINE RUNOUT


!****f* e0pro/main
! NAME 
!   OUTCHE
! PURPOSE
!   Control the output
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTCHE(ptime,pout,pfrom,pto,pstep, &
  ktime,kout,kfrom,kto,kstep,kmode,knum,konoff)
!---------------------------------------------------------------------
  IMPLICIT NONE
!---------------------------------------------------------------------
  REAL(KIND=8)  ptime, pout, pfrom, pto, pstep
  INTEGER       ktime, kout, kfrom, kto, kstep, kmode, knum, konoff
!---------------------------------------------------------------------
!      1.        Initial output
  konoff=0
  if(kmode.eq.1) then
    if(pstep.gt.0d0) then
      if(ptime.eq.pfrom) then
        konoff=1
        knum=1
        pout=pfrom+pstep
      else if(ptime.lt.pfrom) then
        konoff=0
        knum=0
        pout=pfrom
      else
        konoff=0
        knum=int((ptime-pfrom)/pstep)+1
        pout=pfrom+float(knum)*pstep
      endif
    else if(kstep.gt.0) then
      if(ktime.eq.kfrom) then
        konoff=1
        knum=1
        kout=kfrom+kstep
      else if(ktime.lt.kfrom) then
        konoff=0
        knum=0
        kout=kfrom
      else
        konoff=0
        knum=int((ktime-kfrom)/kstep)+1
        kout=kfrom+knum*kstep
      endif
    endif
!---------------------------------------------------------------------
!      2.        Periodic output
  else
    if(pstep.gt.0d0) then
      if(pfrom.lt.pto) then
        if((ptime.ge.pfrom).and.(ptime.lt.pto+pstep)) then
          if(ptime.ge.pout) konoff=1
        endif
      else
        if((pfrom.eq.0d0).and.(pto.eq.0d0)) then
          if(ptime.ge.pout) konoff=1
        endif
      endif
      if(konoff.eq.1) pout=pout+pstep
    else if(kstep.gt.0) then
      if(kfrom.lt.kto) then
        if((ktime.ge.kfrom).and.(ktime.le.kto)) then
          if(ktime.ge.kout) konoff=1
        endif
      else
        if((kfrom.eq.0).and.(kto.eq.0)) then
          if(ktime.ge.kout) konoff=1
        endif
      endif
      if(konoff.eq.1) kout=kout+kstep
    endif
    if(konoff.eq.1) knum=knum+1
  endif
!---------------------------------------------------------------------
!      4.        Return
RETURN
END SUBROUTINE OUTCHE


!****f* e0pro/main
! NAME 
!   OUTDIA
! PURPOSE
!   Output of diagnostics
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTDIA(kmode)
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE commet
  IMPLICIT NONE
  REAL(KIND=8) ztime, zdt
  INTEGER kmode
#ifdef DIAGNO
    call EXPERT(3001,1)
#endif
!---------------------------------------------------------------------
!      1.        Initial output
  if(myproc.eq.0) then
    if(kmode.eq.1) then
      write(*,'(4x,a)') '---------------------------------------------'
      if((tstop-tstart).lt.36000d0) then
        write(*,'(16x,3(a11))') 'time (s)','dt (s)','cflmax'
      elseif((tstop-tstart).gt.864000d0) then
        write(*,'(16x,3(a11))') 'time (d)','dt (d)','cflmax'
      else
        write(*,'(16x,3(a11))') 'time (h)','dt (h)','cflmax'
      endif
      write(*,'(4x,a)') '---------------------------------------------'
    endif
!---------------------------------------------------------------------
!      2.        Periodic output
    if(kmode.eq.2) then
      if((tstop-tstart).lt.36000d0) then
        ztime=time
        zdt=dt
       elseif((tstop-tstart).gt.864000d0) then
        ztime=time/86400d0
        zdt=dt/86400d0
      else
        ztime=time/3600d0
        zdt=dt/3600d0
      endif
      write(*,'(4x,a6,i6,3(1pe11.3))') 'step  ',  &
        nstep,ztime,zdt,cflmax
    endif
!---------------------------------------------------------------------
!      3.        Final output
    if(kmode.eq.3) then
      write(*,'(4x,a)') '---------------------------------------------'
      if((tstop-tstart).lt.36000d0) then
        write(*,'(16x,3(a11))') 'time (s)','dt (s)','cflmax'
      elseif((tstop-tstart).gt.864000d0) then
        write(*,'(16x,3(a11))') 'time (d)','dt (d)','cflmax'
      else
        write(*,'(16x,3(a11))') 'time (h)','dt (h)','cflmax'
      endif
      write(*,'(4x,a)') '---------------------------------------------'
    endif
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(3001,0)
#endif                                                                  
RETURN
END SUBROUTINE OUTDIA
