!****f* e0pro/main
! NAME 
!   OUTFIN
! PURPOSE
!   Output at time levels
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTFIN 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE commet
  USE comout
  USE netcdf
  IMPLICIT NONE
!---------------------------------------------------------------------
#ifdef PARAMESH
  INTEGER,DIMENSION(2) :: istart1,icount1,              &
                          istart2,icount2,              &
                          istart3,icount3,              &
                          istart4,icount4,              &
                          istart5,icount5,              &
                          istart6,icount6
  DATA istart1/1,1/, icount1/ncc1,maxblk/
  DATA istart2/1,1/, icount2/ncc2,maxblk/
  DATA istart3/1,1/, icount3/ncc3,maxblk/
  DATA istart4/1,1/, icount4/nhh1,maxblk/
  DATA istart5/1,1/, icount5/nhh2,maxblk/
  DATA istart6/1,1/, icount6/nhh3,maxblk/
#endif
  INTEGER,DIMENSION(2) :: idims1,              &
                          idims2,              &
                          idims3,              &
                          idims4,              &
                          idims5,              &
                          idims6
  DATA idims1/1,8/
  DATA idims2/2,8/
  DATA idims3/3,8/
  DATA idims4/4,8/
  DATA idims5/5,8/
  DATA idims6/6,8/
!---------------------------------------------------------------------
! INTEGER,DIMENSION(4) :: idim
! DATA idim /1,2,3,8 /
  INTEGER,DIMENSION(5) :: idims
  DATA idims /7,1,2,3,8 /
#ifdef staggered
    INTEGER,DIMENSION(5)  ::  idims1h,istart1h,icount1h
    DATA idims1h/7,4,2,3,8/, istart1h/1,1,1,1,1/
    DATA icount1h/ nnvar,nhh1,ncc2,ncc3,maxblk/
    INTEGER,DIMENSION(5) ::  idims2h,istart2h,icount2h
    DATA idims2h/7,1,5,3,8/, istart2h/1,1,1,1,1/
    DATA icount2h/ nnvar,ncc1,nhh2,ncc3,maxblk/
    INTEGER,DIMENSION(5) ::  idims3h,istart3h,icount3h
    DATA idims3h/7,1,2,6,8/, istart3h/1,1,1,1,1/
    DATA icount3h/nnvar,ncc1,ncc2,nhh3,maxblk/
#endif
!---------------------------------------------------------------------
  INTEGER  iret,ifile,id
! INTEGER  jb
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(3002,1)
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
!      nblk=0
!      do jb=1,lnblocks
!        if(nodetype(jb) .eq. 1) nblk=nblk+1
!      enddo
    nblk=lnblocks
    icount1(2)=nblk
    icount2(2)=nblk
    icount3(2)=nblk
    icount4(2)=nblk
    icount5(2)=nblk
    icount6(2)=nblk
#else
    nblk=1
#endif
!---------------------------------------------------------------------
!      1.        Create or open the file
!        1.1       Create the new file
  iret=NF90_CREATE(lffin,nf90_clobber,ifile)
!---------------------------------------------------------------------
!      2.        Set global attributes
  iret=NF90_PUT_ATT(ifile,nf90_global,'type','FIN')
  iret=NF90_PUT_ATT(ifile,nf90_global,'title','Values for restart')
#include <attribut.src>
!---------------------------------------------------------------------
!      3.        Set array dimensions
  iret=NF90_DEF_DIM(ifile,'n1',nc1,id)
  iret=NF90_DEF_DIM(ifile,'n2',nc2,id)
  iret=NF90_DEF_DIM(ifile,'n3',nc3,id)
  iret=NF90_DEF_DIM(ifile,'n1h',nh1,id)
  iret=NF90_DEF_DIM(ifile,'n2h',nh2,id)
  iret=NF90_DEF_DIM(ifile,'n3h',nh3,id)
  iret=NF90_DEF_DIM(ifile,'nvar',nnvar,id)
  iret=NF90_DEF_DIM(ifile,'nblk',nblk,id)
!---------------------------------------------------------------------
!      4.        Define values
!        4.2       Mesh structure
  iret=NF90_DEF_VAR (ifile,'X1',nf90_double,idims1,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X1-cell positions')
  iret=NF90_PUT_ATT(ifile,id,'units','m')
  iret=NF90_DEF_VAR(ifile,'X2',nf90_double,idims2,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X2-cell positions')
  if(lgeom.eq.'cartesian') then
    iret=NF90_PUT_ATT(ifile,id,'units','m')
  else
    iret=NF90_PUT_ATT(ifile,id,'units','radian')
  endif
  iret=NF90_DEF_VAR (ifile,'X3',nf90_double,idims3,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X3-cell positions')
  if(lgeom.eq.'spherical') then
    iret=NF90_PUT_ATT(ifile,id,'units','radian')
  else
    iret=NF90_PUT_ATT(ifile,id,'units','m')
  endif
!        4.2       Mesh structure
  iret=NF90_DEF_VAR(ifile,'X1H',nf90_double,idims4,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X1-interface positions')
  iret=NF90_PUT_ATT(ifile,id,'units','m')
  iret=NF90_DEF_VAR(ifile,'X2H',nf90_double,idims5,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X2-interface positions')
  if(lgeom.eq.'cartesian') then
    iret=NF90_PUT_ATT(ifile,id,'units','m')
  else
    iret=NF90_PUT_ATT(ifile,id,'units','radian')
  endif
  iret=NF90_DEF_VAR(ifile,'X3H',nf90_double,idims6,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X3-interface positions')
  if(lgeom.eq.'spherical') then
    iret=NF90_PUT_ATT(ifile,id,'units','radian')
  else
    iret=NF90_PUT_ATT(ifile,id,'units','m')
  endif
!        4.3       Time
  iret=NF90_DEF_VAR(ifile,'TIME',nf90_double,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
  iret=NF90_PUT_ATT(ifile,id,'units','s')
  iret=NF90_DEF_VAR(ifile,'DT',nf90_double,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
  iret=NF90_PUT_ATT(ifile,id,'units','s')
  iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
!        4.4       Unknown
  iret=NF90_DEF_VAR(ifile,'UNK',nf90_double,idims,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','Unknown')
!Leave define mode
  iret=NF90_ENDDEF(ifile)
!---------------------------------------------------------------------
!      5.        Write values
!        5.1       Mesh structure
#ifdef PARAMESH
    iret=NF90_PUT_VAR(ifile,1,xc1,start=istart1,count=icount1)
    iret=NF90_PUT_VAR(ifile,2,xc2,start=istart2,count=icount2)
    iret=NF90_PUT_VAR(ifile,3,xc3,start=istart3,count=icount3)
#else
    iret=NF90_PUT_VAR(ifile,1,xc1)
    iret=NF90_PUT_VAR(ifile,2,xc2)
    iret=NF90_PUT_VAR(ifile,3,xc3)
#endif
!        5.2       Mesh structure
#ifdef PARAMESH
    iret=NF90_PUT_VAR(ifile,4,xh1,start=istart4,count=icount4)
    iret=NF90_PUT_VAR(ifile,5,xh2,start=istart5,count=icount5)
    iret=NF90_PUT_VAR(ifile,6,xh3,start=istart6,count=icount6)
#else
    iret=NF90_PUT_VAR(ifile,4,xh1)
    iret=NF90_PUT_VAR(ifile,5,xh2)
    iret=NF90_PUT_VAR(ifile,6,xh3)
#endif
!        5.3       Time
  iret=NF90_PUT_VAR(ifile,7,time)
  iret=NF90_PUT_VAR(ifile,8,dt)
  iret=NF90_PUT_VAR(ifile,9,nstep)
!        5.4       Unknown
  iret=NF90_PUT_VAR(ifile,10,unk)
#ifdef mhd
#ifdef staggered
      iret=NF90_PUT_VAR(ifile,11,b1hcc)
      iret=NF90_PUT_VAR(ifile,12,b2chc)
      iret=NF90_PUT_VAR(ifile,13,b3cch)
#endif
#endif
!---------------------------------------------------------------------
!      6.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!      9.        Return
#ifdef DIAGNO
    call EXPERT(3002,0)
#endif                                                                  
RETURN 
END SUBROUTINE OUTFIN                         
