!****f* e0pro/main
! NAME 
!   OUTEVH
! PURPOSE
!   Output at heliospheric points
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTEVH(kmode) 
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comevo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(KIND=4)  ztime
  INTEGER  iret, ifile, id, kmode
  INTEGER, DIMENSION(2)  ::  idims, istart, icount
  LOGICAL ilexis
  DATA  idims/1,2/
  CHARACTER(len=nf90_max_name) :: iname
#ifdef DIAGNO
    call EXPERT(3004,1)
#endif
!---------------------------------------------------------------------
!      1.        Create the new file
  if(kmode.eq.1) then
    ilexis=.false.
    INQUIRE(file=lfevh,exist=ilexis)
    if(ilexis) then
      iret=NF90_OPEN(lfevh,nf90_write,ifile)
      iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,mevo)
      iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      do while((ztime.gt.real(time)).and.(mevo.ge.1))
        mevo=mevo-1
        iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      enddo
    else
      mevo=0
      iret=NF90_CREATE(lfevh,nf90_clobber,ifile)
!---------------------------------------------------------------------
!      2.        Set global attributes
      iret=NF90_PUT_ATT(ifile,nf90_global,'type','EVH')
      iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
        'Evolution at points in heliosphere')
#include <attribut.src>
!---------------------------------------------------------------------
!      3.        Set array dimensions
      iret=NF90_DEF_DIM(ifile,'nhel',nhel,id)
      iret=NF90_DEF_DIM(ifile,'nevo',nf90_unlimited,id)
!---------------------------------------------------------------------
!      4.        Define constant values
      iret=NF90_DEF_VAR(ifile,'X1',nf90_float,1,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-mesh positions')
      iret=NF90_PUT_ATT(ifile,id,'units','m')
      iret=NF90_DEF_VAR(ifile,'X2',nf90_float,1,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-mesh positions')
      if(lgeom.eq.'cartesian') then
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      endif
      iret=NF90_DEF_VAR(ifile,'X3',nf90_float,1,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-mesh positions')
      if(lgeom.eq.'spherical') then
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      endif
!---------------------------------------------------------------------
!      5.        Define time-dependent values
      iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'DT',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
      iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
      iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
      iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
      iret=NF90_PUT_ATT(ifile,id,'units','K')
      iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
        iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X1-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X2-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X3-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
        iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Density of plasma cloud')
#endif
#ifdef mhdcon
        iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Polarity of magnetic field')
#endif
      if(nldiav) then
        iret=NF90_DEF_VAR(ifile,'E1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X1-electric field')
        iret=NF90_DEF_VAR(ifile,'E2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X2-electric field')
        iret=NF90_DEF_VAR(ifile,'E3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X3-electric field')
        iret=NF90_DEF_VAR(ifile,'XJ1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X1-eta*current')
        iret=NF90_DEF_VAR(ifile,'XJ2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X2-eta*current')
        iret=NF90_DEF_VAR(ifile,'XJ3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X3-eta*current')
        iret=NF90_PUT_ATT(ifile, id,'long_name','Total mass')
        iret=NF90_DEF_VAR(ifile,'ENETOT', nf90_float,2,id)
        iret=NF90_PUT_ATT(ifile, id,'long_name','Total energy')
        iret=NF90_DEF_VAR(ifile,'ENETHE',nf90_float,2,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Thermal energy')
        iret=NF90_DEF_VAR(ifile,'ENEKIN',nf90_float,2,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Kintetic energy')
        iret=NF90_DEF_VAR(ifile,'ENEMAG',nf90_float,2,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Magnetic energy')
      endif
!Leave define mode
      iret=NF90_ENDDEF(ifile)
!---------------------------------------------------------------------
!      6.        Write constant values
      iret=NF90_PUT_VAR(ifile,1,x1hel,count=(/nhel/))
      iret=NF90_PUT_VAR(ifile,2,x2hel,count=(/nhel/))
      iret=NF90_PUT_VAR(ifile,3,x3hel,count=(/nhel/))
    endif
  else
!---------------------------------------------------------------------
!      7.        Open the existing file
    iret=NF90_OPEN(lfevh,nf90_write,ifile)
    istart(1)=1
    istart(2)=mevo
    icount(1)=nhel
    icount(2)=1
!---------------------------------------------------------------------
!      8.        Write time-dependent values
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,5,dt,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,6,nstep,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,7,dhel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,8,thel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,9,v1hel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,10,v2hel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,11,v3hel,start=istart,count=icount)
    id=11
#ifdef mhd
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b1hel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b2hel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b3hel,start=istart,count=icount)
#endif
#ifdef hydcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,dphel,start=istart,count=icount)
#endif
#ifdef mhdcon
#ifdef mhd
        id=id+1
        iret=NF90_PUT_VAR(ifile,id,bphel,start=istart,count=icount)
#endif
#endif
    if(nldiav) then
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,e1hel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,e3hel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,e3hel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,xj1hel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,xj2hel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,xj3hel,start=istart,count=icount)
    endif
    if(nldiav) then
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,totmas,start=(/mevo/))
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,enetot,start=(/mevo/))
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,enethe,start=(/mevo/))
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,enekin,start=(/mevo/))
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,enemag,start=(/mevo/))
    endif
  endif
!---------------------------------------------------------------------
!      9.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!      10.       Return
#ifdef DIAGNO
    call EXPERT(3004,0)
#endif
RETURN
END SUBROUTINE OUTEVH


!****f* e0pro/main
! NAME 
!   OUTEVL
! PURPOSE
!   Output at points on Sun-Earth line                               
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTEVL(kmode)
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comevo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(KIND=4)  ztime
  INTEGER  iret, ifile, id, kmode
  INTEGER, DIMENSION(2)  ::  idims, istart, icount
  LOGICAL ilexis
  DATA  idims/1,2/
  CHARACTER(len=nf90_max_name) :: iname
#ifdef DIAGNO
    call EXPERT(3004,1)
#endif
!---------------------------------------------------------------------
!      1.        Create the new file
  if(kmode.eq.1) then
    ilexis=.false.
    INQUIRE(file=lfevl,exist=ilexis)
    if(ilexis) then
      iret=NF90_OPEN(lfevl,nf90_write,ifile)
      iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,mevo)
      iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      do while((ztime.gt.real(time)).and.(mevo.ge.1))
        mevo=mevo-1
        iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      enddo
    else
      mevo=0
      iret=NF90_CREATE(lfevl,nf90_clobber,ifile)
!---------------------------------------------------------------------
!      2.        Set global attributes
      iret=NF90_PUT_ATT(ifile,nf90_global,'type','EVL')
      iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
        'Evolution at points on Sun-Earth line')
#include <attribut.src>
!---------------------------------------------------------------------
!      3.        Set array dimensions
      iret=NF90_DEF_DIM(ifile,'nsel',nsel,id)
      iret=NF90_DEF_DIM(ifile,'nevo',nf90_unlimited,id)
!---------------------------------------------------------------------
!      4.        Define time-dependent coordinates
      iret=NF90_DEF_VAR(ifile,'X1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-mesh positions')
      iret=NF90_PUT_ATT(ifile,id,'units','m')
      iret=NF90_DEF_VAR(ifile,'X2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-mesh positions')
      if(lgeom.eq.'cartesian') then
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      endif
      iret=NF90_DEF_VAR(ifile,'X3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-mesh positions')
      if(lgeom.eq.'spherical') then
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      endif
!---------------------------------------------------------------------
!      5.        Define time-dependent values
      iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'DT',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
      iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
      iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
      iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
      iret=NF90_PUT_ATT(ifile,id,'units','K')
      iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
        iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X1-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile, id, 'long_name','X2-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X3-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
        iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Density of plasma cloud')
#endif
#ifdef mhdcon
        iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Polarity of magnetic field')
#endif
!Leave define mode
      iret=NF90_ENDDEF(ifile)
    endif
  else
!---------------------------------------------------------------------
!      6.        Open the existing file
    iret=NF90_OPEN(lfevl,nf90_write,ifile)
    istart(1)=1
    istart(2)=mevo
    icount(1)=nsel
    icount(2)=1
!---------------------------------------------------------------------
!      7.        Write time-dependent values
    iret=NF90_PUT_VAR(ifile,1,x1sel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,2,x2sel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,3,x3sel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,5,dt,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,6,nstep,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,7,dsel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,8,tsel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,9,v1sel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,10,v2sel,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,11,v3sel,start=istart,count=icount)
    id=11
#ifdef mhd
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b1sel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b2sel,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b3sel,start=istart,count=icount)
#endif
#ifdef hydcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,dpsel,start=istart,count=icount)
#endif
#ifdef mhdcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,bpsel,start=istart,count=icount)
#endif
  endif
!---------------------------------------------------------------------
!      8.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!      9.        Return
#ifdef DIAGNO
    call EXPERT(3004,0)
#endif
RETURN
END SUBROUTINE OUTEVL


!****f* e0pro/main
! NAME 
!   OUTEVG
! PURPOSE
!   Outputs at geospace points
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTEVG(kmode)
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comevo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(KIND=4)  ztime
  INTEGER  iret, ifile, id, kmode
  INTEGER, DIMENSION(2)  ::  idims, istart, icount
  LOGICAL ilexis
  DATA  idims/1,2/
  CHARACTER(len=nf90_max_name) :: iname
#ifdef DIAGNO
    call EXPERT(3004,1)
#endif
!---------------------------------------------------------------------
!      1.        Create the new file
  if(kmode.eq.1) then
    ilexis=.false.
    INQUIRE(file=lfevg,exist=ilexis)
    if(ilexis) then
      iret=NF90_OPEN(lfevg,nf90_write,ifile)
      iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,mevo)
      iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      do while((ztime.gt.real(time)).and.(mevo.ge.1))
        mevo=mevo-1
        iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      enddo
    else
      mevo=0
      iret=NF90_CREATE(lfevg,nf90_clobber,ifile)
!---------------------------------------------------------------------
!      2.        Set global attributes
      iret=NF90_PUT_ATT(ifile,nf90_global,'type','EVG')
      iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
        'Evolution at points surrounding the geospace')
#include <attribut.src>
!---------------------------------------------------------------------
!      3.        Set array dimensions
      iret=NF90_DEF_DIM(ifile,'ngeo',5,id)
      iret=NF90_DEF_DIM(ifile,'nevo',nf90_unlimited,id)
!---------------------------------------------------------------------
!      4.        Define time-dependent coordinates
      iret=NF90_DEF_VAR(ifile,'X1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-mesh positions')
      iret=NF90_PUT_ATT(ifile,id,'units','m')
      iret=NF90_DEF_VAR(ifile,'X2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-mesh positions')
      if(lgeom.eq.'cartesian') then
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      endif
      iret=NF90_DEF_VAR(ifile,'X3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-mesh positions')
      if(lgeom.eq.'spherical') then
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      endif
!---------------------------------------------------------------------
!      5.        Define time-dependent values
      iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'DT',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
      iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
      iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
      iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
      iret=NF90_PUT_ATT(ifile,id,'units','K')
      iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
        iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',            &
                                   'X1-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',         &
                                   'X2-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',         &
                                   'X3-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
        iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',         &
                                   'Density of plasma cloud')
#endif
#ifdef mhdcon
        iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',         &
                                   'Polarity of magnetic field')
#endif
!Leave define mode
      iret=NF90_ENDDEF(ifile)
      iret=NF90_CLOSE(ifile)
    endif
  else
!---------------------------------------------------------------------
!      6.        Open the existing file
    iret=NF90_OPEN(lfevg,nf90_write,ifile)
    istart(1)=1
    istart(2)=mevo
    icount(1)=5
    icount(2)=1
!---------------------------------------------------------------------
!      7.        Write time-dependent values
    iret=NF90_PUT_VAR(ifile,1,x1geo,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,2,x2geo,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,3,x3geo,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,5,dt,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,6,nstep,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,7,dgeo,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,8,tgeo,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,9,v1geo,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,10,v2geo,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,11,v3geo,start=istart,count=icount)
    id=11
#ifdef mhd
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b1geo,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b2geo,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b3geo,start=istart,count=icount)
#endif
#ifdef hydcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,dpgeo,start=istart,count=icount)
#endif
#ifdef mhdcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,bpgeo,start=istart,count=icount)
#endif
  endif
!---------------------------------------------------------------------
!      8.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!      9.        Return
#ifdef DIAGNO
    call EXPERT(3004,0)
#endif
RETURN
END SUBROUTINE OUTEVG


!****f* e0pro/main
! NAME 
!   OUTEVP
! PURPOSE
!   Output at planetary positions
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTEVP(kmode)
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comevo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(KIND=4)  ztime
  INTEGER  iret, ifile, id, kmode
  INTEGER, DIMENSION(2)  ::  idims, istart, icount
  LOGICAL ilexis
  DATA  idims/1,2/
  CHARACTER(len=nf90_max_name) :: iname
#ifdef DIAGNO
    call EXPERT(3004,1)
#endif
!---------------------------------------------------------------------
!      1.        Create the new file
  if(kmode.eq.1) then
    ilexis=.false.
    INQUIRE(file=lfevp,exist=ilexis)
    if(ilexis) then
      iret=NF90_OPEN(lfevp,nf90_write,ifile)
      iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,mevo)
      iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      do while((ztime.gt.real(time)).and.(mevo.ge.1))
        mevo=mevo-1
        iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      enddo
    else
      mevo=0
      iret=NF90_CREATE(lfevp,nf90_clobber,ifile)
!---------------------------------------------------------------------
!      2.        Set global attributes
      iret=NF90_PUT_ATT(ifile,nf90_global,'type','EVP')
      iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
        'Evolution at points at planetary positions')
#include <attribut.src>
!---------------------------------------------------------------------
!      3.        Set array dimensions
      iret=NF90_DEF_DIM(ifile,'npla',npla,id)
      iret=NF90_DEF_DIM(ifile,'nevo',nf90_unlimited,id)
!---------------------------------------------------------------------
!      4.        Define time-dependent coordinates
      iret=NF90_DEF_VAR(ifile,'X1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-mesh positions')
      iret=NF90_PUT_ATT(ifile,id,'units','m')
      iret=NF90_DEF_VAR(ifile,'X2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-mesh positions')
      if(lgeom.eq.'cartesian') then
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      endif
      iret=NF90_DEF_VAR(ifile,'X3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-mesh positions')
      if(lgeom.eq.'spherical') then
        iret=NF90_PUT_ATT(ifile,id,'units','radian')
      else
        iret=NF90_PUT_ATT(ifile,id,'units','m')
      endif
!---------------------------------------------------------------------
!      5.        Define time-dependent values
      iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'DT',nf90_float,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,2,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
      iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
      iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
      iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
      iret=NF90_PUT_ATT(ifile,id,'units','K')
      iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
        iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X1-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X2-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X3-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
        iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Density of plasma cloud')
#endif
#ifdef mhdcon
        iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Polarity of magnetic field')
#endif
!Leave define mode
      iret=NF90_ENDDEF(ifile)
    endif
  else
!---------------------------------------------------------------------
!      6.        Open the existing file
    iret=NF90_OPEN(lfevp,nf90_write,ifile)
    istart(1)=1
    istart(2)=mevo
    icount(1)=npla
    icount(2)=1
!---------------------------------------------------------------------
!      7.        Write time-dependent values
    iret=NF90_PUT_VAR(ifile,1,x1pla,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,2,x2pla,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,3,x3pla,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,5,dt,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,6,nstep,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,7,dpla,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,8,tpla,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,9,v1pla,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,10,v2pla,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,11,v3pla,start=istart,count=icount)
    id=11
#ifdef mhd
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b1pla,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b2pla,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b3pla,start=istart,count=icount)
#endif
#ifdef hydcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,dppla,start=istart,count=icount)
#endif
#ifdef mhdcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,bppla,start=istart,count=icount)
#endif
  endif
!---------------------------------------------------------------------
!      8.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!      9.        Return
#ifdef DIAGNO
    call EXPERT(3004,0)
#endif
RETURN
END SUBROUTINE OUTEVP


!****f* e0pro/main
! NAME 
!   OUTEVF
! PURPOSE
!   Output along magnetic field lines
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTEVF(kmode) 
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comevo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(KIND=4)  ztime
  INTEGER  iret, ifile, id, kmode
  INTEGER, DIMENSION(3)  ::  idims, istart, icount
  LOGICAL ilexis
  DATA  idims/1,2,3/
  CHARACTER(len=nf90_max_name) :: iname
#ifdef DIAGNO
    call EXPERT(3004,1)
#endif
!---------------------------------------------------------------------
!      1.        Create the new file
  if(kmode.eq.1) then
    ilexis=.false.
    INQUIRE(file=lfevf,exist=ilexis)
    if(ilexis) then
      iret=NF90_OPEN(lfevf,nf90_write,ifile)
      iret=NF90_INQUIRE_DIMENSION(ifile,3,iname,mevo)
      iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      do while((ztime.gt.real(time)).and.(mevo.ge.1))
        mevo=mevo-1
        iret=NF90_GET_VAR(ifile,4,ztime,start=(/mevo/))
      enddo
    else
      mevo=0
      iret=NF90_CREATE(lfevf,nf90_clobber,ifile)
!---------------------------------------------------------------------
!      2.        Set global attributes
      iret=NF90_PUT_ATT(ifile,nf90_global,'type','EVF')
      iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
        'Evolution at magnetic field lines')
#include <attribut.src>
!---------------------------------------------------------------------
!      3.        Set array dimensions
      iret=NF90_DEF_DIM(ifile,'nfld',maxfld,id)
      iret=NF90_DEF_DIM(ifile,'nfstep',maxfstep,id)
      iret=NF90_DEF_DIM(ifile,'nevo',nf90_unlimited,id)
!---------------------------------------------------------------------
!      4.        Define time-dependent values
      iret=NF90_DEF_VAR(ifile,'X1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X1-field-line positions')
      iret=NF90_PUT_ATT(ifile,id,'units','m')
      iret=NF90_DEF_VAR(ifile,'X2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-field-line positions')
      iret=NF90_PUT_ATT(ifile,id,'units','m')
      iret=NF90_DEF_VAR(ifile,'X3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X3-field-line positions')
      iret=NF90_PUT_ATT(ifile,id,'units','m')
      iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,3,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'DT',nf90_float,3,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
      iret=NF90_PUT_ATT(ifile,id,'units','s')
      iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,3,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
      iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile, id, 'long_name','Density')
      iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
      iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile, id, 'long_name','Temperature')
      iret=NF90_PUT_ATT(ifile,id,'units','K')
      iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile, id, 'long_name','X1-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
      iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile, id, 'long_name','X3-velocity')
      iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
        iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X1-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile, id, 'long_name','X2-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR(ifile,'B3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','X3-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
        iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Density of plasma cloud')
#endif
#ifdef mhdcon
        iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name','Polarity of magnetic field')
#endif
!Leave define mode
      iret=NF90_ENDDEF(ifile)
    endif
  else
!---------------------------------------------------------------------
!      5.        Open the existing file
    iret=NF90_OPEN(lfevf,nf90_write,ifile)
    istart(1)=1
    istart(2)=1
    istart(3)=mevo
    icount(1)=nfld
    icount(2)=maxfstep
    icount(3)=1
!---------------------------------------------------------------------
!      6.        Write time-dependent values
    iret=NF90_PUT_VAR(ifile,1,x1fld,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,2,x2fld,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,3,x3fld,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,5,dt,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,6,nstep,start=(/mevo/))
    iret=NF90_PUT_VAR(ifile,7,dfld,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,8,tfld,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,9,v1fld,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,10,v2fld,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,11,v3fld,start=istart,count=icount)
    id=11
#ifdef mhd
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b1fld,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b2fld,start=istart,count=icount)
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,b3fld,start=istart,count=icount)
#endif
#ifdef hydcon
      id=id+1
      iret=NF90_PUT_VAR(ifile,id,dpfld,start=istart,count=icount)
#endif
#ifdef mhdcon
#ifdef mhd
        id=id+1
        iret=NF90_PUT_VAR(ifile,id,bpfld,start=istart,count=icount)
#endif
#endif
  endif
!---------------------------------------------------------------------
!      9.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!      10.       Return
#ifdef DIAGNO
    call EXPERT(3004,0)
#endif
RETURN
END SUBROUTINE OUTEVF
