!****f* e0pro/main
! NAME 
!   OUTVAL
! PURPOSE
!   Output at time levels
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTVAL(kmode) 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE commet
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(KIND=8)  zuthe, zt
#ifdef PARAMESH
#ifndef ethe
  REAL(KIND=8)  zutot,zgmr,zd,zdv1,zdv2,zdv3
#endif
#ifdef mhd
  REAL(KIND=8)  zb1,zb2,zb3,zumag
#endif
#endif
#ifndef ethe
  REAL(KIND=8)  zukin
#ifdef mhd
  REAL(KIND=8)  zumag
#endif
#endif
  INTEGER  iret,ifile,id,ivar
  INTEGER  kmode
  INTEGER  j1,j2,j3,jb,ib
!---------------------------------------------------------------------
  INTEGER,DIMENSION(2) :: idims1,istart1,icount1,                 &
                          idims2,istart2,icount2,                 &
                          idims3,istart3,icount3,                 &
                          idims4,istart4,icount4,                 &
                          idims5,istart5,icount5,                 &
                          idims6,istart6,icount6
  DATA  idims1/1,7/,istart1/1,1/,icount1/ncc1,maxblk/
  DATA  idims2/2,7/,istart2/1,1/,icount2/ncc2,maxblk/
  DATA  idims3/3,7/,istart3/1,1/,icount3/ncc3,maxblk/
  DATA  idims4/4,7/,istart4/1,1/,icount4/nhh1,maxblk/
  DATA  idims5/5,7/,istart5/1,1/,icount5/nhh2,maxblk/
  DATA  idims6/6,7/,istart6/1,1/,icount6/nhh3,maxblk/
!---------------------------------------------------------------------
  INTEGER,DIMENSION(5) :: idims,istart,icount
  DATA  idims/1,2,3,7,8/
  DATA  istart/1,1,1,1,1/
  DATA  icount/ncc1,ncc2,ncc3,maxblk,1/
! INTEGER,DIMENSION(5) :: idims1h,istart1h,icount1h
! DATA  idims1h/4,2,3,7,8/
! DATA  istart1h/1,1,1,1,1/
! DATA  icount1h/nhh1,ncc2,ncc3,maxblk,1/
! INTEGER,DIMENSION(5) :: idims2h,istart2h,icount2h
! DATA  idims2h/1,5,3,7,8/
! DATA  istart2h/1,1,1,1,1/
! DATA  icount2h/ncc1,nhh2,ncc3,maxblk,1/
! INTEGER,DIMENSION(5) :: idims3h,istart3h,icount3h
! DATA  idims3h/1,2,6,7,8/
! DATA  istart3h/1,1,1,1,1/
! DATA  icount3h/ncc1,ncc2,nhh3,maxblk,1/
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(3003,1)
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
    nblk=0
    do jb=1,lnblocks
      if(nodetype(jb).eq.1) nblk=nblk+1
    enddo
#else
    nblk=1
#endif
  istart(5)=mval
! istart1h(5)=mval
! istart2h(5)=mval
! istart3h(5)=mval
  icount1(2)=nblk
  icount2(2)=nblk
  icount3(2)=nblk
  icount(4)=nblk
! icount1h(4)=nblk
! icount2h(4)=nblk
! icount3h(4)=nblk
!---------------------------------------------------------------------
!      1.        Create or open the file
!        1.1       Create the new file
  if(kmode.eq.1) then
    iret=NF90_CREATE(lfval,nf90_clobber,ifile)
!---------------------------------------------------------------------
!      2.          Set global attributes
    iret=NF90_PUT_ATT(ifile,nf90_global,'type','VAL')
    iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
      'Values at time levels')
#include <attribut.src>
!---------------------------------------------------------------------
!      3.          Set array dimensions
    iret=NF90_DEF_DIM(ifile,'n1',ncc1,id)
    iret=NF90_DEF_DIM(ifile,'n2',ncc2,id)
    iret=NF90_DEF_DIM(ifile,'n3',ncc3,id)
    iret=NF90_DEF_DIM(ifile,'n1h',nhh1,id)
    iret=NF90_DEF_DIM(ifile,'n2h',nhh2,id)
    iret=NF90_DEF_DIM(ifile,'n3h',nhh3,id)
    iret=NF90_DEF_DIM(ifile,'nblk',nblk,id)
    iret=NF90_DEF_DIM(ifile,'nval',nf90_unlimited,id)
!---------------------------------------------------------------------
!      4.        Define constant values
!        4.1       Mesh structure
    iret=NF90_DEF_VAR (ifile,'X1',nf90_float,idims1,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-cell positions')
    iret=NF90_PUT_ATT(ifile,id,'units','m')
    iret=NF90_DEF_VAR (ifile,'X2',nf90_float,idims2,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-cell positions')
    if(lgeom.eq.'cartesian') then
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    endif
    iret=NF90_DEF_VAR (ifile,'X3',nf90_float,idims3,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-cell positions')
    if(lgeom.eq.'spherical') then
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    endif
!        4.2       Mesh structure
    iret=NF90_DEF_VAR (ifile,'X1H',nf90_float,idims4,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-interface &
                             &positions')
    iret=NF90_PUT_ATT(ifile,id,'units','m')
    iret=NF90_DEF_VAR (ifile,'X2H',nf90_float,idims5,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-interface &
                             &positions')
    if(lgeom.eq.'cartesian') then
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    endif
    iret=NF90_DEF_VAR (ifile,'X3H',nf90_float,idims6,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-interface &
                             &positions')
    if(lgeom.eq.'spherical') then
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    endif
!---------------------------------------------------------------------
!      5.        Define time-dependent values
!        5.1       Time
    iret=NF90_DEF_VAR (ifile,'TIME',nf90_float,8,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
    iret=NF90_PUT_ATT(ifile,id,'units','s')
    iret=NF90_DEF_VAR (ifile,'DT',nf90_float,8,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
    iret=NF90_PUT_ATT(ifile,id,'units','s')
    iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,8,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
!        5.2       Unknown
    iret=NF90_DEF_VAR (ifile,'D',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
    iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
    iret=NF90_DEF_VAR (ifile,'T',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
    iret=NF90_PUT_ATT(ifile,id,'units','K')
    iret=NF90_DEF_VAR (ifile,'V1',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR (ifile,'V2',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR (ifile,'V3',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
#ifdef staggered
        iret=NF90_DEF_VAR (ifile,'B1',nf90_float,idims1h,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',        &
                                   'X1H-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR (ifile,'B2',nf90_float,idims2h,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',        &
                                   'X2H-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR (ifile,'B3',nf90_float,idims3h,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',        &
                                  'X3H-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
#else
        iret=NF90_DEF_VAR (ifile,'B1',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',        &
                                   'X1-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR (ifile,'B2',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',        &
                                   'X2-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
        iret=NF90_DEF_VAR (ifile,'B3',nf90_float,idims,id)
        iret=NF90_PUT_ATT(ifile,id,'long_name',        &
                                   'X3-magnetic field')
        iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#endif
#ifdef hydcon
      iret=NF90_DEF_VAR (ifile,'DP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',            &
                                'Density of plasma cloud')
#endif
#ifdef mhdcon
      iret=NF90_DEF_VAR (ifile,'BP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name','Polarity of magnetic field')
#endif
!leave define mode
    iret=NF90_ENDDEF(ifile)
!---------------------------------------------------------------------
!      5.        Write constant values
!        5.1       Block structure
#ifdef PARAMESH
      ib=0
      do jb=1,lnblocks
        if(nodetype(jb).eq.1) then
          ib=ib+1
          do j1=nc1l,nc1r
            zx1(j1-ng1,ib)=xc1(j1,jb)
          enddo
          do j2=nc2l,nc2r
            zx2(j2-ng2,ib)=xc2(j2,jb)
          enddo
          do j3=nc3l,nc3r
            zx3(j3-ng3,ib)=xc3(j3,jb)
          enddo
        endif
      enddo
#else
      do j1=nc1l,nc1r
        zx1(j1-ng1,1)=xc1(j1,1)
        zh1(j1-ng1,1)=xh1(j1,1)
      enddo
        zh1(nc1rp-ng1,1)=xh1(nc1rp,1)
        do j2=nc2l,nc2r
          zx2(j2-ng2,1)=xc2(j2,1)
          zh2(j2-ng2,1)=xh2(j2,1)
        enddo
        zh2 (nc2rp-ng2,1)=xh2(nc2rp,1)
        do j3=nc3l,nc3r
          zx3(j3-ng3,1)=xc3(j3,1)
          zh3(j3-ng3,1)=xh3(j3,1)
        enddo
        zh3(nc3rp-ng3,1)=xh3(nc3rp,1)
#endif
!        5.1       Mesh structure
      iret=NF90_PUT_VAR(ifile,1,zx1,start=istart1,count=icount1)
      iret=NF90_PUT_VAR(ifile,2,zx2,start=istart2,count=icount2)
      iret=NF90_PUT_VAR(ifile,3,zx3,start=istart3,count=icount3)
!        5.2       Mesh structure
      iret=NF90_PUT_VAR(ifile,4,zh1,start=istart4,count=icount4)
      iret=NF90_PUT_VAR(ifile,5,zh2,start=istart5,count=icount5)
      iret=NF90_PUT_VAR(ifile,6,zh3,start=istart6,count=icount6)
!---------------------------------------------------------------------
!      6.        Write time-dependent values
    else
!        6.1       Open the existing file
      iret=NF90_OPEN(lfval,nf90_write,ifile)
!---------------------------------------------------------------------
!      7.        Write time-dependent values
!        7.1       Time
      iret=NF90_PUT_VAR(ifile,7,time,start=(/mevo,1/))
      iret=NF90_PUT_VAR(ifile,8,dt,start=(/mevo,1/))
      iret=NF90_PUT_VAR(ifile,9,nstep,start=(/mevo,1/))
!        7.2       Density
#ifdef PARAMESH
        ib=0
        do jb=1,lnblocks
          if(nodetype(jb).eq.1) then
            ib=ib+1
#else
            ib=1
            jb=1
#endif
            do j3=nc3l,nc3r
              do j2=nc2l,nc2r
                do j1=nc1l,nc1r
                  zx123(j1-ng1,j2-ng2,j3-ng3,ib)=unk(1,j1,j2,j3,jb)
                enddo
              enddo
            enddo
#ifdef PARAMESH
            endif
         enddo
#endif
      iret=NF90_PUT_VAR(ifile,10,zx123,start=istart,count=icount)
!        7.3       Temperature
#ifdef PARAMESH
        ib=0
        do jb=1,lnblocks
          if(nodetype(jb).eq.1) then
            ib=ib+1
#endif
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
          do j1=nc1l,nc1r
#ifdef PARAMESH
#ifdef ethe
                zuthe=unk(2,j1,j2,j3,jb)
#else
                zd=unk(1,j1,j2,j3,jb)
                zutot=unk(2,j1,j2,j3,jb)
                zdv1=unk(3,j1,j2,j3,jb)
                zdv2=unk(4,j1,j2,j3,jb)
                zdv3=unk(5,j1,j2,j3,jb)
                zukin=zhalf*(zdv1*zdv1+zdv2*zdv2+zdv3*zdv3)/zd
#ifdef etot
#ifdef mhd
#ifdef staggered
                      zb1=zhalf*(facevarx(1,j1+1,j2,j3,jb) &
                                +facevarx(1,j1,j2,j3,jb))
                      zb2=zhalf*(facevarx(1,j1,j2+1,j3,jb) &
                                +facevarx(1,j1,j2,j3,jb))
                      zb3=zhalf*(facevarx(1,j1,j2,j3+1,jb) &
                                +facevarx(1,j1,j2,j3,jb))
#else
                      zb1=unk(6,j1,j2,j3,jb)
                      zb2=unk(7,j1,j2,j3,jb)
                      zb3=unk(8,j1,j2,j3,jb)
#endif
                    zumag=zr2mu*(zb1*zb1+zb2*zb2+zb3*zb3)
                    zuthe=zutot-zukin-zumag
#else
                    zuthe=zutot-zukin
#endif
#else
                  zuthe=zutot-zukin
#endif
#ifdef gam3d
                  zgmr=(gamma(j1,j2,j3,jb)-zone)/zgas
#endif
                zt=zgmr*zuthe/unk(1,j1,j2,j3,jb)
                zx123(j1-ng1,j2-ng2,j3-ng3,ib)=zt
#endif
#else
#ifdef ethe
                zuthe=unk(2,j1,j2,j3,jb)
#else
                zukin=zhalf*(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb) &
                            +unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb) &
                            +unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))/ &
                            unk(1,j1,j2,j3,jb)
#ifdef etot
#ifdef mhd
                    zumag=zr2mu*(unk(6,j1,j2,j3,jb)*unk(6,j1,j2,j3,jb) &
                                +unk(7,j1,j2,j3,jb)*unk(7,j1,j2,j3,jb) &
                                +unk(8,j1,j2,j3,jb)*unk(8,j1,j2,j3,jb))
                    zuthe=unk(2,j1,j2,j3,jb)-zukin-zumag
#else
                    zuthe=unk(2,j1,j2,j3,jb)-zukin
#endif
#else
                  zuthe=unk(2,j1,j2,j3,jb)-zukin
#endif
#endif
#ifdef gam3d
                zt=((gamma(j1,j2,j3,jb)-zone)/zgas)*zuthe/unk(1,j1,j2,j3,jb)
#else
                zt=((gamma-zone)/zgas)*zuthe/unk(1,j1,j2,j3,jb)
#endif
              zx123(j1-ng1,j2-ng2,j3-ng3,1)=zt
#endif
                  enddo
               enddo
            enddo
#ifdef PARAMESH
               endif
             enddo
#endif
         iret=NF90_PUT_VAR(ifile,11,zx123,start=istart,count=icount)
!        7.3       X1-velocity
#ifdef PARAMESH
            ib=0
               do jb=1,lnblocks
                  if(nodetype(jb).eq.1) then
                     ib=ib+1
#endif
            do j3=nc3l,nc3r
               do j2=nc2l,nc2r
                  do j1=nc1l,nc1r
                    zx123(j1-ng1,j2-ng2,j3-ng3,ib)=unk(3,j1,j2,j3,jb) &
                                                  /unk(1,j1,j2,j3,jb)
                  enddo
               enddo
            enddo
#ifdef PARAMESH
           endif
         enddo
#endif
         iret=NF90_PUT_VAR (ifile,12,zx123,start=istart,count=icount)
!        7.4       X2-velocity
#ifdef PARAMESH
            ib=0
            do jb=1,lnblocks
               if(nodetype(jb).eq.1) then
                  ib=ib+1
#endif
            do j3=nc3l,nc3r
               do j2=nc2l,nc2r
                  do j1=nc1l,nc1r
                    zx123(j1-ng1,j2-ng2,j3-ng3,ib)=unk(4,j1,j2,j3,jb) &
                                                  /unk(1,j1,j2,j3,jb)
                  enddo
               enddo
            enddo
#ifdef PARAMESH
            endif
          enddo
#endif
         iret=NF90_PUT_VAR (ifile,13,zx123,start=istart,count=icount)
!        7.5       X3-velocity
#ifdef PARAMESH
            ib=0
            do jb=1,lnblocks
               if(nodetype(jb).eq.1) then
                  ib=ib+1
#endif
            do j3=nc3l,nc3r
               do j2=nc2l,nc2r
                  do j1=nc1l,nc1r
                    zx123(j1-ng1,j2-ng2,j3-ng3,ib)=unk(5,j1,j2,j3,jb) &
                                                  /unk(1,j1,j2,j3,jb)
                  enddo
               enddo
            enddo
#ifdef PARAMESH
             endif
            enddo
#endif
         iret=NF90_PUT_VAR(ifile,14,zx123,start=istart,count=icount)
         id=13
         ivar=5
#ifdef mhd
!        7.6       X1-magnetic field
#ifdef PARAMESH
                ib=0
                do jb=1,lnblocks
                   if(nodetype(jb).eq.1) then
                      ib=ib+1
#endif
            do j3=nc3l,nc3r
               do j2=nc2l,nc2r
                  do j1=nc1l,nc1r
#ifdef staggered
                        zxh23(j1-ng1,j2-ng2,j3-ng3,ib)=              &
                                              facevarx(1,j1,j2,j3,jb)
#else
                       zx123(j1-ng1,j2-ng2,j3-ng3,ib)=unk(6,j1,j2,j3,jb)
#endif
                  enddo
               enddo
            enddo
#ifdef PARAMESH
               endif
              enddo
#endif

            iret=NF90_PUT_VAR(ifile,15,zx123,start=istart,count=icount)
!        7.7       X2-magnetic field
#ifdef PARAMESH
               ib=0
                  do jb=1,lnblocks
                     if(nodetype(jb).eq.1) then
                        ib=ib+1
#endif
            do j3=nc3l,nc3r
               do j2=nc2l,nc2r
                  do j1=nc1l,nc1r
#ifdef staggered
                        zx1h3(j1-ng1,j2-ng2,j3-ng3,ib)=              &
                                                facevary(1,j1,j2,j3,jb)
#else
                        zx123(j1-ng1,j2-ng2,j3-ng3,ib)=              &
                                                unk(7,j1,j2,j3,jb)
#endif
                  enddo
               enddo
            enddo
#ifdef PARAMESH
              endif
             enddo
#endif
            iret=NF90_PUT_VAR(ifile,16,zx123,start=istart,count=icount)
!        7.8       X3-magnetic field
#ifdef PARAMESH
               ib=0
               do jb=1,lnblocks
                 if(nodetype(jb).eq.1) then
                    ib=ib+1
#endif
            do j3=nc3l,nc3r
               do j2=nc2l,nc2r
                  do j1=nc1l,nc1r
#ifdef staggered
                        zx12h(j1-ng1,j2-ng2,j3-ng3,ib)=              &
                                               facevarz(1,j1,j2,j3,jb)
#else
                        zx123(j1-ng1,j2-ng2,j3-ng3,ib)=              &
                                                    unk(8,j1,j2,j3,jb)
#endif
                  enddo
               enddo
            enddo
#ifdef PARAMESH
              endif
             enddo
#endif
            iret=NF90_PUT_VAR(ifile,17,zx123,start=istart,count=icount)
            id=17
            ivar=8
#endif
#ifdef hydcon
!        7.9       Unknown
            id=id+1
            ivar=ivar+1
#ifdef PARAMESH
               ib=0
               do jb=1,lnblocks
                  if(nodetype(jb).eq.1) then
                     ib=ib+1
#endif
            do j3=nc3l,nc3r
               do j2=nc2l,nc2r
                  do j1=nc1l,nc1r
                     zx123(j1-ng1,j2-ng2,j3-ng3,ib)=                 &
                                               unk(ivar,j1,j2,j3,jb)
                  enddo
               enddo
            enddo
#ifdef PARAMESH
             endif
           enddo
#endif
         iret=NF90_PUT_VAR(ifile,id,zx123,start=istart,count=icount)
#endif
#ifdef mhdcon
!        7.10      Unknown
         id=id+1
         ivar=ivar+1
#ifdef PARAMESH
            ib=0
               do jb=1,lnblocks
                  if(nodetype(jb).eq.1) then
                     ib=ib+1
#endif
         do j3=nc3l,nc3r
            do j2=nc2l,nc2r
               do j1=nc1l,nc1r
                 zx123(j1-ng1,j2-ng2,j3-ng3,ib)=unk(ivar,j1,j2,j3,jb)
               enddo
            enddo
         enddo
#ifdef PARAMESH
          endif
        enddo
#endif
      iret=NF90_PUT_VAR(ifile,id,zx123,start=istart,count=icount)
#endif
  endif
!---------------------------------------------------------------------
!      8.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!      9.        Return
#ifdef DIAGNO
    call EXPERT(3003,0)
#endif                                                                  
RETURN
END SUBROUTINE OUTVAL
