!****f* e0pro/main
! NAME 
!   RUNCAL
! PURPOSE
!   Step on the calculation
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNCAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comtva
  USE comglo
  USE combnd
  USE commet
  USE comevo
  USE comout
!---------------------------------------------------------------------
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
    INTEGER  j1,j2,j3,lb
    INTEGER  iopt,iempty,nlayers
#endif
#ifdef MPI
    include 'mpif.h'
    INTEGER ierr
#endif
#ifdef DIAGNO
    call EXPERT(4000,1)
#endif
!---------------------------------------------------------------------
!      1.        Update time
  nstep=nstep+1
  time=time+dt
  if(nlxn) then
    dtmom=dt/xmass
  else
    dtmom=dt
  endif
!DT coef
  dth=zhalf*dt
  dtq=zhalf*dth
  dthmom=zhalf*dtmom
#ifdef mhd
    dtmag=dt/zmu
    dtene=dt/zmu
    if(nlxn) then
      dtmag=dtmag/xmass
    endif
    dthmag=zhalf*dtmag
    dthene=zhalf*dtene
#endif
!---------------------------------------------------------------------
!      2.        Numerical grid
#ifdef PARAMESH
!        2.1       AMR solution
    mype=0
    iopt=1
    iempty=0
    nlayers=nguard
#ifndef ADVANCE_ALL_LEVELS
      call AMR_RESTRICT(mype,iopt,iempty)
#endif
    if(mod(nstep,2).eq.1) then
      if(lrefine_max.gt.lrefine_min) then
!test refinement levels
        call AMR_TEST_REFINEMENT(mype,lrefine_max,lrefine_min)
!refine grid and apply morton reordering to grid blocks if necessary
        call AMR_REFINE_DEREFINE
!       call SHMEM_BARRIER_ALL()
        call GRID
        call GRID1
        call GRID2
        call GRID3
!prolong solution to any new leaf blocks if necessary
        call AMR_PROLONG(mype,iopt,nlayers)
      endif
    endif
#endif
!---------------------------------------------------------------------
!      3.        Transport processes
#ifdef heat
      call HEAT1
#endif
#ifdef resis
#ifdef PARAMESH
      call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
      call BNDMPI
#endif
    if((myproc.eq.0).and.nltbc) then
#ifdef ICOMM
      call MASTIM(2)
#else
      call BNDRDR(2)
      call BNDMAP
#endif
      if(nbndin.eq.0) call BNDIN0
      if(nbndin.eq.1) call BNDIN1
      if(nbndin.eq.2) call BNDIN2
      call BND1
    else
      call BND1
    endif
#ifdef x2dim
      call BND2
#endif
#ifdef x3dim
      call BND3
#endif
!        3.1       Curl of B
    call CURLB
!        3.2       Anomaleous resisitivity
    call ANORES
!        3.3       Update MHD variables
    call MHDRES
#endif
!---------------------------------------------------------------------
!      4.        Dynamics-directional splitting
#ifdef split
!        4.1       Save values for div(B) treatment
#ifdef toth
      if((myproc.eq.0).and.nltbc) then
#ifdef ICOMM
        call MASTIM(2)
#else
        call BNDRDR(2)
        call BNDMAP
#endif
        if(nbndin.eq.0) call BNDIN0
        if(nbndin.eq.1) call BNDIN1
        if(nbndin.eq.2) call BNDIN2
      endif
      call BND1
      call BND2
      call BND3
      call TOTH1
#endif
!        4.2       Space splitting 123
    if(nsplit.eq.1) then
      if(nc1.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
          call BNDMPI
#endif
        if((myproc.eq.0).and.nltbc) then
#ifdef ICOMM
          call MASTIM(2)
#else
          call BNDRDR(2)
          call BNDMAP
#endif
          if(nbndin.eq.0) call BNDIN0
          if(nbndin.eq.1) call BNDIN1
          if(nbndin.eq.2) call BNDIN2
        endif
        call BND1
        call MHD1
      endif
      if(nc2.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND2
        call MHD2
      endif
      if(nc3.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND3
        call MHD3
      endif
      nsplit=2
!        4.3       Space splitting 321
    else
      if(nc3.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND3
        call MHD3
      endif
      if(nc2.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND2
        call MHD2
      endif
      if(nc1.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
          call BNDMPI
#endif
        if((myproc.eq.0).and.nltbc) then
#ifdef ICOMM
          call MASTIM(2)
#else
          call BNDRDR(2)
          call BNDMAP
#endif
          if(nbndin.eq.0) call BNDIN0
          if(nbndin.eq.1) call BNDIN1
          if(nbndin.eq.2) call BNDIN2
        endif
        call BND1
        call MHD1
      endif
      nsplit=1
    endif
!        4.4       Div(B) treatment
#ifdef toth
#ifdef PARAMESH
        call AMR_GUARDCELL(mype,iopt,nlayers)
#else
#ifdef MPI
          call BNDMPI
#endif
        call BND1
#ifdef x2dim
          call BND2
#endif
#ifdef x3dim
          call BND3
#endif
#endif
      call TOTH2
#endif
#endif
!---------------------------------------------------------------------
!      4.        Dynamics-multi-dimensional
#ifndef split
#ifdef PARAMESH
      call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
      call BNDMPI
#endif
    if((myproc.eq.0).and.nltbc) then
#ifdef ICOMM
      call MASTIM(2)
#else
      call BNDRDR(2)
      call BNDMAP
#endif
      if(nbndin.eq.0) call BNDIN0
      if(nbndin.eq.1) call BNDIN1
      if(nbndin.eq.2) call BNDIN2
    endif
    call BND1
#ifdef x2dim
      call BND2
#endif
#ifdef x3dim
      call BND3
#endif
#if staggered
!        4.1       Staggered grid
      call MHS
#else
!        4.2       Cell-centered grid
      call MHC
#endif
#endif
!---------------------------------------------------------------------
!      5.        Div(B) treatment
#ifdef powell
!        5.1       Powell correction
#ifdef PARAMESH
      call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
      call BNDMPI
#endif
    call BND1
#ifdef x2dim
      call BND2
#endif
#ifdef x3dim
      call BND3
#endif
    call POWELL
#endif
#ifdef janhunen
!        5.1       Janhunen correction
#ifdef PARAMESH
      call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
      call BNDMPI
#endif
    call BND1
#ifdef x2dim
      call BND2
#endif
#ifdef x3dim
      call BND3
#endif
    call JANHUNEN
#endif
#ifdef difb
!        5.1       Diffusion
#ifdef PARAMESH
      call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
      call BNDMPI
#endif
    call BND1
#ifdef x2dim
      call BND2
#endif
#ifdef x3dim
      call BND3
#endif
    call DIFMAG
#endif
!---------------------------------------------------------------------
!      6.        Examination and diagnostics
!        6.1       Save values in significant positions
#ifdef MPI
    call MPI_BARRIER(MPI_COMM_WORLD,ierr)
#endif
  if((testep.ne.zero).or.(nestep.ne.0)) then
    if(nhel.ne.0) call HELVAL
    if(nxgeo.gt.0) call GEOVAL
    if(nsel.ne.0) call SELVAL
    if(npla.ne.0) call PLAVAL
!   if(nfld.ne.0) call FLDVAL
  endif
!        6.2       Save diagnostic values
  if(nldiav) call DIAVAL
!        6.3       Save passage values
  if((tpstep.ne.zero).or.(npstep.ne.0)) call PASVAL
!---------------------------------------------------------------------
!      7.        New time step value
!Skip if not required
  if(mod(nstep,2).eq.0) then
    if(nltimc) then
      if(akcfl.ne.zero) then
        call TIMSTP
      endif
    endif
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4000,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RUNCAL


!****f* e0pro/main
! NAME 
!   TIMSTP
! PURPOSE
!   Timestep control
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE TIMSTP
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comphy
  USE commet
  IMPLICIT NONE
#ifdef MPI
    include 'mpif.h'
    INTEGER  ierr
    REAL(KIND=8)  zcflmx
#endif
  REAL(KIND=8)  zamp,zakdt
  INTEGER  jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
    INTEGER  j1,j2,j3,lb
    INTEGER  iopt,nlayers
#endif
#ifdef DIAGNO
    call EXPERT(4002,1)
#endif
!---------------------------------------------------------------------
!      1.        Choose the most restrictive value of DTSTEP
  nlimdt=0
!        1.1       Maximum CFL number
  cflmax=zero
#ifdef PARAMESH
    do jb=1,lnblocks
      if(nodetype(jb).eq.1) then
#else
   jb=1
#endif
#ifdef resis
    cflmx(jb)=dt*max(cfl1mx(jb),cfl2mx(jb),cfl3mx(jb),cflres(jb))
#else
    cflmx(jb)=dt*max(cfl1mx(jb),cfl2mx(jb),cfl3mx(jb))
#endif
#ifdef DIAGNO
  write(*,'(4x,2(i4),2x,a4,3(1pe11.3))') myproc,jb,'cfl',  &
    dt*cfl1mx(jb),dt*cfl2mx(jb),dt*cfl3mx(jb)
  write(*,'(4x,2(i4),2x,a4,3(1pe11.3))') myproc,jb,'dt ',  &
    abs(akcfl)/cfl1mx(jb),abs(akcfl)/cfl2mx(jb),abs(akcfl)/cfl3mx(jb)
#endif
  cflmax=max(cflmax,cflmx(jb))
#ifdef PARAMESH
      endif
    enddo
#endif
!        1.2       Maximum CFL number
#ifdef MPI
    zcflmx=cflmax
!Wait until all procs come at this point
    call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    call MPI_ALLREDUCE(zcflmx,cflmax,1,MPI_DOUBLE_PRECISION,  &
      MPI_MAX,MPI_COMM_WORLD,ierr)
#endif
!---------------------------------------------------------------------
!      2.        Choose the most restrictive value of DTSTEP
!        2.1       Amplification factor
  zamp=abs(akcfl)/cflmax
!        2.2       Limits on DTSTEP amplification
  if(zamp.gt.akdt) then
    zamp=akdt
    nlimdt=1
  else
    zakdt=zone/akdt
    if(zamp.lt.zakdt) then
      zamp=zakdt
      nlimdt=2
    endif
  endif
!        2.3       Form DTSTEP
  dt=zamp*dt
!        2.4       Check maximum/minimum limits
  if(dt.gt.dtmax) then
    dt=dtmax
    nlimdt=3
  else
    if(dt.lt.dtmin) then
      dt=dtmin
      nlimdt=4
    endif
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4002,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE TIMSTP


!****f* e0pro/main
! NAME 
!   PASVAL
! PURPOSE
!   Save passage valeus in time
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE PASVAL
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comreg
  USE comglo
  USE combnd
  IMPLICIT NONE
  INTEGER  iplus,jx1,jx2,jx3
#ifdef DIAGNO
    call EXPERT(4005,1)
#endif
!---------------------------------------------------------------------
!      1.        Save values
  iplus=nxfrom-1
#ifdef bctim2c
    do jx1=1,ng1
      x1p(jx1)=xc1(jx1+iplus,1)
    enddo
#else
    x1p=xc1(1+iplus,1)
#endif
  do jx2=1,nc2
    x2p(jx2)=xc2(jx2+iplus,1)
  enddo
  do jx3=1,nc3
    x3p(jx3)=xc3(jx3+iplus,1)
  enddo
!        1.1       X1-X2-X3 mesh
! do jx1=1,ng1
!   ix1=jx1+iplus
!   do jx2=1,nc2
!     ix2=jx2+ng2
!     do jx3=1,nc3
!       ix3=jx3+ng3
!       unkpv(:,jx1,jx2,jx3,:)=unk(:,ix1,ix2,ix3,:)
!     enddo
!   enddo
! enddo
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4005,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE PASVAL


!****f* e0pro/main
! NAME 
!   RUNTST
! PURPOSE
!   Test for completion of run
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNTST
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comout
  IMPLICIT NONE
#ifdef DIAGNO
    call EXPERT(4006,1)
#endif
!---------------------------------------------------------------------
!      1.        Check physical time
  if(tstop.gt.tstart) then
    if(time.ge.tstop) nlend=.true.
    if((nstep.ge.nstop).and.(nstop.gt.0)) nlend=.true.
  else
    if(nstop.gt.0) then
      if(nstep.ge.nstop) nlend=.true.
    else
      if(myproc.eq.0) then
        write(*,'(a)') '*** Not specified termination of the run'
        write(*,'(a)') '*** Error in subprogram RUNTST'
      endif
      nlend=.true.
      nlstop=.true.
    endif
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4006,0)
#endif                                                                  
RETURN
END SUBROUTINE RUNTST

!****f* e4cal/heat1
! NAME 
!   HEAT1
! PURPOSE
!   Volumetric heating
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE HEAT1
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE commet
  USE comadm
  USE comtva
  IMPLICIT NONE
  REAL(KIND=8)  zthe, zrad
#ifdef mhd
  REAL(KIND=8) zmag
#endif
#ifndef ethe
  REAL(KIND=8)  zkin
#endif
  INTEGER  j1,j2,j3,jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
  call EXPERT(6001,1)
#endif
!---------------------------------------------------------------------
!      1.        Auxiliary values
#ifdef PARAMESH
!loop over leaf grid blocks
  do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
    if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!        2.1       Parameters
    cfl1mx(jb)=zero
!        2.2       Add the heating to the energy
    do j2=nc2l,nc2r
      do j3=nc3l,nc3r
        do j1=nc1l,nc1r
!        6.2       Temperature
#ifndef ethe
      zkin=zhalf*(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb) &
                 +unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb) &
                 +unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))
#endif
#ifdef mhd
      zmag=zr2mu*(unk(6,j1,j2,j3,jb)*unk(6,j1,j2,j3,jb) &
                 +unk(7,j1,j2,j3,jb)*unk(7,j1,j2,j3,jb) &
                 +unk(8,j1,j2,j3,jb)*unk(8,j1,j2,j3,jb))
#endif
#ifdef etot
#ifdef mhd
      zthe=unk(2,j1,j2,j3,jb)-zkin-zmag
#else
      zthe=unk(2,j1,j2,j3,jb)-zkin
#endif
#else
      zthe=unk(2,j1,j2,j3,jb)
#endif
      if(nheat.eq.0) zrad=1d0
      if(nheat.eq.1) zrad=(xc1(1,jb)/xc1(j1,jb))
      if(nheat.eq.2) zrad=(xc1(1,jb)/xc1(j1,jb))**2
      if(nheat.eq.3) zrad=(xc1(1,jb)/xc1(j1,jb))**3
      unk(2,j1,j2,j3,jb)=unk(2,j1,j2,j3,jb)+qheat*zthe*zrad/float(ncc1)
          enddo
        enddo
      enddo
#ifdef PARAMESH
   enddo
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6001,0)
#endif                       
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE HEAT1                          
