!****f* e0pro/main
! NAME 
!   HELVAL
! PURPOSE
!   Save evolution at heliospheric points
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE HELVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
  REAL(KIND=8)  zuthe
#ifndef ethe
  REAL(KIND=8)  zukin
#endif
#ifdef etot
#ifdef mhd
  REAL(KIND=8)  zumag
#endif
#endif
  INTEGER  jhel,ihel,i1,i2,i3,ib,iv
#ifdef MPI
    include 'mpif.h'
    INTEGER ierr, ic, ixhel, jproc
    INTEGER, DIMENSION(MPI_STATUS_SIZE) :: status
    INTEGER, PARAMETER :: ibuf=maxhel*(3+nnvar)
    REAL(KIND=8) zrecv(ibuf), zsend(ibuf)
#endif
#ifdef DIAGNO
    call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!      1.        Save values
  do jhel=1,nxhel
#ifdef MPI
      ihel=nnhel(myproc+1)+jhel
#else
      ihel=jhel
#endif
!        1.1       X1-X2-X3 mesh
    i1=n1hel(ihel)
    i2=n2hel(ihel)
    i3=n3hel(ihel)
    ib=1
    dhel(ihel)=unk(1,i1,i2,i3,ib)
#ifdef ethe
      zuthe=unk(2,i1,i2,i3,ib)
#else
      zukin=zhalf*(unk(3,i1,i2,i3,ib)*unk(3,i1,i2,i3,ib)  &
                  +unk(4,i1,i2,i3,ib)*unk(4,i1,i2,i3,ib)  &
                  +unk(5,i1,i2,i3,ib)*unk(5,i1,i2,i3,ib)) &
                  /unk(1,i1,i2,i3,ib)
#ifdef etot
#ifdef mhd
          zumag=zr2mu*(unk(6,i1,i2,i3,ib)*unk(6,i1,i2,i3,ib)  &
                      +unk(7,i1,i2,i3,ib)*unk(7,i1,i2,i3,ib)  &
                      +unk(8,i1,i2,i3,ib)*unk(8,i1,i2,i3,ib))
          zuthe=unk(2,i1,i2,i3,ib)-zukin-zumag
#else
          zuthe=unk(2,i1,i2,i3,ib)-zukin
#endif
#else
        zuthe=unk(2,i1,i2,i3,ib)-zukin
#endif
#endif
#ifdef gam3d
      thel(ihel)=((gamma(i1,i2,i3,ib)-zone)/zgas)*zuthe/ &
        unk(1,i1,i2,i3,ib)
#else
      thel(ihel)=((gamma-zone)/zgas)*zuthe/unk(1,i1,i2,i3,ib)
#endif
    v1hel(ihel)=unk(3,i1,i2,i3,ib)/unk(1,i1,i2,i3,ib)
    v2hel(ihel)=unk(4,i1,i2,i3,ib)/unk(1,i1,i2,i3,ib)
    v3hel(ihel)=unk(5,i1,i2,i3,ib)/unk(1,i1,i2,i3,ib)
#ifdef mhd
      b1hel(ihel)=unk(6,i1,i2,i3,ib)
      b2hel(ihel)=unk(7,i1,i2,i3,ib)
      b3hel(ihel)=unk(8,i1,i2,i3,ib)
      iv=8
#else
      iv=5
#endif
#ifdef hydcon
      iv=iv+1
      dphel(ihel)=unk(iv,i1,i2,i3,ib)
#endif
#ifdef mhdcon
      iv=iv+1
      bphel(ihel)=unk(iv,i1,i2,i3,ib)
#endif
#ifdef mhd
      if(nldiav) then
        e1hel(ihel)=-v2hel(ihel)*b3hel(ihel)+v3hel(ihel)*b2hel(ihel)
        e2hel(ihel)=-v3hel(ihel)*b1hel(ihel)+v1hel(ihel)*b3hel(ihel)
        e3hel(ihel)=-v1hel(ihel)*b2hel(ihel)+v2hel(ihel)*b1hel(ihel)
!       xj1hel(ihel)=b1a(i1,i2,i3)
!       xj2hel(ihel)=b2a(i1,i2,i3)
!       xj3hel(ihel)=b3a(i1,i2,i3)
      endif
#endif
  enddo
!---------------------------------------------------------------------
!      8.        Copy to the last processor
#ifdef MPI
    if(nxhel.gt.0) then
      ic=0
      do jhel=1,nxhel
        ihel=nnhel(myproc+1)+jhel
        zsend(ic+01)=x1hel(ihel)
        zsend(ic+02)=x2hel(ihel)
        zsend(ic+03)=x3hel(ihel)
        zsend(ic+04)=dhel(ihel)
        zsend(ic+05)=thel(ihel)
        zsend(ic+06)=v1hel(ihel)
        zsend(ic+07)=v2hel(ihel)
        zsend(ic+08)=v3hel(ihel)
#ifdef mhd
          zsend(ic+9)=b1hel(ihel)
          zsend(ic+10)=b2hel(ihel)
          zsend(ic+11)=b3hel(ihel)
#ifdef hydcon
            zsend(ic+12)=dphel(ihel)
#ifdef mhdcon
              zsend(ic+13)=bphel(ihel)
              ic=ic+13
#else
              ic=ic+12
#endif
#else
#ifdef mhdcon
              zsend(ic+12)=bphel(ihel)
              ic=ic+12
#else
              ic=11
#endif
#endif
#else
#ifdef hydcon
            zsend(ic+9)=dphel(ihel)
            ic=ic+9
#else
            ic=8
#endif
#endif
      enddo
      call MPI_SEND(zsend,ibuf,MPI_DOUBLE_PRECISION,0,  &
        510,MPI_COMM_WORLD,ierr)
!     call MPI_GATHER(zsend,ic,MPI_DOUBLE_PRECISION, &
!                     zrecv,ic,MPI_DOUBLE_PRECISION, &
!                     0,MPI_COMM_WORLD,ierr)
    endif
!   call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    if(myproc.eq.0) then
      do jproc=2,nprocs
        if(nnxhel(jproc).gt.0) then
          call MPI_RECV(zrecv,ibuf,MPI_DOUBLE_PRECISION,jproc-1,  &
            510,MPI_COMM_WORLD,status,ierr)
          ic=0
          do jhel=1,nnxhel(jproc)
            ihel=nnhel(jproc)+jhel
            x1hel(ihel)=zrecv(ic+01)
            x2hel(ihel)=zrecv(ic+02)
            x3hel(ihel)=zrecv(ic+03)
            dhel(ihel)=zrecv(ic+04)
            thel(ihel)=zrecv(ic+05)
            v1hel(ihel)=zrecv(ic+06)
            v2hel(ihel)=zrecv(ic+07)
            v3hel(ihel)=zrecv(ic+08)
#ifdef mhd
              b1hel(ihel)=zrecv(ic+9)
              b2hel(ihel)=zrecv(ic+10)
              b3hel(ihel)=zrecv(ic+11)
#ifdef hydcon
                dphel(ihel)=zrecv(ic+12)
#ifdef mhdcon
                  bphel(ihel)=zrecv(ic+13)
                  ic=ic+13
#else
                  ic=ic+12
#endif
#else
#ifdef mhdcon
                  bphel(ihel)=zrecv(ic+12)
                  ic=ic+12
#else
                  ic=11
#endif
#endif
#else
#ifdef hydcon
                dphel(ihel)=zrecv(ic+9)
                ic=ic+9
#else
                ic=8
#endif
#endif
          enddo
        endif
      enddo
    endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE HELVAL


!****f* e0pro/main
! NAME 
!   DIAVAL
! PURPOSE
!   Save diagnostic values in time
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE DIAVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  IMPLICIT NONE
  INTEGER  j1,j2,j3,jb
#ifdef DIAGNO
    call EXPERT(4004,1)
#endif
!---------------------------------------------------------------------
!      1.        Clear values
  totmas=zero
  enetot=zero
  enethe=zero
  enekin=zero
#ifdef mhd
    enemag=zero
#endif
!---------------------------------------------------------------------
!      2.        Save values
!        2.1       X1-X2-X3 mesh
#ifdef PARAMESH
    do jb=1,lnblocks
    if(nodetype(jb).eq.1) then
#else
    jb=1
#endif
      do j3=1,nc3
        do j2=1,nc2
          do j1=1,nc1
            totmas=totmas+unk(1,j1,j2,j3,jb)
            enekin=enekin+(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb) &
                          +unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb) &
                          +unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))/ &
                          unk(1,j1,j2,j3,jb)
#ifdef mhd
              enemag=enemag+unk(6,j1,j2,j3,jb)*unk(6,j1,j2,j3,jb) &
                           +unk(7,j1,j2,j3,jb)*unk(7,j1,j2,j3,jb) &
                           +unk(8,j1,j2,j3,jb)*unk(8,j1,j2,j3,jb)
#endif
#ifdef ethe
              enethe=enethe+unk(2,j1,j2,j3,jb)
#else
              enetot=enetot+unk(2,j1,j2,j3,jb)
#endif
          enddo
        enddo
      enddo
#ifdef PARAMESH
    endif
  enddo
#endif
!---------------------------------------------------------------------
!      3.        Adjust
  enekin=zhalf*enekin
  if(nlxn) then
    totmas=totmas*xmass
    enekin=enekin*xmass
  endif
#ifdef mhd
    enemag=zr2mu*enemag
#endif
#ifdef etot
    enethe=enetot-enekin-enemag
#else
    enetot=enethe+enekin+enemag
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4004,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE DIAVAL


!****f* e0pro/main
! NAME 
!   SELVAL
! PURPOSE
!   Save evolution at points along Sun-Earth line
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE SELVAL
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
#ifndef ethe
    REAL(KIND=8)  zukin,zumag,zuthe
#endif
  REAL(KIND=8)  incl, nn, ll, gg, lambda, omega
  REAL(KIND=8)  zdate, bo, bocos, bosin
  REAL(KIND=8)  zmm, zmp, zpm, zpp, zzz
  INTEGER  jsel,isel,i1,i2m,i2p,i3m,i3p,ib,iv
  INTEGER  i2sel
#ifdef x2dim
  INTEGER  j2
#endif
#ifdef MPI
    include 'mpif.h'
    INTEGER ierr, ic, ixsel, jproc
    INTEGER, DIMENSION(MPI_STATUS_SIZE) :: status
    INTEGER, PARAMETER :: ibuf=maxsel*(3+nnvar)
    REAL(KIND=8) zrecv(ibuf), zsend(ibuf)
#endif
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!      1.        Compute the heliographic latitude
  if(refdate.gt.0d0) then
!Modified Julian date
    zdate=refdate+time/86400d0
!Inclination of the rotation axis to the ecliptic
    incl=7.25d0
!Number of days from J2000.0 (starts on 1 January 2000 at 12:00 UT)
    nn=zdate-51544.5d0
!Mean longitude of Sun, corrected for aberration
    ll=280.472d0+0.9856474d0*nn
!Mean anomaly
    gg=357.528d0+0.9856003d0*nn
    gg=gg*cpi/180d0
!Ecliptic longitude
    lambda=ll+1.915d0*sin(gg)+0.020d0*sin(2d0*gg)
!Longitude of the ascending node
    omega=73d0+40d0/60d0+50.25d0*(zdate+3243.75d0)/365.25d0/3600d0
!Heliographic latitude
    Bosin=sin((lambda-omega)*cpi/180d0)*sin(incl*cpi/180d0)
    Bocos=sqrt(1d0-Bosin**2)
    Bo=0.5d0*cpi-atan(Bosin/Bocos)
  else
    Bo=0.5d0*cpi
  endif
!---------------------------------------------------------------------
!      2.        Positions of Sun-Earth-line points
#ifdef x2dim
    i2sel=nc2l
    do j2=nc2l,nc2r-1
      if(xc2(j2,1).le.Bo) i2sel=j2
    enddo
#else
    i2sel=1
#endif
!---------------------------------------------------------------------
!      3.        Save values
  i2m=1
  i3m=1
  ib=1
  do jsel=1,nxsel
#ifdef MPI
      isel=nnsel(myproc+1)+jsel
#else
      isel=jsel
#endif
    x2sel(isel)=Bo
!        3.1       X1-X2-X3 mesh
    i1=n1sel(isel)
    i2m=i2sel
    i3m=n3sel(isel)
    i2p=i2m
    i3p=i3m
#ifdef x2dim
#ifdef x3dim
        if(i2m.lt.nc2r-1) i2p=i2m+1
        if(i3m.lt.nc3r-1) i3p=i3m+1
        zzz=(xc2(i2p,ib)-xc2(i2m,ib))*(xc3(i3p,ib)-xc3(i3m,ib))
        zmm=(xc2(i2p,ib)-x2sel(isel))*(xc3(i3p,ib)-x3sel(isel))/zzz
        zmp=(xc2(i2p,ib)-x2sel(isel))*(x3sel(isel)-xc3(i3m,ib))/zzz
        zpm=(x2sel(isel)-xc2(i2m,ib))*(xc3(i3p,ib)-x3sel(isel))/zzz
        zpp=(x2sel(isel)-xc2(i2m,ib))*(x3sel(isel)-xc3(i3m,ib))/zzz
#else
        if(i2m.lt.nc2r-1) i2p=i2m+1
        zzz=xc2(i2p,ib)-xc2(i2m,ib)
        zmm=0.5d0*(xc2(i2p,ib)-x2sel(isel))/zzz
        zmp=0.5d0*(xc2(i2p,ib)-x2sel(isel))/zzz
        zpm=0.5d0*(x2sel(isel)-xc2(i2m,ib))/zzz
        zpp=0.5d0*(x2sel(isel)-xc2(i2m,ib))/zzz
#endif
#else
#ifdef x3dim
        if(i3m.lt.nc3r-1) i3p=i3m+1
        zzz=xc3(i3p,ib)-xc3(i3m,ib)
        zmm=0.5d0*(xc3(i3p,ib)-x3sel(isel))/zzz
        zmp=0.5d0*(x3sel(isel)-xc3(i3m,ib))/zzz
        zpm=0.5d0*(xc3(i3p,ib)-x3sel(isel))/zzz
        zpp=0.5d0*(x3sel(isel)-xc3(i3m,ib))/zzz
#else
        zmm=0.25d0
        zmp=0.25d0
        zpm=0.25d0
        zpp=0.25d0
#endif
#endif
    dsel(isel)=zmm*unk(1,i1,i2m,i3m,ib)+zmp*unk(1,i1,i2m,i3p,ib) &
              +zpm*unk(1,i1,i2p,i3m,ib)+zpp*unk(1,i1,i2p,i3p,ib)
    v1sel(isel)=zmm*(unk(3,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
               +zmp*(unk(3,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
               +zpm*(unk(3,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
               +zpp*(unk(3,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
    v2sel(isel)=zmm*(unk(4,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
               +zmp*(unk(4,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
               +zpm*(unk(4,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
               +zpp*(unk(4,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
    v3sel(isel)=zmm*(unk(5,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
               +zmp*(unk(5,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
               +zpm*(unk(5,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
               +zpp*(unk(5,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
#ifdef mhd
      b1sel(isel)=zmm*unk(6,i1,i2m,i3m,ib)+zmp*unk(6,i1,i2m,i3p,ib) &
                 +zpm*unk(6,i1,i2p,i3m,ib)+zpp*unk(6,i1,i2p,i3p,ib)
      b2sel(isel)=zmm*unk(7,i1,i2m,i3m,ib)+zmp*unk(7,i1,i2m,i3p,ib) &
                 +zpm*unk(7,i1,i2p,i3m,ib)+zpp*unk(7,i1,i2p,i3p,ib)
      b3sel(isel)=zmm*unk(8,i1,i2m,i3m,ib)+zmp*unk(8,i1,i2m,i3p,ib) &
                 +zpm*unk(8,i1,i2p,i3m,ib)+zpp*unk(8,i1,i2p,i3p,ib)
#endif
#ifdef etot
#ifdef mhd
        zukin=zhalf*(dsel(isel)*v1sel(isel)*v1sel(isel)  &
                    +dsel(isel)*v2sel(isel)*v2sel(isel)  &
                    +dsel(isel)*v3sel(isel)*v3sel(isel))
        zumag=zr2mu*(b1sel(isel)*b1sel(isel)+b2sel(isel)*b2sel(isel) &
          +b3sel(isel)*b3sel(isel))
        zuthe=zmm*unk(2,i1,i2m,i3m,ib) &
             +zmp*unk(2,i1,i2m,i3p,ib) &
             +zpm*unk(2,i1,i2p,i3m,ib) &
             +zpp*unk(2,i1,i2p,i3p,ib) &
             -zukin-zumag
#ifdef gam3d
          tsel(isel)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)*zuthe/dsel(isel)
#else
          tsel(isel)=((gamma-zone)/zgas)*zuthe/dsel(isel)
#endif
#else
        zukin=zhalf*(dsel(isel)*v1sel(isel)*v1sel(isel)  &
                    +dsel(isel)*v2sel(isel)*v2sel(isel)  &
                    +dsel(isel)*v3sel(isel)*v3sel(isel))
        zuthe=zmm*unk(2,i1,i2m,i3m,ib) &
             +zmp*unk(2,i1,i2m,i3p,ib) &
             +zpm*unk(2,i1,i2p,i3m,ib) &
             +zpp*unk(2,i1,i2p,i3p,ib) &
             -zukin
#ifdef gam3d
          tsel(isel)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)*zuthe/dsel(isel)
#else
          tsel(isel)=((gamma-zone)/zgas)*zuthe/dsel(isel)
#endif
#endif
#else
#ifdef gam3d
        tsel(isel)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)* &
            (zmm*(unk(2,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
            +zmp*(unk(2,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
            +zpm*(unk(2,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
            +zpp*(unk(2,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib)))
#else
        tsel(isel)=((gamma-zone)/zgas)* &
            (zmm*(unk(2,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
            +zmp*(unk(2,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
            +zpm*(unk(2,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
            +zpp*(unk(2,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib)))
#endif
#endif
#ifdef mhd
      iv=8
#else
      iv=5
#endif
#ifdef hydcon
      iv=iv+1
      dpsel(isel)=zmm*unk(iv,i1,i2m,i3m,ib)+zmp*unk(iv,i1,i2m,i3p,ib) &
                 +zpm*unk(iv,i1,i2p,i3m,ib)+zpp*unk(iv,i1,i2p,i3p,ib)
#endif
#ifdef mhdcon
      iv=iv+1
      bpsel(isel)=zmm*unk(iv,i1,i2m,i3m,ib)+zmp*unk(iv,i1,i2m,i3p,ib) &
                 +zpm*unk(iv,i1,i2p,i3m,ib)+zpp*unk(iv,i1,i2p,i3p,ib)
#endif
  enddo
!---------------------------------------------------------------------
!      8.        Copy to the last processor
#ifdef MPI
    if(nxsel.gt.0) then
      ic=0
      do jsel=1,nxsel
        isel=nnsel(myproc+1)+jsel
        zsend(ic+1)=x1sel(isel)
        zsend(ic+2)=x2sel(isel)
        zsend(ic+3)=x3sel(isel)
        zsend(ic+4)=dsel(isel)
        zsend(ic+5)=tsel(isel)
        zsend(ic+6)=v1sel(isel)
        zsend(ic+7)=v2sel(isel)
        zsend(ic+8)=v3sel(isel)
#ifdef mhd
          zsend(ic+9)=b1sel(isel)
          zsend(ic+10)=b2sel(isel)
          zsend(ic+11)=b3sel(isel)
#ifdef hydcon
            zsend(ic+12)=dpsel(isel)
#ifdef mhdcon
              zsend(ic+13)=bpsel(isel)
              ic=ic+13
#else
              ic=ic+12
#endif
#else
#ifdef mhdcon
              zsend(ic+12)=bpsel(isel)
              ic=ic+12
#else
              ic=11
#endif
#endif
#else
#ifdef hydcon
            zsend(ic+9)=dpsel(isel)
            ic=ic+9
#else
            ic=8
#endif
#endif
      enddo
      call MPI_SEND(zsend,ibuf,MPI_DOUBLE_PRECISION,0,  &
        520,MPI_COMM_WORLD,ierr)
!     call MPI_GATHER(zsend,ic,MPI_DOUBLE_PRECISION, &
!                     zrecv,ic,MPI_DOUBLE_PRECISION, &
!                     0,MPI_COMM_WORLD,ierr)
    endif
!   call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    if(myproc.eq.0) then
      do jproc=2,nprocs
        if(nnxsel(jproc).gt.0) then
          call MPI_RECV(zrecv,ibuf,MPI_DOUBLE_PRECISION,jproc-1,  &
            520,MPI_COMM_WORLD,status,ierr)
          ic=0
          do jsel=1,nnxsel(jproc)
            isel=nnsel(jproc)+jsel
            x1sel(isel)=zrecv(ic+1)
            x2sel(isel)=zrecv(ic+2)
            x3sel(isel)=zrecv(ic+3)
            dsel(isel)=zrecv(ic+4)
            tsel(isel)=zrecv(ic+5)
            v1sel(isel)=zrecv(ic+6)
            v2sel(isel)=zrecv(ic+7)
            v3sel(isel)=zrecv(ic+8)
#ifdef mhd
              b1sel(isel)=zrecv(ic+9)
              b2sel(isel)=zrecv(ic+10)
              b3sel(isel)=zrecv(ic+11)
#ifdef hydcon
                dpsel(isel)=zrecv(ic+12)
#ifdef mhdcon
                  bpsel(isel)=zrecv(ic+13)
                  ic=ic+13
#else
                  ic=ic+12
#endif
#else
#ifdef mhdcon
                  bpsel(isel)=zrecv(ic+12)
                  ic=ic+12
#else
                  ic=11
#endif
#endif
#else
#ifdef hydcon
                dpsel(isel)=zrecv(ic+9)
                ic=ic+9
#else
                ic=8
#endif
#endif
          enddo
        endif
      enddo
    endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE SELVAL


!****f* e0pro/main
! NAME 
!   GEOVAL
! PURPOSE
!   Save evolution at geospace points
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE GEOVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
#ifndef ethe
    REAL(KIND=8)  zukin,zumag,zuthe
#endif
  REAL(KIND=8)  incl, nn, ll, gg, lambda, omega
  REAL(KIND=8)  zdate, bo, bocos, bosin
  REAL(KIND=8)  zmm, zmp, zpm, zpp, zzz
  INTEGER  i2m,i2p,i3m,i3p
  INTEGER  js,i1,i2,i3,ib,iv
  INTEGER  i2geo
#ifdef x2dim
  INTEGER  j2
#endif
#ifdef DIAGNO
    call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!      1.        Compute the heliographic latitude
  if(refdate.gt.0d0) then
!Modified Julian date
    zdate=refdate+time/86400d0
!Inclination of the rotation axis to the ecliptic
    incl=7.25d0
!Number of days from J2000.0 (starts on 1 January 2000 at 12:00 UT)
    nn=zdate-51544.5d0
!Mean longitude of Sun, corrected for aberration
    ll=280.472d0+0.9856474d0*nn
!Mean anomaly
    gg=357.528d0+0.9856003d0*nn
    gg=gg*cpi/180d0
!Ecliptic longitude
    lambda=ll+1.915d0*sin(gg)+0.020d0*sin(2d0*gg)
!Longitude of the ascending node
    omega=73d0+40d0/60d0+50.25d0*(zdate+3243.75d0)/365.25d0/3600d0
!Heliographic latitude
    Bosin=sin((lambda-omega)*cpi/180d0)*sin(incl*cpi/180d0)
    Bocos=sqrt(1d0-Bosin**2)
    Bo=0.5d0*cpi-atan(Bosin/Bocos)
  else
    Bo=0.5d0*cpi
  endif
!---------------------------------------------------------------------
!      2.        Positions of Geospace points
  x2geo(1)=Bo
#ifdef x2dim
    i2geo=nc2l
    do j2=nc2l,nc2r-1
      if(xc2(j2,1).le.Bo) i2geo=j2
    enddo
#else
    i2geo=1
#endif
  do js=2,5
    if((js.eq.2).or.(js.eq.4)) then
      n2geo(js)=i2geo
      x2geo(js)=xc2(i2geo,1)
    else
      n2geo(js)=i2geo+1
      x2geo(js)=xc2(i2geo+1,1)
    endif
  enddo
!---------------------------------------------------------------------
!      3.        Save values
  i2m=1
  i3m=1
  ib=1
!        3.1       X1-X2-X3 mesh
  i1=n1geo(1)
  i2m=i2geo
  i3m=n3geo(1)
  i2p=i2m
  i3p=i3m
#ifdef x2dim
#ifdef x3dim
      if(i2m.lt.nc2r-1) i2p=i2m+1
      if(i3m.lt.nc3r-1) i3p=i3m+1
      zzz=(xc2(i2p,ib)-xc2(i2m,ib))*(xc3(i3p,ib)-xc3(i3m,ib))
      zmm=(xc2(i2p,ib)-x2geo(1))*(xc3(i3p,ib)-x3geo(1))/zzz
      zmp=(xc2(i2p,ib)-x2geo(1))*(x3geo(1)-xc3(i3m,ib))/zzz
      zpm=(x2geo(1)-xc2(i2m,ib))*(xc3(i3p,ib)-x3geo(1))/zzz
      zpp=(x2geo(1)-xc2(i2m,ib))*(x3geo(1)-xc3(i3m,ib))/zzz
#else
      if(i2m.lt.nc2r-1) i2p=i2m+1
      zzz=xc2(i2p,ib)-xc2(i2m,ib)
      zmm=0.5d0*(xc2(i2p,ib)-x2geo(1))/zzz
      zmp=0.5d0*(xc2(i2p,ib)-x2geo(1))/zzz
      zpm=0.5d0*(x2geo(1)-xc2(i2p,ib))/zzz
      zpp=0.5d0*(x2geo(1)-xc2(i2p,ib))/zzz
#endif
#else
#ifdef x3dim
      if(i3m.lt.nc3r-1) i3p=i3m+1
      zzz=xc3(i3p,ib)-xc3(i3m,ib)
      zmm=0.5d0*(xc3(i3p,ib)-x3geo(1))/zzz
      zmp=0.5d0*(x3geo(1)-xc3(i3m,ib))/zzz
      zpm=0.5d0*(xc3(i3p,ib)-x3geo(1))/zzz
      zpp=0.5d0*(x3geo(1)-xc3(i3p,ib))/zzz
#else
      zmm=0.25d0
      zmp=0.25d0
      zpm=0.25d0
      zpp=0.25d0
#endif
#endif
  dgeo(1)=zmm*unk(1,i1,i2m,i3m,ib)+zmp*unk(1,i1,i2m,i3p,ib) &
         +zpm*unk(1,i1,i2p,i3m,ib)+zpp*unk(1,i1,i2p,i3p,ib)
  v1geo(1)=zmm*(unk(3,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
          +zmp*(unk(3,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
          +zpm*(unk(3,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
          +zpp*(unk(3,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
  v2geo(1)=zmm*(unk(4,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
          +zmp*(unk(4,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
          +zpm*(unk(4,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
          +zpp*(unk(4,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
  v3geo(1)=zmm*(unk(5,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
          +zmp*(unk(5,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
          +zpm*(unk(5,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
          +zpp*(unk(5,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
#ifdef mhd
    b1geo(1)=zmm*unk(6,i1,i2m,i3m,ib)+zmp*unk(6,i1,i2m,i3p,ib) &
            +zpm*unk(6,i1,i2p,i3m,ib)+zpp*unk(6,i1,i2p,i3p,ib)
    b2geo(1)=zmm*unk(7,i1,i2m,i3m,ib)+zmp*unk(7,i1,i2m,i3p,ib) &
            +zpm*unk(7,i1,i2p,i3m,ib)+zpp*unk(7,i1,i2p,i3p,ib)
    b3geo(1)=zmm*unk(8,i1,i2m,i3m,ib)+zmp*unk(8,i1,i2m,i3p,ib) &
            +zpm*unk(8,i1,i2p,i3m,ib)+zpp*unk(8,i1,i2p,i3p,ib)
#endif
#ifdef etot
#ifdef mhd
      zukin=zhalf*(dgeo(1)*v1geo(1)*v1geo(1)  &
                  +dgeo(1)*v2geo(1)*v2geo(1)  &
                  +dgeo(1)*v3geo(1)*v3geo(1))
      zumag=zr2mu*(b1geo(1)*b1geo(1)+b2geo(1)*b2geo(1)+b3geo(1)*b3geo(1))
      zuthe=zmm*unk(2,i1,i2m,i3m,ib) &
           +zmp*unk(2,i1,i2m,i3p,ib) &
           +zpm*unk(2,i1,i2p,i3m,ib) &
           +zpp*unk(2,i1,i2p,i3p,ib) &
           -zukin-zumag
#ifdef gam3d
        tgeo(1)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)*zuthe/dgeo(1)
#else
        tgeo(1)=((gamma-zone)/zgas)*zuthe/dgeo(1)
#endif
#else
      zukin=zhalf*(dgeo(1)*v1geo(1)*v1geo(1)  &
                  +dgeo(1)*v2geo(1)*v2geo(1)  &
                  +dgeo(1)*v3geo(1)*v3geo(1))
      zuthe=zmm*unk(2,i1,i2m,i3m,ib) &
           +zmp*unk(2,i1,i2m,i3p,ib) &
           +zpm*unk(2,i1,i2p,i3m,ib) &
           +zpp*unk(2,i1,i2p,i3p,ib) &
           -zukin
#ifdef gam3d
        tgeo(1)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)*zuthe/dgeo(1)
#else
        tgeo(1)=((gamma-zone)/zgas)*zuthe/dgeo(1)
#endif
#endif
#else
#ifdef gam3d
      tgeo(1)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)* &
          (zmm*(unk(2,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
          +zmp*(unk(2,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
          +zpm*(unk(2,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
          +zpp*(unk(2,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib)))
#else
      tgeo(1)=((gamma-zone)/zgas)* &
          (zmm*(unk(2,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
          +zmp*(unk(2,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
          +zpm*(unk(2,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
          +zpp*(unk(2,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib)))
#endif
#endif
#ifdef mhd
    iv=8
#else
    iv=5
#endif
#ifdef hydcon
    iv=iv+1
    dpgeo(1)=zmm*unk(iv,i1,i2m,i3m,ib)+zmp*unk(iv,i1,i2m,i3p,ib) &
            +zpm*unk(iv,i1,i2p,i3m,ib)+zpp*unk(iv,i1,i2p,i3p,ib)
#endif
#ifdef mhdcon
    iv=iv+1
    bpgeo(1)=zmm*unk(iv,i1,i2m,i3m,ib)+zmp*unk(iv,i1,i2m,i3p,ib) &
            +zpm*unk(iv,i1,i2p,i3m,ib)+zpp*unk(iv,i1,i2p,i3p,ib)
#endif
!---------------------------------------------------------------------
!      4.        Save values
  do js=2,5
!        4.1       X1-X2-X3 mesh
    i1=n1geo(js)
    i2=n2geo(js)
    i3=n3geo(js)
    ib=1
    dgeo(js)=unk(1,i1,i2,i3,ib)
#ifdef etot
#ifdef mhd
        zukin=zhalf*(unk(3,i1,i2,i3,ib)*unk(3,i1,i2,i3,ib)  &
                    +unk(4,i1,i2,i3,ib)*unk(4,i1,i2,i3,ib)  &
                    +unk(5,i1,i2,i3,ib)*unk(5,i1,i2,i3,ib)) &
                    /unk(1,i1,i2,i3,ib)
        zumag=zr2mu*(unk(6,i1,i2,i3,ib)*unk(6,i1,i2,i3,ib)  &
                    +unk(7,i1,i2,i3,ib)*unk(7,i1,i2,i3,ib)  &
                    +unk(8,i1,i2,i3,ib)*unk(8,i1,i2,i3,ib))
        zuthe=unk(2,i1,i2,i3,ib)-zukin-zumag
#ifdef gam3d
          tgeo(js)=((gamma(i1,i2,i3,ib)-zone)/zgas)*zuthe/ &
            unk(1,i1,i2,i3,ib)
#else
          tgeo(js)=((gamma-zone)/zgas)*zuthe/unk(1,i1,i2,i3,ib)
#endif
#else
        zukin=zhalf*(unk(3,i1,i2,i3,ib)*unk(3,i1,i2,i3,ib)  &
                    +unk(4,i1,i2,i3,ib)*unk(4,i1,i2,i3,ib)  &
                    +unk(5,i1,i2,i3,ib)*unk(5,i1,i2,i3,ib)) &
                    /unk(1,i1,i2,i3,ib)
        zuthe=unk(2,i1,i2,i3,ib)-zukin
#ifdef gam3d
          tgeo(js)=((gamma(i1,i2,i3,ib)-zone)/zgas)*zuthe/ &
            unk(1,i1,i2,i3,ib)
#else
          tgeo(js)=((gamma-zone)/zgas)*zuthe/unk(1,i1,i2,i3,ib)
#endif
#endif
#else
#ifdef gam3d
        tgeo(js)=((gamma(i1,i2,i3,ib)-zone)/zgas)*unk(2,i1,i2,i3,ib)/ &
          unk(1,i1,i2,i3,ib)
#else
        tgeo(js)=((gamma-zone)/zgas)*unk(2,i1,i2,i3,ib)/unk(1,i1,i2,i3,ib)
#endif
#endif
    v1geo(js)=unk(3,i1,i2,i3,ib)/unk(1,i1,i2,i3,ib)
    v2geo(js)=unk(4,i1,i2,i3,ib)/unk(1,i1,i2,i3,ib)
    v3geo(js)=unk(5,i1,i2,i3,ib)/unk(1,i1,i2,i3,ib)
#ifdef mhd
      b1geo(js)=unk(6,i1,i2,i3,ib)
      b2geo(js)=unk(7,i1,i2,i3,ib)
      b3geo(js)=unk(8,i1,i2,i3,ib)
      iv=8
#else
      iv=5
#endif
#ifdef hydcon
      iv=iv+1
      dpgeo(js)=unk(iv,i1,i2,i3,ib)
#endif
#ifdef mhdcon
      iv=iv+1
      bpgeo(js)=unk(iv,i1,i2,i3,ib)
#endif
#ifdef mhd
      if(nldiav) then
        e1geo(js)=-v2geo(js)*b3geo(js)+v3geo(js)*b2geo(js)
        e2geo(js)=-v3geo(js)*b1geo(js)+v1geo(js)*b3geo(js)
        e3geo(js)=-v1geo(js)*b2geo(js)+v2geo(js)*b1geo(js)
!       xj1geo(js)=b1a(i1,i2,i3)
!       xj2geo(js)=b2a(i1,i2,i3)
!       xj3geo(js)=b3a(i1,i2,i3)
      endif
#endif
  enddo
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE GEOVAL


!****f* e0pro/main
! NAME 
!   PLAVAL
! PURPOSE
!   Save evolution at planetary positions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE PLAVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
#ifndef ethe
    REAL(KIND=8)  zukin,zumag,zuthe
#endif
  REAL(KIND=8)  mjd0, zdate, zx, x3ear
  REAL(KIND=8)  zmm, zmp, zpm, zpp, zzz
  INTEGER  i1pla, i2pla, i3pla
  INTEGER  i1, i2m, i2p, i3m, i3p, iv, ib, ion
  INTEGER  j1, jp
#ifdef x2dim
  INTEGER  j2
#endif
#ifdef x3dim
  INTEGER  j3
#endif
#ifdef MPI
    include 'mpif.h'
    INTEGER ierr, ic, ipla, jproc
    INTEGER, DIMENSION(MPI_STATUS_SIZE) :: status
    INTEGER, PARAMETER :: ibuf=maxhel*(3+nnvar)
    REAL(KIND=8) zrecv(ibuf), zsend(ibuf)
#endif
!---------------------------------------------------------------------
!      1.        Compute the heliographic latitude
!------------------  Initialize common  ----------------------
  REAL(KIND=8)  z0(9), z1(9), z2(9), z3(9), z4(9), z5(9), z6(9), z7(9), z8(9)
  DATA Z0/0.071422,0.027962,0.017202,0.009146,0.00145, 0.000584,0.000205,0.000104,0.000069/
  DATA Z1/3.8484,  3.02812, 1.74022, 4.51234, 4.53364, 4.89884, 2.46615, 3.78556, 3.16948/
  DATA Z2/0.388301,0.013195,0.032044,0.175301,0.090478,0.105558,0.088593,0.016965,0.471239/
  DATA Z3/1.34041, 2.28638, 1.78547, 5.85209, 0.23911, 1.61094, 2.96706, 0.773181,3.91303/
  DATA Z4/0.3871,  0.7233,  1.,      1.5237,  5.2028,  9.5385,  19.182,  30.06,   39.44/
  DATA Z5/0.07974, 0.00506, 0.017,   0.141704,0.249374,0.534156,0.901554,0.27054, 9.86/
  DATA Z6/2.73514, 3.85017, 3.33929, 1.04656, 1.76188, 3.1257,  4.49084, 2.33498, 5.23114/
  DATA Z7/0.122173,0.059341,0.,      0.03142, 0.01972, 0.043633,0.01396, 0.031416,0.300197/
  DATA Z8/0.836013,1.33168, 0.,      0.858702,1.74533, 1.977458,1.28805, 2.29162, 1.91812/
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!      1.        Compute the heliographic latitude
! jd0=2436935d0           ! Julian Date of 1-1-1960
!Modified Julian date
  mjd0=36934.5d0
! pi2 = 2*!PI			; 2 pi.
! nl = N_elements(list)	  	; number of planets to process.
!-----------------  Days since Epoch  ---------------
! frac=hms(0)/24.+hms(1)/1440.+hms(2)/86400.
! nd=jd-jd0+frac
  zdate=refdate+time/86400d0-mjd0
!-----------------  Save Earth azimuth  -------------
  zx=zdate*z0(3)+z1(3)
  if(zx.gt.360d0) zx=zx-360d0
  if(zx.gt.360d0) zx=zx-360d0
  if(zx.gt.360d0) zx=zx-360d0
  if(zx.gt.360d0) zx=zx-360d0
  if(zx.gt.360d0) zx=zx-360d0
  zx=(cpi/180d0)*zx
  zx=zx+z2(3)*sin(zx-z3(3))
! zx=zx mod pi2
  x3ear=zx
!-----------------  Loop thru planets  --------------
  do jp=1,npla
    zx=zdate*z0(jp)+z1(jp)
    if(zx.gt.360d0) zx=zx-360d0
    if(zx.gt.360d0) zx=zx-360d0
    if(zx.gt.360d0) zx=zx-360d0
    if(zx.gt.360d0) zx=zx-360d0
    if(zx.gt.360d0) zx=zx-360d0
    zx=(cpi/180d0)*zx
    zx=zx+z2(jp)*sin(zx-z3(jp))
!   zx=zx mod pi2
!Heliocentric distance (AU)
    x1pla(jp)=1.496d11*(z4(jp)+z5(jp)*sin(zx-z6(jp)))
!Heliocentric latitude (radian)
    x2pla(jp)=z7(jp)*sin(zx-z8(jp))
!Heliocentric longitude (radian)
    zx=cpi+zx-x3ear
    if(zx.lt.0d0) zx=zx+2d0*cpi
    if(zx.gt.2d0*cpi) zx=zx-2d0*cpi
    x3pla(jp)=zx
  enddo
!---------------------------------------------------------------------
!      1.        Positions of Sun-Earth-line points
  do jp=1,npla
    ion=0
    if(myproc.eq.0) then
      if((x1pla(jp).ge.xh1(nc1l,1)).and.(x1pla(jp).le.xh1(nc1r+1,1))) ion=jp
    else
      if((x1pla(jp).gt.xh1(nc1l,1)).and.(x1pla(jp).le.xh1(nc1r+1,1))) ion=jp
    endif
!---------------------------------------------------------------------
!      1.        Positions of planets
    if(ion.gt.0) then
!        1.1       Radial position
      i1pla=nc1l
      do j1=nc1l,nc1r-1
        if(xc1(j1,1).le.x1pla(jp)) i1pla=j1
      enddo
!        1.2       Meridional position
#ifdef x2dim
        i2pla=nc2l
        do j2=nc2l,nc2r-1
          if(xc2(j2,1).le.x2pla(jp)) i2pla=j2
        enddo
#else
        i2pla=1
#endif
!        1.3       Azimuthal position
#ifdef x3dim
        i3pla=nc3l
        do j3=nc3l,nc3r-1
          if(xc3(j3,1).le.x3pla(jp)) i3pla=j3
        enddo
#else
        i3pla=1
#endif
!---------------------------------------------------------------------
!      2.        Save values
      i2m=1
      i3m=1
      ib=1
!        2.1       X1-X2-X3 mesh
      i1=i1pla
      i2m=i2pla
      i3m=i3pla
      i2p=i2m
      i3p=i3m
#ifdef x2dim
#ifdef x3dim
          if(i2m.lt.nc2r-1) then
            i2p=i2m+1
          else
            i2p=nc2l
          endif
          if(i3m.lt.nc3r-1) then
            i3p=i3m+1
          else
            i3p=nc3l
          endif
          zzz=(xc2(i2p,ib)-xc2(i2m,ib))*(xc3(i3p,ib)-xc3(i3m,ib))
          zmm=(xc2(i2p,ib)-x2pla(jp))*(xc3(i3p,ib)-x3pla(jp))/zzz
          zmp=(xc2(i2p,ib)-x2pla(jp))*(x3pla(jp)-xc3(i3m,ib))/zzz
          zpm=(x2pla(jp)-xc2(i2m,ib))*(xc3(i3p,ib)-x3pla(jp))/zzz
          zpp=(x2pla(jp)-xc2(i2m,ib))*(x3pla(jp)-xc3(i3m,ib))/zzz
#else
          if(i2m.lt.nc2r-1) then
            i2p=i2m+1
          else
            i2p=nc2l
          endif
          zzz=xc2(i2p,ib)-xc2(i2m,ib)
          zmm=0.5d0*(xc2(i2p,ib)-x2pla(jp))/zzz
          zmp=0.5d0*(xc2(i2p,ib)-x2pla(jp))/zzz
          zpm=0.5d0*(x2pla(jp)-xc2(i2m,ib))/zzz
          zpp=0.5d0*(x2pla(jp)-xc2(i2m,ib))/zzz
#endif
#else
#ifdef x3dim
          if(i3m.lt.nc3r-1) then
            i3p=i3m+1
          else
            i3p=nc3l
          endif
          zzz=xc3(i3p,ib)-xc3(i3m,ib)
          zmm=0.5d0*(xc3(i3p,ib)-x3pla(jp))/zzz
          zmp=0.5d0*(x3pla(jp)-xc3(i3m,ib))/zzz
          zpm=0.5d0*(xc3(i3p,ib)-x3pla(jp))/zzz
          zpp=0.5d0*(x3pla(jp)-xc3(i3m,ib))/zzz
#else
          zmm=0.25d0
          zmp=0.25d0
          zpm=0.25d0
          zpp=0.25d0
#endif
#endif
      dpla(jp)=zmm*unk(1,i1,i2m,i3m,ib)+zmp*unk(1,i1,i2m,i3p,ib) &
              +zpm*unk(1,i1,i2p,i3m,ib)+zpp*unk(1,i1,i2p,i3p,ib)
      v1pla(jp)=zmm*(unk(3,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
               +zmp*(unk(3,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
               +zpm*(unk(3,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
               +zpp*(unk(3,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
      v2pla(jp)=zmm*(unk(4,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
               +zmp*(unk(4,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
               +zpm*(unk(4,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
               +zpp*(unk(4,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
      v3pla(jp)=zmm*(unk(5,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
               +zmp*(unk(5,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
               +zpm*(unk(5,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
               +zpp*(unk(5,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib))
#ifdef mhd
        b1pla(jp)=zmm*unk(6,i1,i2m,i3m,ib)+zmp*unk(6,i1,i2m,i3p,ib) &
                 +zpm*unk(6,i1,i2p,i3m,ib)+zpp*unk(6,i1,i2p,i3p,ib)
        b2pla(jp)=zmm*unk(7,i1,i2m,i3m,ib)+zmp*unk(7,i1,i2m,i3p,ib) &
                 +zpm*unk(7,i1,i2p,i3m,ib)+zpp*unk(7,i1,i2p,i3p,ib)
        b3pla(jp)=zmm*unk(8,i1,i2m,i3m,ib)+zmp*unk(8,i1,i2m,i3p,ib) &
                 +zpm*unk(8,i1,i2p,i3m,ib)+zpp*unk(8,i1,i2p,i3p,ib)
#endif
#ifdef etot
#ifdef mhd
          zukin=zhalf*(dpla(jp)*v1pla(jp)*v1pla(jp)  &
                      +dpla(jp)*v2pla(jp)*v2pla(jp)  &
                      +dpla(jp)*v3pla(jp)*v3pla(jp))
          zumag=zr2mu*(b1pla(jp)*b1pla(jp)+b2pla(jp)*b2pla(jp)+b3pla(jp)*b3pla(jp))
          zuthe=zmm*unk(2,i1,i2m,i3m,ib) &
                +zmp*unk(2,i1,i2m,i3p,ib) &
                +zpm*unk(2,i1,i2p,i3m,ib) &
                +zpp*unk(2,i1,i2p,i3p,ib) &
                -zukin-zumag
#ifdef gam3d
            tpla(jp)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)*zuthe/dpla(jp)
#else
            tpla(jp)=((gamma-zone)/zgas)*zuthe/dpla(jp)
#endif
#else
          zukin=zhalf*(dpla(jp)*v1pla(jp)*v1pla(jp)  &
                      +dpla(jp)*v2pla(jp)*v2pla(jp)  &
                      +dpla(jp)*v3pla(jp)*v3pla(jp))
          zuthe=zmm*unk(2,i1,i2m,i3m,ib) &
               +zmp*unk(2,i1,i2m,i3p,ib) &
               +zpm*unk(2,i1,i2p,i3m,ib) &
               +zpp*unk(2,i1,i2p,i3p,ib) &
               -zukin
#ifdef gam3d
            tpla(jp)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)*zuthe/dpla(jp)
#else
            tpla(jp)=((gamma-zone)/zgas)*zuthe/dpla(jp)
#endif
#endif
#else
#ifdef gam3d
          tpla(jp)=((gamma(i1,i2m,i3m,ib)-zone)/zgas)* &
            (zmm*(unk(2,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
            +zmp*(unk(2,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
            +zpm*(unk(2,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
            +zpp*(unk(2,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib)))
#else
          tpla(jp)=((gamma-zone)/zgas)* &
            (zmm*(unk(2,i1,i2m,i3m,ib)/unk(1,i1,i2m,i3m,ib)) &
            +zmp*(unk(2,i1,i2m,i3p,ib)/unk(1,i1,i2m,i3p,ib)) &
            +zpm*(unk(2,i1,i2p,i3m,ib)/unk(1,i1,i2p,i3m,ib)) &
            +zpp*(unk(2,i1,i2p,i3p,ib)/unk(1,i1,i2p,i3p,ib)))
#endif
#endif
#ifdef mhd
        iv=8
#else
        iv=5
#endif
#ifdef hydcon
        iv=iv+1
        dppla(jp)=zmm*unk(iv,i1,i2m,i3m,ib)+zmp*unk(iv,i1,i2m,i3p,ib) &
                 +zpm*unk(iv,i1,i2p,i3m,ib)+zpp*unk(iv,i1,i2p,i3p,ib)
#endif
#ifdef mhdcon
        iv=iv+1
        bppla(jp)=zmm*unk(iv,i1,i2m,i3m,ib)+zmp*unk(iv,i1,i2m,i3p,ib) &
                 +zpm*unk(iv,i1,i2p,i3m,ib)+zpp*unk(iv,i1,i2p,i3p,ib)
#endif
!---------------------------------------------------------------------
    else
      x1pla(jp)=0d0
    endif
  enddo
!---------------------------------------------------------------------
!      8.        Copy to the last processor
#ifdef MPI
    if(myproc.gt.0) then
      ic=-1
      do jp=1,npla
        ic=ic+1
        zsend(ic+1)=x1pla(jp)
        zsend(ic+2)=x2pla(jp)
        zsend(ic+3)=x3pla(jp)
        zsend(ic+4)=dpla(jp)
        zsend(ic+5)=tpla(jp)
        zsend(ic+6)=v1pla(jp)
        zsend(ic+7)=v2pla(jp)
        zsend(ic+8)=v3pla(jp)
#ifdef mhd
          zsend(ic+9)=b1pla(ipla)
          zsend(ic+10)=b2pla(ipla)
          zsend(ic+11)=b3pla(ipla)
#ifdef hydcon
            zsend(ic+12)=dppla(ipla)
#ifdef mhdcon
              zsend(ic+13)=bppla(ipla)
              ic=ic+13
#else
              ic=ic+12
#endif
#else
#ifdef mhdcon
              zsend(ic+12)=bppla(ipla)
              ic=ic+12
#else
              ic=ic+11
#endif
#endif
#else
#ifdef hydcon
            zsend(ic+9)=dppla(ipla)
            ic=ic+9
#else
            ic=ic+8
#endif
#endif
      enddo
      call MPI_SEND(zsend,ibuf,MPI_DOUBLE_PRECISION,0,  &
                    520,MPI_COMM_WORLD,ierr)
!     call MPI_GATHER(zsend,ic,MPI_DOUBLE_PRECISION, &
!                     zrecv,ic,MPI_DOUBLE_PRECISION, &
!                     0,MPI_COMM_WORLD,ierr)
!     call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    endif
!---------------------------------------------------------------------
!      9.        Return or stop
    if(myproc.eq.0) then
      do jproc=2,nprocs
        call MPI_RECV(zrecv,ibuf,MPI_DOUBLE_PRECISION,jproc-1,  &
          520,MPI_COMM_WORLD,status,ierr)
        ic=-1
        do jp=1,npla
          ic=ic+1
          if(zrecv(ic).gt.0d0) then
            x1pla(jp)=zrecv(ic+1)
            x2pla(jp)=zrecv(ic+2)
            x3pla(jp)=zrecv(ic+3)
            dpla(jp)=zrecv(ic+4)
            tpla(jp)=zrecv(ic+5)
            v1pla(jp)=zrecv(ic+6)
            v2pla(jp)=zrecv(ic+7)
            v3pla(jp)=zrecv(ic+8)
#ifdef mhd
              b1pla(jp)=zrecv(ic+9)
              b2pla(jp)=zrecv(ic+10)
              b3pla(jp)=zrecv(ic+11)
#ifdef hydcon
                dppla(jp)=zrecv(ic+12)
#ifdef mhdcon
                  bppla(jp)=zrecv(ic+13)
                  ic=ic+13
#else
                  ic=ic+12
#endif
#else
#ifdef mhdcon
                  bppla(jp)=zrecv(ic+12)
                  ic=ic+12
#else
                  ic=ic+11
#endif
#endif
#else
#ifdef hydcon
                dppla(jp)=zrecv(ic+9)
                ic=ic+9
#else
                ic=ic+8
#endif
#endif
          endif
        enddo
      enddo
    endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE PLAVAL
