!****f* e0pro/main
! NAME 
!   BND2
! PURPOSE
!   X2-boundary conditions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND2 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif                                                                  
  USE comoly 
  USE comdim 
  USE commet
  USE comreg
  USE comglo
  USE comtva 
  IMPLICIT NONE
#ifdef DIAGNO                                                           
    call EXPERT(5002,1)
#endif
!---------------------------------------------------------------------
!      1.        X2-left boundary
#ifdef x2dim
#ifdef bc2l_free
!        1.1       Free flow (X1-X2-X3 array)
    unk(:,:,1,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
    unk(:,:,2,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,1,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,1,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,1,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
        facevarx(:,:,2,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,2,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,2,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2l_free0
!        1.2       Free flow (X1-X2-X3 array)
    unk(:,:,1,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
    unk(:,:,2,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
    unk(4,:,1,nc3l:nc3r,:)=zero
    unk(4,:,2,nc3l:nc3r,:)=zero
#ifdef mhd
#ifdef staggered
        facevarx(:,:,1,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,1,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,1,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
        facevarx(:,:,2,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,2,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,2,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2l_flow
!        1.3       Free flow (X1-X2-X3 array)
    unk(:,:,1,nc3l:nc3r,:)=unk(:,:,4,nc3l:nc3r,:)
    unk(:,:,2,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,1,nc3l:nc3r,:)=facevarx(:,:,4,nc3l:nc3r,:)
        facevary(:,:,1,nc3l:nc3r,:)=facevary(:,:,4,nc3l:nc3r,:)
        facevarz(:,:,1,nc3l:nc3r,:)=facevarz(:,:,4,nc3l:nc3r,:)
        facevarx(:,:,2,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,2,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,2,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2l_solid
!        1.4       Solid wall (X1-X2-X3 array)
    unk(:,:,1,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
    unk(4,:,1,nc3l:nc3r,:)=-unk(4,:,3,nc3l:nc3r,:)
    unk(:,:,2,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
    unk(4,:,2,nc3l:nc3r,:)=-unk(4,:,3,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,1,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,1,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,1,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
        facevarx(:,:,2,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,2,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,2,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2l_periodic
!        1.5       Periodic system (X1-X2-X3 array)
    unk(:,:,1,nc3l:nc3r,:)=unk(:,:,nc2-3,nc3l:nc3r,:)
    unk(:,:,2,nc3l:nc3r,:)=unk(:,:,nc2-2,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,1,nc3l:nc3r,:)=facevarx(:,:,nc2-3,nc3l:nc3r,:)
        facevary(:,:,1,nc3l:nc3r,:)=facevary(:,:,nc2-3,nc3l:nc3r,:)
        facevarz(:,:,1,nc3l:nc3r,:)=facevarz(:,:,nc2-3,nc3l:nc3r,:)
#endif
#endif
#endif
!---------------------------------------------------------------------
!      2.        X2-right boundary
#ifdef bc2r_free
!        2.1       Free flow (X1-X2-X3 array)
    unk(:,:,nc2-1,nc3l:nc3r,:)=unk(:,:,nc2-2,nc3l:nc3r,:)
    unk(:,:,nc2,nc3l:nc3r,:)=unk(:,:,nc2-2,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,nc2-1,nc3l:nc3r,:)=facevarx(:,:,nc2-2,nc3l:nc3r,:)
        facevary(:,:,nc2-1,nc3l:nc3r,:)=facevary(:,:,nc2-2,nc3l:nc3r,:)
        facevarz(:,:,nc2-1,nc3l:nc3r,:)=facevarz(:,:,nc2-2,nc3l:nc3r,:)
        facevarx(:,:,nc2,nc3l:nc3r,:)=facevarx(:,:,nc2-2,nc3l:nc3r,:)
        facevary(:,:,nc2,nc3l:nc3r,:)=facevary(:,:,nc2-2,nc3l:nc3r,:)
        facevarz(:,:,nc2,nc3l:nc3r,:)=facevarz(:,:,nc2-2,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2r_free0
!        2.2       Free flow (X1-X2-X3 array)
    unk(:,:,nc2-1,nc3l:nc3r,:)=unk(:,:,nc2-2,nc3l:nc3r,:)
    unk(:,:,nc2,nc3l:nc3r,:)=unk(:,:,nc2-2,nc3l:nc3r,:)
    unk(4,:, nc2-1,nc3l:nc3r,:)=zero
    unk(4,:, nc2,nc3l:nc3r,:)=zero
#ifdef mhd
#ifdef staggered
        facevarx(:,:,nc2-1,nc3l:nc3r,:)=facevarx(:,:,nc2-2,nc3l:nc3r,:)
        facevary(:,:,nc2-1,nc3l:nc3r,:)=facevary(:,:,nc2-2,nc3l:nc3r,:)
        facevarz(:,:,nc2-1,nc3l:nc3r,:)=facevarz(:,:,nc2-2,nc3l:nc3r,:)
        facevarx(:,:,nc2,nc3l:nc3r,:)=facevarx(:,:,nc2-2,nc3l:nc3r,:)
        facevary(:,:,nc2,nc3l:nc3r,:)=facevary(:,:,nc2-2,nc3l:nc3r,:)
        facevarz(:,:,nc2,nc3l:nc3r,:)=facevarz(:,:,nc2-2,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2r_flow
!        2.3       Free flow (X1-X2-X3 array)
    unk(:,:,nc2-1,nc3l:nc3r,:)=unk(:,:,nc2-2,nc3l:nc3r,:)
    unk(:,:,nc2,nc3l:nc3r,:)=unk(:,:,nc2-3,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,nc2-1,nc3l:nc3r,:)=facevarx(:,:,nc2-2,nc3l:nc3r,:)
        facevary(:,:,nc2-1,nc3l:nc3r,:)=facevary(:,:,nc2-2,nc3l:nc3r,:)
        facevarz(:,:,nc2-1,nc3l:nc3r,:)=facevarz(:,:,nc2-2,nc3l:nc3r,:)
        facevarx(:,:,nc2,nc3l:nc3r,:)=facevarx(:,:,nc2-3,nc3l:nc3r,:)
        facevary(:,:,nc2,nc3l:nc3r,:)=facevary(:,:,nc2-3,nc3l:nc3r,:)
        facevarz(:,:,nc2,nc3l:nc3r,:)=facevarz(:,:,nc2-3,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2r_solid
!        2.4       Solid wall (X1-X2-X3 array)
    unk(:,:,nc2-1,nc3l:nc3r,:)=unk(:,:,nc2-2,nc3l:nc3r,:)
    unk(4,:,nc2-1,nc3l:nc3r,:)=-unk(4,:,nc2-2,nc3l:nc3r,:)
    unk(:,:,nc2,nc3l:nc3r,:)=unk(:,:,nc2-3,nc3l:nc3r,:)
    unk(4,:,nc2,nc3l:nc3r,:)=- unk(4,:,nc2-3,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,nc2-1,nc3l:nc3r,:)=facevarx(:,:,nc2-2,nc3l:nc3r,:)
        facevary(:,:,nc2-1,nc3l:nc3r,:)=facevary(:,:,nc2-2,nc3l:nc3r,:)
        facevarz(:,:,nc2-1,nc3l:nc3r,:)=facevarz(:,:,nc2-2,nc3l:nc3r,:)
        facevarx(:,:,nc2,nc3l:nc3r,:)=facevarx(:,:,nc2-3,nc3l:nc3r,:)
        facevary(:,:,nc2,nc3l:nc3r,:)=facevary(:,:,nc2-3,nc3l:nc3r,:)
        facevarz(:,:,nc2,nc3l:nc3r,:)=facevarz(:,:,nc2-3,nc3l:nc3r,:)
#endif
#endif
#endif
#ifdef bc2r_periodic
!        2.5       Periodic system (X1-X2-X3 array)
    unk(:,:,nc2-1,nc3l:nc3r,:)=unk(:,:,3,nc3l:nc3r,:)
    unk(:,:,nc2,nc3l:nc3r,:)=unk(:,:,4,nc3l:nc3r,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,nc2-1,nc3l:nc3r,:)=facevarx(:,:,3,nc3l:nc3r,:)
        facevary(:,:,nc2-1,nc3l:nc3r,:)=facevary(:,:,3,nc3l:nc3r,:)
        facevarz(:,:,nc2-1,nc3l:nc3r,:)=facevarz(:,:,3,nc3l:nc3r,:)
        facevarx(:,:,nc2,nc3l:nc3r,:)=facevarx(:,:,4,nc3l:nc3r,:)
        facevary(:,:,nc2,nc3l:nc3r,:)=facevary(:,:,4,nc3l:nc3r,:)
        facevarz(:,:,nc2,nc3l:nc3r,:)=facevarz(:,:,4,nc3l:nc3r,:)
#endif
#endif
#endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5002,0) 
#endif                                                                  
RETURN
END SUBROUTINE BND2
