!****f* e0pro/main
! NAME 
!   BND3
! PURPOSE
!   X3-boundary conditions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND3 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif        
  USE comoly 
  USE comdim 
  USE commet
  USE comreg
  USE comglo
  USE comtva 
  IMPLICIT NONE
#ifdef bc3l_aperiodic
  INTEGER i2, j1, j2 
#endif
#ifdef DIAGNO                                                           
    call EXPERT(5003,0)
#endif
!---------------------------------------------------------------------
!      1.        X3-left boundary
#ifdef x3dim
#ifdef bc3l_free
!        1.1       Free flow (X1-X2-X3 array)
    unk(:,:,:,1,:)=unk(:,:,:,3,:)
    unk(:,:,:,2,:)=unk(:,:,:,3,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,:,1,:)=facevarx(:,:,:,3,:)
        facevary(:,:,:,1,:)=facevary(:,:,:,3,:)
        facevarz(:,:,:,1,:)=facevarz(:,:,:,3,:)
        facevarx(:,:,:,2,:)=facevarx(:,:,:,3,:)
        facevary(:,:,:,2,:)=facevary(:,:,:,3,:)
        facevarz(:,:,:,2,:)=facevarz(:,:,:,3,:)
#endif
#endif
#endif
#ifdef bc3l_flow
!        1.2       Free flow (X1-X2-X3 array)
    unk(:,:,:,1,:)=unk(:,:,:,4,:)
    unk(:,:,:,2,:)=unk(:,:,:,3,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,:,1,:)=facevarx(:,:,:,4,:)
        facevary(:,:,:,1,:)=facevary(:,:,:,4,:)
        facevarz(:,:,:,1,:)=facevarz(:,:,:,4,:)
        facevarx(:,:,:,2,:)=facevarx(:,:,:,3,:)
        facevary(:,:,:,2,:)=facevary(:,:,:,3,:)
        facevarz(:,:,:,2,:)=facevarz(:,:,:,3,:)
#endif
#endif
#endif
#ifdef bc3l_solid
!        1.3       Solid wall (X1-X2-X3 array)
    unk(:,:,:,1,:)=unk(:,:,:,4,:)
    unk(5,:,:,1,:)=-unk(5,:,:,4,:)
    unk(:,:,:,2,:)=unk(:,:,:,3,:)
    unk(5,:,:,2,:)=-unk(5,:,:,3,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,:,1,:)=facevarx(:,:,:,4,:)
        facevary(:,:,:,1,:)=facevary(:,:,:,4,:)
        facevarz(:,:,:,1,:)=facevarz(:,:,:,4,:)
        facevarx(:,:,:,2,:)=facevarx(:,:,:,3,:)
        facevary(:,:,:,2,:)=facevary(:,:,:,3,:)
        facevarz(:,:,:,2,:)=facevarz(:,:,:,3,:)
#endif
#endif
#endif
#ifdef bc3l_periodic
!        1.4       Periodic system (X1-X2-X3 array)
    unk(:,:,:,1,:)=unk(:,:,:,nc3-3,:)
    unk(:,:,:,2,:)=unk(:,:,:,nc3-2,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,:,1,:)=facevarx(:,:,:,nc3-3,:)
        facevary(:,:,:,1,:)=facevary(:,:,:,nc3-3,:)
        facevarz(:,:,:,1,:)=facevarz(:,:,:,nc3-3,:)
        facevarx(:,:,:,2,:)=facevarx(:,:,:,nc3-2,:)
        facevary(:,:,:,2,:)=facevary(:,:,:,nc3-2,:)
        facevarz(:,:,:,2,:)=facevarz(:,:,:,nc3-2,:)
#endif
#endif
#endif
#ifdef bc3l_aperiodic
!        1.5       Aperiodic system (X1-X2-X3 array)
    do j2=nc2l,nc2r
      i2=nc2-j2+1
      do j1=nc1l,nc1r
        unk(:,:,j2,1,:)=unk(:,:,i2,nc3-3,:)
        unk(:,:,j2,2,:)=unk(:,:,i2,nc3-2,:)
#ifdef mhd
#ifdef staggered
            facevarx(:,:,j2,1,:)=facevarx(:,:,i2,nc3-3,:)
            facevary(:,:,j2,1,:)=facevary(:,:,i2,nc3-3,:)
            facevarz(:,:,j2,1,:)=facevarz(:,:,i2,nc3-3,:)
            facevarx(:,:,j2,2,:)=facevarx(:,:,i2,nc3-2,:)
            facevary(:,:,j2,2,:)=facevary(:,:,i2,nc3-2,:)
            facevarz(:,:,j2,2,:)=facevarz(:,:,i2,nc3-2,:)
#endif
#endif
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      2.        X1-right boundary
#ifdef bc3r_free
!        2.1       Free flow (X1-X2-X3 array)
    unk(:,:,:,nc3-1,:)=unk(:,:,:,nc3-2,:)
    unk(:,:,:,nc3,:)=unk(:,:,:,nc3-2,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,:,nc3-1,:)=facevarx(:,:,:,nc3-2,:)
        facevary(:,:,:,nc3-1,:)=facevary(:,:,:,nc3-2,:)
        facevarz(:,:,:,nc3-1,:)=facevarz(:,:,:,nc3-2,:)
        facevarx(:,:,:,nc3,:)=facevarx(:,:,:,nc3-2,:)
        facevary(:,:,:,nc3,:)=facevary(:,:,:,nc3-2,:)
        facevarz(:,:,:,nc3,:)=facevarz(:,:,:,nc3-2,:)
#endif
#endif
#endif
#ifdef bc3r_flow
!        2.2       Free flow (X1-X2-X3 array)
    unk(:,:,:,nc3-1,:)=unk(:,:,:,nc3-2,:)
    unk(:,:,:,nc3,:)=unk(:,:,:,nc3-3,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,:,nc3-1,:)=facevarx(:,:,:,nc3-2,:)
        facevary(:,:,:,nc3-1,:)=facevary(:,:,:,nc3-2,:)
        facevarz(:,:,:,nc3-1,:)=facevarz(:,:,:,nc3-2,:)
        facevarx(:,:,:,nc3,:)=facevarx(:,:,:,nc3-3,:)
        facevary(:,:,:,nc3,:)=facevary(:,:,:,nc3-3,:)
        facevarz(:,:,:,nc3,:)=facevarz(:,:,:,nc3-3,:)
#endif
#endif
#endif
#ifdef bc3r_solid
!        2.3       Solid wall (X1-X2-X3 array)
    unk(:,:,:,nc3-1,:)=unk(:,:,:,nc3-2,:)
    unk(5,:,:,nc3-1,:)=-unk(5,:,:,nc3-2,:)
    unk(:,:,:,nc3,:)=unk(:,:,:,nc3-3,:)
    unk(5,:,:,nc3,:)=-unk(5,:,:,nc3-3,:)
#ifdef mhd
#ifdef staggered
        facevarx(:,:,:,nc3-1,:)=facevarx(:,:,:,nc3-2,:)
        facevary(:,:,:,nc3-1,:)=facevary(:,:,:,nc3-2,:)
        facevarz(:,:,:,nc3-1,:)=facevarz(:,:,:,nc3-2,:)
        facevarx(:,:,:,nc3,:)=facevarx(:,:,:,nc3-3,:)
        facevary(:,:,:,nc3,:)=facevary(:,:,:,nc3-3,:)
        facevarz(:,:,:,nc3,:)=facevarz(:,:,:,nc3-3,:)
#endif
#endif
#endif
#ifdef bc3r_periodic
!        2.4       Periodic system (X1-X2-X3 array)
      unk(:,:,:,nc3-1,:)=unk(:,:,:,3,:)
      unk(:,:,:,nc3,:)=unk(:,:,:,4,:)
#ifdef mhd
#ifdef staggered
          facevarx(:,:,:,nc3-1,:)=facevarx(:,:,:,3,:)
          facevary(:,:,:,nc3-1,:)=facevary(:,:,:,3,:)
          facevarz(:,:,:,nc3-1,:)=facevarz(:,:,:,3,:)
          facevarx(:,:,:,nc3,:)=facevarx(:,:,:,4,:)
          facevary(:,:,:,nc3,:)=facevary(:,:,:,4,:)
          facevarz(:,:,:,nc3,:)=facevarz(:,:,:,4,:)
#endif
#endif
#endif
#ifdef bc3r_aperiodic
!        2.5       Aperiodic system (X1-X2-X3 array)
      do j2=nc2l,nc2r
        i2=nc2-j2+1
        do j1=nc1l,nc1r
          unk(:,:,j2,nc3-1,:)=unk(:,:,i2,3,:)
          unk(:,:,j2,nc3,:)=unk(:,:,i2,4,:)
#ifdef mhd
#ifdef staggered
              facevarx(:,:,j2,nc3-1,:)=facevarx(:,:,i2,3,:)
              facevary(:,:,j2,nc3-1,:)=facevary(:,:,i2,3,:)
              facevarz(:,:,j2,nc3-1,:)=facevarz(:,:,i2,3,:)
              facevarx(:,:,j2,nc3,:)=facevarx(:,:,i2,4,:)
              facevary(:,:,j2,nc3,:)=facevary(:,:,i2,4,:)
              facevarz(:,:,j2,nc3,:)=facevarz(:,:,i2,4,:)
#endif
#endif
        enddo
      enddo
#endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5003,0) 
#endif        
RETURN
END SUBROUTINE BND3
