!****f* e0pro/main
! NAME 
!   BNDRDR
! PURPOSE
!   Read in boundary values
! INPUTS
!   kmode = 1,2 for initial, periodic action   
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDRDR(kmode) 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comout
  USE comreg
  USE combnd
  USE commet
  USE netcdf
  IMPLICIT NONE
  REAL(KIND=8)  ztime
  INTEGER  kmode, iret, ifile, j2,j3
  CHARACTER(len=nf90_max_name) :: iname
#ifdef bctim2c
  INTEGER  j1
#endif
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(5004,1)
#endif
!---------------------------------------------------------------------
!      1.        Initialize boundary values
  if(kmode.eq.1) then
!        1.1       Open the file
    iret=0
#ifdef bctim2c
      iret=NF90_OPEN(lfpas,nf90_nowrite,ifile)
#else
      iret=NF90_OPEN(lfbnd,nf90_nowrite,ifile)
#endif
!        1.2       Get number of the time levels
    iret=NF90_INQUIRE_DIMENSION(ifile,7,iname,nbndmx)
!        1.3       Read in the first time level
    iret=NF90_GET_VAR(ifile,7,ztime,start=(/1/))
    iret=NF90_CLOSE(ifile)
!        1.4       Case 1
    if(tstart.lt.ztime) then
      nbnd=1
#ifdef bctim2c
        call RESPAS
#else
        call RESBND
#endif
      nbnd=0
    endif
!        1.5       Case 2
    if(tstart.eq.ztime) then
      nbnd=1
#ifdef bctim2c
        call RESPAS
#else
        call RESBND
#endif
    endif
!        1.6       Case 3
    if((tstart.gt.ztime).and.(nbndmx.eq.1)) then
      nbnd=1
#ifdef bctim2c
        call RESPAS
#else
        call RESBND
#endif
      nbnd=0
    endif
!        1.6       Case 4
    if((tstart.gt.ztime).and.(nbndmx.gt.1)) then
#ifdef bctim2c
        iret=NF90_OPEN(lfpas,nf90_nowrite,ifile)
#else
        iret=NF90_OPEN(lfbnd,nf90_nowrite,ifile)
#endif
      nbnd=1
      do while((ztime.le.tstart).and.(nbnd.le.nbndmx))
        nbnd=nbnd+1
        iret=NF90_GET_VAR(ifile,7,ztime,start=(/nbnd/))
      enddo
      if(nbnd.gt.1) nbnd=nbnd-1
      iret=NF90_CLOSE(ifile)
#ifdef bctim2c
        call RESPAS
#else
        call RESBND
#endif
    endif
  endif
!---------------------------------------------------------------------
!      2.        Boundary values in time
  if((kmode.eq.1).or.((nbnd.lt.nbndmx).and.(time.ge.timebn))) then
!        2.1       Shift to old X1-X2-X3 values
    timebo=timebn
    do j3=1,nb3
      x3bo(j3)=x3bn(j3)
      do j2=1,nb2
        x2bo(j2)=x2bn(j2)
#ifdef bctim2c
          do j1=1,nb1
            dbo(j1,j2,j3)=dbn(j1,j2,j3)
            tbo(j1,j2,j3)=tbn(j1,j2,j3)
            v1bo(j1,j2,j3)=v1bn(j1,j2,j3)
            v2bo(j1,j2,j3)=v2bn(j1,j2,j3)
            v3bo(j1,j2,j3)=v3bn(j1,j2,j3)
#ifdef mhd
              b1bo(j1,j2,j3)=b1bn(j1,j2,j3)
              b2bo(j1,j2,j3)=b2bn(j1,j2,j3)
              b3bo(j1,j2,j3)=b3bn(j1,j2,j3)
#endif
#ifdef hydcon
              dpbo(j1,j2,j3)=dpbn(j1,j2,j3)
#endif
#ifdef mhdcon
              bpbo(j1,j2,j3)=bpbn(j1,j2,j3)
#endif
          enddo
#else
          dbo(j2,j3)=dbn(j2,j3)
          tbo(j2,j3)=tbn(j2,j3)
          v1bo(j2,j3)=v1bn(j2,j3)
          v2bo(j2,j3)=v2bn(j2,j3)
          v3bo(j2,j3)=v3bn(j2,j3)
#ifdef mhd
            b1bo(j2,j3)=b1bn(j2,j3)
            b2bo(j2,j3)=b2bn(j2,j3)
            b3bo(j2,j3)=b3bn(j2,j3)
#endif
#ifdef hydcon
            dpbo(j2,j3)=dpbn(j2,j3)
#endif
#ifdef mhdcon
            bpbo(j2,j3)=bpbn(j2,j3)
#endif
#endif
      enddo
    enddo
!        2.2       Read in the next level
    nbnd=nbnd+1
#ifdef bctim2c
      call RESPAS
#else
      call RESBND
#endif
  endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5004,0)
#endif
RETURN
END SUBROUTINE BNDRDR


!****f* e0pro/main
! NAME 
!   BNDMAP
! PURPOSE
!   Map boundary positions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDMAP
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comadm
  IMPLICIT NONE
  REAL(KIND=8)  zdif, zxmin, zxmax, zxold, zxnew
  INTEGER  ix3, ib3o, ib3n
  INTEGER  jb3, jc3
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(5005,1)
#endif
!---------------------------------------------------------------------
!      1.        Map X2
#ifdef x2dim
!    zxmin=x2bo(1)
!    zxmax=x2bo(nb2)
!    do jc2=1,ncc2
!        ix2=ng2+jc2
!        zxold=xc2(ix2,1)
!        zxnew=xc2(ix2,1)
!        if(zxold.lt.zxmin) zxold=zxmin
!        if(zxold.gt.zxmax) zxold=zxmax
!        if(zxnew.lt.zxmin) zxnew=zxmin
!        if(zxnew.gt.zxmax) zxnew=zxmax
!        ib2o=0
!        ib2n=0
!        do jb2=1,nb2
!          if(x2bo(jb2).lt.zxold) ib2o=jb2
!          if(x2bn(jb2).lt.zxnew) ib2n=jb2
!        enddo
! if((ib2o.eq.0).or.(ib2o.eq.nb2)) then
!   if(ib2o.eq.0) then
!            nx2fom(jc2)=1
!            nx2fop(jc2)=2
!            x2fom(jc2)=zone
!            x2fop(jc2)=zero
!   else
!            nx2fom(jc2)=nb2-1
!            nx2fop(jc2)=nb2
!            x2fom(jc2)=zero
!            x2fop(jc2)=zone
!   endif
! else
!          nx2fom(jc2)=ib2o
!          nx2fop(jc2)=ib2o+1
!          zdif=zxold-x2bo(ib2o)
!          x2fop(jc2)=zdif/(x2bo(ib2o+1)-x2bo(ib2o))
!          x2fom(jc2)=zone-x2fop(jc2)
!        endif
! if((ib2n.eq.0).or.(ib2n.eq.nb2)) then
!   if(ib2n.eq.0) then
!            nx2fnm(jc2)=1
!            nx2fnp(jc2)=2
!            x2fnm(jc2)=zone
!            x2fnp(jc2)=zero
!   else
!            nx2fnm(jc2)=nb2-1
!            nx2fnp(jc2)=nb2
!            x2fnm(jc2)=zero
!            x2fnp(jc2)=zone
!   endif
! else
!          nx2fnm(jc2)=ib2n
!          nx2fnp(jc2)=ib2n+1
!          x2fnp(jc2)=zdif/(x2bn(ib2n+1)-x2bn(ib2n))
!          x2fnm(jc2)=zone-x2fnp(jc2)
!        endif
!      enddo
#endif
!---------------------------------------------------------------------
!      2.        Map X3
#ifdef x3dim
    zxmin=0d0
    zxmax=360d0*vux3
    do jc3=1,ncc3
      ix3=ng3+jc3
      zxold=xc3(ix3,1)-vrot*(time-timebo)
      zxnew=xc3(ix3,1)+ vrot*(timebn-time)
      do while(zxold.lt.zxmin)
        zxold=zxmax-(zxmin-zxold)
      enddo
      do while(zxold.gt.zxmax)
        zxold=zxmin+(zxold-zxmax)
      enddo
      do while(zxnew.lt.zxmin)
        zxnew=zxmax-(zxmin-zxnew)
      enddo
      do while(zxnew.gt.zxmax)
        zxnew=zxmin+(zxnew-zxmax)
      enddo
      ib3o=0
      ib3n=0
      do jb3=1,nb3
        if(x3bo(jb3).lt.zxold) ib3o=jb3
        if(x3bn(jb3).lt.zxnew) ib3n=jb3
      enddo
      if((ib3o.eq.0).or.(ib3o.eq.nb3)) then
        nx3fom(jc3)=nb3
        nx3fop(jc3)=1
        if(ib3o.eq.0) then
          zdif=zxold-zxmin+zxmax-x3bo(nb3)
          x3fop(jc3)=zdif/(x3bo(1)-zxmin+zxmax-x3bo(nb3))
        else
          zdif=zxold-x3bo(nb3)
          x3fop(jc3)=zdif/(x3bo(1)-zxmin+zxmax-x3bo(nb3))
        endif
      else
        nx3fom(jc3)=ib3o
        nx3fop(jc3)=ib3o+1
        zdif=zxold-x3bo(ib3o)
        x3fop(jc3)=zdif/(x3bo(ib3o+1)-x3bo(ib3o))
      endif
      x3fom(jc3)=zone-x3fop(jc3)
      if((ib3n.eq.0).or.(ib3n.eq.nb3)) then
        nx3fnm(jc3)=nb3
        nx3fnp(jc3)=1
        if(ib3n.eq.0) then
          zdif=zxnew-zxmin+zxmax-x3bn(nb3)
          x3fnp(jc3)=zdif/(x3bn(1)-zxmin+zxmax-x3bn(nb3))
        else
          zdif=zxnew-x3bn(nb3)
          x3fnp(jc3)=zdif/(x3bn(1)-zxmin+zxmax-x3bn(nb3))
        endif
      else
        nx3fnm(jc3)=ib3n
        nx3fnp(jc3)=ib3n+1
        zdif=zxnew-x3bn(ib3n)
        x3fnp(jc3)=zdif/(x3bn(ib3n+1)-x3bn(ib3n))
      endif
      x3fnm(jc3)=zone-x3fnp(jc3)
    enddo
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5005,0)
#endif
RETURN
END SUBROUTINE BNDMAP


!****f* e0pro/main
! NAME 
!   BNDIN0
! PURPOSE
!   Interpolate boundary values
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDIN0
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE comadm
  IMPLICIT NONE
  REAL(KIND=8)  ztimeo, ztimen, zrgm
#ifdef bctim2c
  INTEGER  j1
#endif
  INTEGER  j2,j3
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(5006,1)
#endif
!---------------------------------------------------------------------
!      1.        Time interpolation factors
  zrgm=rgas/gammam
!        1.1       Before
  ztimeo=zone
  ztimen=zero
!        1.2       Within
  if((time.gt.timebo).and.(time.lt.timebn)) then
    ztimen=(time-timebo)/(timebn-timebo)
    ztimeo=zone-ztimen
  endif
!        1.3       After
  if(time.ge.timebn) then
    ztimeo=zero
    ztimen=zone
  endif
!---------------------------------------------------------------------
!      2.        Interpolate boundary values
!        2.1       X1-X2-X3 array
  do j3=1,ncc3
    do j2=1,ncc2
#ifndef bctim2c
        dpv(j2,j3)=ztimeo*dbo(j2,j3)+ztimen*dbn(j2,j3)
        tpv(j2,j3)=ztimeo*tbo(j2,j3)+ztimen*tbn(j2,j3)
        v1pv(j2,j3)=ztimeo*v1bo(j2,j3)+ztimen*v1bn(j2,j3)
        v2pv(j2,j3)=ztimeo*v2bo(j2,j3)+ztimen*v2bn(j2,j3)
        v3pv(j2,j3)=ztimeo*v3bo(j2,j3)+ztimen*v3bn(j2,j3)
#ifdef mhd
          b1pv(j2,j3)=ztimeo*b1bo(j2,j3)+ztimen*b1bn(j2,j3)
          b2pv(j2,j3)=ztimeo*b2bo(j2,j3)+ztimen*b2bn(j2,j3)
          b3pv(j2,j3)=ztimeo*b3bo(j2,j3)+ztimen*b3bn(j2,j3)
#endif
#ifdef hydcon
          dppv(j2,j3)=ztimeo*dpbo(j2,j3)+ztimen*dpbn(j2,j3)
#endif
#ifdef mhdcon
          bppv(j2,j3)=ztimeo*bpbo(j2,j3)+ztimen*bpbn(j2,j3)
#endif
#ifdef ethe
          uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)
#else
#ifdef etot
#ifdef mhd
              uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)       &
                +zhalf*dpv(j2,j3)*(v1pv(j2,j3)*v1pv(j2,j3) &
                                 +v2pv(j2,j3)*v2pv(j2,j3)  &
                                 +v3pv(j2,j3)*v3pv(j2,j3)) &
                +zr2mu*(b1pv(j2,j3)*b1pv(j2,j3) &
                       +b2pv(j2,j3)*b2pv(j2,j3) &
                       +b3pv(j2,j3)*b3pv(j2,j3))
#else
              uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)       &
                +zhalf*dpv(j2,j3)*(v1pv(j2,j3)*v1pv(j2,j3) &
                                  +v2pv(j2,j3)*v2pv(j2,j3) &
                                  +v3pv(j2,j3)*v3pv(j2,j3))
#endif
#else
            uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)        &
              +zhalf*dpv(j2,j3)*(v1pv(j2,j3)*v1pv(j2,j3)  &
                                +v2pv(j2,j3)*v2pv(j2,j3)  &
                                +v3pv(j2,j3)*v3pv(j2,j3))
#endif
#endif
        dv1pv(j2,j3)=dpv(j2,j3)*v1pv(j2,j3)
        dv2pv(j2,j3)=dpv(j2,j3)*v2pv(j2,j3)
        dv3pv(j2,j3)=dpv(j2,j3)*v3pv(j2,j3)
#else
        do j1=1,nb1
          dpv(j1,j2,j3)=ztimeo*dbo(j1,j2,j3)+ztimen*dbn(j1,j2,j3)
          tpv(j1,j2,j3)=ztimeo*tbo(j1,j2,j3)+ztimen*tbn(j1,j2,j3)
          v1pv(j1,j2,j3)=ztimeo*v1bo(j1,j2,j3)+ztimen*v1bn(j1,j2,j3)
          v2pv(j1,j2,j3)=ztimeo*v2bo(j1,j2,j3)+ztimen*v2bn(j1,j2,j3)
          v3pv(j1,j2,j3)=ztimeo*v3bo(j1,j2,j3)+ztimen*v3bn(j1,j2,j3)
#ifdef mhd
            b1pv(j1,j2,j3)=ztimeo*b1bo(j1,j2,j3)+ztimen*b1bn(j1,j2,j3)
            b2pv(j1,j2,j3)=ztimeo*b2bo(j1,j2,j3)+ztimen*b2bn(j1,j2,j3)
            b3pv(j1,j2,j3)=ztimeo*b3bo(j1,j2,j3)+ztimen*b3bn(j1,j2,j3)
#endif
#ifdef hydcon
            dppv(j1,j2,j3)=ztimeo*dpbo(j1,j2,j3)+ztimen*dpbn(j1,j2,j3)
#endif
#ifdef mhdcon
            bppv(j1,j2,j3)=ztimeo*bpbo(j1,j2,j3)+ztimen*bpbn(j1,j2,j3)
#endif
#ifdef ethe
            uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)
#else
#ifdef etot
#ifdef mhd
                uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)        &
                  +zhalf*dpv(j1,j2,j3)*(v1pv(j1,j2,j3)*v1pv(j1,j2,j3)  &
                                       +v2pv(j1,j2,j3)*v2pv(j1,j2,j3)  &
                                       +v3pv(j1,j2,j3)*v3pv(j1,j2,j3)) &
                  +zr2mu*(b1pv(j1,j2,j3)*b1pv(j1,j2,j3)  &
                         +b2pv(j1,j2,j3)*b2pv(j1,j2,j3)  &
                         +b3pv(j1,j2,j3)*b3pv(j1,j2,j3))
#else
                uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)        &
                  +zhalf*dpv(j1,j2,j3)*(v1pv(j1,j2,j3)*v1pv(j1,j2,j3)  &
                                       +v2pv(j1,j2,j3)*v2pv(j1,j2,j3)  &
                                       +v3pv(j1,j2,j3)*v3pv(j1,j2,j3))
#endif
#else
              uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)        &
                +zhalf*dpv(j1,j2,j3)*(v1pv(j1,j2,j3)*v1pv(j1,j2,j3)  &
                                     +v2pv(j1,j2,j3)*v2pv(j1,j2,j3)  &
                                     +v3pv(j1,j2,j3)*v3pv(j1,j2,j3))
#endif
#endif
          dv1pv(j1,j2,j3)=dpv(j1,j2,j3)*v1pv(j1,j2,j3)
          dv2pv(j1,j2,j3)=dpv(j1,j2,j3)*v2pv(j1,j2,j3)
          dv3pv(j1,j2,j3)=dpv(j1,j2,j3)*v3pv(j1,j2,j3)
        enddo
#endif
    enddo
  enddo
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5006,0)
#endif
RETURN
END SUBROUTINE BNDIN0


!****f* e0pro/main
! NAME 
!   BNDIN1
! PURPOSE
!   Interpolate boundary values
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDIN1
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE comadm
  IMPLICIT NONE
  REAL(KIND=8)  ztimeo, ztimen, zrgm
  REAL(KIND=8)  zo3m, zo3p, zn3m, zn3p
  INTEGER  io3m, io3p, in3m, in3p
#ifdef bctim2c
  INTEGER  j1
#endif
  INTEGER  j2,j3
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(5006,1)
#endif
!---------------------------------------------------------------------
!      1.        Time interpolation factors
  zrgm=rgas/gammam
!        1.1       Before
  ztimeo=zone
  ztimen=zero
!        1.2       Within
  if((time.gt.timebo).and.(time.lt.timebn)) then
    ztimen=(time-timebo)/(timebn-timebo)
    ztimeo=zone-ztimen
  endif
!        1.3       After
  if(time.ge.timebn) then
    ztimeo=zero
    ztimen=zone
  endif
!---------------------------------------------------------------------
!      2.        Interpolate boundary values
!        2.1       X1-X2-X3 array
  do j3=1,ncc3
#ifdef x3dim
      io3m=nx3fom(j3)
      io3p=nx3fop(j3)
      in3m=nx3fnm(j3)
      in3p=nx3fnp(j3)
      zo3m=x3fom(j3)
      zo3p=x3fop(j3)
      zn3m=x3fnm(j3)
      zn3p=x3fnp(j3)
#else
      io3m=1
      io3p=1
      in3m=1
      in3p=1
      zo3m=1d0
      zo3p=0d0
      zn3m=1d0
      zn3p=0d0
#endif
    zo3m=ztimeo*zo3m
    zo3p=ztimeo*zo3p
    zn3m=ztimen*zn3m
    zn3p=ztimen*zn3p
    do j2=1,ncc2
#ifndef bctim2c
        dpv(j2,j3)=zo3m*dbo(j2,io3m)+zo3p*dbo(j2,io3p)+  &
                      zn3m*dbn(j2,in3m)+zn3p*dbn(j2,in3p)
        tpv(j2,j3)=zo3m*tbo(j2,io3m)+zo3p*tbo(j2,io3p)+  &
                      zn3m*tbn(j2,in3m)+zn3p*tbn(j2,in3p)
        v1pv(j2,j3)=zo3m*v1bo(j2,io3m)+zo3p*v1bo(j2,io3p)+ &
                       zn3m*v1bn(j2,in3m)+zn3p*v1bn(j2,in3p)
        v2pv(j2,j3)=zo3m*v2bo(j2,io3m)+zo3p*v2bo(j2,io3p)+ &
                       zn3m*v2bn(j2,in3m)+zn3p*v2bn(j2,in3p)
        v3pv(j2,j3)=zo3m*v3bo(j2,io3m)+zo3p*v3bo(j2,io3p)+ &
                       zn3m*v3bn(j2,in3m)+zn3p*v3bn(j2,in3p)
!       uupv(j2,j3)=zo3m*uubo(j2,io3m)+zo3p*uubo(j2,io3p)+  &
!                      zn3m*uubn(j2,in3m)+zn3p*uubn(j2,in3p)
!       dv1pv(j2,j3)=zo3m*dv1bo(j2,io3m)+zo3p*dv1bo(j2,io3p)+ &
!                       zn3m*dv1bn(j2,in3m)+zn3p*dv1bn(j2,in3p)
!       dv2pv(j2,j3)=zo3m*dv2bo(j2,io3m)+zo3p*dv2bo(j2,io3p)+ &
!                       zn3m*dv2bn(j2,in3m)+zn3p*dv2bn(j2,in3p)
!       dv3pv(j2,j3)=zo3m*dv3bo(j2,io3m)+zo3p*dv3bo(j2,io3p)+ &
!                       zn3m*dv3bn(j2,in3m)+zn3p*dv3bn(j2,in3p)
#ifdef mhd
          b1pv(j2,j3)=zo3m*b1bo(j2,io3m)+zo3p*b1bo(j2,io3p)+  &
                         zn3m*b1bn(j2,in3m)+zn3p*b1bn(j2,in3p)
          b2pv(j2,j3)=zo3m*b2bo(j2,io3m)+zo3p*b2bo(j2,io3p)+  &
                         zn3m*b2bn(j2,in3m)+zn3p*b2bn(j2,in3p)
          b3pv(j2,j3)=zo3m*b3bo(j2,io3m)+zo3p*b3bo(j2,io3p)+  &
                         zn3m*b3bn(j2,in3m)+zn3p*b3bn(j2,in3p)
#endif
#ifdef hydcon
          dppv(j2,j3)=zo3m*dpbo(j2,io3m)+zo3p*dpbo(j2,io3p)+  &
                      zn3m*dpbn(j2,in3m)+zn3p*dpbn(j2,in3p)
#endif
#ifdef mhdcon
          bppv(j2,j3)=zo3m*bpbo(j2,io3m)+zo3p*bpbo(j2,io3p)+  &
                         zn3m*bpbn(j2,in3m)+zn3p*bpbn(j2,in3p)
#endif
#ifdef ethe
          uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)
#else
#ifdef etot
#ifdef mhd
              uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)        &
                +zhalf*dpv(j2,j3)*(v1pv(j2,j3)*v1pv(j2,j3)  &
                                  +v2pv(j2,j3)*v2pv(j2,j3)  &
                                  +v3pv(j2,j3)*v3pv(j2,j3)) &
                +zr2mu*(b1pv(j2,j3)*b1pv(j2,j3)  &
                       +b2pv(j2,j3)*b2pv(j2,j3)  &
                       +b3pv(j2,j3)*b3pv(j2,j3))
#else
              uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)        &
                +zhalf*dpv(j2,j3)*(v1pv(j2,j3)*v1pv(j2,j3)  &
                                  +v2pv(j2,j3)*v2pv(j2,j3)  &
                                  +v3pv(j2,j3)*v3pv(j2,j3))
#endif
#else
            uupv(j2,j3)=zrgm*dpv(j2,j3)*tpv(j2,j3)        &
              +zhalf*dpv(j2,j3)*(v1pv(j2,j3)*v1pv(j2,j3)  &
                                +v2pv(j2,j3)*v2pv(j2,j3)  &
                                +v3pv(j2,j3)*v3pv(j2,j3))
#endif
#endif
        dv1pv(j2,j3)=dpv(j2,j3)*v1pv(j2,j3)
        dv2pv(j2,j3)=dpv(j2,j3)*v2pv(j2,j3)
        dv3pv(j2,j3)=dpv(j2,j3)*v3pv(j2,j3)
#else
        do j1=1,nb1
          dpv(j1,j2,j3)=zo3m*dbo(j1,j2,io3m)+zo3p*dbo(j1,j2,io3p)+  &
                        zn3m*dbn(j1,j2,in3m)+zn3p*dbn(j1,j2,in3p)
          tpv(j1,j2,j3)=zo3m*tbo(j1,j2,io3m)+zo3p*tbo(j1,j2,io3p)+  &
                        zn3m*tbn(j1,j2,in3m)+zn3p*tbn(j1,j2,in3p)
          v1pv(j1,j2,j3)=zo3m*v1bo(j1,j2,io3m)+zo3p*v1bo(j1,j2,io3p)+ &
                         zn3m*v1bn(j1,j2,in3m)+zn3p*v1bn(j1,j2,in3p)
          v2pv(j1,j2,j3)=zo3m*v2bo(j1,j2,io3m)+zo3p*v2bo(j1,j2,io3p)+ &
                         zn3m*v2bn(j1,j2,in3m)+zn3p*v2bn(j1,j2,in3p)
          v3pv(j1,j2,j3)=zo3m*v3bo(j1,j2,io3m)+zo3p*v3bo(j1,j2,io3p)+ &
                         zn3m*v3bn(j1,j2,in3m)+zn3p*v3bn(j1,j2,in3p)
!         uupv(j1,j2,j3)=zo3m*uubo(j1,j2,io3m)+zo3p*uubo(j1,j2,io3p)+  &
!                        zn3m*uubn(j1,j2,in3m)+zn3p*uubn(j1,j2,in3p)
!         dv1pv(j1,j2,j3)=zo3m*dv1bo(j1,j2,io3m)+zo3p*dv1bo(j1,j2,io3p)+ &
!                         zn3m*dv1bn(j1,j2,in3m)+zn3p*dv1bn(j1,j2,in3p)
!         dv2pv(j1,j2,j3)=zo3m*dv2bo(j1,j2,io3m)+zo3p*dv2bo(j1,j2,io3p)+ &
!                         zn3m*dv2bn(j1,j2,in3m)+zn3p*dv2bn(j1,j2,in3p)
!         dv3pv(j1,j2,j3)=zo3m*dv3bo(j1,j2,io3m)+zo3p*dv3bo(j1,j2,io3p)+ &
!                         zn3m*dv3bn(j1,j2,in3m)+zn3p*dv3bn(j1,j2,in3p)
#ifdef mhd
            b1pv(j1,j2,j3)=zo3m*b1bo(j1,j2,io3m)+zo3p*b1bo(j1,j2,io3p)+  &
                           zn3m*b1bn(j1,j2,in3m)+zn3p*b1bn(j1,j2,in3p)
            b2pv(j1,j2,j3)=zo3m*b2bo(j1,j2,io3m)+zo3p*b2bo(j1,j2,io3p)+  &
                           zn3m*b2bn(j1,j2,in3m)+zn3p*b2bn(j1,j2,in3p)
            b3pv(j1,j2,j3)=zo3m*b3bo(j1,j2,io3m)+zo3p*b3bo(j1,j2,io3p)+  &
                           zn3m*b3bn(j1,j2,in3m)+zn3p*b3bn(j1,j2,in3p)
#endif
#ifdef hydcon
            dppv(j1,j2,j3)=zo3m*dpbo(j1,j2,io3m)+zo3p*dpbo(j1,j2,io3p)+  &
                           zn3m*dpbn(j1,j2,in3m)+zn3p*dpbn(j1,j2,in3p)
#endif
#ifdef mhdcon
            bppv(j1,j2,j3)=zo3m*bpbo(j1,j2,io3m)+zo3p*bpbo(j1,j2,io3p)+  &
                           zn3m*bpbn(j1,j2,in3m)+zn3p*bpbn(j1,j2,in3p)
#endif
#ifdef ethe
            uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)
#else
#ifdef etot
#ifdef mhd
                uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)        &
                  +zhalf*dpv(j1,j2,j3)*(v1pv(j1,j2,j3)*v1pv(j1,j2,j3)  &
                                       +v2pv(j1,j2,j3)*v2pv(j1,j2,j3)  &
                                       +v3pv(j1,j2,j3)*v3pv(j1,j2,j3)) &
                  +zr2mu*(b1pv(j1,j2,j3)*b1pv(j1,j2,j3)  &
                         +b2pv(j1,j2,j3)*b2pv(j1,j2,j3)  &
                         +b3pv(j1,j2,j3)*b3pv(j1,j2,j3))
#else
                uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)        &
                  +zhalf*dpv(j1,j2,j3)*(v1pv(j1,j2,j3)*v1pv(j1,j2,j3)  &
                                       +v2pv(j1,j2,j3)*v2pv(j1,j2,j3)  &
                                       +v3pv(j1,j2,j3)*v3pv(j1,j2,j3)) &
#endif
#else
              uupv(j1,j2,j3)=zrgm*dpv(j1,j2,j3)*tpv(j1,j2,j3)        &
                +zhalf*dpv(j1,j2,j3)*(v1pv(j1,j2,j3)*v1pv(j1,j2,j3)  &
                                     +v2pv(j1,j2,j3)*v2pv(j1,j2,j3)  &
                                     +v3pv(j1,j2,j3)*v3pv(j1,j2,j3)) &
#endif
#endif
          dv1pv(j1,j2,j3)=dpv(j1,j2,j3)*v1pv(j1,j2,j3)
          dv2pv(j1,j2,j3)=dpv(j1,j2,j3)*v2pv(j1,j2,j3)
          dv3pv(j1,j2,j3)=dpv(j1,j2,j3)*v3pv(j1,j2,j3)
        enddo
#endif
    enddo
  enddo
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5006,0)
#endif
RETURN
END SUBROUTINE BNDIN1


!****f* e0pro/main
! NAME 
!   BNDIN2
! PURPOSE
!   Interpolate boundary values
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDIN2
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE comadm
  IMPLICIT NONE
  REAL(KIND=8)  ztimeo, ztimen
  REAL(KIND=8)  zo2m, zo2p, zn2m, zn2p
  REAL(KIND=8)  zo3m, zo3p, zn3m, zn3p
  REAL(KIND=8)  zo2m3m, zo2m3p, zo2p3m, zo2p3p
  REAL(KIND=8)  zn2m3m, zn2m3p, zn2p3m, zn2p3p
  INTEGER  io2m, io2p, in2m, in2p
  INTEGER  io3m, io3p, in3m, in3p
#ifdef bctim2c
  INTEGER  j1
#endif
  INTEGER  jc2,jc3
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(5006,1)
#endif
!---------------------------------------------------------------------
!      1.        Time interpolation factors
!        1.1       Before
  ztimeo=zone
  ztimen=zero
!        1.2       Within
  if((time.gt.timebo).and.(time.lt.timebn)) then
    ztimen=(time-timebo)/(timebn-timebo)
    ztimeo=zone-ztimen
  endif
!        1.3       After
  if(time.ge.timebn) then
    ztimeo=zero
    ztimen=zone
  endif
!---------------------------------------------------------------------
!      2.        Interpolate boundary values
!        2.1       X1-X2-X3 array
  do jc3=1,ncc3
#ifdef x3dim
      io3m=nx3fom(jc3)
      io3p=nx3fop(jc3)
      in3m=nx3fnm(jc3)
      in3p=nx3fnp(jc3)
      zo3m=x3fom(jc3)
      zo3p=x3fop(jc3)
      zn3m=x3fnm(jc3)
      zn3p=x3fnp(jc3)
#else
      io3m=1
      io3p=1
      in3m=1
      in3p=1
      zo3m=1d0
      zo3p=0d0
      zn3m=1d0
      zn3p=0d0
#endif
    do jc2=1,ncc2
#ifdef x2dim
        io2m=nx2fom(jc2)
        io2p=nx2fop(jc2)
        in2m=nx2fnm(jc2)
        in2p=nx2fnp(jc2)
        zo2m=x2fom(jc2)
        zo2p=x2fop(jc2)
        zn2m=x2fnm(jc2)
        zn2p=x2fnp(jc2)
#else
        io2m=1
        io2p=1
        in2m=1
        in2p=1
        zo2m=1d0
        zo2p=0d0
        zn2m=1d0
        zn2p=0d0
#endif
      io2m=1
      io2p=1
      in2m=1
      in2p=1
      zo2m=1d0
      zo2p=0d0
      zn2m=1d0
      zn2p=0d0
      zo2m3m=ztimeo*zo2m*zo3m
      zo2m3p=ztimeo*zo2m*zo3p
      zo2p3m=ztimeo*zo2p*zo3m
      zo2p3p=ztimeo*zo2p*zo3p
      zn2m3m=ztimen*zn2m*zn3m
      zn2m3p=ztimen*zn2m*zn3p
      zn2p3m=ztimen*zn2p*zn3m
      zn2p3p=ztimen*zn2p*zn3p
!     if((jc2.eq.1).and.(jc3.eq.1)) then
!       write(*,*) '--io',io2m,io2p,io3m,io3p
!       write(*,*) '--in',in2m,in2p,in3m,in3p
!       write(*,*) '--zo',zo2m3m,zo2m3p,zo2p3m,zo2p3p
!       write(*,*) '--zn',zn2m3m,zn2m3p,zn2p3m,zn2p3p
!     endif
#ifndef bctim2c
        dpv(jc2,jc3)=zo2m3m*dbo(io2m,io3m)+             &
                              zo2m3p*dbo(io2m,io3p)+             &
                              zo2p3m*dbo(io2p,io3m)+             &
                              zo2p3p*dbo(io2p,io3p)+             &
                              zn2m3m*dbn(in2m,in3m)+             &
                              zn2m3p*dbn(in2m,in3p)+             &
                              zn2p3m*dbn(in2p,in3m)+             &
                              zn2p3p*dbn(in2p,in3p)
        uupv(jc2,jc3)=zo2m3m*uubo(io2m,io3m)+           &
                               zo2m3p*uubo(io2m,io3p)+           &
                               zo2p3m*uubo(io2p,io3m)+           &
                               zo2p3p*uubo(io2p,io3p)+           &
                               zn2m3m*uubn(in2m,in3m)+           &
                               zn2m3p*uubn(in2m,in3p)+           &
                               zn2p3m*uubn(in2p,in3m)+           &
                               zn2p3p*uubn(in2p,in3p)
        dv1pv(jc2,jc3)=zo2m3m*dv1bo(io2m,io3m)+         &
                                zo2m3p*dv1bo(io2m,io3p)+         &
                                zo2p3m*dv1bo(io2p,io3m)+         &
                                zo2p3p*dv1bo(io2p,io3p)+         &
                                zn2m3m*dv1bn(in2m,in3m)+         &
                                zn2m3p*dv1bn(in2m,in3p)+         &
                                zn2p3m*dv1bn(in2p,in3m)+         &
                                zn2p3p*dv1bn(in2p,in3p)
        dv2pv(jc2,jc3)=zo2m3m*dv2bo(io2m,io3m)+         &
                                zo2m3p*dv2bo(io2m,io3p)+         &
                                zo2p3m*dv2bo(io2p,io3m)+         &
                                zo2p3p*dv2bo(io2p,io3p)+         &
                                zn2m3m*dv2bn(in2m,in3m)+         &
                                zn2m3p*dv2bn(in2m,in3p)+         &
                                zn2p3m*dv2bn(in2p,in3m)+         &
                                zn2p3p*dv2bn(in2p,in3p)
        dv3pv(jc2,jc3)=zo2m3m*dv3bo(io2m,io3m)+         &
                                zo2m3p*dv3bo(io2m,io3p)+         &
                                zo2p3m*dv3bo(io2p,io3m)+         &
                                zo2p3p*dv3bo(io2p,io3p)+         &
                                zn2m3m*dv3bn(in2m,in3m)+         &
                                zn2m3p*dv3bn(in2m,in3p)+         &
                                zn2p3m*dv3bn(in2p,in3m)+         &
                                zn2p3p*dv3bn(in2p,in3p)
#ifdef mhd
          b1pv(jc2,jc3)=zo2m3m*b1bo(io2m,io3m)+        &
                                  zo2m3p*b1bo(io2m,io3p)+        &
                                  zo2p3m*b1bo(io2p,io3m)+        &
                                  zo2p3p*b1bo(io2p,io3p)+        &
                                  zn2m3m*b1bn(in2m,in3m)+        &
                                  zn2m3p*b1bn(in2m,in3p)+        &
                                  zn2p3m*b1bn(in2p,in3m)+        &
                                  zn2p3p*b1bn(in2p,in3p)
          b2pv(jc2,jc3)=zo2m3m*b2bo(io2m,io3m)+        &
                                  zo2m3p*b2bo(io2m,io3p)+        &
                                  zo2p3m*b2bo(io2p,io3m)+        &
                                  zo2p3p*b2bo(io2p,io3p)+        &
                                  zn2m3m*b2bn(in2m,in3m)+        &
                                  zn2m3p*b2bn(in2m,in3p)+        &
                                  zn2p3m*b2bn(in2p,in3m)+        &
                                  zn2p3p*b2bn(in2p,in3p)
          b3pv(jc2,jc3)=zo2m3m*b3bo(io2m,io3m)+        &
                                  zo2m3p*b3bo(io2m,io3p)+        &
                                  zo2p3m*b3bo(io2p,io3m)+        &
                                  zo2p3p*b3bo(io2p,io3p)+        &
                                  zn2m3m*b3bn(in2m,in3m)+        &
                                  zn2m3p*b3bn(in2m,in3p)+        &
                                  zn2p3m*b3bn(in2p,in3m)+        &
                                  zn2p3p*b3bn(in2p,in3p)
#endif
#ifdef hydcon
          dppv(jc2,jc3)=zo2m3m*dpbo(io2m,io3m)+        &
                                  zo2m3p*dpbo(io2m,io3p)+        &
                                  zo2p3m*dpbo(io2p,io3m)+        &
                                  zo2p3p*dpbo(io2p,io3p)+        &
                                  zn2m3m*dpbn(in2m,in3m)+        &
                                  zn2m3p*dpbn(in2m,in3p)+        &
                                  zn2p3m*dpbn(in2p,in3m)+        &
                                  zn2p3p*dpbn(in2p,in3p)
#endif
#ifdef mhdcon
          bppv(jc2,jc3)=zo2m3m*bpbo(io2m,io3m)+        &
                                  zo2m3p*bpbo(io2m,io3p)+        &
                                  zo2p3m*bpbo(io2p,io3m)+        &
                                  zo2p3p*bpbo(io2p,io3p)+        &
                                  zn2m3m*bpbn(in2m,in3m)+        &
                                  zn2m3p*bpbn(in2m,in3p)+        &
                                  zn2p3m*bpbn(in2p,in3m)+        &
                                  zn2p3p*bpbn(in2p,in3p)
#endif
#else
      do j1=1,nb1
        dpv(j1,jc2,jc3)=zo2m3m*dbo(j1,io2m,io3m)+             &
                              zo2m3p*dbo(j1,io2m,io3p)+             &
                              zo2p3m*dbo(j1,io2p,io3m)+             &
                              zo2p3p*dbo(j1,io2p,io3p)+             &
                              zn2m3m*dbn(j1,in2m,in3m)+             &
                              zn2m3p*dbn(j1,in2m,in3p)+             &
                              zn2p3m*dbn(j1,in2p,in3m)+             &
                              zn2p3p*dbn(j1,in2p,in3p)
        uupv(j1,jc2,jc3)=zo2m3m*uubo(j1,io2m,io3m)+           &
                               zo2m3p*uubo(j1,io2m,io3p)+           &
                               zo2p3m*uubo(j1,io2p,io3m)+           &
                               zo2p3p*uubo(j1,io2p,io3p)+           &
                               zn2m3m*uubn(j1,in2m,in3m)+           &
                               zn2m3p*uubn(j1,in2m,in3p)+           &
                               zn2p3m*uubn(j1,in2p,in3m)+           &
                               zn2p3p*uubn(j1,in2p,in3p)
        dv1pv(j1,jc2,jc3)=zo2m3m*dv1bo(j1,io2m,io3m)+         &
                                zo2m3p*dv1bo(j1,io2m,io3p)+         &
                                zo2p3m*dv1bo(j1,io2p,io3m)+         &
                                zo2p3p*dv1bo(j1,io2p,io3p)+         &
                                zn2m3m*dv1bn(j1,in2m,in3m)+         &
                                zn2m3p*dv1bn(j1,in2m,in3p)+         &
                                zn2p3m*dv1bn(j1,in2p,in3m)+         &
                                zn2p3p*dv1bn(j1,in2p,in3p)
        dv2pv(j1,jc2,jc3)=zo2m3m*dv2bo(j1,io2m,io3m)+         &
                                zo2m3p*dv2bo(j1,io2m,io3p)+         &
                                zo2p3m*dv2bo(j1,io2p,io3m)+         &
                                zo2p3p*dv2bo(j1,io2p,io3p)+         &
                                zn2m3m*dv2bn(j1,in2m,in3m)+         &
                                zn2m3p*dv2bn(j1,in2m,in3p)+         &
                                zn2p3m*dv2bn(j1,in2p,in3m)+         &
                                zn2p3p*dv2bn(j1,in2p,in3p)
        dv3pv(j1,jc2,jc3)=zo2m3m*dv3bo(j1,io2m,io3m)+         &
                                zo2m3p*dv3bo(j1,io2m,io3p)+         &
                                zo2p3m*dv3bo(j1,io2p,io3m)+         &
                                zo2p3p*dv3bo(j1,io2p,io3p)+         &
                                zn2m3m*dv3bn(j1,in2m,in3m)+         &
                                zn2m3p*dv3bn(j1,in2m,in3p)+         &
                                zn2p3m*dv3bn(j1,in2p,in3m)+         &
                                zn2p3p*dv3bn(j1,in2p,in3p)
#ifdef mhd
          b1pv(j1,jc2,jc3)=zo2m3m*b1bo(j1,io2m,io3m)+        &
                                  zo2m3p*b1bo(j1,io2m,io3p)+        &
                                  zo2p3m*b1bo(j1,io2p,io3m)+        &
                                  zo2p3p*b1bo(j1,io2p,io3p)+        &
                                  zn2m3m*b1bn(j1,in2m,in3m)+        &
                                  zn2m3p*b1bn(j1,in2m,in3p)+        &
                                  zn2p3m*b1bn(j1,in2p,in3m)+        &
                                  zn2p3p*b1bn(j1,in2p,in3p)

          b2pv(j1,jc2,jc3)=zo2m3m*b2bo(j1,io2m,io3m)+        &
                                  zo2m3p*b2bo(j1,io2m,io3p)+        &
                                  zo2p3m*b2bo(j1,io2p,io3m)+        &
                                  zo2p3p*b2bo(j1,io2p,io3p)+        &
                                  zn2m3m*b2bn(j1,in2m,in3m)+        &
                                  zn2m3p*b2bn(j1,in2m,in3p)+        &
                                  zn2p3m*b2bn(j1,in2p,in3m)+        &
                                  zn2p3p*b2bn(j1,in2p,in3p)

          b3pv(j1,jc2,jc3)=zo2m3m*b3bo(j1,io2m,io3m)+        &
                                  zo2m3p*b3bo(j1,io2m,io3p)+        &
                                  zo2p3m*b3bo(j1,io2p,io3m)+        &
                                  zo2p3p*b3bo(j1,io2p,io3p)+        &
                                  zn2m3m*b3bn(j1,in2m,in3m)+        &
                                  zn2m3p*b3bn(j1,in2m,in3p)+        &
                                  zn2p3m*b3bn(j1,in2p,in3m)+        &
                                  zn2p3p*b3bn(j1,in2p,in3p)
#endif
#ifdef hydcon
          dppv(j1,jc2,jc3)=zo2m3m*dpbo(j1,io2m,io3m)+        &
                                  zo2m3p*dpbo(j1,io2m,io3p)+        &
                                  zo2p3m*dpbo(j1,io2p,io3m)+        &
                                  zo2p3p*dpbo(j1,io2p,io3p)+        &
                                  zn2m3m*dpbn(j1,in2m,in3m)+        &
                                  zn2m3p*dpbn(j1,in2m,in3p)+        &
                                  zn2p3m*dpbn(j1,in2p,in3m)+        &
                                  zn2p3p*dpbn(j1,in2p,in3p)
#endif
#ifdef mhdcon
          bppv(j1,jc2,jc3)=zo2m3m*bpbo(j1,io2m,io3m)+        &
                                  zo2m3p*bpbo(j1,io2m,io3p)+        &
                                  zo2p3m*bpbo(j1,io2p,io3m)+        &
                                  zo2p3p*bpbo(j1,io2p,io3p)+        &
                                  zn2m3m*bpbn(j1,in2m,in3m)+        &
                                  zn2m3p*bpbn(j1,in2m,in3p)+        &
                                  zn2p3m*bpbn(j1,in2p,in3m)+        &
                                  zn2p3p*bpbn(j1,in2p,in3p)
#endif
      enddo
#endif
    enddo
  enddo
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5006,0)
#endif                             
RETURN
END SUBROUTINE BNDIN2
