!****f* e0pro/main
! NAME 
!   BNDMPI
! PURPOSE
!   Transfer boundary values between slabs
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDMPI 
!---------------------------------------------------------------------
  USE comoly 
  USE comdim 
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE comadm
  IMPLICIT NONE
#ifdef MPI                                                              
    include 'mpif.h' 
    INTEGER ierr 
    INTEGER, DIMENSION(MPI_STATUS_SIZE)  ::  status
    INTEGER, PARAMETER  ::  ibuf=ng1*ncc2*ncc3*nnvar
    REAL(KIND=8)  zrecvl(ibuf), zrecvr(ibuf)
    REAL(KIND=8)  zsendl(ibuf), zsendr(ibuf)
    INTEGER jghost, jc2, jc3, jx1, jx2, jx3, jv
    INTEGER icount, ic
#endif
#ifdef DIAGNO
    call EXPERT(5007,1)
#endif
#ifdef MPI
!---------------------------------------------------------------------
!      1.        Fill buffers
!        1.1       Put buffers into the left ghost layers
    ic=0
    do jc3=1,ncc3
      do jc2=1,ncc2
        do jghost=1,ng1
          do jv=1,nnvar
            ic=ic+1
            zsendl(ic)=unk(jv,ng1+jghost,ng2+jc2,ng3+jc3,1)
            zsendr(ic)=unk(jv,ncc1+jghost,ng2+jc2,ng3+jc3,1)
          enddo
        enddo
      enddo
    enddo
!---------------------------------------------------------------------
!      2.        Wait until all procs come at this point
    call MPI_BARRIER(MPI_COMM_WORLD,ierr)
!---------------------------------------------------------------------
!      3.        Interpolate boundary values
    if(mod(myproc,2).ne.0) then
!        3.1       X1-X2-X3 array
      call MPI_SEND(zsendl,ibuf,MPI_DOUBLE_PRECISION,myproc-1,   &
        100,MPI_COMM_WORLD,ierr)
      call MPI_RECV(zrecvl,ibuf,MPI_DOUBLE_PRECISION,myproc-1,   &
        200,MPI_COMM_WORLD,status,ierr)

      if(myproc.lt.nprocs-1) then
        call MPI_SEND(zsendr,ibuf,MPI_DOUBLE_PRECISION,myproc+1, &
          300,MPI_COMM_WORLD,ierr)
        call MPI_RECV(zrecvr,ibuf,MPI_DOUBLE_PRECISION,myproc+1, &
          400,MPI_COMM_WORLD,status,ierr)
      endif
    else
!        3.2       X1-X2-X3 array
      call MPI_RECV(zrecvr,ibuf,MPI_DOUBLE_PRECISION,myproc+1,   &
        100,MPI_COMM_WORLD,status,ierr)
      call MPI_SEND(zsendr,ibuf,MPI_DOUBLE_PRECISION,myproc+1,   &
        200,MPI_COMM_WORLD,ierr)
      if(myproc.gt.0) then
        call MPI_RECV(zrecvl,ibuf,MPI_DOUBLE_PRECISION,myproc-1, &
          300,MPI_COMM_WORLD,status,ierr)
        call MPI_SEND(zsendl,ibuf,MPI_DOUBLE_PRECISION,myproc-1, &
          400,MPI_COMM_WORLD,ierr)
      endif
    endif
!---------------------------------------------------------------------
!      4.        Extract buffers
!        4.1       Put buffers into the left ghost layers
    if(myproc.ne.0) then
      ic=0
      do jc3=1,ncc3
        do jc2=1,ncc2
          do jghost=1,ng1
            do jv=1,nnvar
              ic=ic+1
              unk(jv,jghost,ng2+jc2,ng3+jc3,1)=zrecvl(ic)
            enddo
          enddo
        enddo
      enddo
    endif
!        4.2       Put buffers into the right ghost layers
    if(myproc.ne.nprocs-1) then
      ic=0
      do jc3=1,ncc3
        do jc2=1,ncc2
          do jghost=1,ng1
            do jv=1,nnvar
              ic=ic+1
              unk(jv,nc1-ng1+jghost,ng2+jc2,ng3+jc3,1)=zrecvr(ic)
            enddo
          enddo
        enddo
      enddo
    endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination - return
#ifdef DIAGNO
    call EXPERT(5007,0)
#endif                                                                  
RETURN
END SUBROUTINE BNDMPI
