!****f* e0pro/main
! NAME 
!   MHC
! PURPOSE
!   MHD X1,X2,X3-solution
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE MHC 
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE commet
  USE comadm
  USE comtva
  IMPLICIT NONE
#ifndef split
  REAL(KIND=8)  zclsp,zclsph,zsp,zsph,zcot,zgdt,zngdt,zgdth
  REAL(KIND=8)  zngdth,zs,zrd,zcdt,zcsc,zcsc1,zcsc2
  REAL(KIND=8)  zavldt,zabldt
  REAL(KIND=8)  zavrdt,zabrdt
#ifdef vcen
  REAL(KIND=8)  zvel,zden,zpthe,zpmag
#else
#ifdef con
  REAL(KIND=8)  zvel
#endif
#endif
  REAL(KIND=8)  zel,zer
  REAL(KIND=8),DIMENSION(nc)  ::  rvol2,rvol3
  INTEGER  nxlmm,nxlm,nxl,nxlp,nxrm,nxr,nxrp,nxrpp
  INTEGER  j1,j2,j3,jv,iv,jb
#endif
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
!---------------------------------------------------------------------
#ifdef DIAGNO
    call EXPERT(6001,1)
#endif
#ifndef split
!---------------------------------------------------------------------
!      1.        Auxiliary values
!        1.1       Geometry terms parameters
#ifdef cartesian
    zclsp=zero
    zclsph=zero
#else
    zclsp=dt
    zclsph=dth
#endif
#ifdef spherical
    zsp=dt
    zsph=dth
#else
    zsp=zero
    zsph=zero
#endif
  zcot=zero
!        1.2       Gravity terms parameters
  if(nlgrav) then
    zgdt =-grav*dt
  else
    zgdt=zero
  endif
  if(nlxn) then
    zngdt=zgdt*xmass
  else
    zngdt=zgdt
  endif
  zgdth=zhalf*zgdt
  zngdth=zhalf*zngdt
!---------------------------------------------------------------------
!      2.        Prologue
#ifdef PARAMESH
!loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
        if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!        2.1       Parameters
  cfl1mx(jb)=zero
  cfl2mx(jb)=zero
  cfl3mx(jb)=zero
!---------------------------------------------------------------------
!      3.        Equation of state (half step)
  do j3=1,nc3
    do j2=1,nc2
      do j1=1,nc1
#ifdef gam3d
          if(nlxn) then
            gammam=gamma(j1,j2,j3,jb)-zone
            zgamma=gamma(j1,j2,j3,jb)/xmass
          else
            gammam=gamma(j1,j2,j3,jb)-zone
            zgamma=gamma(j1,j2,j3,jb)
          endif
#endif
!        3.1       Get old values
        do jv=1,nnvar
          uo(j1,j2,j3,jv)=unk(jv,j1,j2,j3,jb)
        enddo
!        3.2       Velocity
        zrd=zone/max(zmin,uo(j1,j2,j3,1))
        vpo(j1,j2,j3,1)=uo(j1,j2,j3,3)*zrd
        vpo(j1,j2,j3,2)=uo(j1,j2,j3,4)*zrd
        vpo(j1,j2,j3,3)=uo(j1,j2,j3,5)*zrd
#ifndef con
!        3.3       Pressure
#ifdef mhd
            vpo(j1,j2,j3,5)=zr2mu*(uo(j1,j2,j3,6)*uo(j1,j2,j3,6) &
                                  +uo(j1,j2,j3,7)*uo(j1,j2,j3,7) &
                                  +uo(j1,j2,j3,8)*uo(j1,j2,j3,8))
#endif
#ifdef ethe
            vpo(j1,j2,j3,4)=gammam*uo(j1,j2,j3,2)
#else
#ifdef etot
#ifdef mhd
                vpo(j1,j2,j3,4)=gammam*(uo(j1,j2,j3,2)  &
                  -z05*(uo(j1,j2,j3,3)*vpo(j1,j2,j3,1)  &
                       +uo(j1,j2,j3,4)*vpo(j1,j2,j3,2)  &
                       +uo(j1,j2,j3,5)*vpo(j1,j2,j3,3)) &
                       -vpo(j1,j2,j3,5))
#else
                vpo(j1,j2,j3,4)=gammam*(uo(j1,j2,j3,2)  &
                  -z05*(uo(j1,j2,j3,3)*vpo(j1,j2,j3,1)  &
                       +uo(j1,j2,j3,4)*vpo(j1,j2,j3,2)  &
                       +uo(j1,j2,j3,5)*vpo(j1,j2,j3,3)))
#endif
#else
              vpo(j1,j2,j3,4)=gammam*(uo(j1,j2,j3,2)  &
                -z05*(uo(j1,j2,j3,3)*vpo(j1,j2,j3,1)  &
                     +uo(j1,j2,j3,4)*vpo(j1,j2,j3,2)  &
                     +uo(j1,j2,j3,5)*vpo(j1,j2,j3,3)))
#endif
#endif
#ifdef mhd
            vpo(j1,j2,j3,6)=vpo(j1,j2,j3,4)+vpo(j1,j2,j3,5)
#endif
#endif
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      4.        X1-Limited interface values
  do j2=nc2lm,nc2rp
#ifdef spherical
      zcot=cott(j2,jb)
#endif
    do j3=nc3lm,nc3rp
!        4.1       Main variables
      do jv=1,nnvar
        do j1=2,nc1
          difh(j1)=uo(j1,j2,j3,jv)-uo(j1-1,j2,j3,jv)
        enddo
        do j1=2,nc1-1
#ifdef minmod
            yg(j1,jv)=sign(zhalf,difh(j1))*max(zero, &
              min(abs(difh(j1)),sign(zone,difh(j1))*difh(j1+1)))
#endif
#ifdef woodward
            zs=sign(zone,difh(j1))
            yg(j1,jv)=zhalf*zs*max(zero,  &
              min(abs(difh(j1)+difh(j1)), &
              zs*(difh(j1+1)+difh(j1+1)), &
              zhalf*zs*(difh(j1)+difh(j1+1))))
#endif
        enddo
      enddo
!        4.2       Auxiliary variables
      do jv=1,npvar
        do j1=2,nc1
          difh(j1)=vpo(j1,j2,j3,jv)-vpo(j1-1,j2,j3,jv)
        enddo
        do j1=2,nc1-1
#ifdef minmod
            ygg(j1,jv)=sign(zhalf,difh(j1))*max(zero, &
              min(abs(difh(j1)),sign(zone,difh(j1))*difh(j1+1)))
#endif
#ifdef woodward
            zs=sign(zone,difh(j1))
            ygg(j1,jv)=zhalf*zs*max(zero, &
              min(abs(difh(j1)+difh(j1)), &
              zs*(difh(j1+1)+difh(j1+1)), &
              zhalf*zs*(difh(j1)+difh(j1+1))))
#endif
        enddo
      enddo
!---------------------------------------------------------------------
!      5.        X1-Predictor
      do j1=2,nc1-1
        do jv=1,nnvar
          ul(jv)=uo(j1,j2,j3,jv)-yg(j1,jv)
          ur(jv)=uo(j1,j2,j3,jv)+yg(j1,jv)
        enddo
        do jv=1,npvar
          vpl(jv)=vpo(j1,j2,j3,jv)-ygg(j1,jv)
          vpr(jv)=vpo(j1,j2,j3,jv)+ygg(j1,jv)
        enddo
!        4.1       Auxiliary terms
        zavrdt=rvol1(j1,jb)*dth*ah1(j1+1,jb)*vpr(1)
        zavldt=rvol1(j1,jb)*dth*ah1(j1,jb)*vpl(1)
#ifdef mhd
          zabrdt=rvol1(j1,jb)*dthmag*ah1(j1+1,jb)*ur(6)
          zabldt=rvol1(j1,jb)*dthmag*ah1(j1,jb)*ul(6)
#endif
!        4.2       Density
        ua(j1,j2,j3,1)=uo(j1,j2,j3,1)-zavrdt*ur(1)+zavldt*ul(1)
#ifdef con
          ua(j1,j2,j3,3)=uo(j1,j2,j3,1)*vpo(j1,j2,j3,1)
          ua(j1,j2,j3,4)=uo(j1,j2,j3,1)*vpo(j1,j2,j3,2)
          ua(j1,j2,j3,5)=uo(j1,j2,j3,1)*vpo(j1,j2,j3,3)
#else
#ifdef ethe
!        4.3       Thermal energy
          ua(j1,j2,j3,2)=uo(j1,j2,j3,2)-zavrdt*ur(2)+zavldt*ul(2) &
            +vpo(j1,j2,j3,4)*(-zavrdt+zavldt)
#else
#ifdef etot
!        4.3       Total energy
#ifdef mhd
              zer=ur(6)*vpr(1)+ur(7)*vpr(2)+ur(8)*vpr(3)
              zel=ul(6)*vpl(1)+ul(7)*vpl(2)+ul(8)*vpl(3)
              ua(j1,j2,j3,2)=uo(j1,j2,j3,2)+rvol1(j1,jb)*dthene* &
                (ah1(j1+1,jb)*ur(6)*zer-ah1(j1,jb)*ul(6)*zel)    &
                -zavrdt*(ur(2)+vpr(6))+zavldt*(ul(2)+vpl(6))     &
                +zngdth*uo(j1,j2,j3,2)*rxx1(j1,jb)
#else
              ua(j1,j2,j3,2)=uo(j1,j2,j3,2)                  &
                -zavrdt*(ur(2)+vpr(4))+zavldt*(ul(2)+vpl(4)) &
                +zngdth*uo(j1,j2,j3,1)*rxx1(j1,jb)
#endif
#else
            ua(j1,j2,j3,2)=uo(j1,j2,j3,2)                  &
              -zavrdt*(ur(2)+vpr(4))+zavldt*(ul(2)+vpl(4)) &
              +zngdth*uo(j1,j2,j3,1)*rxx1(j1,jb)
#endif
#endif
!        4.4       Momentum
#ifdef mhd
          ua(j1,j2,j3,3)=uo(j1,j2,j3,3)                        &
            -zavrdt*ur(3)+zavldt*ul(3)                         &
            +zabrdt*ur(6)-zabldt*ul(6)                         &
            -dtmom*rdx1(j1,jb)*ygg(j1,6)                       &
            +zgdth*uo(j1,j2,j3,1)*rxx1(j1,jb)                  &
            +zclsph*rx1(j1,jb)*(uo(j1,j2,j3,4)*vpo(j1,j2,j3,2) &
              -zrmu*uo(j1,j2,j3,7)*uo(j1,j2,j3,7))             &
            +zsph*rx1(j1,jb)*(uo(j1,j2,j3,5)*vpo(j1,j2,j3,3)   &
              -zrmu*uo(j1,j2,j3,8)*uo(j1,j2,j3,8))
          ua(j1,j2,j3,4)=uo(j1,j2,j3,4)                        &
            -zavrdt*ur(4)+zavldt*ul(4)                         &
            +zabrdt*ur(7)-zabldt*ul(7)                         &
            -zclsph*rx1(j1,jb)*(uo(j1,j2,j3,3)*vpo(j1,j2,j3,2) &
              -zrmu*uo(j1,j2,j3,6)*uo(j1,j2,j3,7))             &
            +zsph*rx1(j1,jb)*(uo(j1,j2,j3,5)*vpo(j1,j2,j3,3)   &
              -zrmu*uo(j1,j2,j3,8)*uo(j1,j2,j3,8))*zcot
          ua(j1,j2,j3,5)=uo(j1,j2,j3,5)             &
            -zavrdt*ur(5)+zavldt*ul(5)              &
            +zabrdt*ur(8)-zabldt*ul(8)              &
            -zsph*rx1(j1,jb)*(vpo(j1,j2,j3,3)*      &
               (uo(j1,j2,j3,3)+uo(j1,j2,j3,4)*zcot) &
               -zrmu*uo(j1,j2,j3,8)*(uo(j1,j2,j3,6)+uo(j1,j2,j3,7)*zcot))
#else
           ua(j1,j2,j3,3)=uo(j1,j2,j3,3)                       &
             -zavrdt*ur(3)+zavldt*ul(3)                        &
             -dtmom*rdx1(j1,jb)*ygg(j1,4)                      &
             +zgdth*uo(j1,j2,j3,1)*rxx1(j1,jb)                 &
             +zclsph*rx1(j1,jb)*uo(j1,j2,j3,4)*vpo(j1,j2,j3,2) &
             +zsph*rx1(j1,jb)*uo(j1,j2,j3,5)*vpo(j1,j2,j3,3)
           ua(j1,j2,j3,4)=uo(j1,j2,j3,4)                       &
             -zavrdt*ur(4)+zavldt*ul(4)                        &
             -zclsph*rx1(j1,jb)*uo(j1,j2,j3,3)*vpo(j1,j2,j3,2) &
             +zsph*rx1(j1,jb)*uo(j1,j2,j3,5)*vpo(j1,j2,j3,3)*zcot
           ua(j1,j2,j3,5)=uo(j1,j2,j3,5)       &
             -zavrdt*ur(5)+zavldt*ul(5)        &
             -zsph*rx1(j1,jb)*vpo(j1,j2,j3,3)* &
               (uo(j1,j2,j3,3)+uo(j1,j2,j3,4)*zcot)
#endif
#ifdef mhd
!        4.5       Magnetic field
            ua(j1,j2,j3,6)=uo(j1,j2,j3,6)
            ua(j1,j2,j3,7)=uo(j1,j2,j3,7)+rvol12(j1)*dth* &
              (ah12(j1+1)*(-ur(7)*vpr(1)+ur(6)*vpr(2))    &
              +ah12(j1)*(ul(7)*vpl(1)-ul(6)*vpl(2)))
            ua(j1,j2,j3,8)=uo(j1,j2,j3,8)+rvol13(j1)*dth* &
              (ah13(j1+1)*(-ur(8)*vpr(1)+ur(6)*vpr(3))    &
              +ah13(j1)*(ul(8)*vpl(1)-ul(6)*vpl(3)))
            iv=8
#else
            iv=5
#endif
!        4.6       Supplementary variables
#ifdef hydcon
            iv=iv+1
            ua(j1,j2,j3,iv)=uo(j1,j2,j3,iv)-zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#ifdef mhdcon
            iv=iv+1
            ua(j1,j2,j3,iv)=uo(j1,j2,j3,iv)-zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#endif
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      4.        X2-Limited interface values
#ifdef x2dim
    do j3=nc3lm,nc3rp
      do j1=nc1lm,nc1rp
!        4.1       Main variables
        do jv=1,nnvar
          do j2=2,nc2
            difh(j2)=uo(j1,j2,j3,jv)-uo(j1,j2-1,j3,jv)
          enddo
          do j2=2,nc2-1
#ifdef minmod
              yg(j2,jv)=sign(zhalf,difh(j2))*max(zero, &
                min(abs(difh(j2)),sign(zone,difh(j2))*difh(j2+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j2))
              yg(j2,jv)=zhalf*zs*max(zero,  &
                min(abs(difh(j2)+difh(j2)), &
                zs*(difh(j2+1)+difh(j2+1)), &
                zhalf*zs*(difh(j2)+difh(j2+1))))
#endif
          enddo
        enddo
!        4.2       Auxiliary variables
        do jv=1,npvar
          do j2=2,nc2
            difh(j2)=vpo(j1,j2,j3,jv)-vpo(j1,j2-1,j3,jv)
          enddo
          do j2=2,nc2-1
#ifdef minmod
              ygg(j2,jv)=sign(zhalf,difh(j2))*max(zero, &
                min(abs(difh(j2)),sign(zone,difh(j2))*difh(j2+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j2))
              ygg(j2,jv)=zhalf*zs*max(zero, &
                min(abs(difh(j2)+difh(j2)), &
                zs*(difh(j2+1)+difh(j2+1)), &
                zhalf*zs*(difh(j2)+difh(j2+1))))
#endif
          enddo
        enddo
!        4.3       Auxiliary terms
        do j2=2,nc2-1
#ifdef cartesian
            rdx(j2)=rdx2(j2,jb)
            rvol(j2)=rdx2(j2,jb)
            rvol23(j2)=rdx2(j2,jb)
            vol(j2)=dx2(j2,jb)
            vol23(j2)=dx2(j2,jb)
#endif
#ifdef cylindrical
            rdx(j2)=rx1(j1,jb)*rdx2(j2,jb)
            rvol(j2)=rdx(j2)
            rvol23(j2)=rdx(j2)
            vol(j2)=xc1(j1,jb)*dx2(j2,jb)
            vol23(j2)=vol2(j2)
#endif
#ifdef spherical
            rdx(j2)=rx1(j1,jb)*rdx2(j2,jb)
            rvol(j2)=rdx(j2)*rsint(j2,jb)
            rvol23(j2)=rdx(j2)
            vol(j2)=xc1(j1,jb)*sint(j2,jb)*dx2(j2,jb)
            vol23(j2)=xc1(j1,jb)*dx2(j2,jb)
#endif
        enddo
!---------------------------------------------------------------------
!      5.          X2-Predictor
        do j2=2,nc2-1
          do jv=1,nnvar
            ul(jv)=uo(j1,j2,j3,jv)-yg(j2,jv)
            ur(jv)=uo(j1,j2,j3,jv)+yg(j2,jv)
          enddo
          do jv=1,npvar
            vpl(jv)=vpo(j1,j2,j3,jv)-ygg(j2,jv)
            vpr(jv)=vpo(j1,j2,j3,jv)+ygg(j2,jv)
          enddo
!        5.1       Auxiliary terms
          zavrdt=rvol(j2)*dth*ah2(j2+1,jb)*vpr(2)
          zavldt=rvol(j2)*dth*ah2(j2,jb)*vpl(2)
#ifdef mhd
            zabrdt=rvol(j2)*dthmag*ah2(j2+1,jb)*ur(7)
            zabldt=rvol(j2)*dthmag*ah2(j2,jb)*ul(7)
#endif
!        5.2       Density
          ua(j1,j2,j3,1)=ua(j1,j2,j3,1)-zavrdt*ur(1)+zavldt*ul(1)
#ifdef con
            ua(j1,j2,j3,3)=ua(j1,j2,j3,1)*vpo(j1,j2,j3,1)
            ua(j1,j2,j3,4)=ua(j1,j2,j3,1)*vpo(j1,j2,j3,2)
            ua(j1,j2,j3,5)=ua(j1,j2,j3,1)*vpo(j1,j2,j3,3)
#else
#ifdef ethe
!        5.3       Thermal energy
              ua(j1,j2,j3,2)=ua(j1,j2,j3,2) &
                -zavrdt*ur(2)+zavldt*ul(2)  &
                +vpo(j1,j2,j3,4)*(-zavrdt+zavldt)
#else
#ifdef etot
!        5.3       Total energy
#ifdef mhd
                  zer=ur(6)*vpr(1)+ur(7)*vpr(2)+ur(8)*vpr(3)
                  zel=ul(6)*vpl(1)+ul(7)*vpl(2)+ul(8)*vpl(3)
                  ua(j1,j2,j3,2)=ua(j1,j2,j3,2)                      &
                    +rvol(j2)*dthene*                                &
                       (ah2(j2+1,jb)*ur(7)*zer-ah2(j2,jb)*ul(7)*zel) &
                    -zavrdt*(ur(2)+vpr(6))+zavldt*(ul(2)+vpl(6))
#else
                  ua(j1,j2,j3,2)=ua(j1,j2,j3,2) &
                    -zavrdt*(ur(2)+vpr(4))+zavldt*(ul(2)+vpl(4))
#endif
#else
                ua(j1,j2,j3,2)=ua(j1,j2,j3,2) &
                  -zavrdt*(ur(2)+vpr(4))+zavldt*(ul(2)+vpl(4))
#endif
#endif
!        5.4       Momentum
#ifdef mhd
              ua(j1,j2,j3,3)=ua(j1,j2,j3,3) &
                -zavrdt*ur(3)+zavldt*ul(3)+zabrdt*ur(6)-zabldt*ul(6)
              ua(j1,j2,j3,4)=ua(j1,j2,j3,4) &
                -zavrdt*ur(4)+zavldt*ul(4)  &
                -dtmom*rdx(j2)*ygg(j2,6)    &
                +zabrdt*ur(7)-zabldt*ul(7)
              ua(j1,j2,j3,5)=ua(j1,j2,j3,5) &
                -zavrdt*ur(5)+zavldt*ul(5)  &
                +zabrdt*ur(8)-zabldt*ul(8)
#else
              ua(j1,j2,j3,3)=ua(j1,j2,j3,3)-zavrdt*ur(3)+zavldt*ul(3)
              ua(j1,j2,j3,4)=ua(j1,j2,j3,4)-zavrdt*ur(4)+zavldt*ul(4) &
                -dtmom*rdx(j2)*ygg(j2,4)
              ua(j1,j2,j3,5)=ua(j1,j2,j3,5)-zavrdt*ur(5)+zavldt*ul(5)
#endif
#ifdef mhd
!        5.5       Magnetic field
              ua(j1,j2,j3,6)=ua(j1,j2,j3,6)+rvol(j2)*dth*  &
                (ah2(j2+1,jb)*(-ur(6)*vpr(2)+ur(7)*vpr(1)) &
                +ah2(j2,jb)*(ul(6)*vpl(2)-ul(7)*vpl(1)))
              ua(j1,j2,j3,8)=ua(j1,j2,j3,8)+rvol23(j2)*dth* &
                ((-ur(8)*vpr(2)+ur(7)*vpr(3))               &
                +(ul(8)*vpl(2)-ul(7)*vpl(3)))
              iv=8
#else
              iv=5
#endif
!        5.6       Supplementary variables
#ifdef hydcon
              iv=iv+1
              ua(j1,j2,j3,iv)=ua(j1,j2,j3,iv)-zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#ifdef mhdcon
              iv=iv+1
              ua(j1,j2,j3,iv)=ua(j1,j2,j3,iv)-zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      6.        X3-Limited interface values
#ifdef x3dim
    do j2=nc2lm,nc2rp
      do j1=nc1lm,nc1rp
!        6.1       Main variables
        do jv=1,nnvar
          do j3=2,nc3
            difh(j3)=uo(j1,j2,j3,jv)-uo(j1,j2,j3-1,jv)
          enddo
          do j3=2,nc3-1
#ifdef minmod
              yg(j3,jv)=sign(zhalf,difh(j3))*max(zero, &
                min(abs(difh(j3)),sign(zone,difh(j3))*difh(j3+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j3))
              yg(j3,jv)=zhalf*zs*max(zero,  &
                min(abs(difh(j3)+difh(j3)), &
                zs*(difh(j3+1)+difh(j3+1)), &
                zhalf*zs*(difh(j3)+difh(j3+1))))
#endif
          enddo
        enddo
!        6.2       Auxiliary variables
        do jv=1,npvar
          do j3=2,nc3
            difh(j3)=vpo(j1,j2,j3,jv)-vpo(j1,j2,j3-1,jv)
          enddo
          do j3=2,nc3-1
#ifdef minmod
              ygg(j3,jv)=sign(zhalf,difh(j3))*max(zero, &
                min(abs(difh(j3)),sign(zone,difh(j3))*difh(j3+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j3))
              ygg(j3,jv)=zhalf*zs*max(zero, &
                min(abs(difh(j3)+difh(j3)), &
                zs*(difh(j3+1)+difh(j3+1)), &
                zhalf*zs*(difh(j3)+difh(j3+1))))
#endif
          enddo
        enddo
!        6.3       Auxiliary terms
        do j3=2,nc3-1
#ifdef cartesian
            rdx(j3)=rdx3(j3,jb)
            rvol(j3)=rdx3(j3,jb)
            vol(j3)=dx3(j3,jb)
#endif
#ifdef cylindrical
            rdx(j3)=rdx3(j3,jb)
            rvol(j3)=rdx3(j3,jb)
            vol(j3)=dx3(j3,jb)
#endif
#ifdef spherical
            rdx(j3)=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)
            rvol(j3)=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)
            vol(j3)=xc1(j1,jb)*sint(j2,jb)*dx3(j3,jb)
#endif
        enddo
!---------------------------------------------------------------------
!      7.        X3-Predictor
        do j3=2,nc3-1
          do jv=1,nnvar
            ul(jv)=uo(j1,j2,j3,jv)-yg(j3,jv)
            ur(jv)=uo(j1,j2,j3,jv)+yg(j3,jv)
          enddo
          do jv=1,npvar
            vpl(jv)=vpo(j1,j2,j3,jv)-ygg(j3,jv)
            vpr(jv)=vpo(j1,j2,j3,jv)+ygg(j3,jv)
          enddo
!        7.1       Auxiliary terms
          zavrdt=rvol(j3)*dth*vpr(3)
          zavldt=rvol(j3)*dth*vpl(3)
#ifdef mhd
            zabrdt=rvol(j3)*dthmag*ur(8)
            zabldt=rvol(j3)*dthmag*ul(8)
#endif
!        7.2       Density
          ua(j1,j2,j3,1)=ua(j1,j2,j3,1)-zavrdt*ur(1)+ zavldt*ul(1)
#ifdef con
            ua(j1,j2,j3,3)=ua(j1,j2,j3,1)*vpo(j1,j2,j3,1)
            ua(j1,j2,j3,4)=ua(j1,j2,j3,1)*vpo(j1,j2,j3,2)
            ua(j1,j2,j3,5)=ua(j1,j2,j3,1)*vpo(j1,j2,j3,3)
#else
#ifdef ethe
!        7.3       Thermal energy
              ua(j1,j2,j3,2)=ua(j1,j2,j3,2) &
                -zavrdt*ur(2)+zavldt*ul(2)+vpo(j1,j2,j3,4)*(-zavrdt+zavldt)
#else
#ifdef etot
!        7.3       Total energy
#ifdef mhd
                  zer=ur(6)*vpr(1)+ur(7)*vpr(2)+ur(8)*vpr(3)
                  zel=ul(6)*vpl(1)+ul(7)*vpl(2)+ul(8)*vpl(3)
                  ua(j1,j2,j3,2)=ua(j1,j2,j3,2)+rvol(j3)*dthene* &
                    (ur(8)*zer-ul(8)*zel)-                       &
                    zavrdt*(ur(2)+vpr(6))+zavldt*(ul(2)+vpl(6))
#else
                  ua(j1,j2,j3,2)=ua(j1,j2,j3,2) &
                    -zavrdt*(ur(2)+vpr(4))+zavldt*(ul(2)+vpl(4))
#endif
#else
                ua(j1,j2,j3,2)=ua(j1,j2,j3,2) &
                  -zavrdt*(ur(2)+vpr(4))+zavldt*(ul(2)+vpl(4))
#endif
#endif
!        7.4       Momentum
#ifdef mhd
              ua(j1,j2,j3,3)=ua(j1,j2,j3,3) &
                -zavrdt*ur(3)+zavldt*ul(3)+zabrdt*ur(6)-zabldt*ul(6)
              ua(j1,j2,j3,4)=ua(j1,j2,j3,4) &
                -zavrdt*ur(4)+zavldt*ul(4)+zabrdt*ur(7)-zabldt*ul(7)
              ua(j1,j2,j3,5)=ua(j1,j2,j3,5) &
                -zavrdt*ur(5)+zavldt*ul(5)  &
                -dtmom*rdx(j3)*ygg(j3,6)    &
                +zabrdt*ur(8)-zabldt*ul(8)
#else
              ua(j1,j2,j3,3)=ua(j1,j2,j3,3)-zavrdt*ur(3)+zavldt*ul(3)
              ua(j1,j2,j3,4)=ua(j1,j2,j3,4)-zavrdt*ur(4)+zavldt*ul(4)
              ua(j1,j2,j3,5)=ua(j1,j2,j3,5)-zavrdt*ur(5)+zavldt*ul(5) &
                -dtmom*rdx(j3)*ygg(j3,4)
#endif
#ifdef mhd
!        7.5       Magnetic field
              ua(j1,j2,j3,6)=ua(j1,j2,j3,6)+rvol(j3)*dth* &
                (-ur(6)*vpr(3)+ur(8)*vpr(1)               &
                 +ul(6)*vpl(3)-ul(8)*vpl(1))

              ua(j1,j2,j3,7)=ua(j1,j2,j3,7)+rvol(j3)*dth* &
                (-ur(7)*vpr(3)+ur(8)*vpr(2)               & 
                 +ul(7)*vpl(3)-ul(8)*vpl(2))
              iv=8
#else
              iv=5
#endif
!        7.6       Supplementary variables
#ifdef hydcon
              iv=iv+1
              ua(j1,j2,j3,iv)=ua(j1,j2,j3,iv)-zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#ifdef mhdcon
              iv=iv+1
              ua(j1,j2,j3,iv)=ua(j1,j2,j3,iv)-zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      8.        Div(B) correction at the inner boundary
! do j3=nc3lm,nc3rp
!   do j2=nc2lm,nc2rp
!     rvol(j2)=rx1(3,jb)*rsint(j2,jb)*rdx2(j2,jb)
!     ua(2,j2,j3,6)=ax1(4,jb)*(ua(4,j2,j3,6)+rvol1(3,jb)*rvol(j2)*    &
!       (ax2(j2-1,jb)*ua(3,j2-1,j3,7)-ax2(j2-1,jb)*ua(3,j2-1,j3,7)))/ &
!       ax1(2,jb)
!   enddo
! enddo
!---------------------------------------------------------------------
!      6.        Equation of state (full step)
  do j3=nc3lm,nc3rp
    do j2=nc2lm,nc2rp
      do j1=nc1lm,nc1rp
!        3.1       Velocity
        zrd=zone/max(zmin,ua(j1,j2,j3,1))
        vp(j1,j2,j3,1)=ua(j1,j2,j3,3)*zrd
        vp(j1,j2,j3,2)=ua(j1,j2,j3,4)*zrd
        vp(j1,j2,j3,3)=ua(j1,j2,j3,5)*zrd
#ifndef con
!        3.2       Pressure
#ifdef mhd
            vp(j1,j2,j3,5)=zr2mu*(ua(j1,j2,j3,6)*ua(j1,j2,j3,6) &
                                 +ua(j1,j2,j3,7)*ua(j1,j2,j3,7) &
                                 +ua(j1,j2,j3,8)*ua(j1,j2,j3,8))
#endif
#ifdef ethe
            vp(j1,j2,j3,4)=gammam*ua(j1,j2,j3,2)
#else
#ifdef etot
#ifdef mhd
                vp(j1,j2,j3,4)=gammam*(ua(j1,j2,j3,2)  &
                  -z05*(ua(j1,j2,j3,3)*vp(j1,j2,j3,1)  &
                       +ua(j1,j2,j3,4)*vp(j1,j2,j3,2)  &
                       +ua(j1,j2,j3,5)*vp(j1,j2,j3,3)) &
                       -vp(j1,j2,j3,5))
#else
                vp(j1,j2,j3,4)=gammam*(ua(j1,j2,j3,2) &
                  -z05*(ua(j1,j2,j3,3)*vp(j1,j2,j3,1) &
                       +ua(j1,j2,j3,4)*vp(j1,j2,j3,2) &
                       +ua(j1,j2,j3,5)*vp(j1,j2,j3,3)))
#endif
#else
              vp(j1,j2,j3,4)=gammam*(ua(j1,j2,j3,2) &
                -z05*(ua(j1,j2,j3,3)*vp(j1,j2,j3,1) &
                     +ua(j1,j2,j3,4)*vp(j1,j2,j3,2) &
                     +ua(j1,j2,j3,5)*vp(j1,j2,j3,3)))
#endif
#endif
#ifdef mhd
             vp(j1,j2,j3,6)=vp(j1,j2,j3,4)+vp(j1,j2,j3,5)
#endif
#endif
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      4.        X1-Limited interface values
  do j2=nc2lm,nc2rp
#ifdef spherical
      zcot=cott(j2,jb)
#endif
    do j3=nc3lm,nc3rp
!        4.1       Main variables
      do jv=1,nnvar
        do j1=2,nc1
          difh(j1)=uo(j1,j2,j3,jv)-uo(j1-1,j2,j3,jv)
        enddo
        do j1=2,nc1-1
#ifdef minmod
            yg(j1,jv)=sign(zhalf,difh(j1))*max(zero, &
              min(abs(difh(j1)),sign(zone,difh(j1))*difh(j1+1)))
#endif
#ifdef woodward
            zs=sign(zone,difh(j1))
            yg(j1,jv)=zhalf*zs*max(zero,  &
              min(abs(difh(j1)+difh(j1)), &
              zs*(difh(j1+1)+difh(j1+1)), &
              zhalf*zs*(difh(j1)+difh(j1+1))))
#endif
        enddo
      enddo
!        4.2       Auxiliary variables
      do jv=1,npvar
        do j1=2,nc1
          difh(j1)=vpo(j1,j2,j3,jv)-vpo(j1-1,j2,j3,jv)
        enddo
        do j1=2,nc1-1
#ifdef minmod
            ygg(j1,jv)=sign(zhalf,difh(j1))*max(zero, &
              min(abs(difh(j1)),sign(zone,difh(j1))*difh(j1+1)))
#endif
#ifdef woodward
            zs=sign(zone,difh(j1))
            ygg(j1,jv)=zhalf*zs*max(zero, &
              min(abs(difh(j1)+difh(j1)), &
              zs*(difh(j1+1)+difh(j1+1)), &
              zhalf*zs*(difh(j1)+difh(j1+1))))
#endif
        enddo
      enddo
!---------------------------------------------------------------------
!      7.        X1-Fluxes at the full step
      do j1=nc1l,nc1rp
!Values at the cell interfaces
#ifdef upwind
          do jv=1,nnvar
            ul(jv)=ua(j1-1,j2,j3,jv)+yg(j1-1,jv)
            ur(jv)=ua(j1,j2,j3,jv)-yg(j1,jv)
          enddo
          do jv=1,npvar
            vpl(jv)=vp(j1-1,j2,j3,jv)+ygg(j1-1,jv)
            vpr(jv)=vp(j1,j2,j3,jv)-ygg(j1,jv)
          enddo
#else
          do jv=1,nnvar
            ul(jv)=ua(j1-1,j2,j3,jv)+yg(j1,jv)
            ur(jv)=ua(j1,j2,j3,jv)-yg(j1-1,jv)
          enddo
          do jv=1,npvar
            vpl(jv)=vp(j1-1,j2,j3,jv)+ygg(j1,jv)
            vpr(jv)=vp(j1,j2,j3,jv)-ygg(j1-1,jv)
          enddo
#endif
#ifdef con
          zcsc1=abs(vpl(1))
          zcsc2=abs(vpr(1))
#else
#ifdef gam3d
#ifdef mhd
              zcsc1=abs(vpl(1))+sqrt((gamma(j1,j2,j3,jb)*max(zero,vpl(4)) &
                +ztwom*vpl(5))/max(zmin,ul(1)))
              zcsc2=abs(vpr(1))+sqrt((gamma(j1,j2,j3,jb)*max(zero,vpr(4)) &
                +ztwom*vpr(5))/max(zmin,ur(1)))
#else
              zcsc1=abs(vpl(1))+sqrt(gamma(j1,j2,j3,jb)*max(zero,vpl(4))/ &
                max(zmin,ul(1)))
              zcsc2=abs(vpr(1))+sqrt(gamma(j1,j2,j3,jb)*max(zero,vpr(4))/ &
                max(zmin,ur(1)))
#endif
#else
#ifdef mhd
              zcsc1=abs(vpl(1))+sqrt((gamma*max(zero,vpl(4)) &
                +ztwom*vpl(5))/max(zmin,ul(1)))
              zcsc2=abs(vpr(1))+sqrt((gamma*max(zero,vpr(4)) &
                +ztwom*vpr(5))/max(zmin,ur(1)))
#else
              zcsc1=abs(vpl(1))+sqrt(gamma*max(zero,vpl(4))/ &
                max(zmin,ul(1)))
              zcsc2=abs(vpr(1))+sqrt(gamma*max(zero,vpr(4))/ &
                max(zmin,ur(1)))
#endif
#endif
          zcsc=max(zcsc1,zcsc2)
          cfl1mx(jb)=max(cfl1mx(jb),rdx1(j1,jb)*zcsc)
#endif
!        6.3       Fluxes
        zavrdt=dth*ah1(j1,jb)*(vpr(1)-zcsc)
        zavldt=dth*ah1(j1,jb)*(vpl(1)+zcsc)
        zcdt=dth*zcsc
#ifdef mhd
          zabrdt=dthmag*ah1(j1,jb)*ur(6)
          zabldt=dthmag*ah1(j1,jb)*ul(6)
          zer=ur(6)*vpr(1)+ur(7)*vpr(2)+ur(8)*vpr(3)
          zel=ul(6)*vpl(1)+ul(7)*vpl(2)+ul(8)*vpl(3)
#endif
!        6.4       Fluxes
        flxg(j1,1)=zavrdt*ur(1)+zavldt*ul(1)
#ifndef con
!        6.7       Fluxes
#ifdef ethe
            flxg(j1,2)=zavrdt*ur(2)+zavldt*ul(2)
#else
#ifdef etot
#ifdef mhd
                flxg(j1,2)=zavrdt*ur(2)+zavldt*ul(2)            &
                  +dth*ah1(j1,jb)*(vpr(6)*vpr(1)+vpl(6)*vpl(1)) &
                  -dthene*ah1(j1,jb)*(zer*ur(6)+zel*ul(6))
#else
                flxg(j1,2)=zavrdt*ur(2)+zavldt*ul(2) &
                  +dth*ah1(j1,jb)*(vpr(4)*vpr(1)+vpl(4)*vpl(1))
#endif
#else
              flxg(j1,2)=zavrdt*ur(2)+zavldt*ul(2) &
                +dth*ah1(j1,jb)*(vpr(4)*vpr(1)+vpl(4)*vpl(1))
#endif
#endif
          flxh(j1)=zavrdt+zavldt
!        6.5       Fluxes
#ifdef mhd
            flxg(j1,3)=zavrdt*ur(3)+zavldt*ul(3) &
                      -zabrdt*ur(6)-zabldt*ul(6)
            flxg(j1,4)=zavrdt*ur(4)+zavldt*ul(4) &
                      -zabrdt*ur(7)-zabldt*ul(7)
            flxg(j1,5)=zavrdt*ur(5)+zavldt*ul(5) &
                      -zabrdt*ur(8)-zabldt*ul(8)
            difh(j1)=dthmom*(vpr(6)+vpl(6))
#else
            flxg(j1,3)=zavrdt*ur(3)+zavldt*ul(3)
            flxg(j1,4)=zavrdt*ur(4)+zavldt*ul(4)
            flxg(j1,5)=zavrdt*ur(5)+zavldt*ul(5)
            difh(j1)=dthmom*(vpr(4)+vpl(4))
#endif
!        6.6       Fluxes
#ifdef mhd
            flxg(j1,7)=ah12(j1)*(dth*                               &
              (ur(7)*vpr(1)+ul(7)*vpl(1)-ur(6)*vpr(2)-ul(6)*vpl(2)) &
              +zcdt*(ul(7)-ur(7)))
            flxg(j1,8)=ah13(j1)*(dth*                               &
              (ur(8)*vpr(1)+ul(8)*vpl(1)-ur(6)*vpr(3)-ul(6)*vpl(3)) &
              +zcdt*(ul(8)-ur(8)))
            iv=8
#else
            iv=5
#endif
!        6.7       Fluxes
#ifdef hydcon
            iv=iv+1
            flxg(j1,iv)=zavrdt*ur(iv)+zavldt*ul(iv)
#endif

#ifdef mhdcon
            iv=iv+1
            flxg(j1,iv)=zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#endif
      enddo
!---------------------------------------------------------------------
!      7.        X1-Corrector
      do j1=nc1l,nc1r
!        7.2       Continuity equations
        un(j1,j2,j3,1)=uo(j1,j2,j3,1)  &
          +rvol1(j1,jb)*(-flxg(j1+1,1)+flxg(j1,1))
#ifdef con
          un(j1,j2,j3,3)=uo(j1,j2,j3,1)*vp(j1,j2,j3,1)
          un(j1,j2,j3,4)=uo(j1,j2,j3,1)*vp(j1,j2,j3,2)
          un(j1,j2,j3,5)=uo(j1,j2,j3,1)*vp(j1,j2,j3,3)
#else
!        7.3       Energy equations
#ifdef etot
            un(j1,j2,j3,2)=uo(j1,j2,j3,2)              &
              +rvol1(j1,jb)*(-flxg(j1+1,2)+flxg(j1,2)) &
              +zngdt*ua(j1,j2,j3,3)*rxx1(j1,jb)
#else
            un(j1,j2,j3,2)=uo(j1,j2,j3,2)                             &
              +rvol1(j1,jb)*(-flxg(j1+1,2)+flxg(j1,2)+vp(j1,j2,j3,4)* &
                 (-flxh(j1+1)+flxh(j1)))
#endif
!        7.4       Momentum equations
#ifdef mhd
            un(j1,j2,j3,3)=uo(j1,j2,j3,3)                      &
              +rvol1(j1,jb)*(-flxg(j1+1,3)+flxg(j1,3))         &
              +rdx1(j1,jb)*(-difh(j1+1)+difh(j1))              &
              +zgdt*ua(j1,j2,j3,1)*rxx1(j1,jb)                 &
              +zclsp*rx1(j1,jb)*(ua(j1,j2,j3,4)*vp(j1,j2,j3,2) &
                 -zrmu*ua(j1,j2,j3,7)*ua(j1,j2,j3,7))          &
              +zsp*rx1(j1,jb)*(ua(j1,j2,j3,5)*vp(j1,j2,j3,3)   &
                 -zrmu*ua(j1,j2,j3,8)*ua(j1,j2,j3,8))
            un(j1,j2,j3,4)=uo(j1,j2,j3,4)                      &
              +rvol1(j1,jb)*(-flxg(j1+1,4)+flxg(j1,4))         &
              -zclsp*rx1(j1,jb)*(ua(j1,j2,j3,3)*vp(j1,j2,j3,2) &
                -zrmu*ua(j1,j2,j3,6)*ua(j1,j2,j3,7))           &
              +zsp*rx1(j1,jb)*(ua(j1,j2,j3,5)*vp(j1,j2,j3,3)   &
                -zrmu*ua(j1,j2,j3,8)*ua(j1,j2,j3,8))*zcot
            un(j1,j2,j3,5)=uo(j1,j2,j3,5)              &
              +rvol1(j1,jb)*(-flxg(j1+1,5)+flxg(j1,5)) &
              -zsp*rx1(j1,jb)*(vp(j1,j2,j3,3)*         &
                 (ua(j1,j2,j3,3)+ua(j1,j2,j3,4)*zcot)  &
                 -zrmu*ua(j1,j2,j3,8)*                 &
                 (ua(j1,j2,j3,6)+ua(j1,j2,j3,7)*zcot))
#else
            un(j1,j2,j3,3)=uo(j1,j2,j3,3)                     &
              +rvol1(j1,jb)*(-flxg(j1+1,3)+flxg(j1,3))        &
              +rdx1(j1,jb)*(-difh(j1+1)+difh(j1))             &
              +zgdt*ua(j1,j2,j3,1)*rxx1(j1,jb)                &
              +zclsp*rx1(j1,jb)*ua(j1,j2,j3,4)*vp(j1,j2,j3,2) &
              +zsp*rx1(j1,jb)*ua(j1,j2,j3,5)*vp(j1,j2,j3,3)
            un(j1,j2,j3,4)=uo(j1,j2,j3,4)                     &
              +rvol1(j1,jb)*(-flxg(j1+1,4)+flxg(j1,4))        &
              -zclsp*rx1(j1,jb)*ua(j1,j2,j3,3)*vp(j1,j2,j3,2) &
              +zsp*rx1(j1,jb)*ua(j1,j2,j3,4)*vp(j1,j2,j3,3)*zcot
            un(j1,j2,j3,5)=uo(j1,j2,j3,5)              &
              +rvol1(j1,jb)*(-flxg(j1+1,5)+flxg(j1,5)) &
              -zsp*rx1(j1,jb)*vp(j1,j2,j3,3)*          &
                (ua(j1,j2,j3,3)+ua(j1,j2,j3,4)*zcot)
#endif
!        7.5       Magnetic field equations
#ifdef mhd
            un(j1,j2,j3,6)=uo(j1,j2,j3,6)
            un(j1,j2,j3,7)=uo(j1,j2,j3,7) &
              +rvol12(j1)*(-flxg(j1+1,7)+flxg(j1,7))
            un(j1,j2,j3,8)=uo(j1,j2,j3,8) &
              +rvol13(j1)*(-flxg(j1+1,8)+flxg(j1,8))
            iv=8
#else
            iv=5
#endif
!        7.6       Continuity equations
#ifdef hydcon
            iv=iv+1
            un(j1,j2,j3,iv)=uo(j1,j2,j3,iv) &
              +rvol1(j1,jb)*(-flxg(j1+1,iv)+flxg(j1,iv))
#endif
#ifdef mhdcon
            iv=iv+1
            un(j1,j2,j3,iv)=uo(j1,j2,j3,iv) &
              +rvol1(j1,jb)*(-flxg(j1+1,iv)+flxg(j1,iv))
#endif
#endif
      enddo
#ifdef PARAMESH
#ifdef consflux
!        8.1       Save fluxes at block boundaries
          do jv=1,nnvar
            flux_x(jv,1,j2,j3,jb)=flxg(nc1l,jv)
            flux_x(jv,2,j2,j3,jb)=flxg(nc1rp,jv)
          enddo
          flux_x(3,1,j2,j3,jb)=flux_x(3,1,j2,j3,jb)+difh(nc1l)
          flux_x(3,2,j2,j3,jb)=flux_x(3,2,j2,j3,jb)+difh(nc1rp)
#endif
#endif
    enddo
  enddo
!---------------------------------------------------------------------
!      4.        X2-Limited interface values
#ifdef x2dim
    do j3=nc3lm,nc3rp
      do j1=nc1lm,nc1rp
!        4.1       Main variables
        do jv=1,nnvar
          do j2=2,nc2
            difh(j2)=uo(j1,j2,j3,jv)-uo(j1,j2-1,j3,jv)
          enddo
          do j2=2,nc2-1
#ifdef minmod
              yg(j2,jv)=sign(zhalf,difh(j2))*max(zero, &
                min(abs(difh(j2)),sign(zone,difh(j2))*difh(j2+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j2))
              yg(j2,jv)=zhalf*zs*max(zero,  &
                min(abs(difh(j2)+difh(j2)), &
                zs*(difh(j2+1)+difh(j2+1)), &
                zhalf*zs*(difh(j2)+difh(j2+1))))
#endif
          enddo
        enddo
!        4.2       Auxiliary variables
        do jv=1,npvar
          do j2=2,nc2
            difh(j2)=vpo(j1,j2,j3,jv)-vpo(j1,j2-1,j3,jv)
          enddo
          do j2=2,nc2-1
#ifdef minmod
              ygg(j2,jv)=sign(zhalf,difh(j2))*max(zero, &
                min(abs(difh(j2)),sign(zone,difh(j2))*difh(j2+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j2))
              ygg(j2,jv)=zhalf*zs*max(zero, &
                min(abs(difh(j2)+difh(j2)), &
                zs*(difh(j2+1)+difh(j2+1)), &
                zhalf*zs*(difh(j2)+difh(j2+1))))
#endif
          enddo
        enddo
!        4.5       Auxiliary terms
        do j2=2,nc2-1
#ifdef cartesian
            rdx(j2)=rdx2(j2,jb)
            rvol(j2)=rdx2(j2,jb)
            rvol23(j2)=rdx2(j2,jb)
            vol(j2)=dx2(j2,jb)
            vol23(j2)=dx2(j2,jb)
#endif
#ifdef cylindrical
            rdx(j2)=rx1(j1,jb)*rdx2(j2,jb)
            rvol(j2)=rdx(j2)
            rvol23(j2)=rdx(j2)
            vol(j2)=xc1(j1,jb)*dx2(j2,jb)
            vol23(j2)=xc1(j1,jb)*dx2(j2,jb)
#endif
#ifdef spherical
            rdx(j2)=rx1(j1,jb)*rdx2(j2,jb)
            rvol(j2)=rdx(j2)*rsint(j2,jb)
            rvol23(j2)=rdx(j2)
            vol(j2)=xc1(j1,jb)*sint(j2,jb)*dx2(j2,jb)
            vol23(j2)=xc1(j1,jb)*dx2(j2,jb)
#endif
        enddo
!---------------------------------------------------------------------
!      7.        X2-Fluxes at the full step
        do j2=nc2l,nc2rp
!Values at the cell interfaces
#ifdef upwind
            do jv=1,nnvar
              ul(jv)=ua(j1,j2-1,j3,jv)+yg(j2-1,jv)
              ur(jv)=ua(j1,j2,j3,jv)-yg(j2,jv)
            enddo
            do jv=1,npvar
              vpl(jv)=vp(j1,j2-1,j3,jv)+ygg(j2-1,jv)
              vpr(jv)=vp(j1,j2,j3,jv)-ygg(j2,jv)
            enddo
#else
            do jv=1,nnvar
              ul(jv)=ua(j1,j2-1,j3,jv)+yg(j2,jv)
              ur(jv)=ua(j1,j2,j3,jv)-yg(j2-1,jv)
            enddo
            do jv=1,npvar
              vpl(jv)=vp(j1,j2-1,j3,jv)+ygg(j2,jv)
              vpr(jv)=vp(j1,j2,j3,jv)-ygg(j2-1,jv)
            enddo
#endif
#ifdef con
            zcsc1=abs(vpl(2))
            zcsc2=abs(vpr(2))
#else
#ifdef gam3d
#ifdef mhd
                zcsc1=abs(vpl(2))+sqrt((gamma(j1,j2,j3,jb)*max(zero,vpl(4)) &
                  +ztwom*vpl(5))/max(zmin,ul(1)))
                zcsc2=abs(vpr(2))+sqrt((gamma(j1,j2,j3,jb)*max(zero,vpr(4)) &
                  +ztwom*vpr(5))/max(zmin,ur(1)))
#else
                zcsc1=abs(vpl(2))+sqrt(gamma(j1,j2,j3,jb)*max(zero,vpl(4))/ &
                  max(zmin,ul(1)))
                zcsc2=abs(vpr(2))+sqrt(gamma(j1,j2,j3,jb)*max(zero,vpr(4))/ &
                  max(zmin,ur(1)))
#endif
#else
#ifdef mhd
                zcsc1=abs(vpl(2))+sqrt((gamma*max(zero,vpl(4)) &
                  +ztwom*vpl(5))/max(zmin,ul(1)))
                zcsc2=abs(vpr(2))+sqrt((gamma*max(zero,vpr(4)) &
                  +ztwom*vpr(5))/max(zmin,ur(1)))
#else
                zcsc1=abs(vpl(2))+sqrt(gamma*max(zero,vpl(4))/ &
                  max(zmin,ul(1)))
                zcsc2=abs(vpr(2))+sqrt(gamma*max(zero,vpr(4))/ &
                  max(zmin,ur(1)))
#endif
#endif
            zcsc=max(zcsc1,zcsc2)
            cfl2mx(jb)=max(cfl2mx(jb),rdx(j2)*zcsc)
#endif

!        6.3       Fluxes
          zavrdt=dth*ah2(j2,jb)*(vpr(2)-zcsc)
          zavldt=dth*ah2(j2,jb)*(vpl(2)+zcsc)
          zcdt=dth*zcsc
#ifdef mhd
            zabrdt=dthmag*ah2(j2,jb)*ur(7)
            zabldt=dthmag*ah2(j2,jb)*ul(7)
            zer=ur(6)*vpr(1)+ur(7)*vpr(2)+ur(8)*vpr(3)
            zel=ul(6)*vpl(1)+ul(7)*vpl(2)+ul(8)*vpl(3)
#endif
!        6.4       Fluxes
          flxg(j2,1)=zavrdt*ur(1)+zavldt*ul(1)

#ifndef con
!        6.7       Fluxes
#ifdef etot
#ifdef mhd
                flxg(j2,2)=zavrdt*ur(2)+zavldt*ul(2)             &
                  +dth*ah2(j2,jb)*(vpr(6)*vpr(2)+vpl(6)*vpl(2))  &
                  -dthene*ah2(j2,jb)*(zer*ur(7)+zel*ul(7))
#else
                flxg(j2,2)=zavrdt*ur(2)+zavldt*ul(2) &
                  +dth*ah2(j2,jb)*(vpr(4)*vpr(2)+vpl(4)*vpl(2))
#endif
#else
              flxg(j2,2)=zavrdt*ur(2)+zavldt*ul(2)
              flxh(j2)=zavrdt+zavldt
#endif
!        6.5       Fluxes
#ifdef mhd
              flxg(j2,3)=zavrdt*ur(3)+zavldt*ul(3) &
                        -zabrdt*ur(6)-zabldt*ul(6)
              flxg(j2,4)=zavrdt*ur(4)+zavldt*ul(4) &
                        -zabrdt*ur(7)-zabldt*ul(7)
              flxg(j2,5)=zavrdt*ur(5)+zavldt*ul(5) &
                        -zabrdt*ur(8)-zabldt*ul(8)
              difh(j2)=dthmom*(vpr(6)+vpl(6))
#else
              flxg(j2,3)=zavrdt*ur(3)+zavldt*ul(3)
              flxg(j2,4)=zavrdt*ur(4)+zavldt*ul(4)
              flxg(j2,5)=zavrdt*ur(5)+zavldt*ul(5)
              difh(j2)=dthmom*(vpr(4)+vpl(4))
#endif
!        6.6       Fluxes
#ifdef mhd
              flxg(j2,6)=ah2(j2,jb)*(dth*(ur(6)*vpr(2)+ul(6)*vpl(2)  &
                                         -ur(7)*vpr(1)-ul(7)*vpl(1)) &
                                   +zcdt*(ul(6)-ur(6)))
              flxg(j2,8)=dth*(ur(8)*vpr(2)+ul(8)*vpl(2)  &
                             -ur(7)*vpr(3)-ul(7)*vpl(3)) &
                       +zcdt*(ul(8)-ur(8))
              iv=8
#else
              iv=5
#endif
!        6.7       Fluxes
#ifdef hydcon
              iv=iv+1
              flxg(j2,iv)=zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#ifdef mhdcon
              iv=iv+1
              flxg(j2,iv)=zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#endif
        enddo
!---------------------------------------------------------------------
!      7.        X2-Corrector
        do j2=nc2l,nc2r
!        7.2       Continuity equations
          un(j1,j2,j3,1)=un(j1,j2,j3,1) &
            +rvol(j2)*(-flxg(j2+1,1)+flxg(j2,1))
#ifdef con
            un(j1,j2,j3,3)=un(j1,j2,j3,1)*vp(j1,j2,j3,1)
            un(j1,j2,j3,4)=un(j1,j2,j3,1)*vp(j1,j2,j3,2)
            un(j1,j2,j3,5)=un(j1,j2,j3,1)*vp(j1,j2,j3,3)
#else
!        7.3       Energy equations
#ifdef etot
              un(j1,j2,j3,2)=un(j1,j2,j3,2) &
                +rvol(j2)*(-flxg(j2+1,2)+flxg(j2,2))
#else
              un(j1,j2,j3,2)=un(j1,j2,j3,2)         &
                +rvol(j2)*(-flxg(j2+1,2)+flxg(j2,2) &
                +vp(j1,j2,j3,4)*(-flxh(j2+1)+flxh(j2)))
#endif
!        7.4       Momentum equations
            un(j1,j2,j3,3)=un(j1,j2,j3,3) &
              +rvol(j2)*(-flxg(j2+1,3)+flxg(j2,3))
            un(j1,j2,j3,4)=un(j1,j2,j3,4)          &
              +rvol(j2)*(-flxg(j2+1,4)+flxg(j2,4)) &
              +rdx(j2)*(-difh(j2+1)+ difh(j2))
            un(j1,j2,j3,5)=un(j1,j2,j3,5) &
              +rvol(j2)*(-flxg(j2+1,5)+flxg(j2,5))
!        7.5       Magnetic field equations
#ifdef mhd
              un(j1,j2,j3,6)=un(j1,j2,j3,6) &
                +rvol(j2)*(-flxg(j2+1,6)+flxg(j2,6))
              un(j1,j2,j3,8)=un(j1,j2,j3,8) &
                +rvol23(j2)*(-flxg(j2+1,8)+flxg(j2,8))
              iv=8
#else
              iv=5
#endif
!        7.6       Continuity equations
#ifdef hydcon
              iv=iv+1
              un(j1,j2,j3,iv)=un(j1,j2,j3,iv) &
                +rvol(j2)*(-flxg(j2+1,iv)+flxg(j2,iv))
#endif
#ifdef mhdcon
              iv=iv+1
              un(j1,j2,j3,iv)=un(j1,j2,j3,iv) &
                +rvol(j2)*(-flxg(j2+1,iv)+flxg(j2,iv))
#endif
#endif
        enddo
#ifdef PARAMESH
#ifdef consflux
!        8.1       Save fluxes at block boundaries
            do jv=1,nnvar
              flux_y(jv,j1,1,j3,jb)=flxg(nc2l,jv)
              flux_y(jv,j1,2,j3,jb)=flxg(nc2rp,jv)
            enddo
            flux_y(4,j1,1,j3,jb)=flux_y(4,j1,1,j3,jb)+difh(nc2l)
            flux_y(4,j1,2,j3,jb)=flux_y(4,j1,2,j3,jb)+difh(nc2rp)
#endif
#endif
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      4.        X3-Limited interface values
#ifdef x3dim
    do j2=nc2lm,nc2rp
      do j1=nc1lm,nc1rp
!        4.1       Main variables
        do jv=1,nnvar
          do j3=2,nc3
            difh(j3)=uo(j1,j2,j3,jv)-uo(j1,j2,j3-1,jv)
          enddo
          do j3=2,nc3-1
#ifdef minmod
              yg(j3,jv)=sign(zhalf,difh(j3))*max(zero, &
                min(abs(difh(j3)),sign(zone,difh(j3))*difh(j3+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j3))
              yg(j3,jv)=zhalf*zs*max(zero,  &
                min(abs(difh(j3)+difh(j3)), &
                zs*(difh(j3+1)+difh(j3+1)), &
                zhalf*zs*(difh(j3)+difh(j3+1))))
#endif
          enddo
        enddo
!        4.2       Auxiliary variables
        do jv=1,npvar
          do j3=2,nc3
            difh(j3)=vpo(j1,j2,j3,jv)-vpo(j1,j2,j3-1,jv)
          enddo

          do j3=2,nc3-1
#ifdef minmod
              ygg(j3,jv)=sign(zhalf,difh(j3))*max(zero, &
                min(abs(difh(j3)),sign(zone,difh(j3))*difh(j3+1)))
#endif
#ifdef woodward
              zs=sign(zone,difh(j3))
              ygg(j3,jv)=zhalf*zs*max(zero, &
                min(abs(difh(j3)+difh(j3)), &
                zs*(difh(j3+1)+difh(j3+1)), &
                zhalf*zs*(difh(j3)+difh(j3+1))))
#endif
          enddo
        enddo
!        4.5       Auxiliary terms
        do j3=2,nc3-1
#ifdef cartesian
            rdx(j3)=rdx3(j3,jb)
            rvol(j3)=rdx3(j3,jb)
            vol(j3)=dx3(j3,jb)
#endif

#ifdef cylindrical
            rdx(j3)=rdx3(j3,jb)
            rvol(j3)=rdx3(j3,jb)
            vol(j3)=dx3(j3,jb)
#endif
#ifdef spherical
            rdx(j3)=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)
            rvol(j3)=rdx(j3)
            vol(j3)=xc1(j1,jb)*sint(j2,jb)*dx3(j3,jb)
#endif
        enddo
!---------------------------------------------------------------------
!      7.        X3-Fluxes at the full step
        do j3=nc3l,nc3rp
!Values at the cell interfaces
#ifdef upwind
            do jv=1,nnvar
              ul(jv)=ua(j1,j2,j3-1,jv)+yg(j3-1,jv)
              ur(jv)=ua(j1,j2,j3,jv)-yg(j3,jv)
            enddo
            do jv=1,npvar
              vpl(jv)=vp(j1,j2,j3-1,jv)+ygg(j3-1,jv)
              vpr(jv)=vp(j1,j2,j3,jv)-ygg(j3,jv)
            enddo
#else
            do jv=1,nnvar
              ul(jv)=ua(j1,j2,j3-1,jv)+yg(j3,jv)
              ur(jv)=ua(j1,j2,j3,jv)-yg(j3-1,jv)
            enddo
            do jv=1,npvar
              vpl(jv)=vp(j1,j2,j3-1,jv)+ygg(j3,jv)
              vpr(jv)=vp(j1,j2,j3,jv)-ygg(j3-1,jv)
            enddo
#endif
#ifdef con
            zcsc1=abs(vpl(3))
            zcsc2=abs(vpr(3))
#else
#ifdef gam3d
#ifdef mhd
                 zcsc1=abs(vpl(3))+sqrt((gamma(j1,j2,j3,jb)*max(zero,vpl(4)) &
                      +ztwom*vpl(5))/max(zmin,ul(1)))
                 zcsc2=abs(vpr(3))+sqrt((gamma(j1,j2,j3,jb)*max(zero,vpr(4)) &
                      +ztwom*vpr(5))/max(zmin,ur(1)))
#else
                 zcsc1=abs(vpl(3))+sqrt(gamma(j1,j2,j3,jb)*max(zero,vpl(4)) &
                      /max(zmin,ul(1)))
                 zcsc2=abs(vpr(3))+sqrt(gamma(j1,j2,j3,jb)*max(zero,vpr(4)) &
                      /max(zmin,ur(1)))
#endif
#else
#ifdef mhd
                 zcsc1=abs(vpl(3))+sqrt((gamma*max(zero,vpl(4)) &
                      +ztwom*vpl(5))/max(zmin,ul(1)))
                 zcsc2=abs(vpr(3))+sqrt((gamma*max(zero,vpr(4)) &
                      +ztwom*vpr(5))/max(zmin,ur(1)))
#else
                 zcsc1=abs(vpl(3))+sqrt(gamma*max(zero,vpl(4)) &
                      /max(zmin,ul(1)))
                 zcsc2=abs(vpr(3))+sqrt(gamma*max(zero,vpr(4)) &
                      /max(zmin,ur(1)))
#endif
#endif
            zcsc=max(zcsc1,zcsc2)
            cfl3mx(jb)=max(cfl3mx(jb),rdx(j3)*zcsc)
#endif
!        6.3       Fluxes
          zavrdt=dth*(vpr(3)-zcsc)
          zavldt=dth*(vpl(3)+zcsc)
          zcdt=dth*zcsc
#ifdef mhd
            zabrdt=dthmag*ur(8)
            zabldt=dthmag*ul(8)
            zer=ur(6)*vpr(1)+ur(7)*vpr(2)+ur(8)*vpr(3)
            zel=ul(6)*vpl(1)+ul(7)*vpl(2)+ul(8)*vpl(3)
#endif
!        6.4       Fluxes
          flxg(j3,1)=zavrdt*ur(1)+zavldt*ul(1)

#ifndef con
!        6.7       Fluxes
#ifdef etot
#ifdef mhd
                flxg(j3,2)=zavrdt*ur(2)+zavldt*ul(2) &
                  +dth*(vpr(6)*vpr(3)+vpl(6)*vpl(3)) &
                  -dthene*(zer*ur(8)+zel*ul(8))
#else
                flxg(j3,2)=zavrdt*ur(2)+zavldt*ul(2) &
                  +dth*(vpr(4)*vpr(3)+vpl(4)*vpl(3))
#endif
#else
              flxg(j3,2)=zavrdt*ur(2)+zavldt*ul(2)
              flxh(j3)=zavrdt+zavldt
#endif
!        6.5       Fluxes
#ifdef mhd
              flxg(j3,3)=zavrdt*ur(3)+zavldt*ul(3) &
                        -zabrdt*ur(6)-zabldt*ul(6)
              flxg(j3,4)=zavrdt*ur(4)+zavldt*ul(4) &
                        -zabrdt*ur(7)-zabldt*ul(7)
              flxg(j3,5)=zavrdt*ur(5)+zavldt*ul(5) &
                        -zabrdt*ur(8)-zabldt*ul(8)
              difh(j3)=dthmom*(vpr(6)+vpl(6))
#else
              flxg(j3,3)=zavrdt*ur(3)+zavldt*ul(3)
              flxg(j3,4)=zavrdt*ur(4)+zavldt*ul(4)
              flxg(j3,5)=zavrdt*ur(5)+zavldt*ul(5)
              difh(j3)=dthmom*(vpr(4)+vpl(4))
#endif
!        6.6       Fluxes
#ifdef mhd
              flxg(j3,6)=dth*(ur(6)*vpr(3)+ul(6)*vpl(3)  &
                             -ur(8)*vpr(1)-ul(8)*vpl(1)) &
                +zcdt*(ul(6)-ur(6))
              flxg(j3,7)=dth*(ur(7)*vpr(3)+ul(7)*vpl(3)  &
                             -ur(8)*vpr(2)-ul(8)*vpl(2)) &
                +zcdt*(ul(7)-ur(7))
              iv=8
#else
              iv=5
#endif
!        6.7       Fluxes
#ifdef hydcon
              iv=iv+1
              flxg(j3,iv)=zavrdt*ur(iv)+zavldt*ul(iv)
#endif

#ifdef mhdcon
              iv=iv+1
              flxg(j3,iv)=zavrdt*ur(iv)+zavldt*ul(iv)
#endif
#endif
        enddo
!---------------------------------------------------------------------
!      7.        X3-Corrector
        do j3=nc3l,nc3r
!        7.2       Continuity equations
          un(j1,j2,j3,1)=un(j1,j2,j3,1) &
            +rvol(j3)*(-flxg(j3+1,1)+flxg(j3,1))
#ifdef con
            un(j1,j2,j3,3)=un(j1,j2,j3,1)*vp(j1,j2,j3,1)
            un(j1,j2,j3,4)=un(j1,j2,j3,1)*vp(j1,j2,j3,2)
            un(j1,j2,j3,5)=un(j1,j2,j3,1)*vp(j1,j2,j3,3)
#else
!        7.3       Energy equations
#ifdef etot
              un(j1,j2,j3,2)=un(j1,j2,j3,2) &
                +rvol(j3)*(-flxg(j3+1,2)+flxg(j3,2))
#else
              un(j1,j2,j3,2)=un(j1,j2,j3,2)         &
                +rvol(j3)*(-flxg(j3+1,2)+flxg(j3,2) &
                  +vp(j1,j2,j3,4)*(-flxh(j3+1)+flxh(j3)))
#endif
!        7.4       Momentum equations
            un(j1,j2,j3,3)=un(j1,j2,j3,3) &
              +rvol(j3)*(-flxg(j3+1,3)+flxg(j3,3))
            un(j1,j2,j3,4)=un(j1,j2,j3,4) &
              +rvol(j3)*(-flxg(j3+1,4)+flxg(j3,4))
            un(j1,j2,j3,5)=un(j1,j2,j3,5)          &
              +rvol(j3)*(-flxg(j3+1,5)+flxg(j3,5)) &
              +rdx(j3)*(-difh(j3+1)+difh(j3))
!        7.5       Magnetic field equations
#ifdef mhd
              un(j1,j2,j3,6)=un(j1,j2,j3,6) &
                +rvol(j3)*(-flxg(j3+1,6)+flxg(j3,6))
              un(j1,j2,j3,7)=un(j1,j2,j3,7) &
                +rvol(j3)*(-flxg(j3+1,7)+flxg(j3,7))
              iv=8
#else
              iv=5
#endif
!        7.6       Continuity equations
#ifdef hydcon
              iv=iv+1
              un(j1,j2,j3,iv)=un(j1,j2,j3,iv) &
                +rvol(j3)*(-flxg(j3+1,iv)+flxg(j3,iv))
#endif
#ifdef mhdcon
              iv=iv+1
              un(j1,j2,j3,iv)=un(j1,j2,j3,iv) &
                +rvol(j3)*(-flxg(j3+1,iv)+flxg(j3,iv))
#endif
#endif
        enddo
#ifdef PARAMESH
#ifdef consflux
!        8.1       Save fluxes at block boundaries
            do jv=1,nnvar
              flux_z(jv,j1,j2,1,jb)=flxg(nc3l,jv)
              flux_z(jv,j1,j2,2,jb)=flxg(nc3rp,jv)
            enddo
            flux_z(5,j1,j2,1,jb)=flux_z(5,j1,j2,1,jb)+difh(nc3l)
            flux_z(5,j1,j2,2,jb)=flux_z(5,j1,j2,2,jb)+difh(nc3rp)
#endif
#endif
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      8.        Epilogue
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        do jv=1,nnvar
          unk(jv,j1,j2,j3,jb)=un(j1,j2,j3,jv)
        enddo
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      8.        Toth method
#ifdef toth
! hyd_bnd,un,bnd1l,bnd1r,bnd2l,bnd2r
    if(nc1.gt.1) call BND1
    if(nc2.gt.1) call BND2
    if(nc3.gt.1) call BND3
    do j3=1,nc3
      do j2=1,nc2
        do j1=1,nc1
          zrd=1d0/unk(1,j1,j2,j3,jb)
          vp(j1,j2,j3,1)=unk(3,j1,j2,j3,jb)*zrd
          vp(j1,j2,j3,2)=unk(4,j1,j2,j3,jb)*zrd
          vp(j1,j2,j3,3)=unk(5,j1,j2,j3,jb)*zrd
          un(j1,j2,j3,6)=unk(6,j1,j2,j3,jb)
          un(j1,j2,j3,7)=unk(7,j1,j2,j3,jb)
          un(j1,j2,j3,8)=unk(8,j1,j2,j3,jb)
        enddo
      enddo
    enddo
!cell-centered electric field
    do j3=nc3lm,nc3rp
      do j2=nc2lm,nc2rp
        do j1=nc1lm,nc1rp
          e1cch(j1,j2,j3)=-uo(j1,j2,j3,8)*vpo(j1,j2,j3,2) &
                          +uo(j1,j2,j3,7)*vpo(j1,j2,j3,3) &
                          -un(j1,j2,j3,8)*vp(j1,j2,j3,2)  &
                          +un(j1,j2,j3,7)*vp(j1,j2,j3,3)
          e2cch(j1,j2,j3)=-uo(j1,j2,j3,6)*vpo(j1,j2,j3,3) &
                          +uo(j1,j2,j3,8)*vpo(j1,j2,j3,1) &
                          -un(j1,j2,j3,6)*vp(j1,j2,j3,3)  &
                          +un(j1,j2,j3,8)*vp(j1,j2,j3,1)
          e3hcc(j1,j2,j3)=-uo(j1,j2,j3,7)*vpo(j1,j2,j3,1) &
                          +uo(j1,j2,j3,6)*vpo(j1,j2,j3,2) &
                          -un(j1,j2,j3,7)*vp(j1,j2,j3,1)  &
                          +un(j1,j2,j3,6)*vp(j1,j2,j3,2)
        enddo
      enddo
    enddo
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
!        4.5       Auxiliary terms
#ifdef cartesian
            rvol23(j2)=rdx2(j2,jb)
            rvol2(j2)=rvol23(j2)
#endif
#ifdef cylindrical
            rvol23(j2)=rx1(j1,jb)*rdx2(j2,jb)
            rvol2(j2)=rvol23(j2)
#endif
#ifdef spherical
            rvol23(j2)=rx1(j1,jb)*rdx2(j2,jb)
            rvol2(j2)=rvol23(j2)*rsint(j2,jb)
#endif
!        4.5       Auxiliary terms
#ifdef cartesian
            rvol3(j3)=rdx3(j3,jb)
#endif
#ifdef cylindrical
            rvol3(j3)=rdx3(j3,jb)
#endif
#ifdef spherical
            rvol3(j3)=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)
#endif
!        4.5       New values
          un(j1,j2,j3,6)=uo(j1,j2,j3,6)                      &
            +dtq*rvol2(j2)*(-ax2(j2+1,jb)*e3hcc(j1,j2+1,j3)  &
                            +ax2(j2-1,jb)*e3hcc(j1,j2-1,j3)) &
            +dtq*rvol3(j3)*(e2cch(j1,j2,j3+1)-e2cch(j1,j2,j3-1))
#ifdef x2dim
#endif
#ifdef x3dim
#endif
          un(j1,j2,j3,7)=uo(j1,j2,j3,7)                           &
            +dtq*rvol3(j3)*(-e1cch(j1,j2,j3+1)+e1cch(j1,j2,j3-1)) &
            +dtq*rvol12(j1)*(ax12(j1+1)*e3hcc(j1+1,j2,j3)         &
                            -ax12(j1-1)*e3hcc(j1-1,j2,j3))
#ifdef x3dim
#endif
            un(j1,j2,j3,8)=uo(j1,j2,j3,8)                     &
              +dtq*rvol13(j1)*(-ax13(j1+1)*e2cch(j1+1,j2,j3)  &
                               +ax13(j1-1)*e2cch(j1-1,j2,j3)) &
              +dtq*rvol23(j2)*(e1cch(j1,j2+1,j3)-e1cch(j1,j2-1,j3))
#ifdef x2dim
#endif
        enddo
      enddo
    enddo
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
          do jv=1,nnvar
            unk(jv,j1,j2,j3,jb)=un(j1,j2,j3,jv)
          enddo
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      8.        Epilogue
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
      endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
#ifdef PARAMESH
#ifdef consflux
      call AMR_FLUX_CONSERVE(myproc,1)
      do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
        if(nodetype(jb).eq.1) then
#endif
!        8.1       X1-boundary
      do jv=1,5
        unk(jv,nc1l,:,:,jb)=unk(jv,nc1l,:,:,jb) &
          +rvol(nc1l)*(flux_x(jv,1,:,:,jb)-tflux_x(jv,1,:,:,jb))
        unk(jv,nc1r,:,:,jb)=unk(jv,nc1r,:,:,jb) &
          +rvol(nc1r)*(tflux_x(jv,2,:,:,jb)-flux_x(jv,2,:,:,jb))
      enddo
#ifdef mhd
        do jv=7,8
          unk(jv,nc1l,:,:,jb)=unk(jv,nc1l,:,:,jb) &
            +rvol(nc1l)*(flux_x(jv,1,:,:,jb)-tflux_x(jv,1,:,:,jb))
          unk(jv,nc1r,:,:,jb)=unk(jv,nc1r,:,:,jb) &
            +rvol(nc1r)*(tflux_x(jv,2,:,:,jb)-flux_x(jv,2,:,:,jb))
        enddo
#endif
!        8.2       X2-boundary
#ifdef x2dim
        rvol(nc2l)=rdx2(nc2l,jb)
        rvol(nc2r)=rdx2(nc2r,jb)
        do jv=1,5
          unk(jv,:,nc2l,:,jb)=unk(jv,:,nc2l,:,jb) &
            +rvol(nc2l)*(flux_y(jv,:,1,:,jb)-tflux_y(jv,:,1,:,jb))
          unk(jv,:,nc2r,:,jb)=unk(jv,:,nc2r,:,jb) &
            +rvol(nc2r)*(tflux_y(jv,:,2,:,jb)-flux_y(jv,:,2,:,jb))
        enddo
#ifdef mhd
          do jv=6,8,2
            unk(jv,:,nc2l,:,jb)=unk(jv,:,nc2l,:,jb) &
              +rvol(nc2l)*flux_y(jv,:,1,:,jb)-tflux_y(jv,:,1,:,jb))
            unk(jv,:,nc2r,:,jb)=unk(jv,:,nc2r,:,jb) &
              +rvol(nc2r)*(tflux_y(jv,:,2,:,jb)-flux_y(jv,:,2,:,jb))
          enddo
#endif
#endif
!        8.3       X3-boundary
#ifdef x3dim
          rvol(nc3l)=rdx3(nc3l,jb)
          rvol(nc3r)=rdx3(nc3r,jb)
          do jv=1,5
            unk(jv,:,:,nc3l,jb)=unk(jv,:,:,nc3l,jb) &
              +rvol(nc3l)*(flux_z(jv,:,:,1,jb)-tflux_z(jv,:,:,1,jb))
            unk(jv,:,:,nc3r,jb)=unk(jv,:,:,nc3r,jb) &
              +rvol(nc3r)*(tflux_z(jv,:,:,2,jb)-flux_z(jv,:,:,2,jb))
          enddo
#ifdef mhd
            do jv=6,7
              unk(jv,:,:,nc3l,jb)=unk(jv,:,:,nc3l,jb) &
                +rvol(nc3l)*(flux_z(jv,:,:,1,jb)-tflux_z(jv,:,:,1,jb))
              unk(jv,:,:,nc3r,jb)=unk(jv,:,:,nc3r,jb) &
                +rvol(nc3r)*(tflux_z(jv,:,:,2,jb)-flux_z(jv,:,:,2,jb))
            enddo
#endif
#endif
#ifndef ADVANCE_ALL_LEVELS
          endif
#endif
      enddo
#endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
#endif
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6001,0)
#endif 
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE MHC                            
