!****f* e0pro/main
! NAME 
!   MHD1
! PURPOSE
!   MHD X1-solution
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE MHD1 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE commet
  USE comadm
  USE comtva
  IMPLICIT NONE
  REAL(KIND=8)  zclsp,zclsph,zsp,zsph,zcot,zgdt,zngdt,zgdth
  REAL(KIND=8)  zrd,zcdt,zcsc,zcsc1,zcsc2
  REAL(KIND=8)  zngdth
#ifdef woodward
  REAL(KIND=8)  zs
#endif
  REAL(KIND=8)  zv1l,zv2l,zv3l,zpthel,zavldt,zabldt
  REAL(KIND=8)  zv1r,zv2r,zv3r,zpther,zavrdt,zabrdt
  REAL(KIND=8)  zpmagl,zpmagr,zptotl,zptotr
#ifdef vcen
  REAL(KIND=8)  zvel,zden,zpthe,zpmag
#else
#ifdef con
  REAL(KIND=8)  zvel
#endif
#endif
#ifdef etot
#ifdef mhd
  REAL(KIND=8)  zel,zer
#endif
#endif
!      real*8 z1,z2,z3,z4
  INTEGER  jx,j2,j3,jvar,ivar,jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
    call EXPERT(6001,1)
#endif
!---------------------------------------------------------------------
!      1.        Auxiliary values
!        1.2       Geometry terms parameters
#ifdef cartesian
    zclsp=zero
    zclsph=zero
#else
    zclsp=dt
    zclsph=dth
#endif
#ifdef spherical
    zsp=dt
    zsph=dth
#else
    zsp=zero
    zsph=zero
#endif
  zcot=zero
!        1.3       Gravity terms parameters
  if(nlgrav) then
    zgdt=-grav*dt
  else
    zgdt=zero
  endif
  if(nlxn) then
    zngdt=zgdt*xmass
  else
    zngdt=zgdt
  endif
  zgdth=zhalf*zgdt
  zngdth=zhalf*zngdt
!---------------------------------------------------------------------
!      2.        Prologue
#ifdef PARAMESH
!loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
        if(nodetype(jb).eq.1) then
#endif
#else
      jb=1
#endif
!        2.1       Parameters
      cfl1mx(jb)=zero
!        2.2       Pick up 1-D buffers from X1-X2-X3 arrays
      do j2=nc2l,nc2r
#ifdef spherical
          zcot=cott(j2,jb)
#endif
        do j3=nc3l,nc3r
          do jx=1,nc1
!        2.3       Pick up 1-D buffers from 3-D arrays
            do jvar=1,nnvar
              yyo(jx,jvar)=unk(jvar,jx,j2,j3,jb)
              yy(jx,jvar)=yyo(jx,jvar)
            enddo
          enddo
!---------------------------------------------------------------------
!      3.        Equation of state (half step)
          do jx=1,nc1
#ifdef gam3d
#ifdef hydcon
#ifdef mhdcon
                  ivar=10
#else
                  ivar=9
#endif
                if(yyo(jx,ivar).gt.zero) then
                  gamma(jx,j2,j3,jb)=gamcld
                else
                  gamma(jx,j2,j3,jb)=gamma(1,1,1,jb)
                endif
                if(nlxn) then
                  gammam=gamma(jx,j2,j3,jb)-zone
                  zgamma=gamma(jx,j2,j3,jb)/xmass
                else
                  gammam=gamma(jx,j2,j3,jb)-zone
                  zgamma=gamma(jx,j2,j3,jb)
                endif
#endif
#endif
#include <stateold.src>
!        3.3       Fluxes
            yya(jx,1)=vel1(jx)
            yya(jx,2)=vel2(jx)
            yya(jx,3)=vel3(jx)
            yya(jx,4)=pthe(jx)
#ifdef mhd
              yya(jx,5)=pmag(jx)
              yya(jx,6)=pthe(jx)+pmag(jx)
#endif
          enddo
!---------------------------------------------------------------------
!      5.        Limited interface values
!        5.1       Main variables
          do jvar=1,nnvar
            do jx=2,nc1
              difh(jx)=yyo(jx,jvar)-yyo(jx-1,jvar)
            enddo
            do jx=2,nc1-1
#ifdef minmod
                yg(jx,jvar)=sign(zhalf,difh(jx))*max(zero, &
                  min(abs(difh(jx)),sign(zone,difh(jx))*difh(jx+1)))
#endif
#ifdef woodward
                zs=sign(zone,difh(jx))
                yg(jx,jvar)=zhalf*zs*max(zero, &
                  min(abs(difh(jx)+difh(jx)),  &
                  zs*(difh(jx+1)+difh(jx+1)),  &
                  zhalf*zs*(difh(jx)+difh(jx+1))))
#endif
            enddo
          enddo
!        5.2       Auxiliary variables
          do jvar=1,npvar
            do jx=2,nc1
              flxh(jx)=yya(jx,jvar)-yya(jx-1,jvar)
            enddo
            do jx=2,nc1-1
#ifdef minmod
                ygg(jx,jvar)=sign(zhalf,flxh(jx))*max(zero, &
                  min(abs(flxh(jx)),sign(zone,flxh(jx))*flxh(jx+1)))
#endif
#ifdef woodward
                zs=sign(zone,flxh(jx))
                ygg(jx,jvar)=zhalf*zs*max(zero, &
                  min(abs(flxh(jx)+flxh(jx)),   &
                  zs*(flxh(jx+1)+flxh(jx+1)),   &
                  zhalf*zs*(flxh(jx)+flxh(jx+1))))
#endif
            enddo
          enddo
!---------------------------------------------------------------------
!      6.        Predictor
            do jx=2,nc1-1
              do jvar=1,nnvar
                yyl(jvar)=yyo(jx,jvar)-yg(jx,jvar)
                yyr(jvar)=yyo(jx,jvar)+yg(jx,jvar)
              enddo
              zv1l=yya(jx,1)-ygg(jx,1)
              zv1r=yya(jx,1)+ygg(jx,1)
              zv2l=yya(jx,2)-ygg(jx,2)
              zv2r=yya(jx,2)+ygg(jx,2)
              zv3l=yya(jx,3)-ygg(jx,3)
              zv3r=yya(jx,3)+ygg(jx,3)
              zpthel=yya(jx,4)-ygg(jx,4)
              zpther=yya(jx,4)+ygg(jx,4)
!        4.5       Auxiliary terms
              zavrdt=rvol1(jx,jb)*dth*ah1(jx+1,jb)*zv1r
              zavldt=rvol1(jx,jb)*dth*ah1(jx,jb)*zv1l
#ifdef mhd
                zabrdt=rvol1(jx,jb)*dthmag*ah1(jx+1,jb)*yyr(6)
                zabldt=rvol1(jx,jb)*dthmag*ah1(jx,jb)*yyl(6)
                zptotl=yya(jx,6)-ygg(jx,6)
                zptotr=yya(jx,6)+ygg(jx,6)
#endif
!        4.1       Density
            yyh(jx,1)=yyo(jx,1)-zavrdt*yyr(1)+zavldt*yyl(1)
#ifdef con
              yyh(jx,3)=yyh(jx,1)*vel1(jx)
              yyh(jx,4)=yyh(jx,1)*vel2(jx)
              yyh(jx,5)=yyh(jx,1)*vel3(jx)
#else
#ifdef ethe
!        4.2       Thermal energy
                yyh(jx,2)=yyo(jx,2)-zavrdt*yyr(2)+zavldt*yyl(2) &
                  +pthe(jx)*(-zavrdt+zavldt)
#else
#ifdef etot
!        4.2       Total energy
#ifdef mhd
                    zer=yyr(6)*zv1r+yyr(7)*zv2r+yyr(8)*zv3r
                    zel=yyl(6)*zv1l+yyl(7)*zv2l+yyl(8)*zv3l
                    yyh(jx,2)=yyo(jx,2)+rvol1(jx,jb)*dthene*          &
                      (ah1(jx+1,jb)*yyr(6)*zer-ah1(jx,jb)*yyl(6)*zel) &
                      -zavrdt*(yyr(2)+zptotr)+zavldt*(yyl(2)+zptotl)  &
                      +zngdth*yyo(jx,3)*rxx1(jx,jb)
#else
                    yyh(jx,2)=yyo(jx,2)                              &
                      -zavrdt*(yyr(2)+zpther)+zavldt*(yyl(2)+zpthel) &
                      +zngdth*yyo(jx,3)*rxx1(jx,jb)
#endif
#else
                  yyh(jx,2)=yyo(jx,2)                              &
                    -zavrdt*(yyr(2)+zpther)+zavldt*(yyl(2)+zpthel) &
                    +zngdth*yyo(jx,3)*rxx1(jx,jb)
#endif
#endif
!        4.3       Momentum
#ifdef mhd
                yyh(jx,3)=yyo(jx,3)                      &
                  -zavrdt*yyr(3)+zavldt*yyl(3)           &
                  +zabrdt*yyr(6)-zabldt*yyl(6)           &
                  -dtmom*rdx1(jx,jb)*ygg(jx,6)           &
                  +zgdth*yyo(jx,1)*rxx1(jx,jb)           &
                  +zclsph*rx1(jx,jb)*(yyo(jx,4)*vel2(jx) &
                    -zrmu*yyo(jx,7)*yyo(jx,7))           &
                  +zsph*rx1(jx,jb)*(yyo(jx,5)*vel3(jx)   &
                    -zrmu*yyo(jx,8)*yyo(jx,8))
                yyh(jx,4)=yyo(jx,4)                      &
                  -zavrdt*yyr(4)+zavldt*yyl(4)           &
                  +zabrdt*yyr(7)-zabldt*yyl(7)           &
                  -zclsph*rx1(jx,jb)*(yyo(jx,3)*vel2(jx) &
                    -zrmu*yyo(jx,6)*yyo(jx,7))           &
                  +zsph*rx1(jx,jb)*(yyo(jx,5)*vel3(jx)   &
                    -zrmu*yyo(jx,8)*yyo(jx,8))*zcot
                yyh(jx,5)=yyo(jx,5)                      &
                  -zavrdt*yyr(5)+zavldt*yyl(5)           &
                  +zabrdt*yyr(8)-zabldt*yyl(8)           &
                  -zsph *rx1(jx,jb)*(vel3(jx)*(yyo(jx,3) &
                    +yyo(jx,4)*zcot)                     &
                    -zrmu*yyo(jx,8)*(yyo(jx,6)+yyo(jx,7)*zcot))
#else
                yyh(jx,3)=yyo(jx,3)                     &
                  -zavrdt*yyr(3)+zavldt*yyl(3)          &
                  -dtmom*rdx1(jx,jb)*ygg(jx,4)          &
                  +zgdth*yyo(jx,1)*rxx1(jx,jb)          &
                  +zclsph*rx1(jx,jb)*yyo(jx,4)*vel2(jx) &
                  +zsph*rx1(jx,jb)*yyo(jx,5)*vel3(jx)
                yyh(jx,4)=yyo(jx,4)                     &
                  -zavrdt*yyr(4)+zavldt*yyl(4)          &
                  -zclsph*rx1(jx,jb)*yyo(jx,3)*vel2(jx) &
                  +zsph*rx1(jx,jb)*yyo(jx,5)*vel3(jx)*zcot
                yyh(jx,5)=yyo(jx,5)            &
                  -zavrdt*yyr(5)+zavldt*yyl(5) &
                  -zsph*rx1(jx,jb)*vel3(jx)*(yyo(jx,3)+yyo(jx,4)*zcot)
#endif
#ifdef mhd
!        4.4       Magnetic field
                  yyh(jx,6)=yyo(jx,6)
                  yyh(jx,7)=yyo(jx,7)+rvol12(jx)*dth*      &
                    (ah12(jx+1)*(-yyr(7)*zv1r+yyr(6)*zv2r) &
                    +ah12(jx)*(yyl(7)*zv1l-yyl(6)*zv2l))
                  yyh(jx,8)=yyo(jx,8)+rvol13(jx)*dth*      &
                    (ah13(jx+1)*(-yyr(8)*zv1r+yyr(6)*zv3r) &
                    +ah13(jx)*(yyl(8)*zv1l-yyl(6)*zv3l))
                  ivar=8
#else
                  ivar=5
#endif
!        4.5       Supplementary variables
#ifdef hydcon
                  ivar=ivar+1
                  yyh(jx,ivar)=yyo(jx,ivar) &
                    -zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#ifdef mhdcon
                  ivar=ivar+1
                  yyh(jx,ivar)=yyo(jx,ivar) &
                    -zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#endif
            enddo
!---------------------------------------------------------------------
!      6.        Equation of state (full step)
            do jx=2,nc1-1
#ifdef gam3d
#ifdef hydcon
#ifdef mhdcon
                    ivar=10
#else
                    ivar=9
#endif
                  if(yyh(jx,ivar).gt.zero) then
                    gamma(jx,j2,j3,jb)=gamcld
                  else
                    gamma(jx,j2,j3,jb)=gamma(1,1,1,jb)
                  endif
                  if(nlxn) then
                    gammam=gamma(jx,j2,j3,jb)-zone
                    zgamma=gamma(jx,j2,j3,jb)/xmass
                  else
                    gammam=gamma(jx,j2,j3,jb)-zone
                    zgamma=gamma(jx,j2,j3,jb)
                  endif
#endif
#endif
#include <statehlf.src>
              yya(jx,1)=vel1(jx)
              yya(jx,2)=vel2(jx)
              yya(jx,3)=vel3(jx)
              yya(jx,4)=pthe(jx)
#ifdef mhd
                yya(jx,5)=pmag(jx)
                yya(jx,6)=pthe(jx)+pmag(jx)
#endif
            enddo
!---------------------------------------------------------------------
!      7.        Solve the full step
            do jx=3,nc1-1
!Values at the cell interfaces
#ifdef upwind
                do jvar=1,nnvar
                  yyl(jvar)=yyh(jx-1,jvar)+yg(jx-1,jvar)
                  yyr(jvar)=yyh(jx,jvar)-yg(jx,jvar)
                enddo
                zv1l=yya(jx-1,1)+ygg(jx-1,1)
                zv1r=yya(jx,1)-ygg(jx,1)
                zv2l=yya(jx-1,2)+ygg(jx-1,2)
                zv2r=yya(jx,2)-ygg(jx,2)
                zv3l=yya(jx-1,3)+ygg(jx-1,3)
                zv3r=yya(jx,3)-ygg(jx,3)
                zpthel=yya(jx-1,4)+ygg(jx-1,4)
                zpther=yya(jx,4)-ygg(jx,4)
#ifdef mhd
                  zpmagl=yya(jx-1,5)+ygg(jx-1,5)
                  zpmagr=yya(jx,5)-ygg(jx,5)
                  zptotl=yya(jx-1,6)+ygg(jx-1,6)
                  zptotr=yya(jx,6)-ygg(jx,6)
#endif
#else
                do jvar=1,nnvar
                  yyl(jvar)=yyh(jx-1,jvar)+yg(jx,jvar)
                  yyr(jvar)=yyh(jx,jvar)-yg(jx-1,jvar)
                enddo
                zv1l=yya(jx-1,1)+ygg(jx,1)
                zv1r=yya(jx,1)-ygg(jx-1,1)
                zv2l=yya(jx-1,2)+ygg(jx,2)
                zv2r=yya(jx,2)-ygg(jx-1,2)
                zv3l=yya(jx-1,3)+ygg(jx,3)
                zv3r=yya(jx,3)-ygg(jx-1,3)
                zpthel=yya(jx-1,4)+ygg(jx,4)
                zpther=yya(jx,4)-ygg(jx-1,4)
#ifdef mhd
                  zpmagl=yya(jx-1,5)+ygg(jx,5)
                  zpmagr=yya(jx,5)-ygg(jx-1,5)
                  zptotl=yya(jx-1,6)+ygg(jx,6)
                  zptotr=yya(jx,6)-ygg(jx-1,6)
#endif
#endif
!---------------------------------------------------------------------
!      6.        Auxiliary values (half step)
#ifdef con
                zcsc=abs(zvel)
#else
#ifdef vcen
                  zvel=zhalf*(vel1(jx-1)+vel1(jx))
                  zden=zhalf*(yyh(jx-1,1)+yyh(jx,1))
                  zpthe=zhalf*(pthe(jx-1)+pthe(jx))
#ifdef mhd
                    zpmag=zhalf*(pmag(jx-1)+pmag(jx))
                    zcsc=abs(zvel)+sqrt((zgamma*max(zero,zpthe) &
                      +ztwom*zpmag)/max(zmin,zden))
#else
                    zcsc=abs(zvel)+sqrt(zgamma*max(zero,zpthe)/ &
                      max(zmin,zden))
#endif
#else
#ifdef vmax
#ifdef mhd
                      zcsc1=sqrt(zv1l*zv1l+zv2l*zv2l+zv3l*zv3l) &
                        +sqrt((zgamma*max(zero,zpthel)          &
                        +ztwom*zpmagl)/ max(zmin,yyl(1)))
                      zcsc2=sqrt(zv1r*zv1r+zv2r*zv2r+zv3r*zv3r) &
                        +sqrt((zgamma*max(zero,zpther)          &
                        +ztwom*zpmagr)/max(zmin,yyr(1)))
#else
                      zcsc1=sqrt(zv1l*zv1l+zv2l*zv2l+zv3l*zv3l) &
                        +sqrt(zgamma*max(zero,zpthel)/max(zmin,yyl(1)))
                      zcsc2=sqrt(zv1r*zv1r+zv2r*zv2r+zv3r*zv3r) &
                        +sqrt(zgamma*max(zero,zpther)/max(zmin,yyr(1)))
#endif
#else
#ifdef mhd
                      zcsc1=abs(zv1l) &
                        +sqrt((zgamma*max(zero,zpthel)+ztwom*zpmagl)/ &
                        max(zmin,yyl(1)))
                      zcsc2=abs(zv1r) &
                        +sqrt((zgamma*max(zero,zpther)+ztwom*zpmagr)/ &
                        max(zmin,yyr(1)))
#else
                      zcsc1=abs(zv1l) &
                        +sqrt(zgamma*max(zero,zpthel)/max(zmin,yyl(1)))
                      zcsc2=abs(zv1r) &
                        +sqrt(zgamma*max(zero,zpther)/max(zmin,yyr(1)))
#endif
#endif
                  zcsc=max(zcsc1,zcsc2)
#endif
                cfl1mx(jb)=max(cfl1mx(jb),rdx1(jx,jb)*zcsc)
#endif
!---------------------------------------------------------------------
!      7.        Solve the full step
!        6.3       Fluxes
              zavrdt=dth*ah1(jx,jb)*(zv1r-zcsc)
              zavldt=dth*ah1(jx,jb)*(zv1l+zcsc)
              zcdt=dth*zcsc
#ifdef mhd
                zabrdt=dthmag*ah1(jx,jb)*yyr(6)
                zabldt=dthmag*ah1(jx,jb)*yyl(6)
#ifdef etot
                zer=yyr(6)*zv1r+yyr(7)*zv2r+yyr(8)*zv3r
                zel=yyl(6)*zv1l+yyl(7)*zv2l+yyl(8)*zv3l
#endif
#endif
!        6.4       Fluxes
              flxg(jx,1)=zavrdt*yyr(1)+zavldt*yyl(1)

#ifndef con
!        6.7       Fluxes
#ifdef ethe
                  flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2)
                  flxh(jx)=zavrdt+zavldt
#else
#ifdef etot
#ifdef mhd
                      flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2)      &
                        +dth*ah1(jx,jb)*(zptotr*zv1r+zptotl*zv1l) &
                        -dthene*ah1(jx,jb)*(zer*yyr(6)+zel*yyl(6))
#else
                      flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2) &
                        +dth*ah1(jx,jb)*(zpther*zv1r+zpthel*zv1l)
#endif
#else
                    flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2) &
                      +dth*ah1(jx,jb)*(zpther*zv1r+zpthel*zv1l)
#endif
#endif
!        6.5       Fluxes
#ifdef mhd
                  flxg(jx,3)=zavrdt*yyr(3)+zavldt*yyl(3) &
                            -zabrdt*yyr(6)-zabldt*yyl(6)
                  flxg(jx,4)=zavrdt*yyr(4)+zavldt*yyl(4) &
                            -zabrdt*yyr(7)-zabldt*yyl(7)
                  flxg(jx,5)=zavrdt*yyr(5)+zavldt*yyl(5) &
                            -zabrdt*yyr(8)-zabldt*yyl(8)
                  difh(jx)=dthmom*(zptotr+zptotl)
#else
                  flxg(jx,3)=zavrdt*yyr(3)+zavldt*yyl(3)
                  flxg(jx,4)=zavrdt*yyr(4)+zavldt*yyl(4)
                  flxg(jx,5)=zavrdt*yyr(5)+zavldt*yyl(5)
                  difh(jx)=dthmom*(zpther+zpthel)
#endif
!        6.6       Fluxes
#ifdef mhd
                  flxg(jx,7)=ah12(jx)*(dth*                           &
                    (yyr(7)*zv1r+yyl(7)*zv1l-yyr(6)*zv2r-yyl(6)*zv2l) &
                    +zcdt*(yyl(7)-yyr(7)))
                  flxg(jx,8)=ah13(jx)*(dth*                           &
                    (yyr(8)*zv1r+yyl(8)*zv1l-yyr(6)*zv3r-yyl(6)*zv3l) &
                    +zcdt*(yyl(8)-yyr(8)))
                  ivar=8
#else
                  ivar=5
#endif
!        6.7       Fluxes
#ifdef hydcon
                  ivar=ivar+1
                  flxg(jx,ivar)=zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#ifdef mhdcon
                  ivar=ivar+1
                  flxg(jx,ivar)=zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#endif
            enddo
!---------------------------------------------------------------------
!      7.        Solve the full step
            do jx=3,nc1-2
!        7.2       Continuity equations
              yy(jx,1)=yyo(jx,1)+rvol1(jx,jb)*(-flxg(jx+1,1)+flxg(jx,1))
#ifdef con
                yy(jx,3)=yy(jx,1)*vel1(jx)
                yy(jx,4)=yy(jx,1)*vel2(jx)
                yy(jx,5)=yy(jx,1)*vel3(jx)
#else
!        7.3       Energy equations
#ifdef ethe
                yy(jx,2)=yyo(jx,2)+rvol1(jx,jb)* &
                  (-flxg(jx+1,2)+flxg(jx,2)      &
                  +pthe(jx)*(-flxh(jx+1)+flxh(jx)))
#else
                yy(jx,2)=yyo(jx,2)+rvol1(jx,jb)* &
                  (-flxg(jx+1,2)+flxg(jx,2))     &
                  +zngdt*yyh(jx,3)*rxx1(jx,jb)
#endif
!        7.4       Momentum equations
#ifdef mhd
                yy(jx,3)=yyo(jx,3)                         &
                  +rvol1(jx,jb)*(-flxg(jx+1,3)+flxg(jx,3)) &
                  +rdx1(jx,jb)*(-difh(jx+1)+difh(jx))      &
                  +zgdt*yyh(jx,1)*rxx1(jx,jb)              &
                  +zclsp*rx1(jx,jb)*(yyh(jx,4)*vel2(jx)    &
                    -zrmu*yyh(jx,7)*yyh(jx,7))             &
                  +zsp*rx1(jx,jb)*(yyh(jx,5)*vel3(jx)      &
                    -zrmu*yyh(jx,8)*yyh(jx,8))
                yy(jx,4)=yyo(jx,4)                         &
                  +rvol1(jx,jb)*(-flxg(jx+1,4)+flxg(jx,4)) &
                  -zclsp*rx1(jx,jb)*(yyh(jx,3)*vel2(jx)    &
                    -zrmu*yyh(jx,6)*yyh(jx,7))             &
                  +zsp*rx1(jx,jb)*(yyh(jx,5)*vel3(jx)      &
                    -zrmu*yyh(jx,8)*yyh(jx,8))*zcot
                yy(jx,5)=yyo(jx,5)                                     &
                  +rvol1(jx,jb)*(-flxg(jx+1,5)+flxg(jx,5))             &
                  -zsp*rx1(jx,jb)*(vel3(jx)*(yyh(jx,3)+yyh(jx,4)*zcot) &
                  -zrmu*yyh(jx,8)*(yyh(jx,6)+yyh(jx,7)*zcot))
#else
                yy(jx,3)=yyo(jx,3)                         &
                  +rvol1(jx,jb)*(-flxg(jx+1,3)+flxg(jx,3)) &
                  +rdx1(jx,jb)*(-difh(jx+1)+difh(jx))      &
                  +zgdt*yyh(jx,1)*rxx1(jx,jb)              &
                  +zclsp*rx1(jx,jb)*yyh(jx,4)*vel2(jx)     &
                  +zsp*rx1(jx,jb)*yyh(jx,5)*vel3(jx)
                yy(jx,4)=yyo(jx,4)                         &
                  +rvol1(jx,jb)*(-flxg(jx+1,4)+flxg(jx,4)) &
                  -zclsp*rx1(jx,jb)*yyh(jx,3)*vel2(jx)     &
                  +zsp*rx1(jx,jb)*yyh(jx,5)*vel3(jx)*zcot
                yy(jx,5)=yyo(jx,5)                         &
                  +rvol1(jx,jb)*(-flxg(jx+1,5)+flxg(jx,5)) &
                  -zsp*rx1(jx,jb)*vel3(jx)*(yyh(jx,3)+yyh(jx,4)*zcot)
#endif
!        7.5       Magnetic field equations
#ifdef mhd
                yy(jx,6)=yyo(jx,6)
                yy(jx,7)=yyo(jx,7)+rvol12(jx)*(-flxg(jx+1,7)+flxg(jx,7))
                yy(jx,8)=yyo(jx,8)+rvol13(jx)*(-flxg(jx+1,8)+flxg(jx,8))
                ivar=8
#else
                ivar=5
#endif
!        7.6       Continuity equations
#ifdef hydcon
                ivar=ivar+1
                yy(jx,ivar)=yyo(jx,ivar) &
                  +rvol1(jx,jb)*(-flxg(jx+1,ivar)+flxg(jx,ivar))
#endif
#ifdef mhdcon
                ivar=ivar+1
                yy(jx,ivar)=yyo(jx,ivar) &
                  +rvol1(jx,jb)*(-flxg(jx+1,ivar)+flxg(jx,ivar))
#endif
#endif
          enddo
!---------------------------------------------------------------------
!      8.        Epilogue
#ifdef PARAMESH
#ifdef consflux
!        8.1       Save fluxes at block boundaries
              do jvar=1,nnvar
                flux_x(jvar,1,j2,j3,jb)=flxg (nc1l,jvar)
                flux_x(jvar,2,j2,j3,jb)=flxg (nc1r+1,jvar)
              enddo
              flux_x(3,1,j2,j3,jb)=flux_x(3,1,j2,j3,jb)+difh(nc1l)
              flux_x(3,2,j2,j3,jb)=flux_x(3,2,j2,j3,jb)+difh(nc1r+1)
#endif
#endif
!        8.1       Put 1-D buffers to 3-D arrays
          do jx=3,nc1-2
            do jvar=1,nnvar
              unk(jvar,jx,j2,j3,jb)=yy(jx,jvar)
            enddo
!           unk(1,jx,j2,j3,1)=yy(jx,1)
!           unk(2,jx,j2,j3,1)=yy(jx,2)
!           unk(3,jx,j2,j3,1)=yy(jx,3)
!           unk(4,jx,j2,j3,1)=yy(jx,4)
!           unk(5,jx,j2,j3,1)=yy(jx,5)
!           unk(6,jx,j2,j3,1)=yy(jx,6)
!           unk(7,jx,j2,j3,1)=yy(jx,7)
!           unk(8,jx,j2,j3,1)=yy(jx,8)
          enddo
        enddo
      enddo
#ifdef PARAMESH
!        8.1       Put 3-D arrays into UNK variables
#ifndef ADVANCE_ALL_LEVELS
          endif
#endif
      enddo
#endif
!---------------------------------------------------------------------
!      9.        Update values at block boundaries
#ifdef PARAMESH
#ifdef consflux
        call AMR_RESTRICT(mype,1,0)
        call AMR_GUARDCELL(mype,1,nguard)
        call AMR_FLUX_CONSERVE(mype,1)
        do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
            if(nodetype(jb).eq.1) then
#endif
          rvol(nc1l)=rvol1(nc1l,jb)
          rvol(nc1r)=rvol1(nc1r,jb)
          do jvar=1,5
            unk(jvar,nc1l,:,:,jb)=unk(jvar,nc1l,:,:,jb) &
              +rvol(nc1l)*(flux_x(jvar,1,:,:,jb)-tflux_x(jvar,1,:,:,jb))
            unk(jvar,nc1r,:,:,jb)=unk(jvar,nc1r,:,:,jb) &
              +rvol(nc1r)*(tflux_x(jvar,2,:,:,jb)-flux_x(jvar,2,:,:,jb))
          enddo
#ifdef mhd
          do jvar=7,8
            unk(jvar,nc1l,:,:,jb)=unk(jvar,nc1l,:,:,jb) &
              +rvol(nc1l)*(flux_x(jvar,1,:,:,jb)-tflux_x(jvar,1,:,:,jb))
            unk(jvar,nc1r,:,:,jb)=unk(jvar,nc1r,:,:,jb) &
              +rvol(nc1r)*(tflux_x(jvar,2,:,:,jb)-flux_x(jvar,2,:,:,jb))
           enddo
#endif
#ifndef ADVANCE_ALL_LEVELS
          endif
#endif
      enddo
#endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6001,0)
#endif                       
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE MHD1                           
