!****f* e0pro/main
! NAME 
!   MHD3
! PURPOSE
!   MHD X3-solution
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE MHD3 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE commet
  USE comadm
  USE comtva
  IMPLICIT NONE
  REAL(KIND=8)  zrd,zcdt,zcsc,zcsc1,zcsc2
#ifdef woodward
  REAL(KIND=8)  zs
#endif
  REAL(KIND=8)  zv1l,zv2l,zv3l,zpthel,zavldt,zabldt
  REAL(KIND=8)  zv1r,zv2r,zv3r,zpther,zavrdt,zabrdt
  REAL(KIND=8)  zpmagl,zpmagr,zptotl,zptotr
  REAL(KIND=8)  zvel
#ifdef vcen
  REAL(KIND=8)  zden,zpthe,zpmag
#endif
#ifdef etot
#ifdef mhd
  REAL(KIND=8)  zel,zer
#endif
#endif
  INTEGER j1,j2,jx,jvar,ivar,jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
    INTEGER j3
#endif
#ifdef DIAGNO
    call EXPERT(6003,1)
#endif
!---------------------------------------------------------------------
!      1.        Prologue
#ifdef PARAMESH
!loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
        if(nodetype(jb).eq.1) then
#endif
#else
      jb=1
#endif
!        1.1       Parameters
      cfl3mx(jb)=zero
!        1.1       Pick up 1-D buffers from X1-X2-X3 arrays
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
          do jx=1,nc3
!        1.1       Pick up 1-D buffers from 3-D arrays
            do jvar=1,nnvar
              yyo(jx,jvar)=unk(jvar,j1,j2,jx,jb)
              yy(jx,jvar)=yyo(jx,jvar)
            enddo
#ifdef cartesian
              rdx(jx)=rdx3(jx,jb)
              rvol(jx)=rdx3(jx,jb)
#endif
#ifdef cylindrical
              rdx(jx)=rdx3(jx,jb)
              rvol(jx)=rdx3(jx,jb)
#endif
#ifdef spherical
              rdx(jx)=rx1(j1,jb)*rsint(j2,jb)*rdx3(jx,jb)
              rvol(jx)=rdx(jx)
#endif
          enddo
!---------------------------------------------------------------------
!      2.        Equation of state (half step)
          do jx=1,nc3
#ifdef gam3d
#ifdef hydcon
#ifdef mhdcon
                  ivar=10
#else
                  ivar=9
#endif
                if(yyo(jx,ivar).gt.zero) then
                  gamma(j1,j2,jx,jb)=gamcld
                else
                  gamma(j1,j2,jx,jb)=gamma(1,1,1,jb)
                endif
                if(nlxn) then
                  gammam=gamma(j1,j2,jx,jb)-zone
                  zgamma=gamma(j1,j2,jx,jb)/xmass
                else
                  gammam=gamma(j1,j2,jx,jb)-zone
                  zgamma=gamma(j1,j2,jx,jb)
                endif
#endif
#endif
#include <stateold.src>
            yya(jx,1)=vel1(jx)
            yya(jx,2)=vel2(jx)
            yya(jx,3)=vel3(jx)
            yya(jx,4)=pthe(jx)
#ifdef mhd
              yya(jx,5)=pmag(jx)
              yya(jx,6)=pthe(jx)+pmag(jx)
#endif
          enddo
!---------------------------------------------------------------------
!      3.        Limited interface values
!        3.1       Main variables
          do jvar=1,nnvar
            do jx=2,nc3
              difh(jx)=yyo(jx,jvar)-yyo(jx-1,jvar)
            enddo
            do jx=2,nc3-1
#ifdef minmod
                yg(jx,jvar)=sign(zhalf,difh(jx))*max(zero, &
                  min(abs(difh(jx)),sign(zone,difh(jx))*difh(jx+1)))
#endif
#ifdef woodward
                zs=sign(zone,difh(jx))
                yg(jx,jvar)=zhalf*zs*max(zero, &
                  min(abs(difh(jx)+difh(jx)),  &
                  zs*(difh(jx+1)+difh(jx+1)),  &
                  zhalf*zs*(difh(jx)+difh(jx+1))))
#endif
            enddo
          enddo
!        3.2       Auxiliary variables
          do jvar=1,npvar
            do jx=2,nc3
              flxh(jx)=yya(jx,jvar)-yya(jx-1,jvar)
            enddo
            do jx=2,nc3-1
#ifdef minmod
                ygg(jx,jvar)=sign(zhalf,flxh(jx))*max(zero, &
                  min(abs(flxh(jx)),sign(zone,flxh(jx))*flxh(jx+1)))
#endif
#ifdef woodward
                zs=sign(zone,flxh(jx))
                ygg(jx,jvar)=zhalf*zs*max(zero,  &
                  min(abs(flxh(jx)+flxh(jx)),    &
                  zs*(flxh(jx+1)+flxh(jx+1)),    &
                  zhalf*zs*(flxh(jx)+flxh(jx+1))))
#endif
            enddo
          enddo
!---------------------------------------------------------------------
!      4.        Predictor
          do jx=2,nc3-1
            do jvar=1,nnvar
              yyl(jvar)=yyo(jx,jvar)-yg(jx,jvar)
              yyr(jvar)=yyo(jx,jvar)+yg(jx,jvar)
            enddo
            zv1l=yya(jx,1)-ygg(jx,1)
            zv1r=yya(jx,1)+ygg(jx,1)
            zv2l=yya(jx,2)-ygg(jx,2)
            zv2r=yya(jx,2)+ygg(jx,2)
            zv3l=yya(jx,3)-ygg(jx,3)
            zv3r=yya(jx,3)+ygg(jx,3)
            zpthel=yya(jx,4)-ygg(jx,4)
            zpther=yya(jx,4)+ygg(jx,4)
!        4.5       Auxiliary terms
            zavrdt=rvol(jx)*dth*zv3r
            zavldt=rvol(jx)*dth*zv3l
#ifdef mhd
              zabrdt=rvol(jx)*dthmag*yyr(8)
              zabldt=rvol(jx)*dthmag*yyl(8)
              zptotl=yya(jx,6)-ygg(jx,6)
              zptotr=yya(jx,6)+ygg(jx,6)
#endif
!        4.1       Density
            yyh(jx,1)=yyo(jx,1)-zavrdt*yyr(1)+zavldt*yyl(1)
#ifdef con
              yyh(jx,3)=yyh(jx,1)vel1(jx)
              yyh(jx,4)=yyh(jx,1)vel2(jx)
              yyh(jx,5)=yyh(jx,1)vel3(jx)
#else
#ifdef ethe
!        4.2       Thermal energy
                yyh(jx,2)=yyo(jx,2)-zavrdt*yyr(2)+zavldt*yyl(2) &
                  +pthe(jx)*(-zavrdt+zavldt)
#else
#ifdef etot
!        4.2       Total energy
#ifdef mhd
                    zer=yyr(6)*zv1r+yyr(7)*zv2r+yyr(8)*zv3r
                    zel=yyl(6)*zv1l+yyl(7)*zv2l+yyl(8)*zv3l
                    yyh(jx,2)=yyo(jx,2)                        &
                      +rvol(jx)*dthene*(yyr(8)*zer-yyl(8)*zel) &
                      -zavrdt*(yyr(2)+zptotr)+zavldt*(yyl(2)+zptotl)
#else
                    yyh(jx,2)=yyo(jx,2)-zavrdt*(yyr(2)+zpther) &
                                       +zavldt*(yyl(2)+zpthel)
#endif
#else
                  yyh(jx,2)=yyo(jx,2)-zavrdt*(yyr(2)+zpther) &
                                     +zavldt*(yyl(2)+zpthel)
#endif
#endif
!        4.3       Momentum
#ifdef mhd
                yyh(jx,3)=yyo(jx,3)-zavrdt*yyr(3)+zavldt*yyl(3) &
                                   +zabrdt*yyr(6)-zabldt*yyl(6)
                yyh(jx,4)=yyo(jx,4)-zavrdt*yyr(4)+zavldt*yyl(4) &
                                   +zabrdt*yyr(7)-zabldt*yyl(7)
                yyh(jx,5)=yyo(jx,5)-zavrdt*yyr(5)+zavldt*yyl(5) &
                                   +zabrdt*yyr(8)-zabldt*yyl(8) &
                                   -dtmom*rdx(jx)*ygg(jx,6)
#else
                yyh(jx,3)=yyo(jx,3)-zavrdt*yyr(3)+zavldt*yyl(3)
                yyh(jx,4)=yyo(jx,4)-zavrdt*yyr(4)+zavldt*yyl(4)
                yyh(jx,5)=yyo(jx,5)-zavrdt*yyr(5)+zavldt*yyl(5) &
                                   -dtmom*rdx(jx)*ygg(jx,4)
#endif
#ifdef mhd
!        4.4       Magnetic field
                yyh(jx,6)=yyo(jx,6)+rvol(jx)*dth* &
                  (-yyr(6)*zv3r+yyr(8)*zv1r+yyl(6)*zv3l-yyl(8)*zv1l)
                yyh(jx,7)=yyo(jx,7)+rvol(jx)*dth* &
                  (-yyr(7)*zv3r+yyr(8)*zv2r+yyl(7)*zv3l-yyl(8)*zv2l)
                yyh(jx,8)=yyo(jx,8)
                ivar=8
#else
                ivar=5
#endif
!        4.5       Supplementary variables
#ifdef hydcon
                ivar=ivar+1
                yyh(jx,ivar)=yyo(jx,ivar)-zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#ifdef mhdcon
                ivar=ivar+1
                yyh(jx,ivar)=yyo(jx,ivar)-zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#endif
          enddo
!---------------------------------------------------------------------
!      6.        Equation of state (full step)
          do jx=2,nc3-1
#ifdef gam3d
#ifdef hydcon
#ifdef mhdcon
                  ivar=10
#else
                  ivar=9
#endif
                if(yyh(jx,ivar).gt.zero) then
                  gamma(j1,j2,jx,jb)=gamcld
                else
                  gamma(j1,j2,jx,jb)=gamma(1,1,1,jb)
                endif
                if(nlxn) then
                  gammam=gamma(j1,j2,jx,jb)-zone
                  zgamma=gamma(j1,j2,jx,jb)/xmass
                else
                  gammam=gamma(j1,j2,jx,jb)-zone
                  zgamma=gamma(j1,j2,jx,jb)
                endif
#endif
#endif
#include <statehlf.src>
            yya(jx,1)=vel1(jx)
            yya(jx,2)=vel2(jx)
            yya(jx,3)=vel3(jx)
            yya(jx,4)=pthe(jx)
#ifdef mhd
              yya(jx,5)=pmag(jx)
              yya(jx,6)=pthe(jx)+pmag(jx)
#endif
          enddo
!---------------------------------------------------------------------
!      7.        Solve the full step
          do jx=3,nc3-1
!Values at the cell interfaces
#ifdef upwind
                do jvar=1,nnvar
                  yyl(jvar)=yyh(jx-1,jvar)+yg(jx-1,jvar)
                  yyr(jvar)=yyh(jx,jvar)-yg(jx,jvar)
                enddo
                zv1l=yya(jx-1,1)+ygg(jx-1,1)
                zv1r=yya(jx,1)-ygg(jx,1)
                zv2l=yya(jx-1,2)+ygg(jx-1,2)
                zv2r=yya(jx,2)-ygg(jx,2)
                zv3l=yya(jx-1,3)+ygg(jx-1,3)
                zv3r=yya(jx,3)-ygg(jx,3)
                zpthel=yya(jx-1,4)+ygg(jx-1,4)
                zpther=yya(jx,4)-ygg(jx,4)
#ifdef mhd
                  zpmagl=yya(jx-1,5)+ygg(jx-1,5)
                  zpmagr=yya(jx,5)-ygg(jx,5)
                  zptotl=yya(jx-1,6)+ygg(jx-1,6)
                  zptotr=yya(jx,6)-ygg(jx,6)
#endif
#else
                do jvar=1,nnvar
                  yyl(jvar)=yyh(jx-1,jvar)+yg(jx,jvar)
                  yyr(jvar)=yyh(jx,jvar)-yg(jx-1,jvar)
                enddo
                zv1l=yya(jx-1,1)+ygg(jx,1)
                zv1r=yya(jx,1)-ygg(jx-1,1)
                zv2l=yya(jx-1,2)+ygg(jx,2)
                zv2r=yya(jx,2)-ygg(jx-1,2)
                zv3l=yya(jx-1,3)+ygg(jx,3)
                zv3r=yya(jx,3)-ygg(jx-1,3)
                zpthel=yya(jx-1,4)+ygg(jx,4)
                zpther=yya(jx,4)-ygg(jx-1,4)
#ifdef mhd
                  zpmagl=yya(jx-1,5)+ygg(jx,5)
                  zpmagr=yya(jx,5)-ygg(jx-1,5)
                  zptotl=yya(jx-1,6)+ygg(jx,6)
                  zptotr=yya(jx,6)-ygg(jx-1,6)
#endif
#endif
!---------------------------------------------------------------------
!      6.        Auxiliary values (half step)
            zvel=zhalf*(vel3(jx+1)+vel3(jx))
#ifdef con
              zcsc=abs(zvel)
#else
#ifdef vcen
                zvel=zhalf*(vel3(jx-1)+vel3(jx))
                zden=zhalf*(yyh(jx-1,1)+yyh(jx,1))
                zpthe=zhalf*(pthe(jx-1)+pthe(jx))
#ifdef mhd
                  zpmag=zhalf*(pmag(jx-1)+pmag(jx))
                  zcsc=abs(zvel)                                &
                    +sqrt((zgamma*max(zero,zpthe)+ztwom*zpmag)/ &
                    max(zmin,zden))
#else
                  zcsc=abs(zvel)+sqrt(zgamma*max(zero,zpthe)/ &
                    max(zmin,zden))
#endif
#else
#ifdef vmax
#ifdef mhd
                    zcsc1=sqrt(zv1l*zv1l+zv2l*zv2l+zv3l*zv3l)       &
                      +sqrt((zgamma*max(zero,zpthel)+ztwom*zpmagl)/ &
                      max(zmin,yyl(1)))
                    zcsc2=sqrt(zv1r*zv1r+zv2r*zv2r+zv3r*zv3r)       &
                      +sqrt((zgamma*max(zero,zpther)+ztwom*zpmagr)/ &
                      max(zmin,yyr(1)))
#else
                    zcsc1=sqrt(zv1l*zv1l+zv2l*zv2l+zv3l*zv3l)       &
                      +sqrt(zgamma*max(zero,zpthel)/max(zmin,yyl(1)))
                    zcsc2=sqrt(zv1r*zv1r+zv2r*zv2r+zv3r*zv3r)       &
                      +sqrt(zgamma*max(zero,zpther)/max(zmin,yyr(1)))
#endif
#else
#ifdef mhd
                    zcsc1=abs(zv3l)                                 &
                      +sqrt((zgamma*max(zero,zpthel)+ztwom*zpmagl)/ &
                      max(zmin,yyl(1)))
                    zcsc2=abs(zv3r)                                 &
                      +sqrt((zgamma*max(zero,zpther)+ztwom*zpmagr)/ &
                      max(zmin,yyr(1)))
#else
                    zcsc1=abs(zv3l) &
                      +sqrt(zgamma*max(zero,zpthel)/max(zmin,yyl(1)))
                    zcsc2=abs(zv3r) &
                      +sqrt(zgamma*max(zero,zpther)/max(zmin,yyr(1)))
#endif
#endif
                zcsc=max(zcsc1,zcsc2)
#endif
              cfl3mx(jb)=max(cfl3mx(jb),rdx(jx)*zcsc)
#endif
!---------------------------------------------------------------------
!      7.        Solve the full step
!        6.3       Fluxes
            zavrdt=dth*(zv3r-zcsc)
            zavldt=dth*(zv3l+zcsc)
            zcdt=dth*zcsc
#ifdef mhd
              zabrdt=dthmag*yyr(8)
              zabldt=dthmag*yyl(8)
#ifdef etot
              zer=yyr(6)*zv1r+yyr(7)*zv2r+yyr(8)*zv3r
              zel=yyl(6)*zv1l+yyl(7)*zv2l+yyl(8)*zv3l
#endif
#endif
!        6.4       Fluxes
            flxg(jx,1)=zavrdt*yyr(1)+zavldt*yyl(1)

#ifndef con
!        6.7       Fluxes
#ifdef ethe
                flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2)
                flxh(jx)=zavrdt+zavldt
#else
#ifdef etot
#ifdef mhd
                    flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2) &
                      +dth*(zptotr*zv3r+zptotl*zv3l)       &
                      -dthene*(zer*yyr(8)+zel*yyl(8))
#else
                    flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2) &
                      +dth*(zpther*zv3r+zpthel*zv3l)
#endif
#else
                  flxg(jx,2)=zavrdt*yyr(2)+zavldt*yyl(2) &
                    +dth*(zpther*zv3r+zpthel*zv3l)
#endif
#endif
!        6.5       Fluxes
#ifdef mhd
                flxg(jx,3)=zavrdt*yyr(3)+zavldt*yyl(3) &
                          -zabrdt*yyr(6)-zabldt*yyl(6)
                flxg(jx,4)=zavrdt*yyr(4)+zavldt*yyl(4) &
                          -zabrdt*yyr(7)-zabldt*yyl(7)
                flxg(jx,5)=zavrdt*yyr(5)+zavldt*yyl(5) &
                          -zabrdt*yyr(8)-zabldt*yyl(8)
                difh(jx)=dthmom*(zptotr+zptotl)
#else
                flxg(jx,3)=zavrdt*yyr(3)+zavldt*yyl(3)
                flxg(jx,4)=zavrdt*yyr(4)+zavldt*yyl(4)
                flxg(jx,5)=zavrdt*yyr(5)+zavldt*yyl(5)
                difh(jx)=dthmom*(zpther+zpthel)
#endif
!        6.6       Fluxes
#ifdef mhd
                flxg(jx,6)=dth*(yyr(6)*zv3r+yyl(6)*zv3l  &
                               -yyr(8)*zv1r-yyl(8)*zv1l) &
                          +zcdt*(yyl(6)-yyr(6))
                flxg(jx,7)=dth*(yyr(7)*zv3r+yyl(7)*zv3l  &
                               -yyr(8)*zv2r-yyl(8)*zv2l) &
                          +zcdt*(yyl(7)-yyr(7))
                ivar=8
#else
                ivar=5
#endif
!        6.7       Fluxes
#ifdef hydcon
                ivar=ivar+1
                flxg(jx,ivar)=zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#ifdef mhdcon
                ivar=ivar+1
                flxg(jx,ivar)=zavrdt*yyr(ivar)+zavldt*yyl(ivar)
#endif
#endif
          enddo
!---------------------------------------------------------------------
!      7.        Solve the full step
          do jx=3,nc3-2
!        7.2       Continuity equations
              yy(jx,1)=yyo(jx,1)+rvol(jx)*(-flxg(jx+1,1)+flxg(jx,1))
#ifdef con
              yy(jx,3)=yy(jx,1)*vel1(jx)
              yy(jx,4)=yy(jx,1)*vel2(jx)
              yy(jx,5)=yy(jx,1)*vel3(jx)
#else
!        7.3       Energy equations
#ifdef ethe
                yy(jx,2)=yyo(jx,2)+rvol(jx)*(-flxg(jx+1,2)+flxg(jx,2) &
                  +pthe(jx)*(-flxh(jx+1)+flxh(jx)))
#else
                yy(jx,2)=yyo(jx,2)+rvol(jx)*(-flxg(jx+1,2)+flxg(jx,2))
#endif
!        7.4       Momentum equations
              yy(jx,3)=yyo(jx,3)+rvol(jx)*(-flxg(jx+1,3)+flxg(jx,3))
              yy(jx,4)=yyo(jx,4)+rvol(jx)*(-flxg(jx+1,4)+flxg(jx,4))
              yy(jx,5)=yyo(jx,5)+rvol(jx)*(-flxg(jx+1,5)+flxg(jx,5)) &
                +rdx(jx)*(-difh(jx+1)+difh(jx))
!        7.5       Magnetic field equations
#ifdef mhd
                yy(jx,6)=yyo(jx,6)+rvol(jx)*(-flxg(jx+1,6)+flxg(jx,6))
                yy(jx,7)=yyo(jx,7)+rvol(jx)*(-flxg(jx+1,7)+flxg(jx,7))
                yy(jx,8)=yyo(jx,8)
                ivar=8
#else
                ivar=5
#endif
!        7.6       Continuity equations
#ifdef hydcon
                ivar=ivar+1
                yy(jx,ivar)=yyo(jx,ivar) &
                  +rvol(jx)*(-flxg(jx+1,ivar)+flxg(jx,ivar))
#endif
#ifdef mhdcon
               ivar=ivar+1
               yy(jx,ivar)=yyo(jx,ivar) &
                 +rvol(jx)*(-flxg(jx+1,ivar)+flxg(jx,ivar))
#endif
#endif
        enddo
!---------------------------------------------------------------------
!      8.        Epilogue
#ifdef PARAMESH
#ifdef consflux
!        8.1       Save fluxes at block boundaries
            do jvar=1,nnvar
              flux_z(jvar,j1,j2,1,jb)=flxg(nc3l,jvar)
              flux_z(jvar,j1,j2,2,jb)=flxg(nc3r+1,jvar)
            enddo
            flux_z(5,j1,j2,1,jb)=flux_z(5,j1,j2,1,jb)+difh(nc3l)
            flux_z(5,j1,j2,2,jb)=flux_z(5,j1,j2,2,jb)+difh(nc3r+1)
#endif
#endif
!        8.1       Put 1-D buffers to 3-D arrays
        do jx=3,nc3-2
          do jvar=1,nnvar
            unk(jvar,j1,j2,jx,jb)=yy(jx,jvar)
          enddo
        enddo
      enddo
    enddo
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
        endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
!      9.        Update values at block boundaries
#ifdef PARAMESH
#ifdef consflux
      call AMR_RESTRICT(mype,1,0)
      call AMR_GUARDCELL(mype,1,nguard)
      call AMR_FLUX_CONSERVE(mype,1)
      do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
          if(nodetype(jb).eq.1) then
#endif
        rvol(nc3l)=rdx3(nc3l,jb)
        rvol(nc3r)=rdx3(nc3r,jb)
        do jvar=1,5
          unk(jvar,:,:,nc3l,jb)=unk(jvar,:,:,nc3l,jb) &
            +rvol(nc3l)*(flux_z(jvar,:,:,1,jb)-tflux_z(jvar,:,:,1,jb))
          unk(jvar,:,:,nc3r,jb)=unk(jvar,:,:,nc3r,jb) &
            +rvol(nc3r)*(tflux_z(jvar,:,:,2,jb)-flux_z(jvar,:,:,2,jb))
        enddo
#ifdef mhd
          do jvar=6,7
            unk(jvar,:,:,nc3l,jb)=unk(jvar,:,:,nc3l,jb) &
              +rvol(nc3l)*(flux_z(jvar,:,:,1,jb)-tflux_z(jvar,:,:,1,jb))
            unk(jvar,:,:,nc3r,jb)=unk(jvar,:,:,nc3r,jb) &
             +rvol(nc3r)*(tflux_z(jvar,:,:,2,jb)-flux_z(jvar,:,:,2,jb))
          enddo
#endif
#ifndef ADVANCE_ALL_LEVELS
          endif
#endif
      enddo
#endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6003,0)
#endif     
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE MHD3                           
