!****f* e0pro/main
! NAME 
!   TOTH1
! PURPOSE
!   Toth div(B) correction
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE TOTH1 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif                           
  USE comoly
  USE comdim
  USE commet
  USE comreg
  USE comglo
  USE comtva
  IMPLICIT NONE
#ifdef toth
  REAL(KIND=8)  ze1c, ze1p, ze1m
  REAL(KIND=8)  ze2c, ze2p, ze2m
  REAL(KIND=8)  ze3c, ze3p, ze3m
  INTEGER  jx,j1,j2,j3,jb
#endif
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
    call EXPERT(6004,1)
#endif
!---------------------------------------------------------------------
#ifdef toth
#ifdef mhd
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
!loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
        if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!---------------------------------------------------------------------
!      1.        X1-sweep
!        1.1       X1-grid parameters
#ifdef PARAMESH
    do jx=nc1lm, nc1rp
#ifdef cartesian
        ax12(jx)=zone
        ax13(jx)=zone
        rvol12(jx)=rvol1(jx,jb)
        rvol13(jx)=rvol12(jx)
#endif
#ifdef cylindrical
        ax12(jx)=zone
        ax13(jx)=xc1(jx,jb)
        rvol12(jx)=rdx1(jx,jb)
        rvol13(jx)=rx1(jx,jb)*rvol12(jx)
#endif
#ifdef spherical
        ax12(jx)=xc1(jx,jb)
        ax13(jx)=ax12(jx)
        rvol12(jx)=rx1(jx,jb)*rdx1(jx,jb)
        rvol13(jx)=rvol12(jx)
#endif
    enddo
#endif
!        1.1       X1-X2-X3 model
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      ze3c=ax12(nc1l-1)*                                 &
        (unk(7,nc1l-1,j2,j3,jb)*unk(3,nc1l-1,j2,j3,jb)   &
        -unk(6,nc1l-1,j2,j3,jb)*unk(4,nc1l-1,j2,j3,jb))/ &
        unk(1,nc1l-1,j2,j3,jb)
      ze2c=ax13(nc1l-1)*                                 &
        (unk(6,nc1l-1,j2,j3,jb)*unk(5,nc1l-1,j2,j3,jb)   &
        -unk(8,nc1l-1,j2,j3,jb)*unk(3,nc1l-1,j2,j3,jb))/ &
        unk(1,nc1l-1,j2,j3,jb)
      ze3p=ax12(nc1l)*                               &
        (unk(7,nc1l,j2,j3,jb)*unk(3,nc1l,j2,j3,jb)   &
        -unk(6,nc1l,j2,j3,jb)*unk(4,nc1l,j2,j3,jb))/ &
        unk(1,nc1l,j2,j3,jb)
      ze2p=ax13(nc1l)*                               &
        (unk(6,nc1l,j2,j3,jb)*unk(5,nc1l,j2,j3,jb)   &
        -unk(8,nc1l,j2,j3,jb)*unk(3,nc1l,j2,j3,jb))/ &
        unk(1,nc1l,j2,j3,jb)
      do jx=nc1l,nc1r
        ze3m=ze3c
        ze2m=ze2c
        ze3c=ze3p
        ze2c=ze2p
        ze3p=ax12(jx+1)*                               &
          (unk(7,jx+1,j2,j3,jb)*unk(3,jx+1,j2,j3,jb)   &
          -unk(6,jx+1,j2,j3,jb)*unk(4,jx+1,j2,j3,jb))/ &
          unk(1,jx+1,j2,j3,jb)
        ze2p=ax13(jx+1)*                               &
          (unk(6,jx+1,j2,j3,jb)*unk(5,jx+1,j2,j3,jb)   &
          -unk(8,jx+1,j2,j3,jb)*unk(3,jx+1,j2,j3,jb))/ &
          unk(1,jx+1,j2,j3,jb)
        unka(2,jx,j2,j3,jb)=unk(7,jx,j2,j3,jb) &
          +dtq*rvol12(jx)*(-ze3p+ze3m)
        unka(3,jx,j2,j3,jb)=unk(8,jx,j2,j3,jb) &
          +dtq*rvol13(jx)*(ze2p-ze2m)
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      2.        X2-sweep
#ifdef x2dim
!        2.1       X1-X2-X3 model
    do j1=nc1l,nc1r
      do j3=nc3l,nc3r
        ze3c=ax2(nc2l-1,jb)*                               &
          (unk(7,j1,nc2l-1,j3,jb)*unk(3,j1,nc2l-1,j3,jb)   &
          -unk(6,j1,nc2l-1,j3,jb)*unk(4,j1,nc2l-1,j3,jb))/ &
          unk(1,j1,nc2l-1,j3,jb)
        ze1c=(unk(8,j1,nc2l-1,j3,jb)*unk(4,j1,nc2l-1,j3,jb)   &
             -unk(7,j1,nc2l-1,j3,jb)*unk(5,j1,nc2l-1,j3,jb))/ &
             unk(1,j1,nc2l-1,j3,jb)
        ze3p=ax2(nc2l,jb)*                             &
          (unk(7,j1,nc2l,j3,jb)*unk(3,j1,nc2l,j3,jb)   &
          -unk(6,j1,nc2l,j3,jb)*unk(4,j1,nc2l,j3,jb))/ &
          unk(1,j1,nc2l,j3,jb)
        ze1p=(unk(8,j1,nc2l,j3,jb)*unk(4,j1,nc2l,j3,jb)   &
             -unk(7,j1,nc2l,j3,jb)*unk(5,j1,nc2l,j3,jb))/ &
             unk(1,j1,nc2l,j3,jb)
        do jx=nc2l,nc2r
#ifdef cartesian
            rvol23(jx)=rdx2(jx,jb)
            rvol(jx)=rvol23(jx)
#endif
#ifdef cylindrical
            rvol23(jx)=rx1(j1,jb)*rdx2(jx,jb)
            rvol(jx)=rvol23(jx)
#endif
#ifdef spherical
            rvol23(jx)=rx1(j1,jb)*rdx2(jx,jb)
            rvol(jx)=rvol23(jx)*rsint(jx,jb)
#endif
          ze3m=ze3c
          ze1m=ze1c
          ze3c=ze3p
          ze1c=ze1p
          ze3p=ax2(jx+1,jb)*                             &
            (unk(7,j1,jx+1,j3,jb)*unk(3,j1,jx+1,j3,jb)   &
            -unk(6,j1,jx+1,j3,jb)*unk(4,j1,jx+1,j3,jb))/ &
            unk(1,j1,jx+1,j3,jb)
          ze1p=(unk(8,j1,jx+1,j3,jb)*unk(4,j1,jx+1,j3,jb)   &
               -unk(7,j1,jx+1,j3,jb)*unk(5,j1,jx+1,j3,jb))/ &
               unk(1,j1,jx+1,j3,jb)
          unka(1,j1,jx,j3,jb)=unk(6,j1,jx,j3,jb) &
            +dtq*rvol(jx)*(ze3p-ze3m)
          unka(3,j1,jx,j3,jb)=unka(3,j1,jx,j3,jb) &
            +dtq*rvol23(jx)*(-ze1p+ze1m)
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      3.        X3-sweep
#ifdef x3dim
!        3.1       X1-X2-X3 model
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        ze2c=(unk(6,j1,j2,nc3l-1,jb)*unk(5,j1,j2,nc3l-1,jb)   &
             -unk(8,j1,j2,nc3l-1,jb)*unk(3,j1,j2,nc3l-1,jb))/ &
             unk(1,j1,j2,nc3l-1,jb)
        ze1c=(unk(8,j1,j2,nc3l-1,jb)*unk(4,j1,j2,nc3l-1,jb)   &
             -unk(7,j1,j2,nc3l-1,jb)*unk(5,j1,j2,nc3l-1,jb))/ &
             unk(1,j1,j2,nc3l-1,jb)
        ze2p=(unk(6,j1,j2,nc3l,jb)*unk(5,j1,j2,nc3l,jb)   &
             -unk(8,j1,j2,nc3l,jb)*unk(3,j1,j2,nc3l,jb))/ &
             unk(1,j1,j2,nc3l,jb)
        ze1p=(unk(8,j1,j2,nc3l,jb)*unk(4,j1,j2,nc3l,jb)   &
             -unk(7,j1,j2,nc3l,jb)*unk(5,j1,j2,nc3l,jb))/ &
             unk(1,j1,j2,nc3l,jb)
        do jx=nc3l,nc3r
#ifdef cartesian
            rvol(jx)=rdx3(jx,jb)
#endif
#ifdef cylindrical
            rvol(jx)=rdx3(jx,jb)
#endif
#ifdef spherical
            rvol(jx)=rx1(j1,jb)*rsint(j2,jb)*rdx3(jx,jb)
#endif
          ze2m=ze2c
          ze1m=ze1c
          ze2c=ze2p
          ze1c=ze1p
          ze2p=(unk(6,j1,j2,jx+1,jb)*unk(5,j1,j2,jx+1,jb)   &
               -unk(8,j1,j2,jx+1,jb)*unk(3,j1,j2,jx+1,jb))/ &
               unk(1,j1,j2,jx+1,jb)
          ze1p=(unk(8,j1,j2,jx+1,jb)*unk(4,j1,j2,jx+1,jb)   &
               -unk(7,j1,j2,jx+1,jb)*unk(5,j1,j2,jx+1,jb))/ &
               unk(1,j1,j2,jx+1,jb)
#ifdef x2dim
            unka(1,j1,j2,jx,jb)=unka(1,j1,j2,jx,jb) &
              +dtq*rvol(jx)*(-ze2p+ze2m)
#else
            unka(1,j1,j2,jx,jb)=unk(6,j1,j2,jx,jb) &
              +dtq*rvol(jx)*(-ze2p+ze2m)
#endif
          unka(2,j1,j2,jx,jb)=unka(2,j1,j2,jx,jb) &
            +dtq*rvol(jx)*(ze1p-ze1m)
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
      endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
#endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6004,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE TOTH1


!****f* e0pro/main
! NAME 
!   TOTH2
! PURPOSE
!   Toth div(B) correction
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE TOTH2
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE commet
  USE comreg
  USE comglo
  USE comtva
  IMPLICIT NONE
#ifdef toth
  REAL(KIND=8)  ze1c, ze1p, ze1m
  REAL(KIND=8)  ze2c, ze2p, ze2m
  REAL(KIND=8)  ze3c, ze3p, ze3m
  INTEGER  jx,j1,j2,j3,jb
#endif
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
    call EXPERT(6005,1)
#endif
!---------------------------------------------------------------------
#ifdef toth
#ifdef mhd
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
! loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
        if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!---------------------------------------------------------------------
!      1.        X1-sweep
!        1.1       X1-grid parameters
#ifdef PARAMESH
    do jx=nc1lm,nc1rp
#ifdef cartesian
        ax12(jx)=zone
        ax13(jx)=zone
        rvol12(jx)=rvol1(jx,jb)
        rvol13(jx)=rvol12(jx)
#endif
#ifdef cylindrical
        ax12(jx)=zone
        ax13(jx)=xc1(jx,jb)
        rvol12(jx)=rdx1(jx,jb)
        rvol13(jx)=rx1(jx,jb)*rvol12(jx)
#endif
#ifdef spherical
        ax12(jx)=xc1(jx,jb)
        ax13(jx)=ax12(jx)
        rvol12(jx)=rx1(jx,jb)*rdx1(jx,jb)
        rvol13(jx)=rvol12(jx)
#endif
    enddo
#endif
!        1.1       X1-X2-X3 model
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      ze3c=ax12(nc1l-1)*                                 &
        (unk(7,nc1l-1,j2,j3,jb)*unk(3,nc1l-1,j2,j3,jb)   &
        -unk(6,nc1l-1,j2,j3,jb)*unk(4,nc1l-1,j2,j3,jb))/ &
        unk(1,nc1l-1,j2,j3,jb)
      ze2c=ax13(nc1l-1)*                                 &
        (unk(6,nc1l-1,j2,j3,jb)*unk(5,nc1l-1,j2,j3,jb)   &
        -unk(8,nc1l-1,j2,j3,jb)*unk(3,nc1l-1,j2,j3,jb))/ &
        unk(1,nc1l-1,j2,j3,jb)
      ze3p=ax12(nc1l)*                               &
        (unk(7,nc1l,j2,j3,jb)*unk(3,nc1l,j2,j3,jb)   &
        -unk(6,nc1l,j2,j3,jb)*unk(4,nc1l,j2,j3,jb))/ &
        unk(1,nc1l,j2,j3,jb)
      ze2p=ax13(nc1l)*                               &
        (unk(6,nc1l,j2,j3,jb)*unk(5,nc1l,j2,j3,jb)   &
        -unk(8,nc1l,j2,j3,jb)*unk(3,nc1l,j2,j3,jb))/ &
        unk(1,nc1l,j2,j3,jb)
      do jx=nc1l,nc1r
        ze3m=ze3c
        ze2m=ze2c
        ze3c=ze3p
        ze2c=ze2p
        ze3p=ax12(jx+1)*                               &
          (unk(7,jx+1,j2,j3,jb)*unk(3,jx+1,j2,j3,jb)   &
          -unk(6,jx+1,j2,j3,jb)*unk(4,jx+1,j2,j3,jb))/ &
          unk(1,jx+1,j2,j3,jb)
        ze2p=ax13(jx+1)*                               &
          (unk(6,jx+1,j2,j3,jb)*unk(5,jx+1,j2,j3,jb)   &
          -unk(8,jx+1,j2,j3,jb)*unk(3,jx+1,j2,j3,jb))/ &
          unk(1,jx+1,j2,j3,jb)
        unka(2,jx,j2,j3,jb)=unka(2,jx,j2,j3,jb) &
          +dtq*rvol12(jx)*(-ze3p+ze3m)
        unka(3,jx,j2,j3,jb)=unka(3,jx,j2,j3,jb) &
          +dtq*rvol13(jx)*(ze2p-ze2m)
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      2.        X2-sweep
#ifdef x2dim
!        2.1       X1-X2-X3 model
    do j1=nc1l,nc1r
      do j3=nc3l,nc3r
        ze3c=ax2(nc2l-1,jb)*                               &
          (unk(7,j1,nc2l-1,j3,jb)*unk(3,j1,nc2l-1,j3,jb)   &
          -unk(6,j1,nc2l-1,j3,jb)*unk(4,j1,nc2l-1,j3,jb))/ &
          unk(1,j1,nc2l-1,j3,jb)
        ze1c=(unk(8,j1,nc2l-1,j3,jb)*unk(4,j1,nc2l-1,j3,jb)   &
             -unk(7,j1,nc2l-1,j3,jb)*unk(5,j1,nc2l-1,j3,jb))/ &
             unk(1,j1,nc2l-1,j3,jb)
        ze3p=ax2(nc2l,jb)*                             &
          (unk(7,j1,nc2l,j3,jb)*unk(3,j1,nc2l,j3,jb)   &
          -unk(6,j1,nc2l,j3,jb)*unk(4,j1,nc2l,j3,jb))/ &
          unk(1,j1,nc2l,j3,jb)
        ze1p=(unk(8,j1,nc2l,j3,jb)*unk(4,j1,nc2l,j3,jb) &
          -unk(7,j1,nc2l,j3,jb)*unk(5,j1,nc2l,j3,jb))/  &
          unk(1,j1,nc2l,j3,jb)
        do jx=nc2l,nc2r
#ifdef cartesian
            rvol23(jx)=rdx2(jx,jb)
            rvol(jx)=rvol23(jx)
#endif
#ifdef cylindrical
            rvol23(jx)=rx1(j1,jb)*rdx2(jx,jb)
            rvol(jx)=rvol23(jx)
#endif
#ifdef spherical
            rvol23(jx)=rx1(j1,jb)*rdx2(jx,jb)
            rvol(jx)=rvol23(jx)*rsint(jx,jb)
#endif
          ze3m=ze3c
          ze1m=ze1c
          ze3c=ze3p
          ze1c=ze1p
          ze3p=ax2(jx+1,jb)*                             &
            (unk(7,j1,jx+1,j3,jb)*unk(3,j1,jx+1,j3,jb)   &
            -unk(6,j1,jx+1,j3,jb)*unk(4,j1,jx+1,j3,jb))/ &
            unk(1,j1,jx+1,j3,jb)
          ze1p=(unk(8,j1,jx+1,j3,jb)*unk(4,j1,jx+1,j3,jb)   &
               -unk(7,j1,jx+1,j3,jb)*unk(5,j1,jx+1,j3,jb))/ &
               unk(1,j1,jx+1,j3,jb)
          unka(1,j1,jx,j3,jb)=unka(1,j1,jx,j3,jb) &
            +dtq*rvol(jx)*(ze3p-ze3m)
          unka(3,j1,jx,j3,jb)=unka(3,j1,jx,j3,jb) &
            +dtq*rvol23(jx)*(-ze1p+ze1m)
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      3.        X3-sweep
#ifdef x3dim
!        3.1       X1-X2-X3 model
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        ze2c=(unk(6,j1,j2,nc3l-1,jb)*unk(5,j1,j2,nc3l-1,jb)   &
             -unk(8,j1,j2,nc3l-1,jb)*unk(3,j1,j2,nc3l-1,jb))/ &
             unk(1,j1,j2,nc3l-1,jb)
        ze1c=(unk(8,j1,j2,nc3l-1,jb)*unk(4,j1,j2,nc3l-1,jb)   &
             -unk(7,j1,j2,nc3l-1,jb)*unk(5,j1,j2,nc3l-1,jb))/ &
             unk(1,j1,j2,nc3l-1,jb)
        ze2p=(unk(6,j1,j2,nc3l,jb)*unk(5,j1,j2,nc3l,jb)   &
             -unk(8,j1,j2,nc3l,jb)*unk(3,j1,j2,nc3l,jb))/ &
             unk(1,j1,j2,nc3l,jb)
        ze1p=(unk(8,j1,j2,nc3l,jb)*unk(4,j1,j2,nc3l,jb)   &
             -unk(7,j1,j2,nc3l,jb)*unk(5,j1,j2,nc3l,jb))/ &
             unk(1,j1,j2,nc3l,jb)
        do jx=nc3l,nc3r
#ifdef cartesian
            rvol(jx)=rdx3(jx,jb)
#endif
#ifdef cylindrical
            rvol(jx)=rdx3(jx,jb)
#endif
#ifdef spherical
            rvol(jx)=rx1(j1,jb)*rsint(j2,jb)*rdx3(jx,jb)
#endif
          ze2m=ze2c
          ze1m=ze1c
          ze2c=ze2p
          ze1c=ze1p
          ze2p=(unk(6,j1,j2,jx+1,jb)*unk(5,j1,j2,jx+1,jb)   &
               -unk(8,j1,j2,jx+1,jb)*unk(3,j1,j2,jx+1,jb))/ &
               unk(1,j1,j2,jx+1,jb)
          ze1p=(unk(8,j1,j2,jx+1,jb)*unk(4,j1,j2,jx+1,jb)   &
               -unk(7,j1,j2,jx+1,jb)*unk(5,j1,j2,jx+1,jb))/ &
               unk(1,j1,j2,jx+1,jb)
          unka(1,j1,j2,jx,jb)=unka(1,j1,j2,jx,jb) &
            +dtq*rvol(jx)*(-ze2p+ze2m)
          unka(2,j1,j2,jx,jb)=unka(2,j1,j2,jx,jb) &
            +dtq*rvol(jx)*(ze1p-ze1m)
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      5.        Update new values
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        unk(6,j1,j2,j3,jb)=unka(1,j1,j2,j3,jb)
        unk(7,j1,j2,j3,jb)=unka(2,j1,j2,j3,jb)
        unk(8,j1,j2,j3,jb)=unka(3,j1,j2,j3,jb)
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
      endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
#endif
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6005,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE TOTH2


!****f* e0pro/main
! NAME 
!   POWELH
! PURPOSE
!   Powell div(B) correction
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE POWELH
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE commet
  USE comreg
  USE comglo
  USE comtva
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef powell
  REAL(KIND=8)  zdivb, zrd, zv1, zv2, zv3
  INTEGER  j1,j2,j3,jb
#endif
#ifdef DIAGNO
    call EXPERT(6005,1)
#endif
!---------------------------------------------------------------------
#ifdef powell
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
! loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
      if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!---------------------------------------------------------------------
!      1.        X1-sweep
!        1.1       X1-X2-X3 model
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      do j1=nc1l,nc1rp
        difh(j1)=dth*ah1(j1,jb)* &
          (unk(6,j1,j2,j3,jb)+unk(6,j1-1,j2,j3,jb))
      enddo
      do j1=nc1l,nc1r
        zdivb=rvol1(j1,jb)*(difh(j1+1)-difh(j1))
        zrd=zone/unk(1,j1,j2,j3,jb)
        zv1=unk(3,j1,j2,j3,jb)*zrd
        zv2=unk(4,j1,j2,j3,jb)*zrd
        zv3=unk(5,j1,j2,j3,jb)*zrd
        t(j1,j2,j3)=unk(2,j1,j2,j3,jb)-zdivb* &
          (unk(6,j1,j2,j3,jb)*zv1             &
          +unk(7,j1,j2,j3,jb)*zv2             &
          +unk(8,j1,j2,j3,jb)*zv3)
        v1(j1,j2,j3)=unk(3,j1,j2,j3,jb)-zdivb*unk(6,j1,j2,j3,jb)
        v2(j1,j2,j3)=unk(4,j1,j2,j3,jb)-zdivb*unk(7,j1,j2,j3,jb)
        v3(j1,j2,j3)=unk(5,j1,j2,j3,jb)-zdivb*unk(8,j1,j2,j3,jb)
        b1(j1,j2,j3)=unk(6,j1,j2,j3,jb)-zdivb*zv1
        b2(j1,j2,j3)=unk(7,j1,j2,j3,jb)-zdivb*zv2
        b3(j1,j2,j3)=unk(8,j1,j2,j3,jb)-zdivb*zv3
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      2.        X2-sweep
#ifdef x2dim
!        2.1       X1-X2-X3 model
    do j1=nc1l,nc1r
      do j3=nc3l,nc3r
        do j2=nc2l,nc2rp
          difh(j2)=dth*ah2(j2,jb)* &
            (unk(7,j1,j2,j3,jb)+unk(7,j1,j2-1,j3,jb))
        enddo
        do j2=nc2l,nc2r
#ifdef cartesian
            zdivb=rdx2(j2,jb)*(difh(j2+1)-difh(j2))
#endif
#ifdef cylindrical
            zdivb=rx1(j1,jb)*rdx2(j2,jb)*(difh(j2+1)-difh(j2))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx2(j2,jb)* &
              (difh(j2+1)-difh(j2))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
#ifdef x3dim
            t(j1,j2,j3)=t(j1,j2,j3)-zdivb* &
              (unk(6,j1,j2,j3,jb)*zv1      &
              +unk(7,j1,j2,j3,jb)*zv2      &
              +unk(8,j1,j2,j3,jb)*zv3)
            v1(j1,j2,j3)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
            v2(j1,j2,j3)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
            v3(j1,j2,j3)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
            b1(j1,j2,j3)=b1(j1,j2,j3)-zdivb*zv1
            b2(j1,j2,j3)=b2(j1,j2,j3)-zdivb*zv2
            b3(j1,j2,j3)=b3(j1,j2,j3)-zdivb*zv3
#else
            unk(2,j1,j2,j3,jb)=t(j1,j2,j3)-zdivb* &
              (unk(6,j1,j2,j3,jb)*zv1             &
              +unk(7,j1,j2,j3,jb)*zv2             &
              +unk(8,j1,j2,j3,jb)*zv3)
            unk(3,j1,j2,j3,jb)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
            unk(4,j1,j2,j3,jb)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
            unk(5,j1,j2,j3,jb)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
            unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
            unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
            unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      3.        X3-sweep
#ifdef x3dim
!        3.1       X1-X2-X3 model
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        do j3=nc3l,nc3rp
          difh(j3)=dth*(unk(8,j1,j2,j3,jb)+unk(8,j1,j2,j3-1,jb))
        enddo
        do j3=nc3l,nc3r
#ifdef cartesian
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3))
#endif
#ifdef cylindrical
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)* &
              (difh(j3+1)-difh(j3))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
          unk(2,j1,j2,j3,jb)=t(j1,j2,j3)-zdivb* &
            (unk(6,j1,j2,j3,jb)*zv1             &
            +unk(7,j1,j2,j3,jb)*zv2             &
            +unk(8,j1,j2,j3,jb)*zv3)
          unk(3,j1,j2,j3,jb)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
          unk(4,j1,j2,j3,jb)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
          unk(5,j1,j2,j3,jb)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
          unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
          unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
          unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
        endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6005,0) 
#endif                           
!---------------------------------------------------------------------  
RETURN
END SUBROUTINE POWELH


!****f* e0pro/main
! NAME 
!   POWELC
! PURPOSE
!   Powell div(B) correction
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE POWELC
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE commet
  USE comreg
  USE comglo
  USE comtva
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef powell
  REAL(KIND=8)  zdivb, zrd, zv1, zv2, zv3
  INTEGER  j1,j2,j3,jb
#endif
#ifdef DIAGNO
    call EXPERT(6005,1)
#endif
!---------------------------------------------------------------------
#ifdef powell
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
! loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
      if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!---------------------------------------------------------------------
!      1.        X1-sweep
!        1.1       X1-X2-X3 model
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      do j1=nc1lm,nc1rp
        difh(j1)=dtq*ax1(j1,jb)*unk(6,j1,j2,j3,jb)
      enddo
      do j1=nc1l,nc1r
        zdivb=rvol1(j1,jb)*(difh(j1+1)-difh(j1-1))
        zrd=zone/unk(1,j1,j2,j3,jb)
        zv1=unk(3,j1,j2,j3,jb)*zrd
        zv2=unk(4,j1,j2,j3,jb)*zrd
        zv3=unk(5,j1,j2,j3,jb)*zrd
        t(j1,j2,j3)=unk(2,j1,j2,j3,jb)-zdivb* &
          (unk(6,j1,j2,j3,jb)*zv1             &
          +unk(7,j1,j2,j3,jb)*zv2             &
          +unk(8,j1,j2,j3,jb)*zv3)
        v1(j1,j2,j3)=unk(3,j1,j2,j3,jb)-zdivb*unk(6,j1,j2,j3,jb)
        v2(j1,j2,j3)=unk(4,j1,j2,j3,jb)-zdivb*unk(7,j1,j2,j3,jb)
        v3(j1,j2,j3)=unk(5,j1,j2,j3,jb)-zdivb*unk(8,j1,j2,j3,jb)
        b1(j1,j2,j3)=unk(6,j1,j2,j3,jb)-zdivb*zv1
        b2(j1,j2,j3)=unk(7,j1,j2,j3,jb)-zdivb*zv2
        b3(j1,j2,j3)=unk(8,j1,j2,j3,jb)-zdivb*zv3
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      2.        X2-sweep
#ifdef x2dim
!        2.1       X1-X2-X3 model
    do j1=nc1l,nc1r
      do j3=nc3l,nc3r
        do j2=nc2lm,nc2rp
          difh(j2)=dtq*ax2(j2,jb)*unk(7,j1,j2,j3,jb)
        enddo
        do j2=nc2l,nc2r
#ifdef cartesian
            zdivb=rdx2(j2,jb)*(difh(j2+1)-difh(j2-1))
#endif
#ifdef cylindrical
            zdivb=rx1(j1,jb)*rdx2(j2,jb)*(difh(j2+1)-difh(j2-1))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx2(j2,jb)* &
              (difh(j2+1)-difh(j2-1))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
#ifdef x3dim
            t(j1,j2,j3)=t(j1,j2,j3)-zdivb* &
              (unk(6,j1,j2,j3,jb)*zv1      &
              +unk(7,j1,j2,j3,jb)*zv2      &
              +unk(8,j1,j2,j3,jb)*zv3)
            v1(j1,j2,j3)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
            v2(j1,j2,j3)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
            v3(j1,j2,j3)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
            b1(j1,j2,j3)=b1(j1,j2,j3)-zdivb*zv1
            b2(j1,j2,j3)=b2(j1,j2,j3)-zdivb*zv2
            b3(j1,j2,j3)=b3(j1,j2,j3)-zdivb*zv3
#else
            unk(2,j1,j2,j3,jb)=t(j1,j2,j3)-zdivb* &
              (unk(6,j1,j2,j3,jb)*zv1             &
              +unk(7,j1,j2,j3,jb)*zv2             &
              +unk(8,j1,j2,j3,jb)*zv3)
            unk(3,j1,j2,j3,jb)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
            unk(4,j1,j2,j3,jb)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
            unk(5,j1,j2,j3,jb)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
            unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
            unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
            unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      3.        X3-sweep
#ifdef x3dim
!        3.1       X1-X2-X3 model
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        do j3=nc3lm,nc3rp
          difh(j3)=dtq*unk(8,j1,j2,j3,jb)
        enddo
        do j3=nc3l,nc3r
#ifdef cartesian
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3-1))
#endif
#ifdef cylindrical
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3-1))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)* &
              (difh(j3+1)-difh(j3-1))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
          unk(2,j1,j2,j3,jb)=t(j1,j2,j3)-zdivb* &
            (unk(6,j1,j2,j3,jb)*zv1             &
            +unk(7,j1,j2,j3,jb)*zv2             &
            +unk(8,j1,j2,j3,jb)*zv3)
          unk(3,j1,j2,j3,jb)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
          unk(4,j1,j2,j3,jb)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
          unk(5,j1,j2,j3,jb)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
          unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
          unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
          unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
        endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6005,0) 
#endif                           
!---------------------------------------------------------------------  
RETURN
END SUBROUTINE POWELC


!****f* e0pro/main
! NAME 
!   POWELL
! PURPOSE
!   Powell div(B) correction
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE POWELL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE commet
  USE comreg
  USE comglo
  USE comtva
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef powell
  REAL(KIND=8)  zdivb, zrd, zv1, zv2, zv3
  INTEGER  j1,j2,j3,jb
#endif
#ifdef DIAGNO
    call EXPERT(6005,1)
#endif
!---------------------------------------------------------------------
#ifdef powell
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
! loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
      if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!---------------------------------------------------------------------
!      1.        X1-sweep
!        1.1       X1-X2-X3 model
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      do j1=nc1lm,nc1rp
        difh(j1)=dth*ax1(j1,jb)*unk(6,j1,j2,j3,jb)
      enddo
      do j1=nc1l,nc1r
        zdivb=rvol1(j1,jb)*(difh(j1+1)-difh(j1-1))
        zrd=zone/unk(1,j1,j2,j3,jb)
        zv1=unk(3,j1,j2,j3,jb)*zrd
        zv2=unk(4,j1,j2,j3,jb)*zrd
        zv3=unk(5,j1,j2,j3,jb)*zrd
        t(j1,j2,j3)=unk(2,j1,j2,j3,jb)-zdivb* &
          (unk(6,j1,j2,j3,jb)*zv1             &
          +unk(7,j1,j2,j3,jb)*zv2             &
          +unk(8,j1,j2,j3,jb)*zv3)
        v1(j1,j2,j3)=unk(3,j1,j2,j3,jb)-zdivb*unk(6,j1,j2,j3,jb)
        v2(j1,j2,j3)=unk(4,j1,j2,j3,jb)-zdivb*unk(7,j1,j2,j3,jb)
        v3(j1,j2,j3)=unk(5,j1,j2,j3,jb)-zdivb*unk(8,j1,j2,j3,jb)
        b1(j1,j2,j3)=unk(6,j1,j2,j3,jb)-zdivb*zv1
        b2(j1,j2,j3)=unk(7,j1,j2,j3,jb)-zdivb*zv2
        b3(j1,j2,j3)=unk(8,j1,j2,j3,jb)-zdivb*zv3
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      2.        X2-sweep
#ifdef x2dim
!        2.1       X1-X2-X3 model
    do j1=nc1l,nc1r
      do j3=nc3l,nc3r
        do j2=nc2lm,nc2rp
          difh(j2)=dth*ax2(j2,jb)*unk(7,j1,j2,j3,jb)
        enddo
        do j2=nc2l,nc2r
#ifdef cartesian
            zdivb=rdx2(j2,jb)*(difh(j2+1)-difh(j2-1))
#endif
#ifdef cylindrical
            zdivb=rx1(j1,jb)*rdx2(j2,jb)*(difh(j2+1)-difh(j2-1))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx2(j2,jb)* &
              (difh(j2+1)-difh(j2-1))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
#ifdef x3dim
            t(j1,j2,j3)=t(j1,j2,j3)-zdivb* &
              (unk(6,j1,j2,j3,jb)*zv1      &
              +unk(7,j1,j2,j3,jb)*zv2      &
              +unk(8,j1,j2,j3,jb)*zv3)
            v1(j1,j2,j3)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
            v2(j1,j2,j3)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
            v3(j1,j2,j3)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
            b1(j1,j2,j3)=b1(j1,j2,j3)-zdivb*zv1
            b2(j1,j2,j3)=b2(j1,j2,j3)-zdivb*zv2
            b3(j1,j2,j3)=b3(j1,j2,j3)-zdivb*zv3
#else
            unk(2,j1,j2,j3,jb)=t(j1,j2,j3)-zdivb* &
              (unk(6,j1,j2,j3,jb)*zv1             &
              +unk(7,j1,j2,j3,jb)*zv2             &
              +unk(8,j1,j2,j3,jb)*zv3)
            unk(3,j1,j2,j3,jb)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
            unk(4,j1,j2,j3,jb)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
            unk(5,j1,j2,j3,jb)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
            unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
            unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
            unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      3.        X3-sweep
#ifdef x3dim
!        3.1       X1-X2-X3 model
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        do j3=nc3lm,nc3rp
          difh(j3)=dth*unk(8,j1,j2,j3,jb)
        enddo
        do j3=nc3l,nc3r
#ifdef cartesian
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3-1))
#endif
#ifdef cylindrical
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3-1))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)* &
              (difh(j3+1)-difh(j3-1))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
          unk(2,j1,j2,j3,jb)=t(j1,j2,j3)-zdivb* &
            (unk(6,j1,j2,j3,jb)*zv1             &
            +unk(7,j1,j2,j3,jb)*zv2             &
            +unk(8,j1,j2,j3,jb)*zv3)
          unk(3,j1,j2,j3,jb)=v1(j1,j2,j3)-zdivb*unk(6,j1,j2,j3,jb)
          unk(4,j1,j2,j3,jb)=v2(j1,j2,j3)-zdivb*unk(7,j1,j2,j3,jb)
          unk(5,j1,j2,j3,jb)=v3(j1,j2,j3)-zdivb*unk(8,j1,j2,j3,jb)
          unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
          unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
          unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
        endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6005,0) 
#endif                           
!---------------------------------------------------------------------  
RETURN
END SUBROUTINE POWELL


!****f* e0pro/main
! NAME 
!   JANHUNEN
! PURPOSE
!   Janhunen div(B) correction
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE JANHUNEN
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE commet
  USE comreg
  USE comglo
  USE comtva
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef janhunen
  REAL(KIND=8)  zdivb, zrd, zv1, zv2, zv3
  INTEGER  j1,j2,j3,jb
#endif
#ifdef DIAGNO
    call EXPERT(6005,1)
#endif
!---------------------------------------------------------------------
#ifdef janhunen
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
! loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
      if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!---------------------------------------------------------------------
!      1.        X1-sweep
!        1.1       X1-X2-X3 model
! do j3=nc3l,nc3r
!   do j2=nc2l,nc2r
!     do j1=nc1l,nc1rp
!       difh(j1)=dth*ah1(j1,jb)* &
!         (unk(6,j1,j2,j3,jb)+unk(6,j1-1,j2,j3,jb))
!     enddo
!     do j1=nc1l,nc1r
!       zdivb=rvol1(j1,jb)*(difh(j1+1)-difh(j1))
!       zrd=zone/unk(1,j1,j2,j3,jb)
!       zv1=unk(3,j1,j2,j3,jb)*zrd
!       zv2=unk(4,j1,j2,j3,jb)*zrd
!       zv3=unk(5,j1,j2,j3,jb)*zrd
!       b1(j1,j2,j3)=unk(6,j1,j2,j3,jb)-zdivb*zv1
!       b2(j1,j2,j3)=unk(7,j1,j2,j3,jb)-zdivb*zv2
!       b3(j1,j2,j3)=unk(8,j1,j2,j3,jb)-zdivb*zv3
!     enddo
!   enddo
! enddo
!        1.1       X1-X2-X3 model
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        zdivb=rvol1(j1,jb)*dt*               &
          (ax1(j1+1,jb)*unk(6,j1+1,j2,j3,jb) &
          -ax1(j1-1,jb)*unk(6,j1-1,j2,j3,jb))
        zrd=zone/unk(1,j1,j2,j3,jb)
        zv1=unk(3,j1,j2,j3,jb)*zrd
        zv2=unk(4,j1,j2,j3,jb)*zrd
        zv3=unk(5,j1,j2,j3,jb)*zrd
        b1(j1,j2,j3)=unk(6,j1,j2,j3,jb)-zdivb*zv1
        b2(j1,j2,j3)=unk(7,j1,j2,j3,jb)-zdivb*zv2
        b3(j1,j2,j3)=unk(8,j1,j2,j3,jb)-zdivb*zv3
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      2.        X2-sweep
#ifdef x2dim
!        2.1       X1-X2-X3 model
!   do j1=nc1l,nc1r
!     do j3=nc3l,nc3r
!       do j2=nc2l,nc2rp
!         difh(j2)=dth*ah2(j2,jb)* &
!           (unk(7,j1,j2,j3,jb)+unk(7,j1,j2-1,j3,jb))
!       enddo
!       do j2=nc2l,nc2r
#ifdef cartesian
!           zdivb=rdx2(j2,jb)*(difh(j2+1)-difh(j2))
#endif
#ifdef cylindrical
!           zdivb=rx1(j1,jb)*rdx2(j2,jb)*(difh(j2+1)-difh(j2))
#endif
#ifdef spherical
!           zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx2(j2,jb)* &
!             (difh(j2+1)-difh(j2))
#endif
!         zrd=zone/unk(1,j1,j2,j3,jb)
!         zv1=unk(3,j1,j2,j3,jb)*zrd
!         zv2=unk(4,j1,j2,j3,jb)*zrd
!         zv3=unk(5,j1,j2,j3,jb)*zrd
#ifdef x3dim
!           b1(j1,j2,j3)=b1(j1,j2,j3)-zdivb*zv1
!           b2(j1,j2,j3)=b2(j1,j2,j3)-zdivb*zv2
!           b3(j1,j2,j3)=b3(j1,j2,j3)-zdivb*zv3
#else
!           unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
!           unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
!           unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
#endif
!       enddo
!     enddo
!   enddo
!        2.1       X1-X2-X3 model
    do j1=nc1l,nc1r
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
#ifdef cartesian
            zdivb=rdx2(j2,jb)*dt*                &
              (ax2(j2+1,jb)*unk(7,j1,j2+1,j3,jb) &
              -ax2(j2-1,jb)*unk(7,j1,j2-1,j3,jb))
#endif
#ifdef cylindrical
            zdivb=rx1(j1,jb)*rdx2(j2,jb)*(difh(j2+1)-difh(j2))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx2(j2,jb)* &
              (difh(j2+1)-difh(j2))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
#ifdef x3dim
            b1(j1,j2,j3)=b1(j1,j2,j3)-zdivb*zv1
            b2(j1,j2,j3)=b2(j1,j2,j3)-zdivb*zv2
            b3(j1,j2,j3)=b3(j1,j2,j3)-zdivb*zv3
#else
            unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
            unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
            unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      3.        X3-sweep
#ifdef x3dim
!        3.1       X1-X2-X3 model
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        do j3=nc3l,nc3rp
          difh(j3)=dth*(unk(8,j1,j2,j3,jb)+unk(8,j1,j2,j3-1,jb))
        enddo
        do j3=nc3l,nc3r
#ifdef cartesian
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3))
#endif
#ifdef cylindrical
            zdivb=rdx3(j3,jb)*(difh(j3+1)-difh(j3))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)* &
              (difh(j3+1)-difh(j3))
#endif
          zrd=zone/unk(1,j1,j2,j3,jb)
          zv1=unk(3,j1,j2,j3,jb)*zrd
          zv2=unk(4,j1,j2,j3,jb)*zrd
          zv3=unk(5,j1,j2,j3,jb)*zrd
          unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)-zdivb*zv1
          unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)-zdivb*zv2
          unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)-zdivb*zv3
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
        endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6005,0) 
#endif                           
!---------------------------------------------------------------------  
RETURN
END SUBROUTINE JANHUNEN


!****f* e0pro/main
! NAME 
!   DIFMAG
! PURPOSE
!   Diffusion of magnetic field
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE DIFMAG
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE comdim
  USE commet
  USE comreg
  USE comglo
  USE comtva
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef difb
  REAL(KIND=8)  zdivb, zdx, zdx2, zdx3, zcdif
  INTEGER  j1,j2,j3,jb
#endif
#ifdef DIAGNO
    call EXPERT(6005,1)
#endif
!---------------------------------------------------------------------
#ifdef difb
!---------------------------------------------------------------------
!      1.        Get variables
#ifdef PARAMESH
! loop over leaf grid blocks
    do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
      if(nodetype(jb).eq.1) then
#endif
#else
    jb=1
#endif
!---------------------------------------------------------------------
!      1.        X1-sweep
  zdx=dx1(3,1)
#ifdef x2dim
#ifdef cartesian
      zdx2=dx2(3,jb)
#endif
#ifdef cylindrical
      zdx2=xc1(3,jb)*dx2(3,jb)
#endif
#ifdef spherical
      zdx2=xc1(3,jb)*sint(2,jb)*dx2(3,jb)
#endif
    if(zdx2.lt.zdx) zdx=zdx2
#endif
#ifdef x3dim
#ifdef cartesian
      zdx3=dx3(3,jb)
#endif
#ifdef cylindrical
      zdx3=xc1(3,jb)*dx3(3,jb)
#endif
#ifdef spherical
      zdx3=xc1(3,jb)*sint(1,jb)*dx3(3,jb)
#endif
    if(zdx3.lt.zdx) zdx=zdx3
#endif
  zcdif=cdifb*0.04d0*zdx*zdx
write(*,*) 'cdif',zcdif/dt
!---------------------------------------------------------------------
!      1.        X1-sweep
!        1.1       X1-X2-X3 model
  do j3=nc3l,nc3r
    do j2=nc2l,nc2r
      do j1=nc1l-1,nc1r+1
        difh(j1)=zcdif*rvol1(j1,jb)*         &
          (ax1(j1+1,jb)*unk(6,j1+1,j2,j3,jb) &
          -ax1(j1-1,jb)*unk(6,j1-1,j2,j3,jb))
      enddo
      do j1=nc1l,nc1r
        zdivb=rdx1(j1,jb)*(difh(j1+1)-difh(j1-1))
        b1(j1,j2,j3)=unk(6,j1,j2,j3,jb)+zdivb
        b2(j1,j2,j3)=unk(7,j1,j2,j3,jb)+zdivb
        b3(j1,j2,j3)=unk(8,j1,j2,j3,jb)+zdivb
      enddo
    enddo
  enddo
!---------------------------------------------------------------------
!      2.        X2-sweep
#ifdef x2dim
!        2.1       X1-X2-X3 model
    do j1=nc1l,nc1r
      do j3=nc3l,nc3r
        do j2=nc2l-1,nc2r+1
#ifdef cartesian
            difh(j2)=zcdif*rdx2(j2,jb)*          &
              (ax2(j2+1,jb)*unk(7,j1,j2+1,j3,jb) &
              -ax2(j2-1,jb)*unk(7,j1,j2-1,j3,jb))
#endif
#ifdef cylindrical
            difh(j2)=zcdif*rx1(j1,jb)*rdx2(j2,jb)* &
              (ax2(j2+1,jb)*unk(7,j1,j2+1,j3,jb)   &
              -ax2(j2-1,jb)*unk(7,j1,j2-1,j3,jb))
#endif
#ifdef spherical
            difh(j2)=zcdif*rx1(j1,jb)*rsint(j2,jb)*rdx2(j2,jb)* &
              (ax2(j2+1,jb)*unk(7,j1,j2+1,j3,jb)                &
              -ax2(j2-1,jb)*unk(7,j1,j2-1,j3,jb))
#endif
        enddo
        do j2=nc2l,nc2r
#ifdef cartesian
            zdivb=rdx2(j2,jb)* &
              (difh(j2+1)-difh(j2-1))
#endif
#ifdef cylindrical
            zdivb=rx1(j1,jb)*rdx2(j2,jb)* &
              (difh(j2+1)-difh(j2-1))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx2(j2,jb)* &
              (difh(j2+1)-difh(j2-1))
#endif
#ifdef x3dim
            b1(j1,j2,j3)=b1(j1,j2,j3)+zdivb
            b2(j1,j2,j3)=b2(j1,j2,j3)+zdivb
            b3(j1,j2,j3)=b3(j1,j2,j3)+zdivb
#else
            unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)+zdivb
            unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)+zdivb
            unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)+zdivb
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!      3.        X3-sweep
#ifdef x3dim
!        3.1       X1-X2-X3 model
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        do j3=nc3l-1,nc3r+1
#ifdef cartesian
            difh(j3)=zcdif*rdx3(j3,jb)* &
              (unk(8,j1,j2,j3+1,jb)     &
              -unk(8,j1,j2,j3-1,jb))
#endif
#ifdef cylindrical
            difh(j3)=zcdif*rx1(j1,jb)*rdx3(j3,jb)* &
              (unk(8,j1,j2,j3+1,jb)                &
              -unk(8,j1,j2,j3-1,jb))
#endif
#ifdef spherical
            difh(j3)=zcdif*rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)* &
              (unk(8,j1,j2,j3+1,jb)                             &
              -unk(8,j1,j2,j3-1,jb))
#endif
        enddo
        do j3=nc3l,nc3r
          zdivb=difh(j3+1)-difh(j3-1)
#ifdef cartesian
            zdivb=rdx3(j3,jb)* &
              (difh(j3+1)-difh(j3-1))
#endif
#ifdef cylindrical
            zdivb=rx1(j1,jb)*rdx3(j3,jb)* &
              (difh(j3+1)-difh(j3-1))
#endif
#ifdef spherical
            zdivb=rx1(j1,jb)*rsint(j2,jb)*rdx3(j3,jb)* &
              (difh(j3+1)-difh(j3-1))
#endif
          unk(6,j1,j2,j3,jb)=b1(j1,j2,j3)+zdivb
          unk(7,j1,j2,j3,jb)=b2(j1,j2,j3)+zdivb
          unk(8,j1,j2,j3,jb)=b3(j1,j2,j3)+zdivb
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
#ifndef ADVANCE_ALL_LEVELS
        endif
#endif
    enddo
#endif
!---------------------------------------------------------------------
#endif
!---------------------------------------------------------------------
!      9.        Return or stop
!correct termination-return
#ifdef DIAGNO
    call EXPERT(6005,0) 
#endif                           
!---------------------------------------------------------------------  
RETURN
END SUBROUTINE DIFMAG
