!=====================================================================
PROGRAM MAS2BC
!=====================================================================
  USE comoly
  USE comdim
  USE comphy
  USE comglo
  USE comreg
  USE combnd
  USE commet
  USE netcdf
  IMPLICIT NONE 
!---------------------------------------------------------------------
  character file_b*80, file_v*80
  character lsource*8, ldata*8, lstart*10, lstop*10
  integer   isource,   idata,   istart
!---------------------------------------------------------------------
  integer icrnum
  integer id
  integer iyear, imonth, iday
  integer j, j1, j2, j3, jrec
  integer nbrad
!---------------------------------------------------------------------
  REAL(KIND=8) :: mjd_cr
  REAL(KIND=8) :: timpas, time_cld(22,5)
  REAL(KIND=8) :: zdt, ztimcld, timcld(5), xcld(5)
  REAL(KIND=8) :: dcld(5), tcld(5), vcld(5), radcld(5), x2cld(5), x3cld(5)
  REAL(KIND=8) :: dfast, tfast, vfast, bfast, dvexp, vsint, shift
  REAL(KIND=8) :: zshift, zx2, zx3, zrad, zzz
  REAL(KIND=8) :: z2cld, z3cld, zrcld
  REAL(KIND=8) :: dh1, dh2, dh3
  REAL(KIND=8) :: vud, vut, vuv, vub, trot_day, trot_sec, vrot_radsec
  INTEGER  ncloud, ncld, icld, irec, ireco, nrec, igrd, ibnd
  INTEGER  idin, idir, jc, jcloud
  REAL(KIND=8), DIMENSION(ncc1) :: xcc1
  REAL(KIND=8), DIMENSION(ncc2) :: xcc2
  REAL(KIND=8), DIMENSION(ncc3) :: xcc3
  REAL(KIND=8), DIMENSION(nhh1) :: xhh1
  REAL(KIND=8), DIMENSION(nhh2) :: xhh2
  REAL(KIND=8), DIMENSION(nhh3) :: xhh3
!---------------------------------------------------------------------
  REAL(KIND=8), DIMENSION(10) :: timebo_rec, timebn_rec
  REAL(KIND=8), DIMENSION(ncc2,ncc3,10) :: dbn_rec
  REAL(KIND=8), DIMENSION(ncc2,ncc3,10) :: tbn_rec
  REAL(KIND=8), DIMENSION(ncc2,ncc3,10) :: v1bn_rec
  REAL(KIND=8), DIMENSION(ncc2,ncc3,10) :: b1bn_rec
  REAL(KIND=8), DIMENSION(ncc2,ncc3,10) :: b3bn_rec
  REAL(KIND=8), DIMENSION(ncc2,ncc3,10) :: dpbn_rec
  REAL(KIND=8), DIMENSION(ncc2,ncc3,10) :: bpbn_rec
!---------------------------------------------------------------------
  INTEGER, PARAMETER :: ntheta_b=70, nphi_b=65, ntheta_v=91, nphi_v=181
  REAL(KIND=8), DIMENSION(ntheta_b) :: theta_b
  REAL(KIND=8), DIMENSION(ntheta_v) :: theta_v
  REAL(KIND=8), DIMENSION(nphi_b) :: phi_b
  REAL(KIND=8), DIMENSION(nphi_v) :: phi_v
  REAL(KIND=8), DIMENSION(ntheta_b,nphi_b) :: b
  REAL(KIND=8), DIMENSION(ntheta_v,nphi_v) :: v
!---------------------------------------------------------------------
  NAMELIST/NAMDAT/                                        &
    lsource, ldata, lstart, lstop,                        &
    LDIR, LPROJ, LBND, LINI, LGRD,                        &
    dfast, tfast, vfast, bfast, dvexp, shift, nbrad,      &
    timcld, x2cld, x3cld, radcld, xcld, vcld, dcld, tcld, &
    ncld, ncloud, x1l, x1r, x2l, x2r, x3l, x3r
!---------------------------------------------------------------------
!      1.        Clear variables and set parameters
!        1.1       Clear labels
  do j=1,120
    lblank(j:j)=' '
  enddo
  lalph='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
  lnum='0123456789'
  ldir=lblank(1:52)
  lajob=lblank(1:80)
  lgrd=lblank(1:12)
  lbnd=lblank(1:8)
  lsource=lblank(1:8)
  ldata=lblank(1:8)
  lstart=lblank(1:8)
  lstop=lblank(1:8)
!        1.2       Default computational region
  x1l=0.14d0
  x1r=1.07d0
  x2l=30d0
  x2r=150d0
  x3l=0d0
  x3r=360d0
!        1.3       Constants
  gamma=1.5d0
  xalpha=0d0
!        1.4       Fast stream paramaters
  dfast=150d0
  tfast=0.7d0
  vfast=650d0
  bfast=170d0
  dvexp=2d0
!        1.4       Plasma cloud parameters
  timcld(1)=13d0
  x2cld(1)=90d0
  x3cld(1)=180d0
  radcld(1)=25d0
  xcld(1)=1d0
  dcld(1)=6d0
  tcld(1)=1d0
  vcld(1)=650d0
  ncld=0
  ncloud=0
!        1.5       Azimuthal shift on source surface (days)
  shift=0d0
!        1.6       Modification of magnetic field
  nbrad=2
!        1.7       Units
  vux1=1.496d11
  vux2=0.01745329252d0
  vux3=0.01745329252d0
  vutime=3600d0
  vud=1d6
  vuv=1d3
  vut=1d6
  vub=1d-9
!---------------------------------------------------------------------
!      2.          Read in the JOB input parameters
!        2.1       Read in heading
  read(*,'(A)',end=901,err=902) lajob
  write(*,'(a)') '=== MAS2BC: '//lajob
!        2.2       Read in parameters
  read(*,NAMDAT,end=901,err=902)
!---------------------------------------------------------------------
!      3.        Process the JOB input parameters
!        3.1       Carrington rotation number
  icrnum=0
  do j=1,10
    if(lnum(j:j).eq.ldata(1:1)) icrnum=(j-1)*1000
  enddo
  do j=1,10
    if(lnum(j:j).eq.ldata(2:2)) icrnum=icrnum+(j-1)*100
  enddo
  do j=1,10
    if(lnum(j:j).eq.ldata(3:3)) icrnum=icrnum+(j-1)*10
  enddo
  do j=1,10
    if(lnum(j:j).eq.ldata(4:4)) icrnum=icrnum+j-1
  enddo
  write(*,*) '   crnum = ',icrnum
!        3.2       Calendar date
  iyear=0
  do j=1,10
    if(lnum(j:j).eq.lstart(1:1)) iyear=(j-1)*1000
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(2:2)) iyear=iyear+(j-1)*100
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(3:3)) iyear=iyear+(j-1)*10
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(4:4)) iyear=iyear+j-1
  enddo
  imonth=0
  do j=1,10
    if(lnum(j:j).eq.lstart(6:6)) imonth=imonth+(j-1)*10
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(7:7)) imonth=imonth+j-1
  enddo
  iday=0
  do j=1,10
    if(lnum(j:j).eq.lstart(9:9)) iday=iday+(j-1)*10
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(10:10)) iday=iday+j-1
  enddo
!        3.3       Length of file names
  idir=len_trim(ldir)
  isource=len_trim(lsource)
  idata=len_trim(ldata)
  istart=len_trim(lstart)
  ibnd=len_trim(lbnd)
  igrd=len_trim(lgrd)
!        3.4       Date of CR beginning
  mjd_cr=(dble(icrnum)-1750d0)*27.2753d0+45871.41d0
!        3.4       Reference date
  if(istart.gt.0) then
    if(imonth.le.2) then
      iyear=iyear-1
      imonth=imonth+1+12
    else
      imonth=imonth+1
    endif
    refdate=int(365.25d0*iyear)+int(30.6001d0*imonth)+iday-679004d0
    refdate=refdate-int(0.01d0*iyear)+int(0.25d0*int(0.01d0*iyear))
  else
    refdate=mjd_cr
  endif
  write(*,*) '   refdate_mjd = ',refdate
  write(*,*) '   mjd_cr      = ',mjd_cr
!        3.5       Input directory and file names
  if(isource.gt.0) then
    ldin=ldir(1:idir)//'/'//lsource(1:isource)//'/cr'//ldata(1:idata)
    idin=idir+1+isource+3+idata
    file_b=ldin(1:idin)//'/br_helio_scaled.dat'
    file_v=ldin(1:idin)//'/v_smooth_0.05_0.025.dat'
  else
    file_b=ldir(1:idir)//'/br_helio_scaled.dat'
    file_v=ldir(1:idir)//'/v_smooth_0.05_0.025.dat'
  endif
!        3.6       Output directory and file names
  lfgrd='grd.nc'
  mfgrd=6
  lfbnd='bnd.nc'
  mfbnd=6
!---------------------------------------------------------------------
!      4.        Computational region
!        4.1       Region boundaries
  x1l=x1l*vux1
  x1r=x1r*vux1
  x2l=x2l*vux2
  x2r=x2r*vux2
  x3l=x3l*vux3
  x3r=x3r*vux3
!        4.2       Heliospheric grid
  dh1=(x1r-x1l)/dble(ncc1)
  dh2=(x2r-x2l)/dble(ncc2)
  dh3=(x3r-x3l)/dble(ncc3)
  do j1=1,nhh1
    xhh1(j1)=x1l+dh1*dble(j1-1)
  enddo
#ifdef x2dim
    do j2=1,nhh2
      xhh2(j2)=x2l+dh2*dble(j2-1)
    enddo
#else
    xhh2(1)=x2l
    xhh2(2)=x2r
#endif
#ifdef x3dim
    do j3=1,nhh3
      xhh3(j3)=x3l+dh3*dble(j3-1)
    enddo
#else
    xhh3(1)=x3l
    xhh3(2)=x3r
#endif
!        4.5       Heliospheric grid centers
  do j1=1,ncc1
    xcc1(j1)=0.5d0*(xhh1(j1+1)+xhh1(j1))
  enddo
  do j2=1,ncc2
    xcc2(j2)=0.5d0*(xhh2(j2+1)+xhh2(j2))
  enddo
  do j3=1,ncc3
    xcc3(j3)=0.5d0*(xhh3(j3+1)+xhh3(j3))
  enddo
!        4.6       Boundary grid centers
  x1bn=x1l
#ifdef x2dim
    do j2=1,ncc2
      x2bn(j2)=xcc2(j2)
    enddo
    do j2=1,nhh2
      x2bnh(j2)=xhh2(j2)
    enddo
#else
    x2bn(1)=0.5d0*(x2l+x2r)
    x2bnh(1)=x2l
    x2bnh(2)=x2r
#endif
#ifdef x3dim
    do j3=1,ncc3
      x3bn(j3)=xcc3(j3)
    enddo
    do j3=1,nhh3
      x3bnh(j3)=xhh3(j3)
    enddo
#else
    x3bn(1)=0.5d0*(x3l+x3r)
    x3bnh(1)=x3l
    x3bnh(2)=x3r
#endif
!        4.7       Write numerical grid as the NetCDF file
  call OUTGRD(xcc1,xcc2,xcc3,xhh1,xhh2,xhh3)
!---------------------------------------------------------------------
!      5.        Physical parameters
!        5.1       Constants
  gammam=gamma-1d0
  rgas=((2d0+3d0*xalpha)/(1d0+4d0*xalpha))*(ck/cmass)
  xmass=(1d0+4d0*xalpha)*cmass
!        5.2       Convert to metric SI units
  dfast=xmass*dfast*vud
  tfast=tfast*vut
  vfast=vfast*vuv
  bfast=bfast*vub
  dcld(:)=dcld(:)*dfast
  tcld(:)=tcld(:)*tfast
  vcld(:)=vcld(:)*vuv
  x2cld(:)=x2cld(:)*vux2
  x3cld(:)=x3cld(:)*vux3
  radcld(:)=radcld(:)*vux2
  timcld(:)=timcld(:)*vutime
!        5.3       Solar rotation
  trot_day=27.2753d0
  trot_sec=trot_day*24d0*3600d0
  vrot_radsec=360d0*vux3/trot_sec
!---------------------------------------------------------------------
!      6.        Read in SAIC source-surface data
!        6.1       Radial magnetic field
  write(*,'(a)') '--- input boundary file: '//file_b
  open(1,file=file_b,status='old')
  read(1,*) id,j,j,j,theta_b,phi_b,b
  close(1)
!        6.2       Radial velocity
  write(*,'(a)') '--- input boundary file: '//file_v
  open(1,file=file_v,status='old')
  read(1,*) id,j,j,j,theta_v,phi_v,v
  close(1)
!        6.3       Convert to metric SI units
  b=b*2.205d-4
  v=v*481.37d3 
!        6.4       Shift to account solar rotation
  if((shift.gt.0d0).and.(trot_day.gt.0d0)) then
    zshift=2d0*3.1415927d0*shift/trot_day
  else
    zshift=0d0
  endif
!        6.5       Interpolate values onto new grid
  call INTERP(b,theta_b,phi_b,ntheta_b,nphi_b,b1bo,x2bn,x3bn, &
    nbb2,nbb3,zshift)
  call INTERP(v,theta_v,phi_v,ntheta_v,nphi_v,v1bo,x2bn,x3bn, &
    nbb2,nbb3,zshift)
!---------------------------------------------------------------------
!      7.        Determine the background boundary conditions
!        7.1       Set density and temperature
  do j3=1,nbb3
    do j2=1,nbb2
      dbo(j2,j3)=dfast*(vfast/v1bo(j2,j3))**dvexp
      tbo(j2,j3)=dfast*tfast/dbo(j2,j3)
    enddo
  enddo
!        7.2       Modify magnetic field values
  if(nbrad.eq.0) then
    do j2=1,nbb2
      vsint=x1l*vrot_radsec*sin(x2bn(j2))
      do j3=1,nbb3
        if(b1bo(j2,j3).lt.0d0) then
          b1bo(j2,j3)=-b1bo(j2,j3)
          bpbo(j2,j3)=-1d0
        else
          bpbo(j2,j3)=1d0
        endif
        b3bo(j2,j3)=-b1bo(j2,j3)*vsint/v1bo(j2,j3)
      enddo
    enddo
  endif
  if(nbrad.eq.1) then
    zrad=(x1l-0.5d0*(xcc1(2)-xcc1(1)))/ &
         (x1l+0.5d0*(xcc1(2)-xcc1(1)))
    zrad=zrad*zrad
    do j2=1,nbb2
      vsint=x1l*vrot_radsec*sin(x2bn(j2))
      do j3=1,nbb3
        if(b1bo(j2,j3).lt.0d0) then
          b1bo(j2,j3)=-b1bo(j2,j3)
          bpbo(j2,j3)=-1d0
        else
          bpbo(j2,j3)=1d0
        endif
        b3bo(j2,j3)=-b1bo(j2,j3)*vsint/v1bo(j2,j3)
      enddo
    enddo
  endif
  if(nbrad.eq.2) then
    do j2=1,nbb2
      vsint=x1l*vrot_radsec*sin(x2bn(j2))
      do j3=1,nbb3
        if(b1bo(j2,j3).lt.0d0) then
          b1bo(j2,j3)=-b1bo(j2,j3)
          bpbo(j2,j3)=-1d0
        else
          bpbo(j2,j3)=1d0
        endif
        b3bo(j2,j3)=-bfast*vsint/(0.75d0*vfast)
        b1bo(j2,j3)=-b3bo(j2,j3)*v1bo(j2,j3)/vsint
      enddo
    enddo
  endif
!---------------------------------------------------------------------
!      8.        Time levels
!        8.2       For plasma cloud-1
  if(ncld.eq.1) then
    do jcloud=1,ncloud
      timpas=x1l*2d0*radcld(jcloud)/vcld(jcloud)
      time_cld(1,jcloud)=timcld(jcloud)-0.5d0*timpas-1800d0
      time_cld(2,jcloud)=timcld(jcloud)-0.5d0*timpas+1800d0
      time_cld(3,jcloud)=timcld(jcloud)+0.5d0*timpas-1800d0
      time_cld(4,jcloud)=timcld(jcloud)+0.5d0*timpas+1800d0
      icld=icld+4
    enddo
  endif
!        8.2       For plasma cloud-2
  if(ncld.eq.2) then
    do jcloud=1,ncloud
      timpas=x1l*2d0*radcld(jcloud)/vcld(jcloud)
      if(xcld(jcloud).le.1d0) then
        zdt=xcld(jcloud)*timpas/dble(21)
        do j=1,22
          time_cld(j,jcloud)=timcld(jcloud)-0.5d0*timpas+dble(j-1)*zdt
        enddo
      else
        zdt=timpas/dble(21)
        do j=1,11
          time_cld(j,jcloud)=timcld(jcloud)-0.5d0*timpas+dble(j-1)*zdt
        enddo
        ztimcld=timcld(jcloud)+(xcld(jcloud)-1d0)*timpas
        do j=12,22
          time_cld(j,jcloud)=ztimcld-0.5d0*timpas+dble(j-1)*zdt
        enddo
      endif
    enddo
  endif
!        8.3       Write basic information
  if(ncloud.gt.0) then
    do jcloud=1,ncloud
      write(*,'(a7,i2,2x,a6,2(1pe11.3),2(2x,a5,1pe11.3))')           &
       'cloud =',jcloud,                                             &
       'time =',time_cld(1,jcloud)/3600.,time_cld(22,jcloud)/3600.,  &
       'rad =',radcld(jcloud)*180./3.141529,'vel =',vcld(jcloud)/1000.
    enddo
  endif
!---------------------------------------------------------------------
!      8.        Time levels
!        8.1       For ambient state
  nrec=6
  do jrec=1,nrec
    if(jrec.eq.1) time=-2.5d0*trot_sec
    if(jrec.eq.2) time=-1.5d0*trot_sec
    if(jrec.eq.3) time=-0.5d0*trot_sec
    if(jrec.eq.4) time=0.5d0*trot_sec
    if(jrec.eq.5) time=1.5d0*trot_sec
    if(jrec.eq.6) time=2.5d0*trot_sec
    time=time-(refdate-mjd_cr)*86400d0
    timebn_rec(jrec)=time
    dbn_rec(:,:,jrec)=dbo(:,:)
    tbn_rec(:,:,jrec)=tbo(:,:)
    v1bn_rec(:,:,jrec)=v1bo(:,:)
    b1bn_rec(:,:,jrec)=b1bo(:,:)
    b3bn_rec(:,:,jrec)=b3bo(:,:)
#ifdef hydcon
    dpbn_rec(:,:,jrec)=dpbo(:,:)
#endif
#ifdef mhdcon
    bpbn_rec(:,:,jrec)=bpbo(:,:)
#endif
  enddo
!---------------------------------------------------------------------
!      8.        Write boundary values as the NetCDF file
  if(ncloud.gt.0) then
    do jcloud=1,ncloud
      do jc=1,22
        ireco=1
        irec=1
        timebo_rec=timebn_rec
        do while(ireco.lt.6+jcloud*22-1)
          ireco=ireco+1
          irec=irec+1
          if((time_cld(jc,jcloud).ge.timebo_rec(ireco-1)).and. &
            (time_cld(jc,jcloud).lt.timebo_rec(ireco))) then
            timebn_rec(irec)=time_cld(jc,jcloud)
            irec=irec+1
          endif
          timebn_rec(irec)=timebo_rec(ireco)
        enddo
      enddo
    enddo
    nrec=irec
  endif
!---------------------------------------------------------------------
!      8.        Write boundary values as the NetCDF file
  do jrec=3,nrec
!        8.1       Transient disturbances
    time=timebn_rec(jrec)
    zshift=-6.2831854d0*(time-timebn_rec(2))/trot_sec
    if(zshift.gt.6.2831854d0) zshift=zshift-6.2831854d0
    if(zshift.lt.0d0) zshift=zshift+6.2831854d0
    if(zshift.lt.0d0) zshift=zshift+6.2831854d0
    if(zshift.lt.0d0) zshift=zshift+6.2831854d0
    if(zshift.lt.0d0) zshift=zshift+6.2831854d0
!Interpolate values onto new grid
    call INTERP(dbo,x2bn,x3bn,nbb2,nbb3,dbn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(tbo,x2bn,x3bn,nbb2,nbb3,tbn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(v1bo,x2bn,x3bn,nbb2,nbb3,v1bn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(b1bo,x2bn,x3bn,nbb2,nbb3,b1bn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(b3bo,x2bn,x3bn,nbb2,nbb3,b3bn,x2bn,x3bn,nbb2,nbb3,zshift)
#ifdef mhdcon
    call INTERP(bpbo,x2bn,x3bn,nbb2,nbb3,bpbn,x2bn,x3bn,nbb2,nbb3,zshift)
#endif
!Plasma cloud parameters
#ifdef hydcon
!   dpbn(:,:)=0d0
    do j3=1,ncc3
      do j2=1,ncc2
        dpbn(j2,j3)=0d0
      enddo
    enddo
#endif
    if(ncloud.gt.0) then
      do jcloud=1,ncloud
        if((timebn_rec(jrec).gt.time_cld(1,jcloud)).and. &
           (timebn_rec(jrec).lt.time_cld(22,jcloud))) then
          z2cld=x2cld(jcloud)
          z3cld=x3cld(jcloud)
          zzz=2d0*(timcld(jcloud)-timebn_rec(jrec))/ &
            (time_cld(22,jcloud)-time_cld(1,jcloud))
          zrcld=radcld(jcloud)*sqrt(1d0-abs(zzz))
          do j3=1,ncc3
            do j2=1,ncc2
              zx2=x2bn(j2)-z2cld
              zx3=x3bn(j3)-z3cld
              zrad=sqrt(zx2**2+zx3**2)
              if(zrad.le.zrcld) then
                if(dbn(j2,j3).lt.dcld(jcloud)) dbn(j2,j3)=dcld(jcloud)
                if(tbn(j2,j3).lt.tcld(jcloud)) tbn(j2,j3)=tcld(jcloud)
                if(v1bn(j2,j3).lt.vcld(jcloud)) v1bn(j2,j3)=vcld(jcloud)
#ifdef hydcon
                dpbn(j2,j3)=dbn(j2,j3)
#endif
              endif
            enddo
          enddo
        endif
      enddo
    endif
    dbn_rec(:,:,jrec)=dbn(:,:)
    tbn_rec(:,:,jrec)=tbn(:,:)
    v1bn_rec(:,:,jrec)=v1bn(:,:)
    b1bn_rec(:,:,jrec)=b1bn(:,:)
    b3bn_rec(:,:,jrec)=b3bn(:,:)
#ifdef hydcon
    dpbn_rec(:,:,jrec)=dpbn(:,:)
#endif
#ifdef mhdcon
    bpbn_rec(:,:,jrec)=bpbn(:,:)
#endif
  enddo
!---------------------------------------------------------------------
!      8.        Write boundary values as the NetCDF file
  write(*,'(a)') '--- heliospheric boundary file: '//lfbnd(1:mfbnd)
  do jrec=1,nrec
!        8.1       Time-dependent values
    time=timebn_rec(jrec)
    dbn(:,:)=dbn_rec(:,:,jrec)
    tbn(:,:)=tbn_rec(:,:,jrec)
    v1bn(:,:)=v1bn_rec(:,:,jrec)
    b1bn(:,:)=b1bn_rec(:,:,jrec)
    b3bn(:,:)=b3bn_rec(:,:,jrec)
#ifdef hydcon
    dpbn(:,:)=dpbn_rec(:,:,jrec)
#endif
#ifdef mhdcon
    bpbn(:,:)=bpbn_rec(:,:,jrec)
#endif
!        8.2       Write NetDCF data
    call OUTBND(jrec)
  enddo
!---------------------------------------------------------------------
!      9.        Stop
  write(*,'(a)') '=== END OF MAS2BC'
STOP
  901 continue
      write (*,'(a)') '*** End of input file'
      goto 903
  902 continue
      write (*,'(a)') '*** Error in input file'
  903 continue
      write (*,'(a)') '*** Error in program MAS2BC'
STOP
!---------------------------------------------------------------------
END PROGRAM MAS2BC
