;===============================================================
;+
; NAME:
;    color_bar
; PURPOSE:
;    Produce a color bar
; CATEGORY:
;    Graphics
; CALLING SEQUENCE:
;    color_bar, prange
; INPUTS:
;    None
; INPUT KEYWORD PARAMETERS:
;    nlevels     No. of contour levels
;                (default: nlevels=29)
;    levels      Contour levels
;    min         Minimum value of the scale
;    max         Maximum value of the scale
;    bars        Plot bars at bar levels
;    fill        Fill bars by bar colors
;    c_colors    Color indices of the bar levels
;    color       Color index of labels
;    charsize    Character size of labels   
; OUTPUTS:
;    None
; OUTPUT KEYWORD PARAMETERS:
;    None
; COMMON BLOCKS:
;    None
; MODIFICATION HISTORY:
;    D. Odstrcil, Ondrejov, July 1994
;-
;===============================================================
pro color_bar, prange, nlevels=nlevels, levels=levels, $
 title=title, min=min, max=max, bars=bars, fill=fill, $
 b_color=b_color, c_colors=c_colors, color=color, font=font, $
 charsize=charsize, leftside=leftside, righside=rightside
;===============================================================
;Default keywords
  if(not(keyword_set(title))) then title=' '
  if(n_elements(min) eq 0) then min='min'
  if(n_elements(max) eq 0) then max='max'
  if(n_elements(color) eq 0) then begin
    if(!d.name eq 'X') then color=!d.n_colors-1 else color=0
  endif
  if(not(keyword_set(charsize))) then charsize=0.8
  if(n_elements(font) eq 0) then font=-1
;---------------------------------------------------------------
;Color
  if(n_elements(c_colors) eq 0) then $
    c_colors=fix(!d.n_colors*(0.5*indgen(nlevels+1))/ $
      float(nlevels))
;---------------------------------------------------------------
;Auxiliary array
  case 1 of
    (n_elements(levels) eq 0 and n_elements(nlevels) gt 0): $
       levels=findgen(nlevels+2)
    (n_elements(levels) gt 0 and n_elements(nlevels) eq 0): $
       nlevels=n_elements(levels)-2
    else:
  endcase
;---------------------------------------------------------------
;Position
  xyouts,min,charsize=-charsize,width=width_min,/normal
  xyouts,max,charsize=-charsize,width=width_max,/normal
  xyouts,'W',charsize=-charsize,width=pxchar,/normal
  pychar=pxchar*!d.x_size/!d.y_size
;---------------------------------------------------------------
;Position
  px00=prange(0)+width_min+0.75*pxchar
  px11=prange(2)-width_max-0.75*pxchar
  pxdif=(px11-px00)/(levels(nlevels+1)-levels(0))
  py0=prange(1)
  py1=prange(3)
;---------------------------------------------------------------
;Plot a color_bar
; if(keyword_set(bars)) then begin
;   if(n_elements(b_color) eq 1) then begin
;     polyfill,[px00,px11,px11,px00,px00],[py0,py0,py1,py1,py0], $
;/normal,color=b_color
;   endif
; endif
;---------------------------------------------------------------
;Plot a color_bar
  if(keyword_set(fill)) then begin
    for jcolor=0,nlevels do begin
      px0=px00+pxdif*(levels(jcolor)-levels(0))
      px1=px00+pxdif*(levels(jcolor+1)-levels(0))
      polyfill,[px0,px1,px1,px0,px0],[py0,py0,py1,py1,py0], $
        /normal,color=c_colors(jcolor)
    endfor
    plots,[px00,px11,px11,px00,px00],[py0,py0,py1,py1,py0], $
      /normal,color=color
  endif
;---------------------------------------------------------------
;Plot a color_bar
  if(keyword_set(bars)) then begin
    for jcolor=0,nlevels do begin
      px0=px00+pxdif*(levels(jcolor)-levels(0))
      px1=px00+pxdif*(levels(jcolor+1)-levels(0))
      plots,[px0,px1,px1,px0,px0],[py0,py0,py1,py1,py0], $
        /normal,color=b_color
    endfor
    plots,[px00,px11,px11,px00,px00],[py0,py0,py1,py1,py0], $
      /normal,color=b_color
  endif
;---------------------------------------------------------------
;Label a color bar
  xyouts,prange(0),prange(1),min, $
    /normal,alignment=0.0,charsize=charsize,color=color,font=font
  xyouts,prange(2),prange(1),max, $
    /normal,alignment=1.0,charsize=charsize,color=color,font=font
;---------------------------------------------------------------
;Label a color bar
  if(keyword_set(leftside)) then begin
    xyouts,min,charsize=-charsize,width=pxtext,/normal
    xyouts,prange(0)-pxtext-pxchar,prange(1),title, $
      /normal,alignment=1.0,charsize=charsize,color=color,font=font
  endif else begin
    if(keyword_set(rightside)) then begin
      xyouts,max,charsize=-charsize,width=pxtext,/normal
      xyouts,prange(2)+pxtext+pxchar,prange(1),title, $
        /normal,alignment=0.0,charsize=charsize,color=color,font=font
    endif else begin
      xyouts,0.5*(prange(0)+prange(2)),prange(3)+0.75*pychar,title, $
        /normal,alignment=0.5,charsize=charsize,color=color,font=font
    endelse
  endelse
;---------------------------------------------------------------
end
